/*****************************************************************************
 *  Rohde&Schwarz Vector Network Analyzer instrument driver
 *  Callbacks
 *
 *  Original Release: June 2011
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsvna.h"
#include "rsvna_utility.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter



/*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsvna_queryDataUnknownLengthString_ReadCallback
 * Purpose:  This function overrides standard callback function.
 *****************************************************************************/
ViStatus rsvna_queryDataUnknownLengthString_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: rsvna_empty_ReadCallback
 * Purpose:  This function overrides standard callback function. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsvna_empty_ReadCallback(ViSession          instrSession,
                                  ViConstString      repCapName,
                                  RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
* Function: rsvna_quotedInt_ReadCallback
* Purpose:  This function overrides standard callback function. Reads int
*           returned with apostrophes
*****************************************************************************/
ViStatus rsvna_quotedInt_ReadCallback(ViSession          instrSession,
                                      ViConstString      repCapName,
                                      RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_AnyQuotedParamRead(instrSession, repCapName, attr);
}

/*****************************************************************************
 *- Write Callbacks ---------------------------------------------------------*
 *****************************************************************************/

 /*****************************************************************************
  * Function: rsvna_stringToBin_WriteCallback
  * Purpose:  This function overrides standard callback function.
  *****************************************************************************/
#define BYTE_TO_BINARY_PATTERN "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
#define BYTE_TO_BINARY(val)  \
  (val & 0x80000000 ? '1' : '0'), \
  (val & 0x40000000 ? '1' : '0'), \
  (val & 0x20000000 ? '1' : '0'), \
  (val & 0x10000000 ? '1' : '0'), \
  (val & 0x08000000 ? '1' : '0'), \
  (val & 0x04000000 ? '1' : '0'), \
  (val & 0x02000000 ? '1' : '0'), \
  (val & 0x01000000 ? '1' : '0'), \
  (val & 0x00800000 ? '1' : '0'), \
  (val & 0x00400000 ? '1' : '0'), \
  (val & 0x00200000 ? '1' : '0'), \
  (val & 0x00100000 ? '1' : '0'), \
  (val & 0x00080000 ? '1' : '0'), \
  (val & 0x00040000 ? '1' : '0'), \
  (val & 0x00020000 ? '1' : '0'), \
  (val & 0x00010000 ? '1' : '0'), \
  (val & 0x00008000 ? '1' : '0'), \
  (val & 0x00004000 ? '1' : '0'), \
  (val & 0x00002000 ? '1' : '0'), \
  (val & 0x00001000 ? '1' : '0'), \
  (val & 0x00000800 ? '1' : '0'), \
  (val & 0x00000400 ? '1' : '0'), \
  (val & 0x00000200 ? '1' : '0'), \
  (val & 0x00000100 ? '1' : '0'), \
  (val & 0x00000080 ? '1' : '0'), \
  (val & 0x00000040 ? '1' : '0'), \
  (val & 0x00000020 ? '1' : '0'), \
  (val & 0x00000010 ? '1' : '0'), \
  (val & 0x00000008 ? '1' : '0'), \
  (val & 0x00000004 ? '1' : '0'), \
  (val & 0x00000002 ? '1' : '0'), \
  (val & 0x00000001 ? '1' : '0')
ViStatus rsvna_ViInt32ToBin_WriteCallback(ViSession          instrSession,
                                          ViConstString      repCapName,
                                          RsCoreAttributePtr attr,
                                          void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if ((attr->rangeTableDiscrete))
    {
        ViChar* p2cmdString = NULL;

        // Allowed data type only ViInt32
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }

        RsCore_StrCatDynamicString(instrSession, &cmd, " #B", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        ViChar valueString[67] = ""; // Allow 64 bit values plus 3 bytes
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            snprintf(valueString, 67, "#B" BYTE_TO_BINARY_PATTERN, BYTE_TO_BINARY(*(ViInt32*)value));
            break;
        case RS_VAL_INT64:
            snprintf(valueString, 67, "#B" BYTE_TO_BINARY_PATTERN BYTE_TO_BINARY_PATTERN, BYTE_TO_BINARY((*(ViInt64*)value)>>32), BYTE_TO_BINARY(*(ViInt64*)value));
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            snprintf(valueString, 67, "#B%s", (ViString)value);
            break;
        case RS_VAL_BOOLEAN:
            snprintf(valueString, 67, "#B%c", *(ViBoolean*)value ? '1' : '0');
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_REAL64:
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }

        RsCore_StrCatDynamicString(instrSession, &cmd, " ", valueString, NULL);
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}


/*****************************************************************************
 * Function: rsvna_stringToBin_WriteCallback
 * Purpose:  This function overrides standard callback function.
 *****************************************************************************/
ViStatus rsvna_stringToBin_WriteCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr,
                                         void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " #B", value, "");
}

/*****************************************************************************
 * Function: rsvna_empty_WriteCallback
 * Purpose:  This function overrides standard callback function. Does not write
 *           anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rsvna_empty_WriteCallback(ViSession          instrSession,
                                   ViConstString      repCapName,
                                   RsCoreAttributePtr attr,
                                   void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
* Function: rsvna_doubleQuotedInt_WriteCallback
* Purpose:  This function overrides standard callback function. Writes int
*           with apostrophes
*****************************************************************************/
ViStatus rsvna_doubleQuotedInt_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " \"", value, "\"");
}

/*****************************************************************************
 * Function: rsvna_quotedInt_WriteCallback
 * Purpose:  This function overrides standard callback function. Writes int
 *           with apostrophes
 *****************************************************************************/
ViStatus rsvna_quotedInt_WriteCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr,
                                       void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, "  '", value, "'");
}


/*****************************************************************************
*- Check Callbacks ---------------------------------------------------------*
*****************************************************************************/

/*****************************************************************************
 *- Coerce Callbacks --------------------------------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *- Range Table Callbacks ---------------------------------------------------*
 *****************************************************************************/
