/*****************************************************************************
 *  Rohde&Schwarz Vector Network Analyzer Instrument Driver
 *  LabWindows/CVI Instrument Driver
 *  Original Release: 2011-06-17
 *  By: Martin Krcmar
 *
 *  Modification History:
 *
 *       See ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "rsvna.h"
#include "rsvna_internal.h"

#if defined (_CVI_)
#include <utility.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString inOutModeArr[] = {"S", "D", "C",NULL};
static ViString ratiosArr[] = {"B1/A1", "B2/A1", "B2/A2", "B1/A2",NULL};
static ViString numeratorArr[] = {"A", "B", "AP", "BP",NULL};
static ViString detectorArr[] = {"RMS", "PEAK", "AVG",NULL};
static ViString wquantityArr[] = {"A1D1", "B1D1", "B2D1", "A2D2", "B1D2", "B2D2",NULL};
static ViString dcvArr[] = {"DC+1V", "DC+10V",NULL};
static ViString exportFormatArr[] = {"UNF", "FORM",NULL};
static ViString exportDataArr[] = {"COMP", "LINP", "LOGP",NULL};
static ViString decimalSeparatorArr[] = {"POIN", "COMM", NULL};
static ViString fieldSeparatorArr[] = {"SEM", "COMM", "TAB", "SPAC"};
static ViString calTypeArr[] =
{
    "FNP", "SFTP", "FOP", "OPTP", "REFL", "RSH", "FRTR", "TOSM", "TOM",
    "TSM", "TRM", "TRL", "TNA", "UOSM", "FTR", "RTR", "PTOS", "PTOM",
    "PTSM", "PTRM", "PTRL", "PTNA", "PUOS", "ART", "ARPT", "PFNP", "VUOS",
    "VPU", "CPU", "MTOS", "MUOS", "MOSM", "MTRL", "UMD","PUMD", NULL
};
static ViString calAutoAssignmentArr[] =
{
    "FNP", "", "FOP", "OPTP", "REFL", "RSH", "FRTR", "TOSM", "",
    "", "", "", "", "", "FTR", "RTR", "", "",
    "", "", "", "", "", "", "", "PFNP", "VUOS", "VPU", "CPU",
    "", "", "", "", "UMD","PUMD", NULL
};
static ViString calAutoTypeArr[] =
{
    "FNP", "SFTP", "FOP", "OPTP", "", "RSH", "FRTR", "TOSM", "",
    "", "", "", "", "", "FTR", "RTR", "", "",
    "", "", "", "", "", "", "", "PFNP", "VUOS", "", "CPU",
    "", "", "", "", "UMD","PUMD", NULL
};
//static ViString generalStandardArr[] = {
//    "THR", "OPEN", "SHOR", "MATC", "NET", "ATT", "REFL", "LINE",
//    "LINE2", "LINE3", "SLID", "OSH", "OSH2", "OSH3", "POW",NULL
//};
static ViString calibrationStandardArr[] = {"UTHR", NULL};
static ViString offOnArr[] = {"OFF", "ON",NULL};
static ViString propagationModeArr[] = {"TEM", "WGUI",NULL};
static ViString connectorTypeArr[] = {"GEND", "NGEN",NULL};
static ViString connectorGenderArr[] = {"MALE", "FEM",NULL};
static ViString limitTypeArr[] = {"OFF", "LMAX", "LMIN", NULL};
static ViString horizontalPositionArr[] = {"LEFT", "MID", "RIGH", NULL};
static ViString verticalPositionArr[] = {"TOP", "MID", "BOTT", NULL};
static ViString limitCheckArr[] = {"", "UPP:", "LOW:", "",NULL};
static ViString loadCalKitStandardArr[] =
{
    "MMTH", "FFTH", "MFTH", "MML", "FFL", "MFL",
    "MMLINE2", "FFLINE2", "MFLINE2", "MMLINE3", "FFLINE3",
    "MFLINE3", "MOSH", "FOSH", "MOSHORT2", "FOSHORT2",
    "MOSHORT3", "FOSHORT3", "MMAT", "FFAT", "MFAT",
    "MMSN", "FFSN", "MFSN", "MOP", "FOP", "MSH", "FSH",
    "MREF", "FREF", "MMTC", "FMTC", "MSM", "FSM",NULL
};
static ViString sParameterArr[] = {"S11", "S12", "S21", "S22", NULL};
static ViString connectorKitArr[] = {"N50", "N75", "PC7", "PC35", "PC292", "USER1", "USER2", "SMA",NULL};
static ViString approxArr[] = {"OPEN", "SHORT", "MATC",NULL};
static ViString triggerMeasSequenceArr[] = {"SWE", "SEGM", "POIN", "PPO",NULL};
static ViString multipleTriggerSourceArr[] = {"IMM", "EXT1", "EXT2", "E1A2", "E1O2", "MAN", "EXTA", "EXTB", "EXTC", "EXTD", "UPOR", NULL};
static ViString slopeLevelArr[] = {"NEG", "POS", "HIGH", "LOW",NULL};
static ViString functionTypeArr[] = {"DEEM", "EMB",NULL};
static ViString circuitModelBArr[] =
{
    "FIMP", "STSL", "STSC", "SLST", "SCST", "CSSL", "LSSC",
    "CSSC", "LSSL", "SLCS", "SCLS", "SCCS", "SLLS", "STSG",
    "SGST", "GSSL", "LSSG", "GSSG", "SLGS", "SGLS", "SGGS",NULL
};
static ViString circuitModelSArr[] =
{
    "FIMP", "CSL", "LSC", "CSC", "LSL", "SLC", "SCL", "SCC", "SLL", "SHLC",
    "GSL", "LSG", "GSG", "SGL", "SLG", "SGG",NULL
};
static ViString portCircuitParameterArr[] = {"PMA", "PSEC",NULL};
static ViString traceStyleArr[] = {"SOL", "DASH", "DOTT", "DDOT", "DDD",NULL};
static ViString PAETypeArr[] = {"", "Main", "Secondary",NULL};
static ViString PAEParameterTypeArr[] = {"I", "R", "U",NULL};
static ViString sweepAxisTypeArr[] = {"Frequency", "Power",NULL};
static ViString calSourceArr[] = {"PORT", "GEN", "A1", "A2", "A3", "A4", "ESRC1", "ESRC2", NULL};
static ViString offsetParameterArr[] = {"ONLY", "CPAD",NULL};
static ViString receiverPowerCalArr[] = {"AWAV", "BWAV", "B1", "B2", "B3", "B4",NULL};
static ViString sweepTypeConvArr[] = {"SWE", "CW",NULL};
static ViString AGCModeArr[] = {"AUTO", "LNO", "LDIS", NULL};
static ViString mixerPortArr[] = {"RF", "LO", "LO1", "LO2", "IF", NULL};
//static ViString PowerCalSettingArr[] =
//{
//    "WAVE", "STAR", "STOP", "POIN", "STYP", "ATT", "CPOW", "CFR",
//    "TST", "TVNA", "MVNA", "MTES",NULL
//};
static ViString SignalRoutingArr[] = {"Index", "Rtrigger", NULL};
static ViString genderArr[] = {"NGEN", "MALE", "FEM", NULL};
static ViString detailArr[] = {"CONN", "LAB", "NAME", "GEND", NULL};
static ViString interchangeArr[] = {"FPOR", "IPOR", NULL};
static ViString circuitModelGLArr[] = {"FIMP", "SL", "SC", "SG", NULL};
static ViString circuitModelDiffArr[] = {"FIMP", "SHLC", NULL};
static ViString impedanceModeArr[] = {"CIMP", "PIMP", NULL};
static ViString delayModeArr[] = {"DEL", "ELEN", NULL};
static ViString sourceTypeArr[] = {"PORT","GEN","CLO", NULL};
ViString dataFormatArr[] = { "SDAT", "FDAT", "MDAT", "TSD", "NCD", "UCD" };
static ViString THDFormatArr[] = {"THDF","THDR", NULL};
static ViString TwoToneGroupDelayArr[] = {"MixDly","MixPhas","MixDeriv",
       "Mix_a_LTone","Mix_a_UTone","Mix_b_LTone","Mix_b_Utone", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsvna_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsvna_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsvna_init(ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsvna_InitWithOptions(resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsvna_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsvna_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsvna", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        rsvna_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsvna_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "NRP33SN", VI_FALSE: "NRP" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSVNA_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version for this driver is 4.3.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 3, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsvna_RsCoreRepCapTable, rsvna_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "NRP33SN", false : "NRP"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSVNA_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSVNA_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsvna_reset(instrSession));
    }
    else
    {
        checkErr(rsvna_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSVNA_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsvna_SetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr    attributeId,
                                            ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsvna_SetAttributeViInt64(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr    attributeId,
                                            ViInt64   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsvna_SetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr    attributeId,
                                             ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsvna_SetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr    attributeId,
                                             ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsvna_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  channelName,
                                              ViAttr    attributeId,
                                              ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsvna_GetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr    attributeId,
                                            ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsvna_GetAttributeViInt64(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr    attributeId,
                                            ViInt64*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsvna_GetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr    attributeId,
                                             ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufSize/Attribute Value parameter.
/// HIPAR value/The buffer in which the function returns the current value of the
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsvna_GetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr    attributeId,
                                             ViInt32   bufferSize,
                                             ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
                                        value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsvna_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   channelName,
                                              ViAttr     attributeId,
                                              ViBoolean* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function synchronizes the selected repeated capability with
/// HIFN  the current instrument's state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR repeatedCapability/Selects the repeated capability.
ViStatus _VI_FUNC rsvna_SynchronizeDynamicRepeatedCapability(ViSession instrSession,
                                                             ViInt32   repeatedCapability)
{  
    ViStatus error = VI_SUCCESS;
    ViInt32  repCap_list[1024];
    ViInt32  repCap_value = 0;
    ViInt32  i = 0;
    ViChar   buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = NULL;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repeatedCapability, RSVNA_VAL_ALL, RSVNA_VAL_GROUP),
            2, "Repeated Capability");

    // Channel repeated capability
    if ((repeatedCapability == RSVNA_VAL_CHANNEL || repeatedCapability == RSVNA_VAL_ALL))
    {
        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Channel' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "Channel", rsvna_RsCoreRepCapTable));

        // get all active channels
        checkErr(rsvna_ChannelList(instrSession, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Ch1,2,New_Channel'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        // enter active channel numbers as 'Channel' rep caps
        while (repCap_value < i)
        {
            snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, repCap_list[repCap_value]);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_list[repCap_value]);
            checkErr(RsCore_AddRepCap(instrSession, "Channel", repCap, repCap_cmd));

            repCap_value++;
        }
    }

    // Window repeated capability
    if ((repeatedCapability == RSVNA_VAL_WINDOW || repeatedCapability == RSVNA_VAL_ALL))
    {
        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Window' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "Window", rsvna_RsCoreRepCapTable));

        // get all active Windows
        checkErr(rsvna_DiagramAreaCatalog(instrSession, 1, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Window 1,2,Another Window'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        // enter active window numbers as 'Window' rep caps
        while (repCap_value < i)
        {
            snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", repCap_list[repCap_value]);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_list[repCap_value]);
            checkErr(RsCore_AddRepCap(instrSession, "Window", repCap, repCap_cmd));

            repCap_value++;
        }
    }

    // Segment repeated capability
    if ((repeatedCapability == RSVNA_VAL_SEGMENT || repeatedCapability == RSVNA_VAL_ALL))
    {
        ViInt32 channel = 0;
        ViInt32 segment = 0;
        ViInt32 max_segment = 0;

        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Segment' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "Segment", rsvna_RsCoreRepCapTable));

        // get all active channels
        checkErr(rsvna_ChannelList(instrSession, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Ch1,2,New_Channel'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        for (channel = 0; channel < i; channel++)
        {
            // get all active Segment for each active channel
            checkErr(rsvna_QuerySweepSegmentNumber(instrSession, channel+1, &segment));
            // store the highest segment number
            if (segment > max_segment) max_segment = segment;
        }

        // enter active window numbers as 'Segment' rep caps
        while (repCap_value < max_segment)
        {
            repCap_value++;

            snprintf(repCap, RS_REPCAP_BUF_SIZE, "Seg%ld", repCap_value);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_value);
            checkErr(RsCore_AddRepCap(instrSession, "Segment", repCap, repCap_cmd));
        }
    }

    // Limit Segment repeated capability
    if ((repeatedCapability == RSVNA_VAL_LIMIT_SEGMENT || repeatedCapability == RSVNA_VAL_ALL))
    {
        ViInt32 channel = 0;
        ViInt32 segment = 0;
        ViInt32 max_segment = 0;

        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Limit Segment' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "LimitSegment", rsvna_RsCoreRepCapTable));

        // get all active channels
        checkErr(rsvna_ChannelList(instrSession, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Ch1,2,New_Channel'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        for (channel = 0; channel < i; channel++)
        {
            // get all active Limit Segment for each active channel
            checkErr(rsvna_QueryLimitLineSegmentCount(instrSession, channel+1, &segment));
            // store the highest limit segment number
            if (segment > max_segment) max_segment = segment;
        }

        // enter active limit segment numbers as 'Limit Segment' rep caps
        while (repCap_value < max_segment)
        {
            repCap_value++;

            snprintf(repCap, RS_REPCAP_BUF_SIZE, "LimitSeg%ld", repCap_value);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_value);
            checkErr(RsCore_AddRepCap(instrSession, "LimitSegment", repCap, repCap_cmd));
        }
    }

    // Ripple Segment repeated capability
    if ((repeatedCapability == RSVNA_VAL_RIPPLE_SEGMENT || repeatedCapability == RSVNA_VAL_ALL))
    {
        ViInt32 channel = 0;
        ViInt32 segment = 0;
        ViInt32 max_segment = 0;

        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Ripple Segment' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "RippleSegment", rsvna_RsCoreRepCapTable));

        // get all active channels
        checkErr(rsvna_ChannelList(instrSession, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Ch1,2,New_Channel'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        for (channel = 0; channel < i; channel++)
        {
            // get all active Ripple Segment for each active channel
            checkErr(rsvna_QueryRippleLimitRangesCount(instrSession, channel+1, &segment));
            // store the highest Ripple segment number
            if (segment > max_segment) max_segment = segment;
        }

        // enter active limit segment numbers as 'Ripple Segment' rep caps
        while (repCap_value < max_segment)
        {
            repCap_value++;

            snprintf(repCap, RS_REPCAP_BUF_SIZE, "RippleSeg%ld", repCap_value);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_value);
            checkErr(RsCore_AddRepCap(instrSession, "RippleSegment", repCap, repCap_cmd));
        }
    }

    // Group repeated capability
    if ((repeatedCapability == RSVNA_VAL_GROUP || repeatedCapability == RSVNA_VAL_ALL))
    {
        ViInt32 channel = 0;
        ViInt32 groups = 0;
        ViInt32 max_groups = 0;

        memset((void*)repCap_list, 0, 1024);
        repCap_value = 0;
        i = 0;
        buffer[0] = '\0';
        repCap[0] = '\0';
        repCap_cmd[0] = '\0';

        // reset 'Group' rep cap
        checkErr(RsCore_ResetRepCapID(instrSession, "Group", rsvna_RsCoreRepCapTable));

        // get all active channels
        checkErr(rsvna_ChannelList(instrSession, buffer, RS_MAX_MESSAGE_BUF_SIZE));

        //parse buffer - '1,Ch1,2,New_Channel'.
        p2buf = strtok(buffer, ",");

        while (p2buf != NULL)
        {
            repCap_list[i++] = atoi(p2buf);
            p2buf = strtok(NULL, ",");
            p2buf = strtok(NULL, ",");
        }

        for (channel = 0; channel < i; channel++)
        {
            // get all active Group for each active channel
            checkErr(rsvna_QueryPortGroupsCount(instrSession, channel+1, &groups));
            // store the highest Group number
            if (groups > max_groups) max_groups = groups;
        }

        // enter active Group numbers as 'Group' rep caps
        while (repCap_value < max_groups)
        {
            repCap_value++;

            snprintf(repCap, RS_REPCAP_BUF_SIZE, "Group%ld", repCap_value);
            snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", repCap_value);
            checkErr(RsCore_AddRepCap(instrSession, "Group", repCap, repCap_cmd));
        }
    }

Error:
    return error;
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsvna_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeId, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsvna_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeId,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN  Manages the list of favorite recall sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR favid/Position in the favorites list.
/// HIPAR fileName/Manages the list of favorite recall sets.
ViStatus _VI_FUNC rsvna_FileListofFavoriteRecallSet(ViSession instrSession,
                                                    ViInt32   favid,
                                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, favid, 1, 100),
            2, "FavId");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FILE_LIST_OF_FAVORITE_RECALL_SET,
            "FavId", favid - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "FavId");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_FILE_LIST_OF_FAVORITE_RECALL_SET, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function manage a recall sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operationToBePerformed/This control selects the type of operation to be performed.
/// HIPAR recallSetName/String parameter to specify the name of the manipulated recall set.
ViStatus _VI_FUNC rsvna_RecallSetManager(ViSession instrSession,
                                         ViInt32   operationToBePerformed,
                                         ViString  recallSetName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operationToBePerformed)
    {
    case RSVNA_VAL_RECALL_SET_NEW:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_NEW, recallSetName),
                3, "Recall Set Name");
        break;

    case RSVNA_VAL_RECALL_SET_OPEN:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_LOAD, recallSetName),
                3, "Recall Set Name");

        checkErr(rsvna_SynchronizeDynamicRepeatedCapability(instrSession, RSVNA_VAL_ALL));

        break;

    case RSVNA_VAL_RECALL_SET_SAVE:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_STORE, recallSetName),
                3, "Recall Set Name");
        break;

    case RSVNA_VAL_RECALL_SET_ACTIVE:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_ACTIVE, recallSetName),
                3, "Recall Set Name");
        break;

    case RSVNA_VAL_RECALL_SET_CLOSE:
        if (recallSetName == NULL)
        {
            checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_CLOSE_ALL, NULL));
        }
        else
        {
            viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_RECALL_SET_CLOSE, recallSetName),
                    3, "Recall Set Name");
        }
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operationToBePerformed), 2, "Operation To Be Performed");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function prints the active recall set on selected printer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR printerName/Define printer name, string variable. One of the printers accessible
/// HIPAR printerName/from your instrument.
ViStatus _VI_FUNC rsvna_Print(ViSession instrSession,
                              ViString  printerName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRINT_PRINTER_NAME, printerName),
            2, "Printer Name");

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRINT_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specify how the screen contents are stored to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies a file name to save the screen contents.
/// HIPAR fileFormat/Selects a file format for printer files. Selecting the format is
/// HIPAR fileFormat/recommended to ensure that the file defined via MMEMory:NAME can be
/// HIPAR fileFormat/read or imported by an external application.
ViStatus _VI_FUNC rsvna_PrintToFile(ViSession instrSession,
                                    ViString  fileName,
                                    ViInt32   fileFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRINT_PRINTER_NAME, "MMEM"));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRINT_FILE_NAME, fileName),
            2, "File Name");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_PRINT_FILE_FORMAT, fileFormat),
            3, "File Format");

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRINT_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function provides options to specify how the document should be
/// HIFN  printed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR diagramArea/Defines the number of diagram areas per printed page.
/// HIPAR useColor/Enables color printing.
/// HIPAR pageOrientation/Defines the orientation of the printed page. Switching between
/// HIPAR pageOrientation/LANDscape and PORTrait rotates the hardcopy result by 90 degrees. No
/// HIPAR pageOrientation/other settings are changed.
/// HIPAR leftMargin/Defines the distance between the left edge of the page and the left
/// HIPAR leftMargin/edge of the printed information.
/// HIPAR rightMargin/Defines the distance between the right edge of the page and the right
/// HIPAR rightMargin/edge of the printed information.
/// HIPAR topMargin/Defines the distance between the top of the page and the top of the
/// HIPAR topMargin/printed information.
/// HIPAR bottomMargin/Defines the distance between the bottom of the page and the bottom of
/// HIPAR bottomMargin/the printed information.
ViStatus _VI_FUNC rsvna_ConfigurePrintPageSetup(ViSession instrSession,
                                                ViInt32   diagramArea,
                                                ViBoolean useColor,
                                                ViInt32   pageOrientation,
                                                ViReal64  leftMargin,
                                                ViReal64  rightMargin,
                                                ViReal64  topMargin,
                                                ViReal64  bottomMargin)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_PRINT_DIAGRAM_AREA, diagramArea),
            2, "Diagram Area");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_PRINT_USE_COLOR, useColor),
            3, "Use Color");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_PRINT_PAGE_ORIENTATION, pageOrientation),
            4, "Page Orientation");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "Left", RSVNA_ATTR_PRINT_PAGE_MARGIN, leftMargin),
            5, "Left Margin");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "Right", RSVNA_ATTR_PRINT_PAGE_MARGIN, rightMargin),
            6, "Right Margin");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "Top", RSVNA_ATTR_PRINT_PAGE_MARGIN, topMargin),
            7, "Top Margin");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "Bottom", RSVNA_ATTR_PRINT_PAGE_MARGIN, bottomMargin),
            8, "Bottom Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the color scheme to be used for printing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR colorScheme/Select the color scheme to be used for printing.
ViStatus _VI_FUNC rsvna_ConfigurePrintColorScheme(ViSession instrSession,
                                                  ViInt32   colorScheme)
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_PRINT_COLOR_SCHEME, colorScheme),
            2, "Color Scheme");

Error:
    return error;
}

/// HIFN  This function provides operations which allows to do basic file
/// HIFN  operations on the storage media of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operationtobePerformed/This control selects the type of operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If
/// HIPAR source/the operation requires a single parameter, this control specifies that
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsvna_FileManager(ViSession instrSession,
                                    ViInt32   operationtobePerformed,
                                    ViString  source,
                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (operationtobePerformed)
    {
    case RSVNA_VAL_FILE_MAN_CDRIVE:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CHANGE_DRIVE, source),
                3, "Source");
        break;

    case RSVNA_VAL_FILE_MAN_CDIR:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CHANGE_DIRECTORY, source),
                3, "Source");
        break;

    case RSVNA_VAL_FILE_MAN_MDIR:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CREATE_DIRECTORY, source),
                3, "Source");
        break;

    case RSVNA_VAL_FILE_MAN_RDIR:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_REMOVE_DIRECTORY, source),
                3, "Source");
        break;

    case RSVNA_VAL_FILE_MAN_COPY:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:COPY '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        break;

    case RSVNA_VAL_FILE_MAN_MOVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:MOVE '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        break;

    case RSVNA_VAL_FILE_MAN_DELETE:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_REMOVE, source),
                3, "Source");
        break;

    case RSVNA_VAL_FILE_MAN_CDIR_DEF:
        checkErr(RsCore_Write(instrSession, "MMEM:CDIR DEF"));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operationtobePerformed), 2, "Operation to be Performed");
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the default directory for mass memory storage
/// HIFN  (current directory).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR currentDirectory/Returns the current directory.
ViStatus _VI_FUNC rsvna_GetCurrentDirectory(ViSession instrSession,
                                            ViChar    currentDirectory[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CHANGE_DIRECTORY,
            RS_MAX_MESSAGE_BUF_SIZE, currentDirectory),
            2, "Current Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the indicated directory. According to DOS
/// HIFN  convention, wild card characters can be entered in order to query e.g.
/// HIFN  a list of all files of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which this function will be
/// HIPAR directory/performed.
/// HIPAR catalog/This indicator returns the files in the indicated directory as comma
/// HIPAR catalog/separated strings, for example:
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_FileCatalog(ViSession instrSession,
                                    ViString  directory,
                                    ViChar    catalog[],
                                    ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (directory == NULL)
    {
        checkErr(rsvna_QueryViString(instrSession, "MMEM:CAT:ALL?", bufferSize, catalog));
    }
    else
    {
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CHANGE_DIRECTORY, directory),
                2, "Directory");

        checkErr(rsvna_QueryViString(instrSession, "MMEM:CAT?", bufferSize, catalog));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rsvna_ReadToFileFromInstrument(ViSession instrSession,
                                                 ViString  source,
                                                 ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, source),
                2, "Source");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, destination),
                3, "Destination");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEMory:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the host computer and
/// HIFN  write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rsvna_WriteFromFileToInstrument(ViSession instrSession,
                                                  ViString  source,
                                                  ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, source),
                2, "Source");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, destination),
                3, "Destination");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function for mechanical matrices allows to query the list of
/// HIFN  switch counts for all relays (if supported by the matrix).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR relayCounts/This control returns the list of switch counts for all relays.
/// HIPAR arraySize/Sets the number of elements to be queried in Relay Counts array.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixRelayCount(ViSession instrSession,
                                                    ViInt32   switchMatrixNumber,
                                                    ViInt32   arraySize,
                                                    ViInt32   relayCounts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:REL:SWIT:COUN?", switchMatrixNumber);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, relayCounts, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Scans for external switch matrices connected via USB or Direct
/// HIFN  Control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR switchMatrices/Scans for external switch matrices connected via USB or Direct
/// HIPAR switchMatrices/Control.
ViStatus _VI_FUNC rsvna_QueryExternalSwitchMatrices(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    switchMatrices[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_SCAN_SWITCH_MATRIX,
            bufferSize, switchMatrices),
            3, "Switch Matrices");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the serial number of the related switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR bufferSize/Size of output array.
/// HIPAR serialNumber/Returns the serial number of the related switch matrix.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixSerialNumber(ViSession instrSession,
                                                      ViInt32   switchMatrixNumber,
                                                      ViInt32   bufferSize,
                                                      ViChar    serialNumber[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, switchMatrixNumber, 1), 2, "Switch Matrix Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWITCH_MATRIX_SERIAL_NUMBER,
            "Matrix", switchMatrixNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Switch Matrix Number");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWITCH_MATRIX_SERIAL_NUMBER,
            bufferSize, serialNumber),
            4, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates or redefines a user-defined physical port.
/// HIFN     Note(s):
/// HIFN     (1) Redefining physical ports causes a factory reset and deletes all
/// HIFN  switch matrix RF connections. So the RF configuration for switch
/// HIFN  matrices (see rsvna_SwitchMatrixConfigurationOperation) has to be done
/// HIFN  after the port redefinition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR portRedefinition/String representation '{Source}:{a wave}:{b wave}' where
ViStatus _VI_FUNC rsvna_ConfigureUserDefinedPhysicalPort(ViSession instrSession,
                                                         ViInt32   physicalPort,
                                                         ViString  portRedefinition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_USER_DEFINED_PHYSICAL_PORT,
            "Port", physicalPort -1, RS_REPCAP_BUF_SIZE, repCap_port),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap_port, RSVNA_ATTR_USER_DEFINED_PHYSICAL_PORT, portRedefinition),
            3, "Port Redefinition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function disables user-defined physical ports.
/// HIFN     Note(s):
/// HIFN     (1) Redefining physical ports causes a factory reset and deletes all
/// HIFN  switch matrix RF connections. So the RF configuration for switch
/// HIFN  matrices (see rsvna_SwitchMatrixConfigurationOperation) has to be done
/// HIFN  after the port redefinition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeleteUserDefinedPhysicalPorts(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_USER_DEFINED_PHYSICAL_PORT_STATE, VI_FALSE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds trace <TraceName> to a previously initialized shared memory
/// HIFN  buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Trace name. Identifies the trace within the active recall set.
/// HIPAR dataFormat/The data format of the stored trace values.
/// HIPAR numberOfPoints/Number of trace values to be stored.
ViStatus _VI_FUNC rsvna_DataMemoryAddTrace(ViSession instrSession,
                                           ViString  traceName,
                                           ViInt32   dataFormat,
                                           ViInt32   numberOfPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  dataFormatString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDataFormat,
                    dataFormat, &dataFormatString),
                3, "Data Format");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s',%s,%ld",
             traceName, dataFormatString, numberOfPoints);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DATA_MEMORY_ADD_TRACE, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the shared memory buffers initialized using
/// HIFN  SYSTem:DATA:MEMory:INITalize.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR catalog/Queries the names of the shared memory buffers initialized using
/// HIPAR catalog/SYSTem:DATA:MEMory:INITalize.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryDataMemoryNamesCatalog(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    catalog[],
                                                    ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_DATA_MEMORY_NAMES_CATALOG, RS_VAL_DIRECT_USER_CALL,
                    bufferSize, catalog, responseLength),
                3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initializes a new shared memory buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DataMemoryInitializeNewSharedBuffer(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DATA_MEMORY_INITIALIZE_NEW_SHARED_BUFFER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Completes the setup of a named shared memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Completes the setup of a named shared memory buffer that was
/// HIPAR name/initialized using SYSTem:DATA:MEMory:INITalize an configured using one
/// HIPAR name/or more SYSTem:DATA:MEMory:ADD. The buffer is then available in
/// HIPAR name/Windows.
ViStatus _VI_FUNC rsvna_DataMemoryCommitName(ViSession instrSession,
                                             ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DATA_MEMORY_COMMIT_NAME, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a shared memory buffer <ShMemName> initialized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Deletes a shared memory buffer <ShMemName> initialized using
/// HIPAR name/SYSTem:DATA:MEMory:INITalize.
ViStatus _VI_FUNC rsvna_DataMemoryDelete(ViSession instrSession,
                                         ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DATA_MEMORY_DELETE, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the offset in bytes of the last added trace data block,
/// HIFN  relative to the start of the shared memory buffer.
/// HIFN     The offset is required by processes that want to read the trace data
/// HIFN  from the buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Queries the offset in bytes of the last added trace data block,
/// HIPAR offset/relative to the start of the shared memory buffer.
ViStatus _VI_FUNC rsvna_QueryDataMemoryOffset(ViSession instrSession,
                                              ViInt32*  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DATA_MEMORY_OFFSET, offset),
            2, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all shared memory buffers initialized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DataMemoryDeleteAllSharedBuffers(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DATA_MEMORY_DELETE_ALL_SHARED_BUFFERS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the size of the shared memory buffer that was last created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR size/Queries the size of the shared memory buffer that was last created.
ViStatus _VI_FUNC rsvna_QueryDataMemorySize(ViSession instrSession,
                                            ViInt32*  size)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DATA_MEMORY_SIZE, size),
            2, "Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the HUMS service and data collection on and off, and enables the
/// HIFN  protocols for HUMS access.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR humsEnabled/Turns the HUMS service and data collection on and off.
/// HIPAR restEnabled/Returns information about the HUMS REST status and enables or disables
/// HIPAR restEnabled/the public HUMS REST endpoints.
/// HIPAR snmpVersion/Selects the SNMP version.
ViStatus _VI_FUNC rsvna_ConfigureHUMSEnabled(ViSession instrSession,
                                             ViBoolean humsEnabled,
                                             ViBoolean restEnabled,
                                             ViInt32   snmpVersion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_HUMS_ENABLED, humsEnabled),
            2, "HUMS Enabled");
    if (humsEnabled)
    {
        viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_HUMS_REST_ENABLED, restEnabled),
                3, "REST Enabled");

        viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_VERSION, snmpVersion),
                4, "SNMP Version");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the HUMS history as a ZIP file to the specified path and
/// HIFN  filename on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/A ZIP file path and filename.
ViStatus _VI_FUNC rsvna_HUMSExportHistoryToFile(ViSession instrSession,
                                                ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_EXPORT_HISTORY, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the complete HUMS data. This includes device history, device
/// HIFN  tags, SCPI connections, utilization history and utilizations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSDeleteHistory(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_DELETE_HISTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the complete device history information of the connected
/// HIFN  instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSDeleteDeviceHistory(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_DELETE_DEVICE_HISTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the complete utilization history information from the
/// HIFN  instrument.
/// HIFN     Alternatively, you can delete device history only with
/// HIFN  rsvna_HUMSDeleteDeviceHistory or all data with rsvna_HUMSDeleteHistory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSDeleteUtilizationHistory(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_DELETE_UTILIZATION_HISTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Retrieves the specified HUMS data from the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the format for the queried HUMS data.
/// HIPAR data/Selects the data type to save from the instrument.
/// HIPAR buffersize/The size of the allocated buffer.
/// HIPAR buffer/The buffer in which to store the data returned from the instrument.
/// HIPAR returnedBytes/The number of bytes read from the instrument. If this value is greater
/// HIPAR returnedBytes/than the allcoated buffer size, some data from the instrument was lost.
/// HIPAR returnedBytes/Consider reallocating and querying again.
ViStatus _VI_FUNC rsvna_HUMSQueryDataFromInstrument(ViSession instrSession,
                                                    ViInt32   format,
                                                    ViInt32   data,
                                                    ViInt32   bufferSize,
                                                    ViChar    buffer[],
                                                    ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[RSVNA_VAL_HUMS_DATA_REST_API_ENDPOINTS + 1] =
             {
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_HISTORY,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_UTILIZATION,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_UTILIZATION_HISTORY,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_FOOTPRINT,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SECURITY,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SERVICE_INFORMATION,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_BIOS,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_STORAGE,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SOFTWARE,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SYSTEM_INFO,
             RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SYSTEM_STATUS,
             RSVNA_ATTR_HIDDEN_HUMS_REST_API_ENDPOINTS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_HUMS_DATA_FORMAT, format),
            2, "Format");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, data, RSVNA_VAL_HUMS_DATA_HISTORY, RSVNA_VAL_HUMS_DATA_REST_API_ENDPOINTS),
            3, "Data");
    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    attrArr[data],
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, buffer, returnedBytes),
                3, "Buffer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the specified HUMS data from the instrument to a local file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the format for the queried HUMS data.
/// HIPAR data/Selects the data type to save from the instrument.
/// HIPAR filename/Specifies a local file to save the specified HUMS data to.
ViStatus _VI_FUNC rsvna_HUMSSaveDataFromInstrumentToFile(ViSession instrSession,
                                                         ViInt32   format,
                                                         ViInt32   data,
                                                         ViString  filename)
{
    ViStatus           error = VI_SUCCESS;
    ViPChar            cmd = NULL;
    RsCoreAttributePtr attrArr[RSVNA_VAL_HUMS_DATA_REST_API_ENDPOINTS + 1] =
                       {
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_HISTORY,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_UTILIZATION,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_UTILIZATION_HISTORY,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_FOOTPRINT,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SECURITY,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SERVICE_INFORMATION,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_BIOS,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_STORAGE,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SOFTWARE,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SYSTEM_INFO,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_DEVICE_SYSTEM_STATUS,
                       &g_RSVNA_ATTR_HIDDEN_HUMS_REST_API_ENDPOINTS
                       };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_HUMS_DATA_FORMAT, format),
            2, "Format");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, data, RSVNA_VAL_HUMS_DATA_HISTORY, RSVNA_VAL_HUMS_DATA_REST_API_ENDPOINTS),
            3, "Data");
    viCheckParm(RsCore_BuildAttrCommandString(instrSession, attrArr[data], NULL, &cmd),
                3, "Data");
    viCheckParm(RsCore_InvalidStringLength(instrSession, filename, 1, -1),
            4, "Filename");

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, filename, RS_VAL_TRUNCATE));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SNMP community strings for read-only and read-write access.
/// HIFN     Prerequisites: Select an SNMP version that supports communities
/// HIFN  (RSVNA_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR readOnly/Sets the SNMP community string for read-only access.
/// HIPAR readWrite/Sets the SNMP community string for read-write access.
ViStatus _VI_FUNC rsvna_ConfigureHUMSSNMPCommunityStrings(ViSession instrSession,
                                                          ViString  readOnly,
                                                          ViString  readWrite)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_COMMUNITY_READ_ONLY, readOnly),
            2, "Read Only");

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_COMMUNITY_READ_WRITE, readWrite),
            3, "Read Write");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SNMP system information.
/// HIFN     Note: Not setting the system information via SCPI persistently allows
/// HIFN  to change the string via SNMP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR systemContact/Sets the SNMP contact information for the administrator.
/// HIPAR systemLocation/Sets the SNMP location information for the administrator.
ViStatus _VI_FUNC rsvna_ConfigureHUMSSNMPSystemInfo(ViSession instrSession,
                                                    ViString  systemContact,
                                                    ViString  systemLocation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_SYSTEM_CONTACT, systemContact),
            2, "SystemContact");

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_SYSTEM_LOCATION, systemLocation),
            3, "System Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SNMP system information.
/// HIFN
/// HIFN     Note: Not setting the system information via SCPI persistently allows
/// HIFN  to change the string via SNMP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR buffersSize/Sizes of output array.
/// HIPAR systemContact/Sets the SNMP contact information for the administrator.
/// HIPAR systemLocation/Sets the SNMP location information for the administrator.
ViStatus _VI_FUNC rsvna_QueryHUMSSNMPSystemInfo(ViSession instrSession,
                                                ViInt32   buffersSize,
                                                ViChar    systemContact[],
                                                ViChar    systemLocation[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_HUMS_SNMP_SYSTEM_CONTACT,
                    RS_VAL_DIRECT_USER_CALL,
                    buffersSize, systemContact),
                3, "System Contact");
    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_HUMS_SNMP_SYSTEM_LOCATION,
                    RS_VAL_DIRECT_USER_CALL,
                    buffersSize, systemLocation),
                4, "System Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a SNMP user profile.
/// HIFN     Prerequisites: Select SNMPv3 (RSVNA_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR username/Name of the user.
/// HIPAR access/Defines the access right of the user.
/// HIPAR security/Defines the security level.
/// HIPAR authpwd/String containing the authentication password.
/// HIPAR privpwd/String containing the privacy password.
ViStatus _VI_FUNC rsvna_HUMSSNMPDefineUserProfile(ViSession instrSession,
                                                  ViString  username,
                                                  ViInt32   access,
                                                  ViInt32   security,
                                                  ViString  authPwd,
                                                  ViString  privPwd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar   concatBuf[RS_MAX_MESSAGE_LEN] = "";
    ViPChar  p2access;
    ViPChar  p2security;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngSNMPAccess,
                    access, &p2access),
                3, "Access");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngSNMPSecurityLevel,
                    security, &p2security),
                4, "Security");

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s',%s,%s", username, p2access, p2security);

    switch (security)
    {
    case RSVNA_VAL_SNMP_SECURITY_AUTH:
        snprintf(concatBuf, RS_MAX_MESSAGE_LEN, ",'%s'", authPwd);
        strncat(param, concatBuf, RS_MAX_MESSAGE_LEN - strlen(param));
        break;
    case RSVNA_VAL_SNMP_SECURITY_PRIVACY:
        snprintf(concatBuf, RS_MAX_MESSAGE_LEN, ",'%s','%s'", authPwd, privPwd);
        strncat(param, concatBuf, RS_MAX_MESSAGE_LEN - strlen(param));
        break;
    }

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_HUMS_SNMP_DEFINE_USER_PROFILE,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                2, "Username");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of users and a list of all SNMP users for SNMPv3.
/// HIFN     Prerequisites for this command: Select SNMPv3
/// HIFN  (RSVNA_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR userList/Returns the number of users and a list of all SNMP users for SNMPv3.
/// HIPAR returnedBytes/The number of bytes read from the instrument. If this value is greater
/// HIPAR returnedBytes/than the allcoated buffer size, some data from the instrument was lost.
/// HIPAR returnedBytes/Consider reallocating and querying again.
ViStatus _VI_FUNC rsvna_QueryHUMSSNMPUserList(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViChar    userList[],
                                              ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_HUMS_SNMP_USER_LIST,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, userList, returnedBytes),
                3, "User List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a specific SNMP user profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR user/Deletes a specific SNMP user profile.
ViStatus _VI_FUNC rsvna_HUMSSNMPDeleteUser(ViSession instrSession,
                                           ViString  user)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_DELETE_USER, user),
            2, "User");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all SNMP user profiles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSSNMPDeleteAllUsers(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_SNMP_DELETE_ALL_USERS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds or modifies a key-value pair (device tag).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tagID/Selects a HUMS tag.
/// HIPAR key/The key name.
/// HIPAR value/The value.
ViStatus _VI_FUNC rsvna_HUMSTagDefineKeyValuePair(ViSession instrSession,
                                                  ViInt32   tagID,
                                                  ViString  key,
                                                  ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_HUMS_TAG_DEFINE_KEY_PAIR,
            "HUMSTag", tagID , RS_REPCAP_BUF_SIZE, repCap),
            2, "Tag ID");

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", key, value);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_HUMS_TAG_DEFINE_KEY_PAIR,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Key");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the key-value pair for a given ID or an empty string if the ID
/// HIFN  is unknown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tagID/Selects a HUMS tag.
/// HIPAR bufferSize/Size of output array.
/// HIPAR keyValuePair/Returns the key-value pair for a given ID or an empty string if the ID
/// HIPAR keyValuePair/is unknown.
/// HIPAR returnedBytes/The number of bytes read from the instrument. If this value is greater
/// HIPAR returnedBytes/than the allcoated buffer size, some data from the instrument was lost.
/// HIPAR returnedBytes/Consider reallocating and querying again.
ViStatus _VI_FUNC rsvna_QueryHUMSTagKeyValuePair(ViSession instrSession,
                                                 ViInt32   tagID,
                                                 ViInt32   bufferSize,
                                                 ViChar    keyValuePair[],
                                                 ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_HUMS_TAG_DEFINE_KEY_PAIR,
            "HUMSTag", tagID , RS_REPCAP_BUF_SIZE, repCap),
            2, "Tag ID");
    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, repCap,
                    RSVNA_ATTR_HUMS_TAG_KEY_VALUE_PAIR,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, keyValuePair, returnedBytes),
                3, "Key Value Pair");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all key-value tags that you have assigend to the instrument as
/// HIFN  a comma-separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR deviceTags/Returns all key-value tags that you have assigend to the instrument as
/// HIPAR deviceTags/a comma-separated string.
/// HIPAR returnedBytes/The number of bytes read from the instrument. If this value is greater
/// HIPAR returnedBytes/than the allcoated buffer size, some data from the instrument was lost.
/// HIPAR returnedBytes/Consider reallocating and querying again.
ViStatus _VI_FUNC rsvna_QueryHUMSAllDeviceTags(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    deviceTags[],
                                               ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_HUMS_ALL_DEVICE_TAGS,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, deviceTags, returnedBytes),
                3, "Device Tags");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a certain tag you assigned to your instrument, including its
/// HIFN  key and value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tagID/Selects a HUMS tag.
ViStatus _VI_FUNC rsvna_HUMSTagDelete(ViSession instrSession,
                                      ViInt32   tagID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_HUMS_TAG_DEFINE_KEY_PAIR,
            "HUMSTag", tagID , RS_REPCAP_BUF_SIZE, repCap),
            2, "Tag ID");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HUMS_TAG_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all key-value tags you have assigned to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSTagDeleteAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_TAG_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the activity tracking of utilizations.
/// HIFN     To find out the ID of the utilization, you have to query the complete
/// HIFN  utilization data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR utilizationID/ID number of the utilization activity. To find out the ID of the
/// HIPAR utilizationID/utilization, you have to query the complete utilization data.
/// HIPAR trackingEnabled/Turns tracking of the specified utilization on or off.
ViStatus _VI_FUNC rsvna_HUMSUtilizationTrackingEnable(ViSession instrSession,
                                                      ViInt32   utilizationID,
                                                      ViBoolean trackingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%s", utilizationID, offOnArr[trackingEnabled]);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_HUMS_UTILIZATION_TRACKING_ENABLED,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                2, "Utilization ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the status summary of the complete system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deviceStatus/Returns the status summary of the complete system.
ViStatus _VI_FUNC rsvna_QueryHUMSDeviceStatusSummary(ViSession instrSession,
                                                     ViInt32*  deviceStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_HUMS_DEVICE_STATUS_SUMMARY, deviceStatus),
            2, "Device Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an event to the event history.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR severity/Severity of the event.
/// HIPAR message/Short description of the event.
/// HIPAR detail/More comprehensive description of the event.
ViStatus _VI_FUNC rsvna_HUMSAddEvent(ViSession instrSession,
                                     ViInt32   severity,
                                     ViString  message,
                                     ViString  detail)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Severity;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngHUMSSeverity, severity, &p2Severity),
                2, "Severity");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,'%s','%s'", p2Severity, message, detail);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_HUMS_ADD_EVENT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Changes the current values of the custom utilization (and indirectly
/// HIFN  also the start value).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR id/Identifier of the custom utilization.
/// HIPAR value/Defines the value to be changed.
/// HIPAR mode/Sets the update custom mode.
/// HIPAR state/Initial activity state of custom utilization.
ViStatus _VI_FUNC rsvna_HUMSUpdateCustomEvent(ViSession instrSession,
                                              ViInt32   id,
                                              ViString  value,
                                              ViInt32   mode,
                                              ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Mode;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngHUMSUtilizationUpdateValue, mode, &p2Mode),
                3, "Local Port");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s',%s,%s", id, value, p2Mode, state ? "ON" : "OFF");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_HUMS_UTILIZATION_UPDATE_CUSTOM, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates a custom utilization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR id/Identifier of the custom utilization.
/// HIPAR name/Name of the custom utilization that appears in the user interface.
/// HIPAR description/Description of the custom utilization.
/// HIPAR unit/Unit of the custom utilization that appears in the user interface.
/// HIPAR enabled/Initial activity state of custom utilization.
ViStatus _VI_FUNC rsvna_HUMSUtilizationCreatesCustom(ViSession instrSession,
                                                     ViInt32   id,
                                                     ViString  name,
                                                     ViString  description,
                                                     ViString  unit,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s','%s','%s',%s", id, name, description, unit, enabled ? "ON" : "OFF");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_HUMS_UTILIZATION_CREATES_CUSTOM, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries all custom utilizations currently in use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR customUtilization/Queries all custom utilizations currently in use.
ViStatus _VI_FUNC rsvna_QueryHUMSUtilizationAll(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    customUtilization[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_UTILIZATION_ALL,
            bufferSize, customUtilization),
            3, "Custom Utilization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all custom utilizations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_HUMSUtilizationDeleteCustomAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HUMS_UTILIZATION_DELETE_CUSTOM_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes a single custom utilization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deleteUtilization/Deletes a single custom utilization.
ViStatus _VI_FUNC rsvna_HUMSUtilizationDeleteCustom(ViSession instrSession,
                                                    ViInt32   deleteUtilization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_HUMS_UTILIZATION_DELETE_CUSTOM, deleteUtilization),
            2, "Delete Utilization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries detailed HUMS History.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR id/ID of the utilization you want to get information about.
/// HIPAR startDate/String containing the start date of the evaluation period (ISO8601
/// HIPAR startDate/format).
/// HIPAR endDate/String containing the end date of the evaluation period (ISO8601
/// HIPAR endDate/format).
/// HIPAR granularity/
/// HIPAR history/Queries all custom utilizations currently in use.
ViStatus _VI_FUNC rsvna_QueryHUMSUtilizationHistoryDetailed(ViSession instrSession,
                                                            ViInt32   id,
                                                            ViString  startDate,
                                                            ViString  endDate,
                                                            ViReal64  granularity,
                                                            ViChar    history[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s','%s',%.12lG", id, startDate, endDate, granularity);

    checkErr(RsCore_GetAttributeViStringWithQparamUnknownLen(instrSession, NULL, RSVNA_ATTR_HIDDEN_HUMS_UTILIZATION_HISTORY_DETAILED, params, &history));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets last date and time the instrument was calibrated in ISO 8601
/// HIFN  format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR date/Gets last date and time the instrument was calibrated in ISO 8601
/// HIPAR date/format.
ViStatus _VI_FUNC rsvna_QueryServiceCalibrationLastDate(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    date[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_SERVICE_CALIBRATION_LAST_DATE,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, date),
                3, "Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets the next date and time the instrument needs calibration to be
/// HIFN  done in ISO 8601 format. The response may be empty in case of no fixed
/// HIFN  next calibration due.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR date/Gets the next date and time the instrument needs calibration to be
/// HIPAR date/done in ISO 8601 format. The response may be empty in case of no fixed
/// HIPAR date/next calibration due.
ViStatus _VI_FUNC rsvna_QueryServiceCalibrationDueDate(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    date[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_SERVICE_CALIBRATION_DUE_DATE,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, date),
                3, "Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the state if the calibration is OK or the instrument requires
/// HIFN  calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationDue/Returns the state if the calibration is OK or the instrument requires
/// HIPAR calibrationDue/calibration.
ViStatus _VI_FUNC rsvna_QueryServiceCalibrationDueState(ViSession instrSession,
                                                        ViInt32*  calibrationDue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_SERVICE_CALIBRATION_DUE_STATE, calibrationDue),
            2, "Calibration Due");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the recommended calibration interval in ISO 8601 duration
/// HIFN  format, set by Rohde&Schwarz service.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR interval/Queries the recommended calibration interval in ISO 8601 duration
/// HIPAR interval/format, set by Rohde&Schwarz service.
ViStatus _VI_FUNC rsvna_QueryServiceCalibrationInterval(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    interval[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_SERVICE_CALIBRATION_INTERVAL,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, interval),
                3, "Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the last date and time the instrument was serviced (ISO 8601
/// HIFN  format).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR lastServiceDate/Gets the last date and time the instrument was serviced (ISO 8601
/// HIPAR lastServiceDate/format).
ViStatus _VI_FUNC rsvna_QueryServiceLastServiceDate(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    lastServiceDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViString(instrSession, NULL,
                    RSVNA_ATTR_SERVICE_LAST_SERVICE_DATE,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, lastServiceDate),
                3, "Last Service Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the state if the instrument requires service.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serviceDue/Returns the state if the instrument requires service.
ViStatus _VI_FUNC rsvna_QueryServiceDueState(ViSession instrSession,
                                             ViInt32*  serviceDue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_SERVICE_DUE_STATE, serviceDue),
            2, "Service Due");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the numbers of all configured generic external devices. The
/// HIFN  response is a string containing a comma-separated list of generator
/// HIFN  numbers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR generators/Queries the numbers of all configured generic external devices. The
/// HIPAR generators/response is a string containing a comma-separated list of generator
/// HIPAR generators/numbers.
ViStatus _VI_FUNC rsvna_QueryGenericDeviceCatalog(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    generators[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_GENERIC_DEVICE_CATALOG,
            bufferSize, generators),
            3, "Generators");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes (unregisters) all configured generic external devices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_GenericDeviceDeleteAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_GENERIC_DEVICE_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes (unregisters) generic external device no. <Gdev>.
/// HIFN
/// HIFN     Use SYSTem:COMMunicate:RDEVice:GDEV:DELete:ALL to unregister all
/// HIFN  devices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR genericDevice/Selects Generic Device.
ViStatus _VI_FUNC rsvna_GenericDeviceDelete(ViSession instrSession,
                                            ViInt32   genericDevice)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GENERIC_DEVICE_DELETE,
            "GenericDevice", genericDevice - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generic Device");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_GENERIC_DEVICE_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Registers a generic device at the analyzer or queries its connection
/// HIFN  properties.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR genericDevice/Selects Generic Device.
/// HIPAR interfacetype/Interface type (string parameter).
/// HIPAR address/Interface address (string parameter), depending on the interface type.
ViStatus _VI_FUNC rsvna_ConfigureGenericDevice(ViSession instrSession,
                                               ViInt32   genericDevice,
                                               ViString  interfacetype,
                                               ViString  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_CONFIGURE,
            "GenericDevice", genericDevice - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generic Device");

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s','%s'", interfacetype, address);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_CONFIGURE, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the addresses of the configured generic external devices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR adresses/Returns the addresses of the configured generic external devices.
ViStatus _VI_FUNC rsvna_QueryGenericDeviceAddresses(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    addresses[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_GENERIC_DEVICES_ADDRESSES,
            bufferSize, addresses),
            3, "Addresses");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes generic device command configurations for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR deleteAddress/Deletes generic device command configurations for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureGenericDevicesDeleteAddress(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViString  deleteAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GENERIC_DEVICES_DELETE_ADDRESS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_GENERIC_DEVICES_DELETE_ADDRESS, deleteAddress),
            3, "Delete Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines or queries the generic device command configuration of channel
/// HIFN  <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR address/Use SENSe:GDEVice:ADDRess? to get the address strings of the
/// HIPAR address/configured generic devices.
/// HIPAR commandfilepath/Path to the command file, either absolute or relative to the current
/// HIPAR commandfilepath/directory
/// HIPAR commandid/A unique Command Name.
ViStatus _VI_FUNC rsvna_SelectGenericDevice(ViSession instrSession,
                                            ViInt32   channel,
                                            ViString  address,
                                            ViString  commandfilepath,
                                            ViString  commandid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_SELECT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s','%s','%s'", address, commandfilepath, commandid);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_SELECT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the calibration unit selected using
/// HIFN  RSVNA_ATTR_SELECT_CALIBRATION_UNIT is an inline calibration unit (ICU),
/// HIFN  this attribute sets/queries the characterization to be used for
/// HIFN  calibrations with this ICU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR characterization/If the calibration unit selected using
/// HIPAR characterization/RSVNA_ATTR_SELECT_CALIBRATION_UNIT is an inline calibration unit (ICU),
/// HIPAR characterization/this attribute sets/queries the characterization to be used for
/// HIPAR characterization/calibrations with this ICU.
ViStatus _VI_FUNC rsvna_ConfigureGenericDevicesCharacterization(
                            ViSession instrSession,
                            ViString  characterization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_GENERIC_DEVICE_CHARACTERIZATION, characterization),
            3, "Characterization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If the calibration unit selected using
/// HIFN  RSVNA_ATTR_SELECT_CALIBRATION_UNIT is an inline calibration system
/// HIFN  (ICS), this query returns the addresses of the connected ICUs.
/// HIFN  Otherwise it returns the empty string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR devices/If the calibration unit selected using
/// HIPAR devices/RSVNA_ATTR_SELECT_CALIBRATION_UNIT is an inline calibration system
/// HIPAR devices/(ICS), this query returns the addresses of the connected ICUs.
/// HIPAR devices/Otherwise it returns the empty string.
ViStatus _VI_FUNC rsvna_QueryGenericDeviceConfiguredDevices(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    devices[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_GENERIC_DEVICE_CONFIGURED_DEVICES,
            bufferSize, devices),
            3, "Devices");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines or queries the generic device calibration mixer command
/// HIFN  configuration of channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR genericDevice/Selects Generic Device.
/// HIPAR address/Address string.
/// HIPAR commandfilepath/Path to the command file, either absolute or relative to the current
/// HIPAR commandfilepath/directory.
/// HIPAR mixercommandid/A unique Command Name within command file <CommandFilepath>.
ViStatus _VI_FUNC rsvna_GenericDeviceCalibrationMixer(ViSession instrSession,
                                                      ViInt32   genericDevice,
                                                      ViString  address,
                                                      ViString  commandfilepath,
                                                      ViString  mixercommandid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_CALIBRATION_MIXER,
            "GenericDevice", genericDevice - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s','%s'", address, commandfilepath, mixercommandid);

    checkErr(RsCore_SetAttributeViRawString(instrSession, repCap,
                        RSVNA_ATTR_HIDDEN_GENERIC_DEVICE_CALIBRATION_MIXER, RS_VAL_DIRECT_USER_CALL,
                        param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a comma-separated list of the currently defined converter
/// HIFN  types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR catalog/Returns a comma-separated list of the currently defined converter
/// HIPAR catalog/types.
ViStatus _VI_FUNC rsvna_QueryConverterTypesCatalog(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_CONVERTER_TYPES_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns how many converter types are currently defined (pre-defined or
/// HIFN  user-defined).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Returns how many converter types are currently defined (pre-defined or
/// HIPAR count/user-defined).
ViStatus _VI_FUNC rsvna_QueryConverterCount(ViSession instrSession,
                                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_CONVERTER_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates a converter type with default properties.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR define/Creates a converter type with default properties.
ViStatus _VI_FUNC rsvna_DefineConverterNewType(ViSession instrSession,
                                               ViString  define)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_CONVERTER_DEFINE_NEW_TYPE, define),
            2, "Define");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the related converter type.
/// HIFN     Only user-defined converter types can be deleted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR converterDefinition/Deletes the related converter type.
ViStatus _VI_FUNC rsvna_DeleteConverterDefinition(ViSession instrSession,
                                                  ViString  converterDefinition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_CONVERTER_DEFINITION_DELETE, converterDefinition),
            2, "Converter Definition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the start and stop frequency of (the converter port of)
/// HIFN  the related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR startFrequency/Sets the start frequency.
/// HIPAR stopFrequency/Sets the stop frequency.
ViStatus _VI_FUNC rsvna_ConfigureConverterStartStopFrequency(ViSession instrSession,
                                                             ViString  type,
                                                             ViReal64  startFrequency,
                                                             ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%.12lG,%.12lG", type, startFrequency, stopFrequency);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_START_STOP_FREQUENCY, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the start and stop frequency of (the converter port of) the
/// HIFN  related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR startFrequency/Queries the start frequency.
/// HIPAR stopFrequency/Queries the stop frequency.
ViStatus _VI_FUNC rsvna_QueryConverterStartStopFrequency(ViSession instrSession,
                                                         ViString  type,
                                                         ViReal64* startFrequency,
                                                         ViReal64* stopFrequency)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    params[RS_PARAM_BUF_SIZE]= "";
    ViChar    result[RS_PARAM_BUF_SIZE];
    ViInt32   outResponseLen;
    ViReal64* resArr = NULL;
    ViInt32   count;

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_START_STOP_FREQUENCY,
                                                        params, RS_PARAM_BUF_SIZE, result, &outResponseLen));

    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, result, &resArr, &count));

    *startFrequency = resArr[0];
    *stopFrequency = resArr[1];

Error:
    if (resArr)
        free(resArr);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the fixed intermediate frequency of the related converter
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR ifFrequency/Sets the IF frequency.
ViStatus _VI_FUNC rsvna_ConfigureConverterIFFrequency(ViSession instrSession,
                                                      ViString  type,
                                                      ViReal64  ifFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, ifFrequency);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_IF_FREQUENCY, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the fixed intermediate frequency of the related converter
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR ifFrequency/Queries the IF frequency.
ViStatus _VI_FUNC rsvna_QueryConverterIFFrequency(ViSession instrSession,
                                                  ViString  type,
                                                  ViReal64* ifFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";
    ViChar   result[RS_PARAM_BUF_SIZE];
    ViInt32  outResponseLen;

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_IF_FREQUENCY,
                                                        params, RS_PARAM_BUF_SIZE, result, &outResponseLen));

    *ifFrequency = RsCore_Convert_String_To_ViReal64(result);

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the LO multiplicator of the related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR localMultiplicator/Sets the Local Multiplicator.
ViStatus _VI_FUNC rsvna_ConfigureConverterLocalMultiplicator(ViSession instrSession,
                                                             ViString  type,
                                                             ViInt32   localMultiplicator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%ld", type, localMultiplicator);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_LOCAL_MULTIPLICATOR, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the LO multiplicator of the related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR localMultiplicator/Queries the Local Multiplicator.
ViStatus _VI_FUNC rsvna_QueryConverterLocalMultiplicator(ViSession instrSession,
                                                         ViString  type,
                                                         ViInt32*  localMultiplicator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViInt32WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_LOCAL_MULTIPLICATOR,
                                                        params, localMultiplicator));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the preferred input power at the LO In port of the
/// HIFN  related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR localPower/Sets the Local Power.
/// HIPAR offset/Sets the Local Power Offset.
ViStatus _VI_FUNC rsvna_ConfigureConverterLocalPower(ViSession instrSession,
                                                     ViString  type,
                                                     ViReal64  localPower,
                                                     ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";
    ViChar   params2[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, localPower);
    snprintf (params2, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, offset);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_LOCAL_POWER, params));
    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_LOCAL_POWER_OFFSET, params2));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the preferred input power at the LO In port of the related
/// HIFN  converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR localPower/Queries the Local Power.
/// HIPAR offset/Queries the Local Offset.
ViStatus _VI_FUNC rsvna_QueryConverterLocalPower(ViSession instrSession,
                                                 ViString  type,
                                                 ViReal64* localPower,
                                                 ViReal64* offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_LOCAL_POWER,
                                                        params, localPower));

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_LOCAL_POWER_OFFSET,
                                                        params, offset));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the maximum tolerable input power at the RF In port of the
/// HIFN  related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR maximumInputPower/Sets the Maximum Input Power.
ViStatus _VI_FUNC rsvna_ConfigureConverterMaximumInputPower(ViSession instrSession,
                                                            ViString  type,
                                                            ViReal64  maximumInputPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, maximumInputPower);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_MAXIMUM_INPUT_POWER, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the maximum tolerable input power at the RF In port of the
/// HIFN  related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR maximumInputPower/Queries the Maximum Input Power.
ViStatus _VI_FUNC rsvna_QueryConverterMaximumInputPower(ViSession instrSession,
                                                        ViString  type,
                                                        ViReal64* maximumInputPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_MAXIMUM_INPUT_POWER,
                                                        params, maximumInputPower));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the source multiplicator of the related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR multiplicator/Sets the Source Multiplicator.
ViStatus _VI_FUNC rsvna_ConfigureConverterSourceMultiplicator(
                            ViSession instrSession,
                            ViString  type,
                            ViInt32   multiplicator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%ld", type, multiplicator);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_SOURCE_MULTIPLICATOR, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the source multiplicator of the related converter type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR multiplicator/Queries the Source Multiplicator.
ViStatus _VI_FUNC rsvna_QueryConverterSourceMultiplicator(ViSession instrSession,
                                                          ViString  type,
                                                          ViInt32*  multiplicator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViInt32WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_SOURCE_MULTIPLICATOR,
                                                        params, multiplicator));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the SOurce Nominal Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR nominalPower/Sets the Source Nominal Power.
/// HIPAR nominalPowerOffset/Sets the Source Nominal Power Offset.
ViStatus _VI_FUNC rsvna_ConfigureConverterSourceNominalPower(ViSession instrSession,
                                                             ViString  type,
                                                             ViReal64  nominalPower,
                                                             ViReal64  nominalPowerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";
    ViChar   params2[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, nominalPower);
    snprintf (params2, RS_PARAM_BUF_SIZE, "'%s',%.12lG", type, nominalPowerOffset);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_SOURCE_NOMINAL_POWER, params));
    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_CONVERTER_SOURCE_NOMINAL_POWER_OFFSET, params2));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Source Nominal Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Name of the converter type, whose start and stop frequencies are
/// HIPAR type/set/queried.
/// HIPAR nominalPower/Queries the Source Nominal Power.
/// HIPAR nominalPowerOffset/Queries the Source Nominal Power Offset.
ViStatus _VI_FUNC rsvna_QueryConverterSourceNominalPower(ViSession instrSession,
                                                         ViString  type,
                                                         ViReal64* nominalPower,
                                                         ViReal64* nominalPowerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", type);

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_SOURCE_NOMINAL_POWER,
                                                        params, nominalPower));

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_SOURCE_NOMINAL_POWER_OFFSET,
                                                        params, nominalPowerOffset));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/queries the type of frequency converter at VNA port <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR vnaPort/Sets/queries the type of frequency converter at VNA port <Port>.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyConversionVNAPort(ViSession instrSession,
                                                            ViInt32   physicalPort,
                                                            ViString  vnaPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FREQUENCY_CONVERSION_VNA_PORT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_FREQUENCY_CONVERSION_VNA_PORT, vnaPort),
            3, "VNA Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the source of the LO signal distributed via splitter <Port>.
/// HIFN     Splitter 2 is only useful, if 2 independent 2-port converter
/// HIFN  measurements shall be performed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR splitter/Selects the splitter number.
/// HIPAR localPort/The port type, further specified by its index <LocalIndex>.
/// HIPAR localIndex/The LO source port index, if required.
ViStatus _VI_FUNC rsvna_ConfigureConverterSplitterLOPort(ViSession instrSession,
                                                         ViInt32   splitter,
                                                         ViInt32   localPort,
                                                         ViInt32   localIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2LocalPort;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CONVERTER_SPLITTER_LO_PORT,
            "Splitter", splitter - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Splitter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSplitterLOPort, localPort, &p2LocalPort),
                3, "Local Port");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", p2LocalPort, localIndex);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_CONVERTER_SPLITTER_LO_PORT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the (cable and) splitter loss for splitter no. <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR splitterLoss/Defines the (cable and) splitter loss for splitter no. <Port>.
ViStatus _VI_FUNC rsvna_ConfigureConverterSplitterLoss(ViSession instrSession,
                                                       ViInt32   splitter,
                                                       ViReal64  splitterLoss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_SPLITTER_LOSS,
            "Splitter", splitter - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Splitter");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_SPLITTER_LOSS, splitterLoss),
            3, "Splitter Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures COnverter Default Values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR cableLoss/Default cable (and splitter) loss for the local port of converter
/// HIPAR cableLoss/<Port>.
/// HIPAR slopeFactor/Default slope factor for the local port of converter <Port>.
ViStatus _VI_FUNC rsvna_ConfigureConverterDefault(ViSession instrSession,
                                                  ViInt32   physicalPort,
                                                  ViReal64  cableLoss,
                                                  ViReal64  slopeFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_DEFAULT_CABLE_LOSS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_DEFAULT_CABLE_LOSS, cableLoss),
            3, "Cable Loss");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_DEFAULT_SLOPE_FACTOR, slopeFactor),
            4, "Slope Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Converter IF Input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR ifInput/Selects input connectors to be used for the IF meas and IF ref
/// HIPAR ifInput/connection between the converter and the analyzer port <Port>.
/// HIPAR ifFrequency/Sets/queries the intermediate frequency for converter <Port>.
ViStatus _VI_FUNC rsvna_ConfigureConverterIF(ViSession instrSession,
                                             ViInt32   physicalPort,
                                             ViInt32   ifInput,
                                             ViReal64  ifFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_IF_INPUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CONVERTER_IF_INPUT, ifInput),
            3, "IF Input");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_IF_FREQUENCY, ifFrequency),
            4, "IF Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures converter LO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR loss/Defines the cable loss at port <Port> if used as LO port (without
/// HIPAR loss/splitter).
ViStatus _VI_FUNC rsvna_ConfigureConverterLOLoss(ViSession instrSession,
                                                 ViInt32   physicalPort,
                                                 ViReal64  loss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_LO_LOSS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_LO_LOSS, loss),
            3, "Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the source for the LO In port of converter <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR local/The port type, further specified by its index <LocalIndex>
/// HIPAR index/The port index, if required.
ViStatus _VI_FUNC rsvna_ConfigureConverterLOPort(ViSession instrSession,
                                                 ViInt32   physicalPort,
                                                 ViInt32   local,
                                                 ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2local;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CONVERTER_LO_PORT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngLOPort, local, &p2local),
                3, "Local");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", p2local, index);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_CONVERTER_LO_PORT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the cable loss at the source port or converter <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR portLoss/Defines the cable loss at the source port or converter <Port>.
ViStatus _VI_FUNC rsvna_ConfigureConverterSourcePortLoss(ViSession instrSession,
                                                         ViInt32   physicalPort,
                                                         ViReal64  portLoss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_SOURCE_PORT_LOSS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_SOURCE_PORT_LOSS, portLoss),
            3, "Port Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the VNA source port that drives converter <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR portType/Returns the port type.
/// HIPAR sourcePort/Returns the port index.
ViStatus _VI_FUNC rsvna_QueryConverterSourcePort(ViSession instrSession,
                                                 ViInt32   physicalPort,
                                                 ViInt32*  portType,
                                                 ViInt32*  sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   results[RS_MAX_MESSAGE_LEN] = "";
    ViChar   *p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_SOURCE_PORT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_QUERY_CONVERTER_SOURCE_PORT,
            RS_MAX_MESSAGE_LEN, results),
            3, "Source Port");

    p2buf = strtok(results, ",");
     if (p2buf != NULL)
    {
        checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsvna_rngLOPort, p2buf, portType));
    }
    p2buf = strtok (NULL, ",\n");
    {
        *sourcePort = RsCore_Convert_String_To_ViInt32(p2buf);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates converter <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR enabled/Activates or deactivates converter <Port>.
ViStatus _VI_FUNC rsvna_ConfigureConverterPortEnabled(ViSession instrSession,
                                                      ViInt32   physicalPort,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CONVERTER_PORT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the RF Slope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rfSlope/Sets the RF Slope.
ViStatus _VI_FUNC rsvna_ConfigureConverterPortRFSlope(ViSession instrSession,
                                                      ViInt32   physicalPort,
                                                      ViReal64  rfSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CONVERTER_PORT_RF_SLOPE, rfSlope),
            2, "RF Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the converter connected to physical VNA port <PhyPt>, sets/queries
/// HIFN  whether a leveling dataset shall be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR controlMode/For the converter connected to physical VNA port <PhyPt>, sets/queries
/// HIPAR controlMode/whether a leveling dataset shall be used.
ViStatus _VI_FUNC rsvna_ConfigureLevelingDatasetPowerControlMode(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViInt32   controlMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LEVELING_DATASET_POWER_CONTROL_MODE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LEVELING_DATASET_POWER_CONTROL_MODE, controlMode),
            3, "Control Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the frequency converter connected to physical port <PhyPt>,
/// HIFN  sets/queries whether the factory leveling dataset or a user-defined
/// HIFN  leveling dataset shall be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR mode/For the frequency converter connected to physical port <PhyPt>,
/// HIPAR mode/sets/queries whether the factory leveling dataset or a user-defined
/// HIPAR mode/leveling dataset shall be used.
ViStatus _VI_FUNC rsvna_ConfigureLevelingDatasetMode(ViSession instrSession,
                                                     ViInt32   physicalPort,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LEVELING_DATASET_MODE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LEVELING_DATASET_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a generic leveling dataset for the converter connected to
/// HIFN  physical port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR filename/Defines a generic leveling dataset for the converter connected to
/// HIPAR filename/physical port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureLevelingDatasetFile(ViSession instrSession,
                                                     ViInt32   physicalPort,
                                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONVERTER_SOURCE_PORT_LOSS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_LEVELING_DATASET_FILE, filename),
            3, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the path to the folder containing the leveling dataset for the
/// HIFN  converter connected to physical port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR path/Defines the path to the folder containing the leveling dataset for the
/// HIPAR path/converter connected to physical port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureLevelingDatasetPath(ViSession instrSession,
                                                     ViInt32   physicalPort,
                                                     ViString  path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LEVELING_DATASET_PATH,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_LEVELING_DATASET_PATH, path),
            3, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
ViStatus _VI_FUNC rsvna_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_ClearStatus(instrSession));
    checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", NULL));

    checkErr(RsCore_ResetRepCapTable(instrSession, rsvna_RsCoreRepCapTable, rsvna_RsCoreRepCapTableCount));
    checkErr(rsvna_DefaultInstrSetup(instrSession));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function switches the instrument to standby state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_SystemShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_SYSTEM_SHUTDOWN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsvna_self_test(ViSession instrSession,
                                  ViInt16*  testResult,
                                  ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsvna_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_SPECIFIC_DRIVER_REVISION,
        0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, "", RSVNA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
        0, 256, firmwareRevision));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsvna_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsvna_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
    return rsvna_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  Gets the current number of entries in the SCPI error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR systemErrorCount/Gets the current number of entries in the SCPI error queue.
ViStatus _VI_FUNC rsvna_ErrorCount(ViSession instrSession,
                                   ViInt32*  systemErrorCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_SYSTEM_ERROR_COUNT, systemErrorCount),
            2, "System Error Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsvna_SystemVersion(ViSession instrSession,
                                      ViInt32   length,
                                      ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_SYST_VERSION,
            length, systemVersion),
            2, "System Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument
/// HIFN  (reading of the Standard Event Register and checking it for error). All
/// HIFN  driver functions are using state checking. Switch this function to ON
/// HIFN  when debug your application. For better bus throughput and instrument
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsvna_setCheckStatus(ViSession instrSession,
                                       ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
ViStatus _VI_FUNC rsvna_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function allows a temporary disabling of a named option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionName/This control selects the option to be disabled or put back to
/// HIPAR optionName/operation.
/// HIPAR disableOption/Allows a temporary disabling of a named option.
ViStatus _VI_FUNC rsvna_TemporaryDisableOption(ViSession instrSession,
                                               ViString  optionName,
                                               ViInt32   disableOption)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, disableOption, RSVNA_VAL_REVERT_DISABLING, RSVNA_VAL_DISABLE),
            3, "Disable Option");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:OPT:DIS '%s',%ld", optionName, disableOption);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of IDQueryResponse buffer
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus _VI_FUNC rsvna_IDQueryResponse(ViSession instrSession,
                                        ViUInt32  bufferSize,
                                        ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession))
    {
        strncpy(IDQueryResponse, RSVNA_SIMULATION_ID_QUERY, bufferSize);
        IDQueryResponse[bufferSize - 1] = 0;
        goto Error;
    }

    checkErr(rsvna_QueryViString(instrSession, "*IDN?", bufferSize, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus _VI_FUNC rsvna_QueryOPC(ViSession instrSession,
                                 ViInt32*  opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsvna_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout)
{
    return RsCore_SetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsvna_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout)
{
    return RsCore_GetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument.  The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Specifies the directory of the file to be executed.
/// HIPAR fileToExecute/Name of an application program to be executed or of a file to be
/// HIPAR fileToExecute/opened. Blanks in the file name are not allowed.
/// HIPAR fileParameters/Command name in the selected application.
/// HIPAR waitToFinish/Locks command execution and manual control of the analyzer while a
/// HIPAR waitToFinish/program is running. The analyzer does not execute any further commands
/// HIPAR waitToFinish/or queries until the program is stopped or paused.
ViStatus _VI_FUNC rsvna_ExecuteProgram(ViSession instrSession,
                                       ViString  directory,
                                       ViString  fileToExecute,
                                       ViString  fileParameters,
                                       ViBoolean waitToFinish)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SELECT_APPLICATION, RSVNA_VAL_SELECT_APPL_PROG));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_CHANGE_DIRECTORY, directory),
            2, "Directory");

    if (strcmp(fileParameters, "") == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "PROG:SEL:EXEC '%s'", fileToExecute);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "PROG:SEL:EXEC '%s','%s'", fileToExecute, fileParameters);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

    if (waitToFinish == VI_TRUE)
    {
        checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_WAIT_TO_FINISH, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the return value of an application or process
/// HIFN  started via PROGram[:SELected]:EXECute (see function
/// HIFN  rsvna_ExecuteProgram).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control specifies the size of the buffer for Return Value
/// HIPAR bufferSize/control.
/// HIPAR returnValue/This control queries the return value of an application or started
/// HIPAR returnValue/process.
ViStatus _VI_FUNC rsvna_QueryProgramReturnValue(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    returnValue[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_QUERY_APPLICATION_RETURN_VALUE,
            bufferSize, returnValue),
            3, "Return Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads Distance to Fault (DtF) cable type files (*.rsc) from the
/// HIFN  specified input directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/The directory from which cable definitions will be loaded.
ViStatus _VI_FUNC rsvna_FileLoadDistancetoFaultCableTypes(ViSession instrSession,
                                                          ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FILE_LOAD_DISTANCE_TO_FAULT_CABLE_TYPES, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a comment to be added to (the comment section of) exported
/// HIFN  trace files.
/// HIFN     This is a global setting, i.e. the comment string is used for each
/// HIFN  trace export in every open recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR comment/Defines a comment to be added to (the comment section of) exported
/// HIPAR comment/trace files.
ViStatus _VI_FUNC rsvna_FileAddedComment(ViSession instrSession,
                                         ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FILE_ADDED_COMMENT, comment),
            2, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves all predefined and user-defined Distance to Fault (DtF) cable
/// HIFN  types to the specified output directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/The directory to which cable definitions will be saved.
ViStatus _VI_FUNC rsvna_FileStoreDistancetoFaultCableTypes(ViSession instrSession,
                                                           ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FILE_STORE_DISTANCE_TO_FAULT_CABLE_TYPES, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the export of symmetric (balanced) S-parameters
/// HIFN  for MMEMory:STORe:TRACe:PORTs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symmetricParams/Activates/deactivates the export of symmetric (balanced) S-parameters
/// HIPAR symmetricParams/for MMEMory:STORe:TRACe:PORTs.
ViStatus _VI_FUNC rsvna_ConfigureFileSymmetricParams(ViSession instrSession,
                                                     ViBoolean symmetricParams)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_FILE_SYMMETRIC_PARAMS, symmetricParams),
            2, "Symmetric Params");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of decimal places for values in all supported trace
/// HIFN  data files (Touchstone, ASCII, MatLab).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataDecimalPlaces/Defines the number of decimal places for data values in all supported
/// HIPAR dataDecimalPlaces/trace data files (Touchstone, ASCII, MatLab)
/// HIPAR stimulusDecimalPlaces/Defines the number of decimal places for stimulus values in all
/// HIPAR stimulusDecimalPlaces/supported trace data files (Touchstone, ASCII, MatLab)
ViStatus _VI_FUNC rsvna_ConfigureFileDecimalPlaces(ViSession instrSession,
                                                   ViInt32   dataDecimalPlaces,
                                                   ViInt32   stimulusDecimalPlaces)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FILE_DATA_DECIMAL_PLACES, dataDecimalPlaces),
            2, "Data Decimal Places");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FILE_STIMULUS_DECIMAL_PLACES, stimulusDecimalPlaces),
            3, "Stimulus Decimal Places");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function converts S-parameters into matched-circuit (converted)
/// HIFN  Y-parameters or Z-parameters and vice versa.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR conversion/This control converts S-parameters into matched-circuit (converted)
/// HIPAR conversion/Y-parameters or Z-parameters and vice versa.
ViStatus _VI_FUNC rsvna_ConfigureParametersConversion(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   conversion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PARAMETER_CONVERSION, conversion),
            3, "Conversion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the sweep type and the measurement parameter in a single
/// HIFN  string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepType/Single string parameter defining the sweep type and the parameter to
/// HIPAR sweepType/be measured: "<sweep_type>:<parameter>".
ViStatus _VI_FUNC rsvna_DefineSweepandMeasurement(ViSession instrSession,
                                                  ViString  sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(value, RS_MAX_MESSAGE_LEN, "'%s'", sweepType);
    //SENS:FUNC
    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DEFINE_SWEEP_AND_MEASUREMENT, value),
            2, "Sweep Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the sweep type and the measurement parameter in a single
/// HIFN  string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepType/Single string parameter defining the sweep type and the parameter to
/// HIPAR sweepType/be measured: "<sweep_type>:<parameter>".
/// HIPAR arg1/An additional argument further specifying the definition.
/// HIPAR arg2/An additional argument further specifying the definition.
ViStatus _VI_FUNC rsvna_DefineSweepandMeasurementWithArguments(
                            ViSession instrSession,
                            ViString  sweepType,
                            ViInt32   arg1,
                            ViInt32   arg2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2arg1;
    ViChar*  p2arg2;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSweepMeasurementArg, arg1, &p2arg1),
                3, "Arg1");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSweepMeasurementArg, arg2, &p2arg2),
                4, "Arg2");

    snprintf(value, RS_MAX_MESSAGE_LEN, "'%s', %s, %s", sweepType, p2arg1, p2arg2);
    //SENS:FUNC
    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DEFINE_SWEEP_AND_MEASUREMENT, value),
            2, "Sweep Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the sweep type and the measurement parameter for the selected
/// HIFN  trace in a single string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR bufferSize/Size of output buffer.
/// HIPAR sweepType/Queries the sweep type and the measurement parameter for the selected
/// HIPAR sweepType/trace in a single string.
ViStatus _VI_FUNC rsvna_QuerySweepandMeasurementDefinition(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   bufferSize,
                                                           ViChar    sweepType[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_QUERY_SWEEP_AND_MEASUREMENT_DEFINITION,
            bufferSize, sweepType),
            3, "Sweep Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select one of the four elements of the standard 2-port
/// HIFN  scattering matrix (S-parameters) or select S-parameters for multiport
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectSParameters(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   outPort,
                                          ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','S%02ld%02ld'", traceName, outPort, inPort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select S-parameters for multiport measurements
/// HIFN  (including the 2-port S-parameters) or mixed mode S-parameters. All
/// HIFN  possible combinations of mixed mode parameters (e.g. Sss, Scs, Sds,
/// HIFN  Sdd,...) are provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectMoreSParameters(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  traceName,
                                              ViInt32   outMode,
                                              ViInt32   outPort,
                                              ViInt32   inMode,
                                              ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','S%s%s%02ld%02ld'", traceName, inOutModeArr[outMode], inOutModeArr[inMode], outPort, inPort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select S-parameters for multiport measurements
/// HIFN  (including the 2-port S-parameters) or mixed mode S-parameters. All
/// HIFN  possible combinations of mixed mode parameters (e.g. Sss, Scs, Sds,
/// HIFN  Sdd,...) are provided.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIFN
/// HIFN     Mixed mode parameters are used to distinguish the following three port
/// HIFN  modes:
/// HIFN
/// HIFN     Single-ended (for unbalanced ports)
/// HIFN     Differential mode (for balanced ports)
/// HIFN     Common mode (for balanced ports)
/// HIFN
/// HIFN     The notation of a general S-parameter is S<mout><min><out><in>, where
/// HIFN  <mout> and <min> denote the output and input port modes, <out> and <in>
/// HIFN  denote the output and input port numbers.
/// HIFN
/// HIFN     The selected modes must be compatible with the port configuration. If
/// HIFN  an attempt is made to select an incompatible parameter (e.g. a
/// HIFN  single-ended parameter for a balanced port), the analyzer displays an
/// HIFN  error message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
/// HIPAR detector/Selects the algorithm that is used to calculate the displayed
/// HIPAR detector/measurement points from the raw data.
ViStatus _VI_FUNC rsvna_SelectMoreSParametersWithDetector(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViString  traceName,
                                                          ViInt32   outMode,
                                                          ViInt32   outPort,
                                                          ViInt32   inMode,
                                                          ViInt32   inPort,
                                                          ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','S%s%s%02ld%02ld%s'", traceName, inOutModeArr[outMode], inOutModeArr[inMode], outPort, inPort, detectorArr[detector]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select S-parameters for multiport measurements
/// HIFN  (including the 2-port S-parameters) or mixed mode S-parameters. All
/// HIFN  possible combinations of mixed mode parameters (e.g. Sss, Scs, Sds,
/// HIFN  Sdd,...) are provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectBalancedPortsSParameters(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViString  traceName,
                                                       ViInt32   outMode,
                                                       ViInt32   outPort,
                                                       ViInt32   inMode,
                                                       ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_SINGLE_ENDED_RAW),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_SINGLE_ENDED_RAW),
            6, "In Mode");

    checkErr(rsvna_TraceList(instrSession, channel, rd_buffer, RS_MAX_MESSAGE_BUF_SIZE));

    if (outMode == RSVNA_VAL_SINGLE_ENDED_RAW || inMode == RSVNA_VAL_SINGLE_ENDED_RAW)
    {
        if (strstr(rd_buffer, traceName))
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:MEAS:SEND '%s','S%02ld%02ld'", channel, traceName, outPort, inPort);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:SDEF:SEND '%s','S%02ld%02ld'", channel, traceName, outPort, inPort);
    }
    else
    {
        if (strstr(rd_buffer, traceName))
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:MEAS:SEND '%s','S%s%s%02ld%02ld'",
                    channel, traceName, inOutModeArr[outMode], inOutModeArr[inMode], outPort, inPort);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:SDEF:SEND '%s','S%s%s%02ld%02ld'",
                    channel, traceName, inOutModeArr[outMode], inOutModeArr[inMode], outPort, inPort);
    }
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates the traces for all S-parameters associated with a group
/// HIFN  of logical ports (Sparameter group).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR number_ofLogicalPorts/Sets Ports number.
/// HIPAR logicalPorts/Logical (balanced or unbalanced) port numbers.
ViStatus _VI_FUNC rsvna_CreateTraceSParametersGroup(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   numberOfLogicalPorts,
                                                    ViInt32   logicalPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:DEF:SGR ", channel);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, logicalPorts, numberOfLogicalPorts));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select predefined complex ratios of the standard 2-port
/// HIFN  wave quantities a1, a2, b1, and b2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR ratios/Select predefined complex ratios of the standard 2-port wave
/// HIPAR ratios/quantities a1, a2, b1, and b2.
ViStatus _VI_FUNC rsvna_SelectRatios(ViSession instrSession,
                                     ViInt32   channel,
                                     ViString  traceName,
                                     ViInt32   ratios)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ratios, RSVNA_VAL_RATIO_B1_A1_SRC_PORT_1, RSVNA_VAL_RATIO_B1_A2_SRC_PORT_2),
            4, "Ratios");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));
    sprintf(cmd + offset, "'%s','%s'", traceName, ratiosArr[ratios]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select arbitrary ratios of wave quantities, e.g. for
/// HIFN  different detectors and source ports or more than 2 ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR sourcePort/Source Port selects one of the available test ports of the analyzer as
/// HIPAR sourcePort/a source of the stimulus signal.
/// HIPAR numeratorType/Selects the type of the wave that forms the numerator of the ratio.
/// HIPAR numeratorPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR numeratorPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR numeratorPortNumber/on the analyzer model.
/// HIPAR denominatorType/Selects the type of the wave that forms the denominator of the ratio.
/// HIPAR denominatorPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR denominatorPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR denominatorPortNumber/on the analyzer model.
ViStatus _VI_FUNC rsvna_SelectMoreRatios(ViSession instrSession,
                                         ViInt32   channel,
                                         ViString  traceName,
                                         ViInt32   sourcePort,
                                         ViInt32   numeratorType,
                                         ViInt32   numeratorPortNumber,
                                         ViInt32   denominatorType,
                                         ViInt32   denominatorPortNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numeratorType, RSVNA_VAL_A, RSVNA_VAL_B),
            5, "Numerator Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, denominatorType, RSVNA_VAL_A, RSVNA_VAL_B),
            7, "Denominator Type");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s%ldD%ld/%s%ldD%ld'", traceName,
            numeratorArr[numeratorType],
            numeratorPortNumber,
            sourcePort,
            numeratorArr[denominatorType],
            denominatorPortNumber,
            sourcePort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select arbitrary ratios of wave quantities with
/// HIFN  detector, e.g. for different detectors and source ports or more than 2
/// HIFN  ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR sourcePort/Source Port selects one of the available test ports of the analyzer as
/// HIPAR sourcePort/a source of the stimulus signal.
/// HIPAR numeratorType/Selects the type of the wave that forms the numerator of the ratio.
/// HIPAR numeratorPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR numeratorPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR numeratorPortNumber/on the analyzer model.
/// HIPAR denominatorType/Selects the type of the wave that forms the denominator of the ratio.
/// HIPAR denominatorPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR denominatorPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR denominatorPortNumber/on the analyzer model.
/// HIPAR detector/Selects the algorithm that is used to calculate the displayed
/// HIPAR detector/measurement points from the raw data.
/// HIPAR measurementTime/This control sets the detector observation time ratios and wave
/// HIPAR measurementTime/quantities if a Peak or RMS detector is active.
ViStatus _VI_FUNC rsvna_SelectMoreRatiosWithDetector(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViString  traceName,
                                                     ViInt32   sourcePort,
                                                     ViInt32   numeratorType,
                                                     ViInt32   numeratorPortNumber,
                                                     ViInt32   denominatorType,
                                                     ViInt32   denominatorPortNumber,
                                                     ViInt32   detector,
                                                     ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numeratorType, RSVNA_VAL_A, RSVNA_VAL_B_PRIMED),
            5, "Numerator Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, denominatorType, RSVNA_VAL_A, RSVNA_VAL_B_PRIMED),
            7, "Denominator Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, RSVNA_VAL_DET_RMS, RSVNA_VAL_DET_AVG),
            9, "Detector");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s%ldD%ld/%s%ldD%ld%s'", traceName,
            numeratorArr[numeratorType],
            numeratorPortNumber,
            sourcePort,
            numeratorArr[denominatorType],
            denominatorPortNumber,
            sourcePort,
            detectorArr[detector]);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_DETECTOR_TIME, measurementTime),
            10, "Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the standard 2-port wave quantities a1, a2, b1,
/// HIFN  and b2 for different source ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR waveQuantities/Select the standard 2-port wave quantities a1, a2, b1, and b2 for
/// HIPAR waveQuantities/different source ports.
ViStatus _VI_FUNC rsvna_SelectWaveQuantities(ViSession instrSession,
                                             ViInt32   channel,
                                             ViString  traceName,
                                             ViInt32   waveQuantities)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveQuantities, RSVNA_VAL_WQUANTITY_A1_SRC_PORT_1, RSVNA_VAL_WQUANTITY_B2_SRC_PORT_2),
            4, "Wave Quantities");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s'", traceName, wquantityArr[waveQuantities]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select arbitrary wave quantities, e.g. for different
/// HIFN  detectors and source ports or more than 2 ports, frequency offsets, or
/// HIFN  mixed mode measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR waveQuantityType/Selects the type of the wave quantity.
/// HIPAR waveQuantityPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR waveQuantityPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR waveQuantityPortNumber/on the analyzer model.
/// HIPAR sourcePort/Source Port selects one of the available test ports of the analyzer as
/// HIPAR sourcePort/a source of the stimulus signal. The analyzer places no restriction on
/// HIPAR sourcePort/the combination of source ports and port numbers of the measured wave
/// HIPAR sourcePort/quantity, so it is even possible to measure a2 while the source port is
/// HIPAR sourcePort/port 1 (e.g. in order to estimate the directivity of the coupler).
/// HIPAR showAs/Selects the physical unit of the displayed trace. It is possible to
/// HIPAR showAs/display the measured "Voltage" V or convert the wave quantity into an
/// HIPAR showAs/effective power according to P = V2/Re(Z0). Z0 denotes the reference
/// HIPAR showAs/impedance of the source port (for wave quantities an) or of the receive
/// HIPAR showAs/port (for wave quantities bn).
ViStatus _VI_FUNC rsvna_SelectMoreWaveQuantities(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViString  traceName,
                                                 ViInt32   waveQuantityType,
                                                 ViInt32   waveQuantityPortNumber,
                                                 ViInt32   sourcePort,
                                                 ViInt32   showAs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveQuantityType, RSVNA_VAL_A, RSVNA_VAL_B),
            4, "Wave Quantity Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, showAs, RSVNA_VAL_WAVE_UNIT_TYPE_POWER, RSVNA_VAL_WAVE_UNIT_TYPE_VOLTAGE),
            7, "Show As");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s%ldD%ld'", traceName,
            numeratorArr[waveQuantityType],
            waveQuantityPortNumber,
            sourcePort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_WAVE_QUANTITIES_UNIT_TYPE, showAs),
            7, "Show As");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select arbitrary wave quantities with detector, e.g. for
/// HIFN  different detectors and source ports or more than 2 ports, frequency
/// HIFN  offsets, or mixed mode measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR waveQuantityType/Selects the type of the wave quantity.
/// HIPAR waveQuantityPortNumber/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR waveQuantityPortNumber/number is selected. The range of output and input port numbers depends
/// HIPAR waveQuantityPortNumber/on the analyzer model.
/// HIPAR sourcePort/Source Port selects one of the available test ports of the analyzer as
/// HIPAR sourcePort/a source of the stimulus signal. The analyzer places no restriction on
/// HIPAR sourcePort/the combination of source ports and port numbers of the measured wave
/// HIPAR sourcePort/quantity, so it is even possible to measure a2 while the source port is
/// HIPAR sourcePort/port 1 (e.g. in order to estimate the directivity of the coupler).
/// HIPAR showAs/Selects the physical unit of the displayed trace. It is possible to
/// HIPAR showAs/display the measured "Voltage" V or convert the wave quantity into an
/// HIPAR showAs/effective power according to P = V2/Re(Z0). Z0 denotes the reference
/// HIPAR showAs/impedance of the source port (for wave quantities an) or of the receive
/// HIPAR showAs/port (for wave quantities bn).
/// HIPAR detector/Selects the algorithm that is used to calculate the displayed
/// HIPAR detector/measurement points from the raw data.
/// HIPAR measurementTime/This control sets the detector observation time ratios and wave
/// HIPAR measurementTime/quantities if a Peak or RMS detector is active.
ViStatus _VI_FUNC rsvna_SelectMoreWaveQuantitiesWithDetector(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViString  traceName,
                                                             ViInt32   waveQuantityType,
                                                             ViInt32   waveQuantityPortNumber,
                                                             ViInt32   sourcePort,
                                                             ViInt32   showAs,
                                                             ViInt32   detector,
                                                             ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, waveQuantityType, RSVNA_VAL_A, RSVNA_VAL_B_PRIMED),
            4, "Wave Quantity Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, showAs, RSVNA_VAL_WAVE_UNIT_TYPE_POWER, RSVNA_VAL_WAVE_UNIT_TYPE_VOLTAGE),
            7, "Show As");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, RSVNA_VAL_DET_RMS, RSVNA_VAL_DET_AVG),
            8, "Detector");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s%ldD%ld%s'", traceName,
            numeratorArr[waveQuantityType],
            waveQuantityPortNumber,
            sourcePort, detectorArr[detector]);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_WAVE_QUANTITIES_UNIT_TYPE, showAs),
            7, "Show As");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_DETECTOR_TIME, measurementTime),
            9, "Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables phase normalization for wave quantity traces. If set to ON,
/// HIFN  the phase of the measured wave quantity is normalized to the a-wave of
/// HIFN  the selected driving port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enables phase normalization for wave quantity traces. If set to ON,
/// HIPAR enabled/the phase of the measured wave quantity is normalized to the a-wave of
/// HIPAR enabled/the selected driving port.
ViStatus _VI_FUNC rsvna_ConfigureWaveQuantitiesPhaseNormalizationEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_WAVE_QUANTITIES_PHASE_NORMALIZATION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_WAVE_QUANTITIES_PHASE_NORMALIZATION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the 2-port (or more ports) matched-circuit,
/// HIFN  converted impedances. The parameters describe the impedances of a
/// HIFN  2-port (or more ports) DUT, obtained in forward and reverse
/// HIFN  transmission and reflection measurements:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outPort/Z-S-parameters are expressed as Z-S<out><in>, where <out> and <in>
/// HIPAR outPort/denote the output and input port numbers of the DUT.
/// HIPAR inPort/Z-S-parameters are expressed as Z-S<out><in>, where <out> and <in>
/// HIPAR inPort/denote the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectImpedances(ViSession instrSession,
                                         ViInt32   channel,
                                         ViString  traceName,
                                         ViInt32   outPort,
                                         ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Z-S%02ld%02ld'", traceName,
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select converted, matched-circuit impedance parameters
/// HIFN  for more ports or balanced port measurements. All possible combinations
/// HIFN  of mixed mode parameters (e.g. Z-Sss, Z-Scs, Z-Sds, Z-Sdd,...) are
/// HIFN  provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/Z-S-parameters are expressed as Z-S<out><in>, where <out> and <in>
/// HIPAR outPort/denote the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/Z-S-parameters are expressed as Z-S<out><in>, where <out> and <in>
/// HIPAR inPort/denote the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectMoreImpedances(ViSession instrSession,
                                             ViInt32   channel,
                                             ViString  traceName,
                                             ViInt32   outMode,
                                             ViInt32   outPort,
                                             ViInt32   inMode,
                                             ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Z-S%s%s%02ld%02ld'", traceName,
            inOutModeArr[outMode],
            inOutModeArr[inMode],
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the 2-port (or more ports) converted
/// HIFN  matched-circuit admittance parameters. The parameters describe the
/// HIFN  admittances of a 2-port (or more ports) DUT, obtained in forward and
/// HIFN  reverse transmission and reflection measurements:
/// HIFN     Y11 is the input admittance of a 2-port DUT that is terminated at its
/// HIFN  output with the reference impedance Z0 (matched-circuit admittance
/// HIFN  measured in a forward reflection measurement).
/// HIFN     Y22 is the output admittance of a 2-port DUT that is terminated at its
/// HIFN  input with the reference impedance Z0 (matched-circuit admittance
/// HIFN  measured in a reverse reflection measurement).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outPort/Y-S-parameters are expressed as Y-S<out><in>, where <out> and <in>
/// HIPAR outPort/denote the output and input port numbers of the DUT.
/// HIPAR inPort/Y-S-parameters are expressed as Y-S<out><in>, where <out> and <in>
/// HIPAR inPort/denote the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectAdmittances(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   outPort,
                                          ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Y-S%02ld%02ld'", traceName, outPort, inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select converted, matched-circuit admittance parameters
/// HIFN  for more ports or balanced port measurements. All possible combinations
/// HIFN  of mixed mode parameters (e.g. Y-Sss, Y-Scs, Y-Sds, Y-Sdd,...) are
/// HIFN  provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/Y-S-parameters are expressed as Y-S<out><in>, where <out> and <in>
/// HIPAR outPort/denote the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/Y-S-parameters are expressed as Y-S<out><in>, where <out> and <in>
/// HIPAR inPort/denote the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectMoreAdmittances(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  traceName,
                                              ViInt32   outMode,
                                              ViInt32   outPort,
                                              ViInt32   inMode,
                                              ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Y-S%s%s%02ld%02ld'", traceName,
            inOutModeArr[outMode],
            inOutModeArr[inMode],
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select open-circuit Z-parameters for multiport
/// HIFN  measurements (including the 2-port Z-parameters) or mixed mode
/// HIFN  Z-parameters. All possible combinations of mixed mode parameters (e.g.
/// HIFN  Zss, Zcs, Zds, Zdd,...) are provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/Z-parameters are expressed as Z<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/Z-parameters are expressed as Z<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectZParameters(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   outMode,
                                          ViInt32   outPort,
                                          ViInt32   inMode,
                                          ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Z%s%s%02ld%02ld'", traceName,
            inOutModeArr[outMode],
            inOutModeArr[inMode],
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select short-circuit Y-parameters for multiport
/// HIFN  measurements (including the 2-port Y-parameters) or mixed mode
/// HIFN  Y-parameters. All possible combinations of mixed mode parameters (e.g.
/// HIFN  Yss, Ycs, Yds, Ydd,...) are provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outMode/Define port mode.
/// HIPAR outPort/Y-parameters are expressed as Y<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inMode/Define port mode.
/// HIPAR inPort/Y-parameters are expressed as Y<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectYParameters(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   outMode,
                                          ViInt32   outPort,
                                          ViInt32   inMode,
                                          ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            4, "Out Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inMode, RSVNA_VAL_SINGLE_ENDED, RSVNA_VAL_COMMON),
            6, "In Mode");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Y%s%s%02ld%02ld'", traceName,
            inOutModeArr[outMode],
            inOutModeArr[inMode],
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select an imbalance parameter as a measured quantity for
/// HIFN  the active trace. For a balanced port configuration with k balanced
/// HIFN  ports plus m single-ended ports (2 * k + m = n, where n is the number
/// HIFN  of analyzer ports), the pull-down list provides the full set of k * (k
/// HIFN  - 1 + 2 * m) imbalance parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outPort/Imbalance parameters are expressed as "Imb<out><in>", where <out> and
/// HIPAR outPort/<in> denote the logical output and input port numbers of the DUT.
/// HIPAR inPort/Imbalance parameters are expressed as "Imb<out><in>", where <out> and
/// HIPAR inPort/<in> denote the logical output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectImbalance(ViSession instrSession,
                                        ViInt32   channel,
                                        ViString  traceName,
                                        ViInt32   outPort,
                                        ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','IMB%ld%ld'", traceName,
            outPort,
            inPort);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select one of the three two port stability factors K, u1
/// HIFN  or u2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR dutOut/Selects the test port number of the analyzer to be connected to the
/// HIPAR dutOut/input (DUT Input) and the output of the DUT (DUT Output). The ports can
/// HIPAR dutOut/be arbitrary, however, the stability factor calculation is based on
/// HIPAR dutOut/2-port reflection and transmission S-parameters so that the input and
/// HIPAR dutOut/output port numbers must be different.
/// HIPAR dutIn/Selects the test port number of the analyzer to be connected to the
/// HIPAR dutIn/input (DUT Input) and the output of the DUT (DUT Output). The ports can
/// HIPAR dutIn/be arbitrary, however, the stability factor calculation is based on
/// HIPAR dutIn/2-port reflection and transmission S-parameters so that the input and
/// HIPAR dutIn/output port numbers must be different.
/// HIPAR stabilityFactor/Selects the stability factor to be calculated.
ViStatus _VI_FUNC rsvna_SelectStabilityFactors(ViSession instrSession,
                                               ViInt32   channel,
                                               ViString  traceName,
                                               ViInt32   dutOut,
                                               ViInt32   dutIn,
                                               ViInt32   stabilityFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, stabilityFactor, RSVNA_VAL_SFACTOR_K, RSVNA_VAL_SFACTOR_U2),
            6, "Stability Factor");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    if (stabilityFactor == RSVNA_VAL_SFACTOR_K)
    {
        sprintf(cmd + offset, "'%s','KFAC%ld%ld'", traceName,
                dutOut,
                dutIn);
    }
    else
    {
        sprintf(cmd + offset, "'%s','MUF%ld%ld%ld'", traceName,
                stabilityFactor,
                dutOut,
                dutIn);
    }

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the DC voltages fed to the DC MEAS input
/// HIFN  connectors as measured quantities. The input connectors are located at
/// HIFN  the rear panel of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR dcMeas/Select the DC voltages fed to the DC MEAS input connectors as measured
/// HIPAR dcMeas/quantities.
ViStatus _VI_FUNC rsvna_SelectDCMeasurement(ViSession instrSession,
                                            ViInt32   channel,
                                            ViString  traceName,
                                            ViInt32   dcMeas)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dcMeas, RSVNA_VAL_DC_MEAS_1V, RSVNA_VAL_DC_MEAS_10V),
            4, "DC Meas");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s'", traceName,
            dcvArr[dcMeas]);

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the measurement range of the input
/// HIFN  connectors. For best accuracy, the range should be adjusted to the
/// HIFN  measured signal. To protect the instrument, the range is re-set to "+/-
/// HIFN  20 V" (the widest range) when a new DC connector is selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dcInput/This control selects an DC input connector on the read panel.
/// HIPAR range/This control selects an DC voltage range.
ViStatus _VI_FUNC rsvna_ConfigureDCRange(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   dcInput,
                                         ViInt32   range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dcInput, 1, 4),
            3, "DC Input");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",DCInput%ld", channel, dcInput);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DC_RANGE, range),
            4, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the measurement type for the Power Added Efficiency (PAE)
/// HIFN measurement. The three measurement types involve different test setups,
/// HIFN input parameters,and DC voltage measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR measurementType/This control selects measurement type for the Power Added
/// HIPAR measurementType/Efficiency (PAE) measurement.
ViStatus _VI_FUNC rsvna_ConfigurePAEMeasurementType(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   measurementType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PAE_MEASUREMENT_TYPE, measurementType),
            4, "Measurement Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the first (main) DC INPUT connector for the PAE measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR type/This control selects input type (main/secondary).
/// HIPAR DCInput/This control selects an DC input connector on the rear panel.
ViStatus _VI_FUNC rsvna_ConfigurePAEDCInput(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   type,
                                            ViInt32   DCInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 1, 2),
            3, "Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, PAETypeArr[type]);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PAE_DC_INPUT, DCInput),
            4, "DC Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines DC supply current I, resistance R, or DC supply voltage U
/// HIFN for the PAE measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR parameterType/This control selects parameter type.
/// HIPAR parameterValue/Sets the parameter value.
ViStatus _VI_FUNC rsvna_ConfigurePAEParameters(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   parameterType,
                                               ViReal64  parameterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterType, 0, 2),
            3, "Parameter Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, PAEParameterTypeArr[parameterType]);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PAE_PARAMETERS_VALUE, parameterValue),
            4, "parameter Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a balanced port numbered <Logicalog Port>,
/// HIFN  combining two physical ports <Physical Port 1> and <Physical Port 2>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/Logical port number used to number balanced ports.
/// HIPAR physicalPort1/This control defines the port number.
/// HIPAR physicalPort2/This control defines the port number.
ViStatus _VI_FUNC rsvna_DefineBalancedPort(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   logicalPort,
                                           ViInt32   physicalPort1,
                                           ViInt32   physicalPort2)
{
    return rsvna_DefineLogicalPort(instrSession, channel, logicalPort, physicalPort1, physicalPort2);
}

/// HIFN  This function dissolves selected balanced port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/Logical port number used to number balanced ports.
ViStatus _VI_FUNC rsvna_DeleteBalancedPort(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   logicalPort)
{
    return rsvna_DeleteLogicalPort(instrSession, channel, logicalPort);
}

/// HIFN  This function dissolves all balanced ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteAllBalancedPorts(ViSession instrSession,
                                               ViInt32   channel)
{
    return rsvna_DeleteAllLogicalPorts(instrSession, channel);
}

/// HIFN  This function specifies the complex common mode reference impedance
/// HIFN  for the balanced (logical) port numbered <log_port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/This control defines the logical port number.
/// HIPAR real/This control defines the real part of the port impedance.
/// HIPAR imaginary/This control defines the imaginary part of the port impedance.
ViStatus _VI_FUNC rsvna_ConfigureLogicalPortReferenceImpedanceCommonMode(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   logicalPort,
                            ViReal64  real,
                            ViReal64  imaginary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, real, 1.0e-3, 10.0e6), 4, "Real");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, imaginary, -10.0e6, 10.0e6), 4, "Imaginary");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:LPOR%ld:ZCOM %.12lG,%.12lG", channel, logicalPort, real, imaginary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the complex differential mode reference
/// HIFN  impedance for the balanced (logical) port numbered <log_port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/This control defines the logical port number.
/// HIPAR real/This control defines the real part of the port impedance.
/// HIPAR imaginary/This control defines the imaginary part of the port impedance.
ViStatus _VI_FUNC rsvna_ConfigureLogicalPortReferenceImpedanceDifferentialMode(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   logicalPort,
                            ViReal64  real,
                            ViReal64  imaginary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, real, 1.0e-3, 10.0e6), 4, "Real");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, imaginary, -10.0e6, 10.0e6), 4, "Imaginary");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:LPOR%ld:ZDIF %.12lG,%.12lG", channel, logicalPort, real, imaginary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the theory for the renormalization of port
/// HIFN  impedances. The selection has an impact on the conversion formulas for
/// HIFN  wave quantities and S-parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR theory/This control selects the theory for the renormalization of port
/// HIPAR theory/impedances. The selection has an impact on the conversion formulas for
/// HIPAR theory/wave quantities and S-parameters.
ViStatus _VI_FUNC rsvna_ConfigureLogicalPortReferenceImpedanceRenormalization(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   theory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LOGICAL_PORT_REFERENCE_IMPEDANCE_RENORMALIZATION, theory),
            3, "Theory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows to toggle between default and renormalized
/// HIFN  reference impedance(s) for logical port <LogPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/This control defines the logical port number.
/// HIPAR useDefault/This control allows to toggle between default and renormalized
/// HIPAR useDefault/reference impedance(s) for logical port <LogPt>.
ViStatus _VI_FUNC rsvna_ConfigureLogicalPortReferenceImpedanceDefault(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   logicalPort,
                            ViBoolean useDefault)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",LogPort%ld", channel, logicalPort);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LOGICAL_PORT_REFERENCE_IMPEDANCE_USE_DEFAULT, useDefault),
            4, "Use Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects general intermodulation products.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR order/Defines the order of the intercept point.
/// HIPAR side/Define port mode.
/// HIPAR DUTPort/Define port mode.
ViStatus _VI_FUNC rsvna_SelectIntermodulationProducts(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViString  traceName,
                                                      ViInt32   order,
                                                      ViInt32   side,
                                                      ViInt32   DUTPort,
                                                      ViBoolean relative)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString sideArr[] = {"M", "U", "L"};
    ViString DUTPortArr[] = {"O", "I"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K14"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, order, 3, 9),
            4, "Order");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, side, RSVNA_VAL_INTERMODULATION_MAJOR, RSVNA_VAL_INTERMODULATION_LOWER),
            5, "Side");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    if (relative == VI_TRUE)
        sprintf(cmd + offset, "'%s','IM%ld%s%sR'", traceName, order, sideArr[side], DUTPortArr[DUTPort]);
    else
        sprintf(cmd + offset, "'%s','IM%ld%s%s'", traceName, order, sideArr[side], DUTPortArr[DUTPort]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects general intercept points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR order/Defines the order of the intercept point.
/// HIPAR side/Define port mode.
/// HIPAR DUTPort/Define port mode.
ViStatus _VI_FUNC rsvna_SelectInterceptPoints(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  traceName,
                                              ViInt32   order,
                                              ViInt32   side,
                                              ViInt32   DUTPort)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViString sideArr[] = {"M", "U", "L"};
    ViString DUTPortArr[] = {"O", "I"};
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K14"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, order, 3, 9),
            4, "Order");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, side, RSVNA_VAL_INTERMODULATION_MAJOR, RSVNA_VAL_INTERMODULATION_LOWER),
            5, "Side");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','IP%ld%s%s'", traceName, order, sideArr[side], DUTPortArr[DUTPort]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects noise level of the intermodulation measurement.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_SelectNoiseLevel(ViSession instrSession,
                                         ViInt32   channel,
                                         ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K30"));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','NLO'", traceName);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects noise level of the intermodulation measurement.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR tone/Select the measurement of the two fundamental waves of the
/// HIPAR tone/intermodulation measurement.
ViStatus _VI_FUNC rsvna_SelectMainTone(ViSession instrSession,
                                       ViInt32   channel,
                                       ViString  traceName,
                                       ViInt32   tone)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString toneArr[] = {"LTO", "LTI", "UTO","UTI"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K14"));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s'", traceName, toneArr[tone]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function select an power sensor measurement using a power
/// HIFN meter 'Pmtr<no>' and either an
/// HIFN external generator 'G<no>' or an analyzer source port 'D<no>'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR traceName/Define trace name.
/// HIPAR powerSensor/Power source parameter Pmtr<n>
/// HIPAR source/Selects the source.
/// HIPAR sourcePort/Selectrs the port number of selected Source. 'G<no>' for
/// HIPAR External Generator, 'D<no>' for Analyzer.
ViStatus _VI_FUNC rsvna_SelectPowerSensorMeasurement(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViString  traceName,
                                                     ViInt32   powerSensor,
                                                     ViInt32   source,
                                                     ViInt32   sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  offset;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    switch (source)
    {
    case RSVNA_VAL_PMET_EXT_GEN:
        sprintf(cmd + offset, "'%s','Pmtr%ldG%02ld'", traceName,
                powerSensor, sourcePort);
        break;
    case RSVNA_VAL_PMET_ANALYZER:
        sprintf(cmd + offset, "'%s','Pmtr%ldD%02ld'", traceName,
                powerSensor, sourcePort);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, source), 5, "Source");
    }

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  COnfigures the Infinite Averaging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Turns infinite averaging on or off.
/// HIPAR mode/Selects the quantities to be averaged if infinite averaging is
/// HIPAR mode/enabled.
ViStatus _VI_FUNC rsvna_ConfigureInfiniteAveraging(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled,
                                                   ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INFINITE_AVERAGING_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_INFINITE_AVERAGING_ENABLED, enabled),
            3, "Enabled");

    if (enabled)
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_INFINITE_AVERAGING_MODE, mode),
                4, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects noise figure measurements.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIFN
/// HIFN     Noise figure parameters NF<out><in>, where <out> and <in> denote the
/// HIFN  output and input port numbers of the DUT, and <out>!=<in>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR outPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR outPort/the output and input port numbers of the DUT.
/// HIPAR inPort/S-parameters are expressed as S<out><in>, where <out> and <in> denote
/// HIPAR inPort/the output and input port numbers of the DUT.
ViStatus _VI_FUNC rsvna_SelectNoiseFigure(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   outPort,
                                          ViInt32   inPort)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','NF%ld%ld'", traceName, outPort, inPort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects noise figure DUT gain measurements.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR bWave/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR bWave/number is selected. The range of output and input port numbers depends
/// HIPAR bWave/on the analyzer model.
/// HIPAR aWave/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR aWave/number is selected. The range of output and input port numbers depends
/// HIPAR aWave/on the analyzer model.
/// HIPAR sourcePort/Selects the source port.
ViStatus _VI_FUNC rsvna_SelectNoiseFigureDUTGain(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViString  traceName,
                                                 ViInt32   bWave,
                                                 ViInt32   aWave,
                                                 ViInt32   sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','b%ld/a%ld(P%ld)'", traceName, bWave, aWave, sourcePort);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects receiver noise figure measurements.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR port/Selects the port.
ViStatus _VI_FUNC rsvna_SelectReceiverNoiseFigure(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  traceName,
                                                  ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','NF%ldR'", traceName, port);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects noise (power spectral) density measurements
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR port/Selects the port.
ViStatus _VI_FUNC rsvna_SelectNoiseDensity(ViSession instrSession,
                                           ViInt32   channel,
                                           ViString  traceName,
                                           ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','ND%ld'", traceName, port);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the physical VNA port for the spectrum
/// HIFN  measurement.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR port/S-parameters are expressed as S<port>.
ViStatus _VI_FUNC rsvna_SelectSpectrum(ViSession instrSession,
                                       ViInt32   channel,
                                       ViString  traceName,
                                       ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','SA%ld'", traceName, port);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the spectrum trace at the reference receiver
/// HIFN  port measurement .
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR port/S-parameters are expressed as S<port>.
ViStatus _VI_FUNC rsvna_SelectSpectrumReference(ViSession instrSession,
                                                ViInt32   channel,
                                                ViString  traceName,
                                                ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','SA%ldREF'", traceName, port);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the physical VNA port for the spectrum
/// HIFN  measurement.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR measurement/Selects the input power, output power, or transmission S-parameter
/// HIPAR measurement/value at the <x dB> compression point as the measured quantity, where
/// HIPAR measurement/<x dB> is the specified Compression Value.
/// HIPAR compressionValue/Defines the compression value for a gain compression measurement in
/// HIPAR compressionValue/channel <Ch>.
ViStatus _VI_FUNC rsvna_SelectGainCompression(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  traceName,
                                              ViInt32   measurement,
                                              ViReal64  compressionValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViInt32  offset;
    ViString measArr[] = {"CmpPtPin", "CmpPtPout", "CmpPtS"};
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_VALUE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSVNA_VAL_GAIN_COMPRESSION_POWER_IN, RSVNA_VAL_GAIN_COMPRESSION_S_PARAMETER),
            4, "Measurement");

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s'", traceName, measArr[measurement]);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_VALUE, compressionValue),
            5, "Compression Vlaue");

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function select configures the Harmonic Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR order/Sets the order.
/// HIPAR receiverPort/Sets the receiver port.
/// HIPAR sourcePort/Sets the source port.
/// HIPAR relative/Sets the relative or dirrect settings.
ViStatus _VI_FUNC rsvna_SelectHarmonicMeasurement(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  traceName,
                                                  ViInt32   order,
                                                  ViInt32   receiverPort,
                                                  ViInt32   sourcePort,
                                                  ViBoolean relative)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    if (relative == VI_TRUE)
    {
        sprintf(cmd + offset, "'%s','H%ldB%ldD%ldRSAM'", traceName, order, receiverPort, sourcePort);
    }
    else
    {
        sprintf(cmd + offset, "'%s','H%ldB%ldD%ldSAM'", traceName, order, receiverPort, sourcePort);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select configures the Harmonic Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR order/Sets the order.
/// HIPAR receiverPort/Sets the receiver port.
/// HIPAR sourcePort/Sets the source port.
/// HIPAR mode/Switches between THDf or THDr.
ViStatus _VI_FUNC rsvna_SelectHarmonicMeasurementTHD(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViString  traceName,
                                                     ViInt32   order,
                                                     ViInt32   receiverPort,
                                                     ViInt32   sourcePort,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','H%ldB%ldD%ld%s'", traceName, order, receiverPort, sourcePort, THDFormatArr[mode]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the two-tone group delay measurements.
/// HIFN
/// HIFN     Assigns a measurement parameter to an existing trace or creates a
/// HIFN  trace and assigns a channel number, a name and a measurement parameter
/// HIFN  to it. The trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
/// HIPAR mode/Switches between quantities to be measured.
ViStatus _VI_FUNC rsvna_SelectTwoToneGroupDelayMeasurement(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViString  traceName,
                                                           ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','%s'", traceName, TwoToneGroupDelayArr[mode]);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines how the measured result at any sweep point is post-processed and presented in the graphical display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR format/Defines how the measured data is presented in the graphical display.
/// HIPAR aperturePoints/Defines an aperture for the calculation of the group delay as an integer number of frequency sweep steps.
ViStatus _VI_FUNC rsvna_ConfigureTraceFormat(ViSession instrSession,
                                             ViInt32   channel_Trace,
                                             ViInt32   format,
                                             ViInt32   aperturePoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRACE_FORMAT, format),
            3, "Format");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRACE_APERTURE_POINTS, aperturePoints),
            4, "Aperture Points");

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adjusts the Scale Divisions and the Ref. Value in order
/// HIFN  to display the entire active trace in the diagram area, leaving an
/// HIFN  appropriate display margin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/This control sets the trace name.
ViStatus _VI_FUNC rsvna_AutoScaleTrace(ViSession instrSession,
                                       ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:AUTO ONCE,'%s'", traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures trace parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/This control sets the trace name.
/// HIPAR scaleDivisions/Scale /Div corresponds to the increment between two consecutive grid
/// HIPAR scaleDivisions/lines. The unit depends on the display format: dB for display format dB
/// HIPAR scaleDivisions/Mag, degrees for Phase and Unwrapped Phase, ns for Delay, U (units) for
/// HIPAR scaleDivisions/all other (dimensionless) formats.
/// HIPAR referenceLevel/Sets the reference level (or reference value) for a particular
/// HIPAR referenceLevel/displayed trace. Setting a new reference level does not affect the
/// HIPAR referenceLevel/value of PDIVision.
/// HIPAR referencePosition/Sets the point on the y-axis to be used as the reference position as a
/// HIPAR referencePosition/percentage of the length of the y-axis. The reference position is the
/// HIPAR referencePosition/point on the y-axis which should equal the RLEVel.
ViStatus _VI_FUNC rsvna_ConfigureTrace(ViSession instrSession,
                                       ViString  traceName,
                                       ViReal64  scaleDivisions,
                                       ViReal64  referenceLevel,
                                       ViReal64  referencePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:PDIV %.12lG,'%s'", scaleDivisions, traceName);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:RLEV %.12lG,'%s'", referenceLevel, traceName);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:RPOS %.12lG,'%s'", referencePosition, traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the upper and lower edge of the diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/This control sets the trace name.
/// HIPAR top/Value for the upper (maximum) edge of the diagram area <Wnd>.
/// HIPAR bottom/Value for the lower (minimum) edge of the diagram area <Wnd>.
ViStatus _VI_FUNC rsvna_ConfigureTraceMaxMin(ViSession instrSession,
                                             ViString  traceName,
                                             ViReal64  top,
                                             ViReal64  bottom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:TOP %.12lG,'%s'", top, traceName);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:Y:BOTT %.12lG,'%s'", bottom, traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new trace in the current diagram area and
/// HIFN  assigns it to the selected channel. The new trace is created with the
/// HIFN  default trace and channel settings. Trace is not displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_TraceAdd(ViSession instrSession,
                                 ViInt32   channel,
                                 ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", channel);

    checkErr(RsCore_AddRepCap(instrSession, "Channel", repCap, repCap_cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:SDEF '%s','S21'", channel, traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays or hides an existing trace, identified by its trace name
/// HIFN  (CALCulate<Ch>:PARameter:SDEFine <Trace_Name>), or a group of traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceType/This control selects the type of trace.
/// HIPAR singleTraceName/Define trace name.
/// HIPAR showTrace/This control displays or hides trace(s).
ViStatus _VI_FUNC rsvna_ConfigureTraceDisplayState(ViSession instrSession,
                                                   ViInt32   traceType,
                                                   ViString  singleTraceName,
                                                   ViBoolean showTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (traceType)
    {
    case RSVNA_VAL_DISP_TRAC_DATA:
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:SHOW DALL,%s", showTrace ? "ON" : "OFF");
            break;
        }
    case RSVNA_VAL_DISP_TRAC_MEM:
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:SHOW MALL,%s", showTrace ? "ON" : "OFF");
            break;
        }
    case RSVNA_VAL_DISP_TRAC_SING:
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TRAC:SHOW '%s',%s", singleTraceName, showTrace ? "ON" : "OFF");
            break;
        }
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, traceType), 2, "Trace Type");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays or hides the label of an existing trace, identified by its
/// HIFN  <TraceName>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR singleTraceName/Define trace name.
/// HIPAR displayed/Display or hide the label of the related trace.
ViStatus _VI_FUNC rsvna_ConfigureTraceDisplayLabel(ViSession instrSession,
                                                   ViString  singleTraceName,
                                                   ViBoolean displayed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s',%s", singleTraceName, offOnArr[displayed]);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_TRACE_LABEL, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new trace in a new diagram area and assigns
/// HIFN  the trace to the selected channel. The new trace is created with the
/// HIFN  default trace and channel settings and displayed in the new diagram
/// HIFN  area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_TraceAddDiagramArea(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   channel,
                                            ViString  traceName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_TraceAdd(instrSession, channel, traceName));

    checkErr(rsvna_TraceAssignDiagramArea(instrSession, window, traceName));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns an existing trace to a diagram area, and
/// HIFN  displays the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_TraceAssignDiagramArea(ViSession instrSession,
                                               ViInt32   window,
                                               ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", window);

    checkErr(RsCore_AddRepCap(instrSession, "Window", repCap, repCap_cmd));

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_DIAGRAM_STATE, VI_TRUE));

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_ASSIGN_DIAGRAM, traceName),
            3, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function releases the assignment between a trace and a diagram
/// HIFN  area, as defined by means of DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED
/// HIFN  <Trace_Name> and expressed by the <WndTr> suffix. The trace itself is
/// HIFN  not deleted; this must be done via function rsvna_TraceDelete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR window_Trace/Trace number used to distinguish the traces of the same diagram area
/// HIPAR window_Trace/<Wnd>.
ViStatus _VI_FUNC rsvna_TraceUnassignDiagramArea(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   window_Trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:DEL", window, window_Trace);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects an existing trace as the active trace of the
/// HIFN  channel. All trace commands without explicit reference to the trace
/// HIFN  name act on the active trace (e.g. CALCulate<Ch>:FORMat).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_TraceSelect(ViSession instrSession,
                                    ViInt32   channel,
                                    ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_SELECT_ACTIVE, traceName),
            3, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a trace with a specified trace name and channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_TraceDelete(ViSession instrSession,
                                    ViInt32   channel,
                                    ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_DELETE, traceName),
            3, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes all traces in channel no. <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR channel/Document parameter here.
ViStatus _VI_FUNC rsvna_TraceDeleteAll(ViSession instrSession,
                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes all traces in all channels of the active recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_TraceDeleteAllChannels(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TRACE_DELETE_ALL_CHANNELS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trace names and measurement parameters of
/// HIFN  all traces assigned to a particular channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR catalog/Response is a string parameter with comma-separated list of trace
/// HIPAR catalog/names and measurement parameters, e.g. 'CH4TR1,S11,CH4TR2,S12'. The
/// HIPAR catalog/measurement parameters are returned according to the naming convention
/// HIPAR catalog/of CALCulate<Ch>:PARameter:SDEFine. The order of traces in the list
/// HIPAR catalog/reflects their creation time: The oldest trace is the first, the newest
/// HIPAR catalog/trace is the last trace in the list.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_TraceList(ViSession instrSession,
                                  ViInt32   channel,
                                  ViChar    catalog[],
                                  ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_CHANNEL_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trace names and measurement parameters of
/// HIFN  all traces assigned to a particular channel. In presence of balanced
/// HIFN  ports, this function distinguishes between:
/// HIFN     - single-ended S-parameters for logical ports (SSS<Li><Lj>)
/// HIFN     - "raw" single-ended S-parameters referring to physical ports
/// HIFN  (S<Pi><Pj>)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR catalog/Response is a string parameter with comma-separated list of trace
/// HIPAR catalog/names and measurement parameters, e.g. 'CH4TR1,S11,CH4TR2,S12'. The
/// HIPAR catalog/measurement parameters are returned according to the naming convention
/// HIPAR catalog/of CALCulate<Ch>:PARameter:SDEFine. The order of traces in the list
/// HIPAR catalog/reflects their creation time: The oldest trace is the first, the newest
/// HIPAR catalog/trace is the last trace in the list.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_TraceListBalancedPorts(ViSession instrSession,
                                               ViInt32   channel,
                                               ViChar    catalog[],
                                               ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_CHANNEL_CATALOG_BALANCED_PORTS,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a new name to a trace. The trace does not have
/// HIFN  to be the active trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR oldTraceName/Define old trace name name.
/// HIPAR newTraceName/Define new trace name.
ViStatus _VI_FUNC rsvna_TraceRename(ViSession instrSession,
                                    ViString  oldTraceName,
                                    ViString  newTraceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC:REN '%s','%s'", oldTraceName, newTraceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a new name to the active trace in channel <ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR traceName/Define new trace name.
ViStatus _VI_FUNC rsvna_ChannelTraceRename(ViSession instrSession,
                                           ViInt32   channel,
                                           ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_CHANNEL_RENAME, traceName),
            3, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the numbers and names of all traces in the
/// HIFN  current setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR catalog/Returns string with comma-separated list of trace numbers and names.
/// HIPAR catalog/If all traces have been deleted the response is an empty string ("").
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_TraceListCatalog(ViSession instrSession,
                                         ViChar    catalog[],
                                         ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_TRACE_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the numbers and names of all traces in channel
/// HIFN  no. <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR catalog/Returns the numbers and names of all traces in channel no. <Ch>. The
/// HIPAR catalog/response is a string containing a comma-separated list of trace numbers
/// HIPAR catalog/and names. If all traces have been deleted the response is an empty
/// HIPAR catalog/string ("").
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_TraceListChannelCatalog(ViSession instrSession,
                                                ViInt32   channel,
                                                ViChar    catalog[],
                                                ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_NAME_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trace <Trc> name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceNumber/Trace number.
/// HIPAR traceName/Returns the trace name.
ViStatus _VI_FUNC rsvna_QueryTraceName(ViSession instrSession,
                                       ViInt32   traceNumber,
                                       ViChar    traceName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC%ld:NAME?", traceNumber);
    checkErr(rsvna_QueryViString(instrSession, cmd, RS_MAX_MESSAGE_BUF_SIZE, rd_buffer));
    strcpy(traceName, rd_buffer);
    RsCore_TrimString(traceName, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the trace number (numeric suffix) of a trace
/// HIFN  with known trace name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Sets the trace name.
/// HIPAR traceNumber/Returns the trace number (numeric suffix) of a trace with known trace
/// HIPAR traceNumber/name.
ViStatus _VI_FUNC rsvna_QueryTraceNumber(ViSession instrSession,
                                         ViString  traceName,
                                         ViInt32*  traceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC:NAME:ID? '%s'", traceName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, traceNumber));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of the diagram which the trace is
/// HIFN  assigned to. A zero is returned when the trace is not
/// HIFN  assigned/displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Sets the trace name.
/// HIPAR diagramNumber/Returns the number of the diagram which the trace is assigned to. A
/// HIPAR diagramNumber/zero is returned when the trace is not assigned/displayed.
ViStatus _VI_FUNC rsvna_QueryTraceDiagramNumber(ViSession instrSession,
                                                ViString  traceName,
                                                ViInt32*  diagramNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC:WIND? '%s'", traceName);
    checkErr(rsvna_QueryViInt32(instrSession, cmd, diagramNumber));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the channel name for an existing trace named
/// HIFN  '<Trc_name>'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Sets the trace name.
/// HIPAR channelName/Returns the channel name for an existing trace.
ViStatus _VI_FUNC rsvna_QueryTraceChannelName(ViSession instrSession,
                                              ViString  traceName,
                                              ViChar    channelName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC:CHAN:NAME? '%s'", traceName);
    checkErr(RsCore_QueryViString(instrSession, cmd, channelName));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns channel number (numeric suffix) for an existing
/// HIFN  trace named '<Trc_name>'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Sets the trace name.
/// HIPAR channelNumber/Returns the channel number (numeric suffix) for an existing trace.
ViStatus _VI_FUNC rsvna_QueryTraceChannelNumber(ViSession instrSession,
                                                ViString  traceName,
                                                ViInt32*  channelNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TRAC:CHAN:NAME:ID? '%s'", traceName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, channelNumber));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies the current state of the active data trace to a
/// HIFN  memory trace. If a mathematical trace is active, the data trace
/// HIFN  associated with the mathematical trace is copied. The memory trace is
/// HIFN  named Mem<n>[<Data_Trace>] where <n> counts all data and memory traces
/// HIFN  in the active setup in chronological order, and <Data_Trace> is the
/// HIFN  name of the associated (copied) data trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_TraceDataToMemory(ViSession instrSession,
                                          ViInt32   channel_Trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_DATA_COPY_TO_MEMMORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies a data trace to a memory trace. The trace to be
/// HIFN  copied is specified as a trace with a name (string variable). The
/// HIFN  created memory trace is specified as a memory trace with an arbitrary
/// HIFN  name (string variable). An existing memory trace with the same name is
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR memoryTrace/Name of the memory trace.
/// HIPAR dataTrace/Name of the data trace.
ViStatus _VI_FUNC rsvna_TraceDataToMemoryTrace(ViSession instrSession,
                                               ViString  memoryTrace,
                                               ViString  dataTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:COPY '%s','%s'", memoryTrace, dataTrace);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies a mathematical trace to a memory trace. The trace
/// HIFN  to be copied is specified as a trace with a name (string variable). The
/// HIFN  created memory trace is specified as a memory trace with an arbitrary
/// HIFN  name (string variable). An existing memory trace with the same name is
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR memoryTrace/Name of the memory trace.
/// HIPAR dataTrace/Name of the data trace.
ViStatus _VI_FUNC rsvna_TraceMathToMemoryTrace(ViSession instrSession,
                                               ViString  memoryTrace,
                                               ViString  dataTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:COPY:MATH '%s','%s'", memoryTrace, dataTrace);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all memory traces in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteMemoryTraceAll(ViSession instrSession,
                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MEMORY_TRACE_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all memory traces in all channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeleteMemoryTraceAllChannel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_MEMORY_TRACE_DELETE_ALL_CHANNELS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the mathematical mode where the active data
/// HIFN  trace is divided by the last generated memory trace. The division is
/// HIFN  calculated on a point-to-point basis: Each measurement point of the
/// HIFN  active trace is divided by the corresponding measurement point of the
/// HIFN  memory trace. The result of the division is a mathematical trace and
/// HIFN  replaces the active data trace in the diagram area. The mathematical
/// HIFN  trace is updated as the measurement goes on and the analyzer provides
/// HIFN  new active trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathematicalExpression/String parameter for the mathematical expression, enclosed in
/// HIPAR mathematicalExpression/brackets. Defines a general mathematical relation between traces.
ViStatus _VI_FUNC rsvna_TraceUserDefinedMath(ViSession instrSession,
                                             ViInt32   channel_Trace,
                                             ViString  mathematicalExpression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_EXPRESSION, mathematicalExpression),
            3, "Mathematical Expression");

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_MODE_STATE, VI_TRUE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the mathematical mode where the
/// HIFN  mathematical trace defined via rsvna_TraceUserDefinedMath function is
/// HIFN  calculated and displayed instead of the active data trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathState/Activates or deactivates the mathematical mode (ON - Display the
/// HIPAR mathState/active data trace, OFF - Display the mathematical trace).
ViStatus _VI_FUNC rsvna_ConfigureTraceMathState(ViSession instrSession,
                                                ViInt32   channel_Trace,
                                                ViBoolean mathState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_MODE_STATE, mathState),
            3, "Math State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a simple mathematical relation between the
/// HIFN  active trace and the active memory trace to calculate a new
/// HIFN  mathematical trace and displays the mathematical trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathematicalFunction/This control defines a simple mathematical relation between the active
/// HIPAR mathematicalFunction/trace and the active memory trace to calculate a new mathematical trace
/// HIPAR mathematicalFunction/and displays the mathematical trace.
ViStatus _VI_FUNC rsvna_ConfigureTraceMathFunction(ViSession instrSession,
                                                   ViInt32   channel_Trace,
                                                   ViInt32   mathematicalFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_FUNCTION, mathematicalFunction),
            3, "Mathematical Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the mathematical mode where the active data
/// HIFN  trace is divided by the last generated memory trace. The division is
/// HIFN  calculated on a point-to-point basis: Each measurement point of the
/// HIFN  active trace is divided by the corresponding measurement point of the
/// HIFN  memory trace. The result of the division is a mathematical trace and
/// HIFN  replaces the active data trace in the diagram area. The mathematical
/// HIFN  trace is updated as the measurement goes on and the analyzer provides
/// HIFN  new active trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathematicalExpression/String parameter for the mathematical expression, enclosed in
/// HIPAR mathematicalExpression/brackets. Defines a general mathematical relation between traces.
ViStatus _VI_FUNC rsvna_TraceUserDefinedMathFormatted(ViSession instrSession,
                                                      ViInt32   channel_Trace,
                                                      ViString  mathematicalExpression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACE_MATH_FORMATTED_EXPRESSION,
            "Channel", channel_Trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_FORMATTED_EXPRESSION, mathematicalExpression),
            3, "Mathematical Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the mathematical mode where the
/// HIFN  mathematical trace defined via rsvna_TraceUserDefinedMathFormatted
/// HIFN  function is calculated and displayed instead of the active data trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathState/Activates or deactivates trace math for unformatted and/or formatted
/// HIPAR mathState/trace data.
ViStatus _VI_FUNC rsvna_ConfigureTraceMathFormattedState(ViSession instrSession,
                                                         ViInt32   channel_Trace,
                                                         ViBoolean mathState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACE_MATH_FORMATTED_MODE_ENABLED,
            "Channel", channel_Trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_FORMATTED_MODE_ENABLED, mathState),
            3, "Math State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a simple mathematical relation between the
/// HIFN  active trace and the active memory trace to calculate a new
/// HIFN  mathematical trace and displays the mathematical trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR function/This control defines a simple mathematical relation between the active
/// HIPAR function/trace and the active memory trace to calculate a new mathematical trace
/// HIPAR function/and displays the mathematical trace.
ViStatus _VI_FUNC rsvna_ConfigureTraceMathFormattedFunction(ViSession instrSession,
                                                            ViInt32   channel_Trace,
                                                            ViInt32   function)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACE_MATH_FORMATTED_FUNCTION,
            "Channel", channel_Trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_FORMATTED_FUNCTION, function),
            3, "Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function controls the conversion and formatting of the mathematic
/// HIFN  expression defined via rsvna_TraceUserDefinedMath function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR mathWaveQuantityState/Controls the conversion and formatting of the mathematic expression.
ViStatus _VI_FUNC rsvna_ConfigureTraceMathWaveQuantityState(ViSession instrSession,
                                                            ViInt32   channel_Trace,
                                                            ViBoolean mathWaveQuantityState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_MATH_WAVE_QUANTITY_STATE, mathWaveQuantityState),
            3, "Math Wave Quantity State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the time domain settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR timeDomain/This control determines whether the time domain transformation for
/// HIPAR timeDomain/trace no. <Channel> is enabled.
/// HIPAR type/This control defines time domain transformation type.
/// HIPAR impulseResponse/This control defines window type for filtering the data in the
/// HIPAR impulseResponse/frequency domain prior to the time domain transformation.
/// HIPAR sideLobeLevel/This control sets the sideband suppression for the Dolph-Chebyshev
/// HIPAR sideLobeLevel/window.
/// HIPAR resolutionEnhancement/This control sets the resolution enhancement factor.
ViStatus _VI_FUNC rsvna_ConfigureTimeDomain(ViSession instrSession,
                                            ViInt32   channel,
                                            ViBoolean timeDomain,
                                            ViInt32   type,
                                            ViInt32   impulseResponse,
                                            ViReal64  sideLobeLevel,
                                            ViReal64  resolutionEnhancement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STATE, timeDomain),
            3, "Time Domain");

    switch (type)
    {
    case RSVNA_VAL_TYPE_BPAS_IMP:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_TYPE, RSVNA_VAL_TYPE_BAND_PASS),
                4, "Type");

        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS, RSVNA_VAL_STIMULUS_IMPULSE),
                4, "Type - Stimulus");
        break;

    case RSVNA_VAL_TYPE_LPAS_IMP:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_TYPE, RSVNA_VAL_TYPE_LOW_PASS),
                4, "Type");

        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS, RSVNA_VAL_STIMULUS_IMPULSE),
                4, "Type - Stimulus");
        break;

    case RSVNA_VAL_TYPE_LPAS_STEP:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_TYPE, RSVNA_VAL_TYPE_LOW_PASS),
                4, "Type");

        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS, RSVNA_VAL_STIMULUS_STEP),
                4, "Type - Stimulus");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_IMPULSE_RESPONSE, impulseResponse),
            5, "Impulse Response");

    if (impulseResponse == RSVNA_VAL_FILTER_DCH)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_SIDE_LOBE_LEVEL, sideLobeLevel),
                6, "Side Lobe Level");
    }

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_RESOLUTION_ENHANCEMENT, resolutionEnhancement),
            7, "Resolution Enhancement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables time domain site VSWR measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables time domain site VSWR measurements.
ViStatus _VI_FUNC rsvna_ConfigureTimeDomainVSWREnabled(ViSession instrSession,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_TIME_DOMAIN_VSWR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function calculates the harmonic grid for low pass time domain
/// HIFN  transforms according to one of the three alternative algorithms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR calculationMethod/This control selects the calculation method.
ViStatus _VI_FUNC rsvna_ConfigureHarmonicGridAndKeep(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   calculationMethod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_HARMONIC_GRID_AND_KEEP, calculationMethod),
            3, "Calculation Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function extrapolates the measured trace towards f = 0 and
/// HIFN  overwrites the current DC value
/// HIFN  (CALCulate<Chn>:TRANsform:TIME:LPASs:DCSParam). The function is
/// HIFN  relevant for low pass time domain transforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_ExtrapolateDCValue(ViSession instrSession,
                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_EXTRAPOLATE_DC_VALUE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function calculates the harmonic grid for low pass time domain
/// HIFN  transforms, keeping the stop frequency and the number of points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_CalculateHarmonicGrid(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_CALCULATE_HARMONIC_GRID, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the time gate settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR timeGate/This control determines whether the time gate is enabled.
/// HIPAR timeGateType/This control defines time gate type.
/// HIPAR rangeLines/This control enables or disables permanent display of the gate limits.
/// HIPAR filterType/This control selects the time gate to be applied to the time domain
/// HIPAR filterType/transform.
/// HIPAR sidebandSuppression/This control sets the sideband suppression for the Dolph-Chebyshev
/// HIPAR sidebandSuppression/time gate.
ViStatus _VI_FUNC rsvna_ConfigureTimeGate(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean timeGate,
                                          ViInt32   timeGateType,
                                          ViBoolean rangeLines,
                                          ViInt32   filterType,
                                          ViReal64  sidebandSuppression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TIME_GATE_STATE, timeGate),
            3, "Time Gate");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_GATE_TYPE, timeGateType),
            4, "Time Gate Type");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TIME_GATE_SHOW_RANGE_LINES, rangeLines),
            5, "Range Lines");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_GATE_FILTER_TYPE, filterType),
            6, "Filter Type");

    if (filterType == RSVNA_VAL_FILTER_DCH)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_GATE_SIDEBAND_SUPRESSION, sidebandSuppression),
                7, "Sideband Suppression");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the start and stop times of the time gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR startTime/This control defines the start time of the time gate.
/// HIPAR stopTime/This control defines the stop time of the time gate.
ViStatus _VI_FUNC rsvna_ConfigureTimeGateStartStop(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  startTime,
                                                   ViReal64  stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_GATE_START_TIME, startTime),
            3, "Start Time");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_GATE_STOP_TIME, stopTime),
            4, "Stop Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the center and span times of the time gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR centerTime/This control defines the center time of the time gate.
/// HIPAR span/This control defines the span of the time gate.
ViStatus _VI_FUNC rsvna_ConfigureTimeGateCenterSpan(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViReal64  centerTime,
                                                    ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_GATE_CENTER_TIME, centerTime),
            3, "Center Time");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_GATE_SPAN, span),
            4, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the operating mode where the time gate is moved in the opposite
/// HIFN direction when the "Delay" setting is changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR adjustTimeGate/Enables or disables "Adjust Time Gate".
ViStatus _VI_FUNC rsvna_ConfigureTimeGateOffsetActivation(ViSession instrSession,
                                                          ViBoolean adjustTimeGate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_TIME_GATE_OFFSET_STATE, adjustTimeGate),
            2, "Adjust Time Gate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Time Gate Linking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR timeGateLinking/Sets the Time Gate Linking.
ViStatus _VI_FUNC rsvna_ConfigureTimeGateLinking(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViString  timeGateLinking)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TIME_GATE_LINKING, timeGateLinking),
            3, "Time Gate Linking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If channel <Chn> is performing an unsegmented frequency sweep, this
/// HIFN  attribute allows you to transform its active trace into a Distance to
/// HIFN  Fault (DtF) trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dtfEnabled/If channel <Chn> is performing an unsegmented frequency sweep, this
/// HIPAR dtfEnabled/control allows you to transform its active trace into a Distance to
/// HIPAR dtfEnabled/Fault (DtF) trace.
/// HIPAR cableType/Selects one of the available (predefined or user-defined) cable types
/// HIPAR cableType/for Distance to Fault measurements.
ViStatus _VI_FUNC rsvna_ConfigureDistanceToFault(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean dtfEnabled,
                                                 ViString  cableType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_STATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_STATE, dtfEnabled),
            3, "DTF Enabled");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_CABLE_TYPE, cableType),
            4, "Cable Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the window of the Distance to Fault measurement using a start
/// HIFN  and stop distance. Use rsvna_ConfigureDistanceToFaultCenterSpan to
/// HIFN  specify center and span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR startDistance/Defines the start distance of the Distance to Fault measurement.
/// HIPAR stopDistance/Defines the stop distance of the Distance to Fault measurement.
ViStatus _VI_FUNC rsvna_ConfigureDistanceToFaultStartStop(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  startDistance,
                                                          ViReal64  stopDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_START_DISTANCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_START_DISTANCE, startDistance),
            3, "Start Distance");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_STOP_DISTANCE, stopDistance),
            4, "Stop Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the distance window of the Distance to Fault measurement using
/// HIFN  its center and span. Use rsvna_ConfigureDistanceToFaultStartStop to
/// HIFN  specify start and stop distances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR centerDistance/Defines the distance window of the Distance to Fault measurement using
/// HIPAR centerDistance/its center.
/// HIPAR span/Defines the span of the Distance to Fault measurement around the
/// HIPAR span/window center.
ViStatus _VI_FUNC rsvna_ConfigureDistanceToFaultCenterSpan(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViReal64  centerDistance,
                                                           ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_CENTER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_CENTER, centerDistance),
            3, "Center Distance");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_WINDOW_SPAN, span),
            4, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the minimum number of sweep points that are required to
/// HIFN  provide an unambiguous display of fault locations for the active
/// HIFN  Distance to Fault trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_DistanceToFaultAutoNumberOfPoints(ViSession instrSession,
                                                          ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_AUTO_NUMBER_OF_POINTS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_AUTO_NUMBER_OF_POINTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures  the current Distance to Fault trace to check for spikes
/// HIFN  above the given Fault Limit (red line). The fault limit is defined
/// HIFN  relative to the 0 dB-line in the test diagram, i.e. the peak response
/// HIFN  value for total reflection after proper calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR limitCheckEnabled/Enables/disables checking the current Distance to Fault trace for
/// HIPAR limitCheckEnabled/spikes above the given Fault Limit (red line).
/// HIPAR limit/If Limit Check Enabled is set to On, determines the level above which
/// HIPAR limit/peaks in the active 'Distance to Fault' trace are considered as faults.
ViStatus _VI_FUNC rsvna_ConfigureDistanceToFaultLimitCheck(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean limitCheckEnabled,
                                                           ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_LIMIT_CHECK_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_LIMIT_CHECK_ENABLED, limitCheckEnabled),
            3, "Limit Check Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_LIMIT, limit),
            4, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a new user-defined cable type for Distance to Fault (DtF)
/// HIFN  measurements. The cable's frequency-dependent attenuation is defined
/// HIFN  via attenuation/frequency pairs. At least one pair has to be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cableName/Name of the user-defined cable. Must be unique among all DtF cable
/// HIPAR cableName/types, with case insensitive match.
/// HIPAR permittivity/Relative permittivity eplison_r of the cable type.
/// HIPAR arrayLengths/Size of the attenuation and frequency arrays. At least one pair has to
/// HIPAR arrayLengths/be specified.
/// HIPAR attenuationValues/Array of attenuation values i in the frequency-dependent attenuation
/// HIPAR attenuationValues/table of the cable type. Default unit: dB/m
/// HIPAR frequencyValues/Array of frequencies i in the frequency-dependent attenuation table of
/// HIPAR frequencyValues/the cable type. Default unit: Hz
ViStatus _VI_FUNC rsvna_DefineDistanceToFaultCableType(ViSession instrSession,
                                                       ViString  cableName,
                                                       ViReal64  permittivity,
                                                       ViInt32   arrayLengths,
                                                       ViReal64  attenuationValues[],
                                                       ViReal64  frequencyValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";
    ViChar   cat_str[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cableName, 1, -1),
                2, "Cable Name");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arrayLengths, 1),
                4, "Array Lengths");

    snprintf(value, RS_MAX_MESSAGE_LEN, "'%s',%.12lG", cableName, permittivity);

    for (i = 0; i < arrayLengths; i++)
    {
        snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG,%.12lG", attenuationValues[i], frequencyValues[i]);
        strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
    }

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISTANCE_TO_FAULT_DEFINE_CABLE_TYPE, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the user-defined cable type with the given name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cableType/The name of the user-defined cable type to delete.
ViStatus _VI_FUNC rsvna_DeleteDistanceToFaultCableType(ViSession instrSession,
                                                       ViString  cableType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISTANCE_TO_FAULT_DELETE_CABLE_TYPE, cableType),
            2, "Cable Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of DtF limit violations of this trace, which
/// HIFN  corresponds to the rows in the Fault List Dock Widget Fault Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR peakCount/Returns the number of DtF limit violations of this trace.
ViStatus _VI_FUNC rsvna_QueryDistanceToFaultPeakListCount(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32*  peakCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISTANCE_TO_FAULT_PEAK_LIST_COUNT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DISTANCE_TO_FAULT_PEAK_LIST_COUNT, peakCount),
            3, "Peak Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If DtF limit checking is enabled for the active trace channel <Chn>,
/// HIFN  this function returns the peak data of DtF limit violations of this
/// HIFN  trace from the Fault List Dock Widget Fault Table. Peaks are numbered
/// HIFN  according to their distance (row no. 1 has the smallest distance).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR arrayLengths/The allocated size of the Peak Distance and Response Value arrays. No
/// HIPAR arrayLengths/more than this number of results will be returned. If the value of Peak
/// HIPAR arrayLengths/Count is larger than the allocated array sizes, not all results from
/// HIPAR arrayLengths/the table have been returned.
/// HIPAR peakDistances/The distances of the fault peaks, in meters.
/// HIPAR responseValues/The response values of the fault peaks, in dB.
/// HIPAR peakCount/The number of entries in the DTF list. If this value is greater than
/// HIPAR peakCount/the allocated lengths of the result arrays, not all results from the
/// HIPAR peakCount/table have been returned.
ViStatus _VI_FUNC rsvna_DistanceToFaultPeakListResults(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   arrayLengths,
                                                       ViReal64  peakDistances[],
                                                       ViReal64  responseValues[],
                                                       ViInt32*  peakCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViInt32  loopCount = 0;
    ViReal64 result[2] = {0.0, 0.0};
    ViInt32  resultCount;

    checkErr(RsCore_LockSession(instrSession));

    // Channel range checking performed in rsvna_QueryDistanceToFaultPeakListCount
    checkErr(rsvna_QueryDistanceToFaultPeakListCount(instrSession, channel, peakCount));

    loopCount = (*peakCount < arrayLengths) ? *peakCount : arrayLengths;

    for (ViInt32 i = 0; i < loopCount; i++)
    {
        snprintf(cmd, RS_MAX_MESSAGE_LEN, "CALC%ld:TRAN:DTF:PEAK:DATA%ld?", channel, i + 1);
        checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 2, result, &resultCount));
        if (resultCount != 2)
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        peakDistances[i] = result[0];
        responseValues[i] = result[1];
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays or hides all statistical results and moves the
/// HIFN  statistics info field for the active trace <Chn> to one of nine
/// HIFN  predefined positions in the active diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR infoField/Displays or hides all statistical results in the diagram area of trace
/// HIPAR infoField/no. <Ch/Tr>.
/// HIPAR horizontalPosition/Selects the horizontal position of the info field.
/// HIPAR verticalPosition/Selects the vertical position of the info field.
ViStatus _VI_FUNC rsvna_ConfigureStatisticsInfoField(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean infoField,
                                                     ViInt32   horizontalPosition,
                                                     ViInt32   verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizontalPosition, RSVNA_VAL_HORIZONTAL_LEFT, RSVNA_VAL_HORIZONTAL_RIGHT),
            4, "Horizontal Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalPosition, RSVNA_VAL_VERTICAL_TOP, RSVNA_VAL_VERTICAL_BOTTOM),
            5, "Vertical Position");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_STATISTICS_DISPLAY_ALL_RESULTS, infoField),
            3, "Info Field");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:STAT:STAT:AREA %s,%s", channel,
             horizontalPositionArr[horizontalPosition],
             verticalPositionArr[verticalPosition]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function define the range for the statistical and phase
/// HIFN  evaluation. The evaluation range is a continuous interval of the sweep
/// HIFN  variable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR evaluationRange/Selects one out of 10 evaluation ranges.
/// HIPAR start/Defines the start value of the selected evaluation range.
/// HIPAR stop/Defines the stop value of the selected evaluation range.
ViStatus _VI_FUNC rsvna_ConfigureTraceEvaluationRange(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   evaluationRange,
                                                      ViReal64  start,
                                                      ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STATISTICS_EVALUATION_RANGE, evaluationRange),
            3, "Evaluation Range");

    if (evaluationRange != RSVNA_VAL_FULL_SPAN)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICS_EVALUATION_RANGE_START, start),
                4, "Start");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICS_EVALUATION_RANGE_STOP, stop),
                5, "Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Trace Statistical Reference values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR referenceValue/Defines how the reference value ('small signal value') for the
/// HIPAR referenceValue/compression point calculation is calculated.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalReference(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_VALUE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Manually defines the reference level for compression point calculation
/// HIFN  on dB formatted traces. This only applies if
/// HIFN  rsvna_ConfigureTraceStatisticalReference is set to Manual
/// HIFN  (RSVNA_VAL_STAT_COMP_REF_VALUE_MANUAL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR referenceLevel/Teference level for compression point calculation on dB formatted
/// HIPAR referenceLevel/traces.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalManualReferenceLevel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_LEVEL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Manually defines the reference phase for compression point calculation
/// HIFN  on phase formatted traces. Only applies if
/// HIFN  rsvna_ConfigureTraceStatisticalReference is set to Manual
/// HIFN  (RSVNA_VAL_STAT_COMP_REF_VALUE_MANUAL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR referencePhase/Reference phase for compression point calculation on phase formatted
/// HIPAR referencePhase/traces.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalManualReferencePhase(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  referencePhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_PHASE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_PHASE, referencePhase),
            3, "Reference Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows you to select the marker whose value shall be used as the
/// HIFN  reference ('small signal value') for the compression point calculation
/// HIFN  if rsvna_ConfigureTraceStatisticalReference is set to Marker
/// HIFN  (RSVNA_VAL_STAT_COMP_REF_VALUE_MARKER).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR selectedMarker/The marker whose value shall be used as the reference.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalReferenceMarker(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   selectedMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_MARKER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_MARKER, selectedMarker),
            3, "Selected Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Trace Statistical Reference Range if
/// HIFN  rsvna_ConfigureTraceStatisticalReference is set to Range
/// HIFN  (RSVNA_VAL_STAT_COMP_REF_VALUE_RANGE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR show/Displays or hides range limit lines for the reference range.
/// HIPAR referenceRange/Selects one of the 10 channel-specific user defined ranges as the
/// HIPAR referenceRange/reference range for compression point calculation.
/// HIPAR start/Defines the start value of the range.
/// HIPAR stop/Defines the start value of the range.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalReferenceRange(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean show,
                            ViInt32   referenceRange,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_RANGE_SHOW,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_RANGE_SHOW, show),
            3, "Show");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_RANGE, referenceRange),
            4, "Reference Range");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_RANGE_START, start),
            5, "Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_REFERENCE_RANGE_STOP, stop),
            6, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the compression value x for the compression point measurement
/// HIFN  for phase formatted traces. Use rsvna_QueryTraceCompressionPoint to
/// HIFN  retrieve the compression results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phase/Defines the compression value x for the compression point measurement
/// HIPAR phase/for phase formatted traces.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_ConfigureTraceStatisticalCompressionPhase(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_COMPRESSION_PHASE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_COMPRESSION_PHASE, phase),
            3, "Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a single statistical parameters of the trace no.
/// HIFN  <Chn>. It is not necessary to display the info field before using this
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR statisticalParameter/Define statistical parameter.
/// HIPAR responseValue/The data is returned as a list of real numbers. The unit is the
/// HIPAR responseValue/default unit of the measured parameter.
ViStatus _VI_FUNC rsvna_QueryTraceStatisticalParameter(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   statisticalParameter,
                                                       ViReal64* responseValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_stat[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticalParameter, RSVNA_VAL_MEAN, RSVNA_VAL_FLATNESS),
            3, "Statistical Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STATISTICAL_RESULTS,
            "StatParam", statisticalParameter , RS_REPCAP_BUF_SIZE, repCap_stat),
            3, "Statistical Parameter");

    strcat(repCap, ",");
    strcat(repCap, repCap_stat);

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STATISTICAL_RESULTS, responseValue),
            4, "Response Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns all statistical parameters of the trace no.
/// HIFN  <Chn>. It is not necessary to display the info field before using this
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Response
/// HIPAR bufferSize/Values'.
/// HIPAR responseValues/The data is returned as a list of real numbers. The unit is the
/// HIPAR responseValues/default unit of the measured parameter.
ViStatus _VI_FUNC rsvna_QueryTraceStatisticalAllParameters(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   bufferSize,
                                                           ViReal64  responseValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:STAT:RES? ALL", channel);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, responseValues, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the x-dB compression point of an S-parameter or
/// HIFN  ratio measured in a power sweep. The compression value x is set via
/// HIFN  rsvna_ConfigureTraceStatisticalCompressionPhase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR compressionPointIn/This control returns compression point in. It is the stimulus level at
/// HIPAR compressionPointIn/the compression point in units of dBm
/// HIPAR compressionPointOut/This control returns compression point out. It is the magnitude of the
/// HIPAR compressionPointOut/wave reflected or transmitted by the DUT. If an S-parameter Sij is
/// HIPAR compressionPointOut/measured, then compression point out is the magnitude of the wave bi.
ViStatus _VI_FUNC rsvna_QueryTraceCompressionPoint(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64* compressionPointIn,
                                                   ViReal64* compressionPointOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:STAT:NLIN:COMP:RES?", channel);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, compressionPointIn, compressionPointOut, NULL, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the smoothing function for the active trace,
/// HIFN  which may be a data or a memory trace. With active smoothing function,
/// HIFN  each measurement point is replaced by the arithmetic mean value of all
/// HIFN  measurement points located in a symmetric interval centered on the
/// HIFN  stimulus value. The width of the smoothing interval is referred to as
/// HIFN  the Smoothing Apertureand can be adjusted according to the properties
/// HIFN  of the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR smoothing/Enables or disables smoothing for trace no. <Ch/Tr>.
/// HIPAR aperture/Defines how many measurement points are averaged to smooth the trace.
ViStatus _VI_FUNC rsvna_ConfigureTraceSmoothing(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean smoothing,
                                                ViReal64  aperture)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SMOOTHING_STATE, smoothing),
            3, "Smoothing");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SMOOTHING_APERTURE, aperture),
            4, "Aperture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shifts the active trace in horizontal direction, leaving
/// HIFN  the positions of all markers unchanged. The positive or negative offset
/// HIFN  value for the stimulus variable is entered into an input field. The
/// HIFN  unit depends on the sweep type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR shiftStimulusValue/Stimulus offset value.
ViStatus _VI_FUNC rsvna_TraceShiftStimulusValue(ViSession instrSession,
                                                ViInt32   window,
                                                ViReal64  shiftStimulusValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACE_SHIFT_STIMULUS_VALUE,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TRACE_SHIFT_STIMULUS_VALUE, shiftStimulusValue),
            3, "Shift Stimulus Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function modifies all points of the trace <WndTr> by means of an
/// HIFN  added and/or a multiplied complex constant. The response values M of
/// HIFN  the trace are transformed according to:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR magnitude/The magnitude factor shifts a dB Mag trace in vertical direction,
/// HIPAR magnitude/leaving the phase of a complex parameter unchanged.
/// HIPAR phase/The phase factor rotates a trace that is displayed in a polar diagram
/// HIPAR phase/around the origin, leaving the magnitude unchanged.
/// HIPAR real/The real added constant shifts a real trace in vertical direction,
/// HIPAR real/leaving the imaginary part unchanged.
/// HIPAR imaginary/The imaginary added constant shifts a imaginary trace in vertical
/// HIPAR imaginary/direction, leaving the real part unchanged.
ViStatus _VI_FUNC rsvna_TraceShiftResponseValue(ViSession instrSession,
                                                ViInt32   window,
                                                ViReal64  magnitude,
                                                ViReal64  phase,
                                                ViReal64  real,
                                                ViReal64  imaginary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, magnitude, -300, 300), 3, "Magnitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, -3.4e38, 3.4e38), 4, "Phase");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, real, -3.4e38, 3.4e38), 4, "Real");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, imaginary, -3.4e38, 3.4e38), 4, "Imaginary");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC:Y:OFFS %.12lG,%.12lG,%.12lG,%.12lG", window, magnitude, phase, real, imaginary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the current response values of the active data
/// HIFN  trace or memory trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the current response values of the active data trace
/// HIPAR traceData/or memory trace.
ViStatus _VI_FUNC rsvna_TraceResponseData(ViSession instrSession,
                                          ViInt32   channel_Trace,
                                          ViInt32   arraySize,
                                          ViInt32   dataFormat,
                                          ViInt32*  noofValues,
                                          ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_UNCAL),
            4, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA? %s", channel_Trace, dataFormatArr[dataFormat]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the current response values of the active data
/// HIFN  trace or memory trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR errorTerm/Denotes the error terms generated during a calibration.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the current response values of the active data trace
/// HIPAR traceData/or memory trace.
ViStatus _VI_FUNC rsvna_TraceResponseDataError(ViSession instrSession,
                                               ViInt32   channel_Trace,
                                               ViInt32   errorTerm,
                                               ViInt32   arraySize,
                                               ViInt32*  noofValues,
                                               ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, errorTerm, 1, 27),
            3, "Error Term");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                4, "Array size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA? SCORr%ld", channel_Trace, errorTerm);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the current response values of all traces of the
/// HIFN  current test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns the current response values of all traces of the current test
/// HIPAR traceData/setup.
ViStatus _VI_FUNC rsvna_TraceResponseDataAll(ViSession instrSession,
                                             ViInt32   arraySize,
                                             ViInt32   dataFormat,
                                             ViInt32*  noofValues,
                                             ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array size");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_UNFORMATTED_MATH),
            3, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DATA:ALL? %s", dataFormatArr[dataFormat]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads the current response values of all data traces of the current
/// HIFN  test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns the current response values of all traces of the current test
/// HIPAR traceData/setup.
ViStatus _VI_FUNC rsvna_TraceResponseDataAllData(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViInt32   dataFormat,
                                                 ViInt32*  noofValues,
                                                 ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array size");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_UNFORMATTED_MATH),
            3, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DATA:DALL? %s", dataFormatArr[dataFormat]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Reads the current response values of all S-parameter data traces in channel no. <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noOfValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns the current response values of all S-parameter data traces in channel no. <Ch>.
ViStatus _VI_FUNC rsvna_TraceResponseDataAllSData(ViSession instrSession,
                                                  ViInt32   channel_Trace,
                                                  ViInt32   arraySize,
                                                  ViInt32   dataFormat,
                                                  ViInt32*  noOfValues,
                                                  ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array size");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_UNFORMATTED_MATH),
            3, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA:CALL? %s", channel_Trace, dataFormatArr[dataFormat]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns all traces which are available for rsvna_TraceResponseDataAllSData in channel no.<Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 'Catalog' parameter.
/// HIPAR catalog/Response is a string parameter with all S-parameter traces in
/// HIPAR catalog/the current channel or in the active system error correction.
ViStatus _VI_FUNC rsvna_TraceResponseSDataCatalog(ViSession instrSession,
                                                  ViInt32   channel_Trace,
                                                  ViInt32   bufferSize,
                                                  ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TRACE_RESPONSE_S_DATA_CATALOG,
            bufferSize, catalog),
            4, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the response values of a trace acquired in single
/// HIFN  sweep mode (INITiate<Ch>:CONTinuous OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR sweepNumber/Number of sweep to be read.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the response values of a trace acquired in single
/// HIPAR traceData/sweep mode.
ViStatus _VI_FUNC rsvna_TraceResponseSingleSweepData(ViSession instrSession,
                                                     ViInt32   channel_Trace,
                                                     ViInt32   sweepNumber,
                                                     ViInt32   arraySize,
                                                     ViInt32*  noofValues,
                                                     ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA:NSW? SDATa,%ld", channel_Trace, sweepNumber);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the number of completed sweeps in single sweep
/// HIFN  mode (INITiate<Ch>:CONTinuous OFF). The trace can be any of the traces
/// HIFN  acquired during the single sweep cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR sweepCount/Returns the number of completed sweeps in single sweep mode.
ViStatus _VI_FUNC rsvna_TraceResponseSingleSweepDataCount(ViSession instrSession,
                                                          ViInt32   channel_Trace,
                                                          ViInt32*  sweepCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUERY_TRACE_COMPLETE_SWEEP_COUNT, sweepCount),
            3, "Sweep Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the response values of a trace acquired in single
/// HIFN  sweep mode (INITiate<Ch>:CONTinuous OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR sweepNumber/Number of sweep to be read. 1 denotes the first sweep acquired, 2
/// HIPAR sweepNumber/denotes the second and so forth.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the response values of a trace acquired in single
/// HIPAR traceData/sweep mode.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
ViStatus _VI_FUNC rsvna_TraceResponseSingleSweepDataForward(ViSession instrSession,
                                                            ViInt32   channel_Trace,
                                                            ViInt32   sweepNumber,
                                                            ViInt32   arraySize,
                                                            ViInt32*  noofValues,
                                                            ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA:NSW:FIRS? SDAT,%ld ", channel_Trace, sweepNumber);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the response values of a trace acquired in single
/// HIFN  sweep mode (INITiate<Ch>:CONTinuous OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR firstSweep/Number of first sweep to be read.
/// HIPAR lastSweep/Number of last sweep to be read.
/// HIPAR noOfValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the response values of a trace acquired in single
/// HIPAR traceData/sweep mode.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
ViStatus _VI_FUNC rsvna_TraceResponseSingleSweepDataFirstLast(
                            ViSession instrSession,
                            ViInt32   channel_Trace,
                            ViInt32   firstSweep,
                            ViInt32   lastSweep,
                            ViInt32   arraySize,
                            ViInt32*  noOfValues,
                            ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA:NSW:FIRS? SDAT,%ld,%ld ", channel_Trace, firstSweep, lastSweep);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the current response values of the active data
/// HIFN  trace or memory trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Trace name.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the current response values of the active data trace
/// HIPAR traceData/or memory trace.
ViStatus _VI_FUNC rsvna_ArbitraryTraceResponseData(ViSession instrSession,
                                                   ViString  traceName,
                                                   ViInt32   arraySize,
                                                   ViInt32   dataFormat,
                                                   ViInt32*  noofValues,
                                                   ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_UNFORMATTED_MATH),
            4, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DATA:TRAC? '%s',%s", traceName, dataFormatArr[dataFormat]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the stimulus values of the active data or memory
/// HIFN  trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR noofValues/Returns number of taken trace data values.
/// HIPAR traceData/Returns array of the current stimnulus values of the active data trace
/// HIPAR traceData/or memory trace.
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
ViStatus _VI_FUNC rsvna_TraceStimulusData(ViSession instrSession,
                                          ViInt32   channel_Trace,
                                          ViInt32   arraySize,
                                          ViInt32*  noofValues,
                                          ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1),
                2, "Channel Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DATA:STIM?", channel_Trace);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Uses linear inter-/extrapolation to regrid all memory traces of the
/// HIFN  related channel to the channel's current stimulus values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects repcap Channel.
ViStatus _VI_FUNC rsvna_TraceDataExtrapolation(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DATA_EXTRAPOLATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DATA_EXTRAPOLATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes a memory trace data to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR dataFormat/Selects trace data format.
/// HIPAR noofPoints/Sets the number of trace points to be written to the instrument.
/// HIPAR traceData/Trace data array.
ViStatus _VI_FUNC rsvna_WriteMemoryTraceData(ViSession instrSession,
                                             ViInt32   channel_Trace,
                                             ViInt32   dataFormat,
                                             ViInt32   noofPoints,
                                             ViReal64  traceData[])
{
    ViStatus error = VI_SUCCESS;
             dataFormat++;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_WriteMemoryTraceDataExt(instrSession, channel_Trace,
        RSVNA_UNFORMATTED, noofPoints, traceData));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates LO Suppression for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR loSupression/Activates/deactivates LO Suppression for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureTraceLOSupression(ViSession instrSession,
                                                   ViInt32   channel_Trace,
                                                   ViBoolean loSupression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ACTIVATES_LO_SUPRESSION,
            "Channel", channel_Trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ACTIVATES_LO_SUPRESSION, loSupression),
            3, "LO Supression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads trace data from a specified trace file and assigns
/// HIFN  it to a trace with a specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Name of an existing data trace in the active setup (string parameter).
/// HIPAR traceName/The trace data is loaded into a memory trace associated with the
/// HIPAR traceName/specified data trace. If one or more memory traces are already
/// HIPAR traceName/associated with the specified data trace, the last generated memory
/// HIPAR traceName/trace is overwritten.
/// HIPAR fileName/String parameter to specify the name and directory of the trace file
/// HIPAR fileName/to be loaded. Several file formats for trace files are supported. If no
/// HIPAR fileName/path is specified the analyzer searches the current directory, to be
/// HIPAR fileName/queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_TraceImportData(ViSession instrSession,
                                        ViString  traceName,
                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:TRAC '%s','%s'", traceName, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the specified trace file and automatically
/// HIFN  distributes the imported S-parameter traces Sij to all diagrams in the
/// HIFN  active channel that are currently displaying Sij.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/String parameter to specify the name and directory of the trace file
/// HIPAR traceName/to be loaded. Several file formats for trace files are supported. If no
/// HIPAR traceName/path is specified the analyzer searches the current directory, to be
/// HIPAR traceName/queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_TraceImportDataAutoDistribute(ViSession instrSession,
                                                      ViString  traceName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_IMPORT_TRACE_DATA_AUTO_DISTRIBUTE, traceName),
            2, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the trace data of a specified trace to a trace
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Name of an existing data trace in the active recall set.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/trace file. Several file formats for trace files are supported. If no
/// HIPAR fileName/path is specified the analyzer uses the current directory.
/// HIPAR exportFormat/This control selects the export format.
/// HIPAR exportData/This control selects the export data.
/// HIPAR decimalSeparator/This control selects the decimal separator.
/// HIPAR fieldSeparator/This control selects the field separator.
ViStatus _VI_FUNC rsvna_TraceExportData(ViSession instrSession,
                                        ViString  traceName,
                                        ViString  fileName,
                                        ViInt32   exportFormat,
                                        ViInt32   exportData,
                                        ViInt32   decimalSeparator,
                                        ViInt32   fieldSeparator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_FORMATTED),
            4, "Export Format");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportData, RSVNA_VAL_COMPLEX, RSVNA_VAL_LOGP),
            5, "Export Data");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, decimalSeparator, RSVNA_VAL_POIN, RSVNA_VAL_COMM),
            6, "Decimal Separator");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fieldSeparator, RSVNA_VAL_SEM, RSVNA_VAL_SPAC),
            7, "Field Separator");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TRAC '%s','%s',%s,%s,%s,%s",
            traceName, fileName, exportFormatArr[exportFormat], exportDataArr[exportData], decimalSeparatorArr[decimalSeparator], fieldSeparatorArr[fieldSeparator]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the trace data to a Touchstone file for the
/// HIFN  specified ports. The Touchstone file (.s<n>p where <n> is the number of
/// HIFN  ports) contains a full set of <n>2 single-ended S-parameters for the
/// HIFN  selected ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelNumber/Channel number used to identify the active recall set.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/trace file. Several file formats for trace files are supported. If no
/// HIPAR fileName/path is specified the analyzer uses the current directory.
/// HIPAR exportData/This control selects the export data.
/// HIPAR impedanceMode/This control selects the impedance mode.
/// HIPAR port1/First port number.
/// HIPAR port2/Second port number.
/// HIPAR port3/Third port number.
/// HIPAR port4/Fourth port number.
ViStatus _VI_FUNC rsvna_TraceExportDataPorts(ViSession instrSession,
                                             ViInt32   channelNumber,
                                             ViString  fileName,
                                             ViInt32   exportData,
                                             ViInt32   impedanceMode,
                                             ViInt32   port1,
                                             ViInt32   port2,
                                             ViInt32   port3,
                                             ViInt32   port4)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportData, RSVNA_VAL_COMPLEX, RSVNA_VAL_LOGP),
            4, "Export Data");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, impedanceMode, RSVNA_VAL_COMMON_IMPEDANCE, RSVNA_VAL_PORT_IMPEDANCE),
            5, "Impedance Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TRAC:PORT %ld,'%s',%s,%s,%ld,%ld,%ld,%ld", channelNumber, fileName,
             exportDataArr[exportData], impedanceModeArr[impedanceMode], port1, port2, port3, port4);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the trace data to a Touchstone file for the
/// HIFN  specified ports. The Touchstone file (.s<n>p where <n> is the number of
/// HIFN  ports) contains a full set of <n>2 single-ended S-parameters for the
/// HIFN  selected ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelNumber/Channel number used to identify the active recall set.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/trace file. Several file formats for trace files are supported. If no
/// HIPAR fileName/path is specified the analyzer uses the current directory.
/// HIPAR exportData/This control selects the export data.
/// HIPAR impedanceMode/This control selects the impedance mode.
/// HIPAR arraySize/Sets the number of elements of the Ports array.
/// HIPAR ports/Port numbers in an array
ViStatus _VI_FUNC rsvna_TraceExportDataPortsVariable(ViSession instrSession,
                                                     ViInt32   channelNumber,
                                                     ViString  fileName,
                                                     ViInt32   exportData,
                                                     ViInt32   impedanceMode,
                                                     ViInt32   arraySize,
                                                     ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportData, RSVNA_VAL_COMPLEX, RSVNA_VAL_LOGP),
            4, "Export Data");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, impedanceMode, RSVNA_VAL_COMMON_IMPEDANCE, RSVNA_VAL_PORT_IMPEDANCE),
            5, "Impedance Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TRAC:PORT %ld,'%s',%s,%s ", channelNumber, fileName,
             exportDataArr[exportData], impedanceModeArr[impedanceMode]);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, ports, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the trace data of all data traces in the
/// HIFN  specified channel to a trace file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR selectChannel/This control selects the channel in the active setup.
/// HIPAR channelNumber/Channel number used to identify the active recall set.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/trace file. Several file formats for trace files are supported. If no
/// HIPAR fileName/path is specified the analyzer uses the current directory.
/// HIPAR exportFormat/This control selects the export format.
/// HIPAR exportData/This control selects the export data.
/// HIPAR decimalSeparator/This control selects the decimal separator.
/// HIPAR fieldSeparator/This control selects the field separator.
ViStatus _VI_FUNC rsvna_ChannelTraceExportData(ViSession instrSession,
                                               ViInt32   selectChannel,
                                               ViInt32   channelNumber,
                                               ViString  fileName,
                                               ViInt32   exportFormat,
                                               ViInt32   exportData,
                                               ViInt32   decimalSeparator,
                                               ViInt32   fieldSeparator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   chan[10];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selectChannel, RSVNA_VAL_CHANNEL_TRACE_SINGLE, RSVNA_VAL_CHANNEL_TRACE_ALL),
            2, "Select Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportFormat, RSVNA_VAL_UNFORMATTED, RSVNA_VAL_FORMATTED),
            5, "Export Format");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, exportData, RSVNA_VAL_COMPLEX, RSVNA_VAL_LOGP),
            6, "Export Data");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, decimalSeparator, RSVNA_VAL_POIN, RSVNA_VAL_COMM),
            7, "Decimal Separator");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fieldSeparator, RSVNA_VAL_SEM, RSVNA_VAL_SPAC),
            8, "Field Separator");

    if (selectChannel == RSVNA_VAL_CHANNEL_TRACE_ALL)
        sprintf(chan, "ALL");
    else
        sprintf(chan, "%ld", channelNumber);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TRAC:CHAN %s,'%s',%s,%s,%s,%s",
            chan, fileName, exportFormatArr[exportFormat], exportDataArr[exportData], decimalSeparatorArr[decimalSeparator], fieldSeparatorArr[fieldSeparator]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays or hides the entire limit line (including all
/// HIFN  segments) associated to the active trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR showLimitLine/Displays or hides the entire limit line.
ViStatus _VI_FUNC rsvna_ShowLimitLine(ViSession instrSession,
                                      ViInt32   channel,
                                      ViBoolean showLimitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SHOW_LIMIT_LINE, showLimitLine),
            3, "Show Limit Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the limit check (or individual upper and lower
/// HIFN  limits) on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR limitLine/Select limit line.
/// HIPAR limitCheck/Switches the limit check on or off.
/// HIPAR horizontalPosition/Selects the horizontal position of the info field.
/// HIPAR verticalPosition/Selects the vertical position of the info field.
ViStatus _VI_FUNC rsvna_ConfigureLimitCheck(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   limitLine,
                                            ViBoolean limitCheck,
                                            ViInt32   horizontalPosition,
                                            ViInt32   verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitLine, RSVNA_VAL_LIMIT_UPPER, RSVNA_VAL_LIMIT_ALL),
            3, "Limit Line");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizontalPosition, RSVNA_VAL_HORIZONTAL_LEFT, RSVNA_VAL_HORIZONTAL_RIGHT),
            5, "Horizontal Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalPosition, RSVNA_VAL_VERTICAL_TOP, RSVNA_VAL_VERTICAL_BOTTOM),
            6, "Vertical Position");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM:STAT:AREA %s,%s", channel,
             horizontalPositionArr[horizontalPosition],
             verticalPositionArr[verticalPosition]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_CHECK,
            "LimitCheck", limitLine -1, RS_REPCAP_BUF_SIZE, repCap_limit),
            3, "Limit Line");

    strcat(repCap, ",");
    strcat(repCap, repCap_limit);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LIMIT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a 0 or 1, to indicate whether the LIMit test has
/// HIFN  failed or not. Sweep must be initiated before reading the result when
/// HIFN  in single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR limitCheckResult/Returns a 0 or 1, to indicate whether the LIMit test has failed or
/// HIPAR limitCheckResult/not.
ViStatus _VI_FUNC rsvna_QueryLimitCheckResult(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32*  limitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LIMIT_CHECK_RESULT, limitCheckResult),
            3, "Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a 0 or 1, to indicate whether or not a global,
/// HIFN  composite limit check on several traces has failed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR globalLimitCheck/Returns a 0 or 1, to indicate whether or not a global, composite limit
/// HIPAR globalLimitCheck/check on several traces has failed.
ViStatus _VI_FUNC rsvna_QueryGlobalLimitCheckResult(ViSession instrSession,
                                                    ViInt32*  globalLimitCheck)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_GLOBAL_LIMIT_CHECK_RESULT, globalLimitCheck),
            2, "Global Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clear the limit check results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_ClearLimitCheckResults(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_LIMIT_CHECK_CLEAR, NULL));

    checkErr(RsCore_ResetRepCapID(instrSession, "LimitCheck", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a new segment to the limit line list. In each
/// HIFN  segment the limit line is defined as a straight line connecting two
/// HIFN  points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR reserved/Reserved for future use.
/// HIPAR type/Type indicates whether the segment belongs to an Upper or a Lower
/// HIPAR type/limit line, or if the limit check at the segment is switched Off.
/// HIPAR type/Switching off the limit check does not delete the segment.
/// HIPAR startStimulus/Start Stimulus is the stimulus (x-axis) value of the first point of
/// HIPAR startStimulus/the segment (not necessarily smaller than Stop Stimulus).
/// HIPAR stopStimulus/Stop Stimulus is the stimulus (x-axis) value of the last point of the
/// HIPAR stopStimulus/segment (not necessarily larger than Start Stimulus).
/// HIPAR startResponse/Start Response is the response (y-axis) value of the first point of
/// HIPAR startResponse/the segment.
/// HIPAR stopResponse/Stop Response is the response (y-axis) value of the last point of the
/// HIPAR stopResponse/segment.
ViStatus _VI_FUNC rsvna_AddLimitLineSegment(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   reserved,
                                            ViInt32   type,
                                            ViReal64  startStimulus,
                                            ViReal64  stopStimulus,
                                            ViReal64  startResponse,
                                            ViReal64  stopResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  count = 0, rc_segment = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSVNA_VAL_LIMIT_OFF, RSVNA_VAL_LIMIT_LOWER),
            4, "Type");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "LimitSegment", &count));

    rc_segment = count;

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "LimitSeg%ld", rc_segment);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", rc_segment);

    checkErr(RsCore_AddRepCap(instrSession, "LimitSegment", repCap_segment, repCap_cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM:DATA %ld,%.12lG,%.12lG,%.12lG,%.12lG",
            channel, type, startStimulus, stopStimulus, startResponse, stopResponse);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function edits existing segment of the limit line segments list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR type/Selects the limit line type for a limit line segment.
/// HIPAR startStimulus/Changes the start stimulus value (i.e. the largest or smallest
/// HIPAR startStimulus/stimulus value) of a limit line segment.
/// HIPAR stopStimulus/Changes the stop stimulus value (i.e. the largest or smallest stimulus
/// HIPAR stopStimulus/value) of a limit line segment.
/// HIPAR startResponse/Changes the start response value (i.e. the response value assigned to
/// HIPAR startResponse/the start stimulus value) of a limit line segment.
/// HIPAR stopResponse/Changes the stop response value (i.e. the response value assigned to
/// HIPAR stopResponse/the stop stimulus value) of a limit line segment.
ViStatus _VI_FUNC rsvna_EditLimitLineSegment(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   segment,
                                             ViInt32   type,
                                             ViReal64  startStimulus,
                                             ViReal64  stopStimulus,
                                             ViReal64  startResponse,
                                             ViReal64  stopResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_start_stim[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_stop_stim[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_start_resp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_stop_resp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",LimitSeg%ld", channel, segment);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_SEGMENT_TYPE, type),
            4, "Type");

    strcpy(repCap_start_stim, repCap);
    strcat(repCap_start_stim, ",Stimulus,Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_start_stim, RSVNA_ATTR_LIMIT_LINE_SEGMENT_VALUES, startStimulus),
            5, "Start Stimulus");

    strcpy(repCap_stop_stim, repCap);
    strcat(repCap_stop_stim, ",Stimulus,Stop");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_stop_stim, RSVNA_ATTR_LIMIT_LINE_SEGMENT_VALUES, stopStimulus),
            6, "Stop Stimulus");

    strcpy(repCap_start_resp, repCap);
    strcat(repCap_start_resp, ",Amplitude,Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_start_resp, RSVNA_ATTR_LIMIT_LINE_SEGMENT_VALUES, startResponse),
            7, "Start Response");

    strcpy(repCap_stop_resp, repCap);
    strcat(repCap_stop_resp, ",Amplitude,Stop");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_stop_resp, RSVNA_ATTR_LIMIT_LINE_SEGMENT_VALUES, stopResponse),
            8, "Stop Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of limit line segments, including
/// HIFN  enabled and disabled segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segmentCount/Returns the number of limit line segments, including enabled and
/// HIPAR segmentCount/disabled segments.
ViStatus _VI_FUNC rsvna_QueryLimitLineSegmentCount(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32*  segmentCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_SEGMENT_COUNT, segmentCount),
            3, "Segment Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns entire limit line segment list. The limit line
/// HIFN  segment is calculated as a straight line connecting the two points
/// HIFN  (<Start Stimulus>, <Start Response>) and (<Stop Stimulus>, <Stop
/// HIFN  Response>).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR listSize/Pass the maximum number of segments (segments list size) to be taken
/// HIPAR listSize/out of the segments list.
/// HIPAR segmentsCount/Returns the number of received list segments.
/// HIPAR type/Type indicates whether the segment belongs to an Upper or a Lower
/// HIPAR type/limit line, or if the limit check at the segment is switched Off.
/// HIPAR startStimulus/Start Stimulus is the stimulus (x-axis) value of the first point of
/// HIPAR startStimulus/the segment (not necessarily smaller than Stop Stimulus).
/// HIPAR stopStimulus/Stop Stimulus is the stimulus (x-axis) value of the last point of the
/// HIPAR stopStimulus/segment (not necessarily larger than Start Stimulus).
/// HIPAR startResponse/Start Response is the response (y-axis) value of the first point of
/// HIPAR startResponse/the segment.
/// HIPAR stopResponse/Stop Response is the response (y-axis) value of the last point of the
/// HIPAR stopResponse/segment.
ViStatus _VI_FUNC rsvna_ReadLimitLineSegmentList(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   listSize,
                                                 ViInt32*  segmentsCount,
                                                 ViInt32   type[],
                                                 ViReal64  startStimulus[],
                                                 ViReal64  stopStimulus[],
                                                 ViReal64  startResponse[],
                                                 ViReal64  stopResponse[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  prd_buf = NULL;
    ViChar*  p2buf;
    ViInt32  i, count, j;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM:DATA?", channel);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &prd_buf));

    *segmentsCount = 0;

    p2buf = prd_buf;
    do
    {
        p2buf = strchr(p2buf, ',');
        (*segmentsCount)++;
        if (p2buf)
            p2buf++;
    }
    while (p2buf);

    *segmentsCount /= 5;

    count = (listSize < *segmentsCount) ? listSize : *segmentsCount;

    p2buf = prd_buf;
    for (i = 0; i < count; i++)
    {
        (void)sscanf(p2buf, "%ld,%le,%le,%le,%le", &type[i],
                     &startStimulus[i],
                     &stopStimulus[i],
                     &startResponse[i],
                     &stopResponse[i]);
        for (j = 0; j < 5; j++)
        {
            p2buf = strchr(p2buf, ',');
            if (p2buf)
                p2buf++;
        }
    }

Error:
    if (prd_buf)
        free(prd_buf);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the stimulus values of the limit line and/or
/// HIFN  creates new limit line segments. It also defines the response (y-axis)
/// HIFN  values of the lower (upper) limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR listSize/Pass the number of segments (segments list size).
/// HIPAR type/Selects the limit line type.
/// HIPAR startStimulus/Start Stimulus is the stimulus (x-axis) value of the first point of
/// HIPAR startStimulus/the segment(s) (not necessarily smaller than Stop Stimulus).
/// HIPAR stopStimulus/Stop Stimulus is the stimulus (x-axis) value of the last point of the
/// HIPAR stopStimulus/segment(s) (not necessarily larger than Start Stimulus).
/// HIPAR startResponse/Start Response is the response (y-axis) value of the first point of
/// HIPAR startResponse/the segment(s).
/// HIPAR stopResponse/Stop Response is the response (y-axis) value of the last point of the
/// HIPAR stopResponse/segment(s).
ViStatus _VI_FUNC rsvna_WriteLimitLineSegmentList(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   listSize,
                                                  ViInt32   type,
                                                  ViReal64  startStimulus[],
                                                  ViReal64  stopStimulus[],
                                                  ViReal64  startResponse[],
                                                  ViReal64  stopResponse[])
{
    ViStatus  error = VI_SUCCESS;
    ViChar*   p2buf;
    ViInt32   i = 0,
    index     = 0;
    ViReal64* tempArr = NULL;
    ViChar*   cmd = NULL;
    size_t    size = 0;

    checkErr(RsCore_LockSession(instrSession));

    /*

    Upper limit line segments are assigned odd numbers (1, 3, 5,...).
    Lower limit line segments are assigned even numbers (2, 4, 6,...).

    */
    size = (size_t)(RS_MAX_MESSAGE_BUF_SIZE) + 8 * listSize * 4;  // NOLINT(bugprone-implicit-widening-of-multiplication-result)
    // --- Response data ---
    cmd = (ViChar*)malloc(size);
    snprintf(cmd, size, "CALC%ld:LIM:%sDATA ", channel, limitCheckArr[type]);
    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, cmd, listSize, startResponse, stopResponse, NULL, NULL, NULL, NULL));
    checkErr(rsvna_CheckStatus(instrSession));

    // --- Stimulus data ---
    tempArr = malloc(4 * listSize * sizeof(ViReal64));  // NOLINT(bugprone-implicit-widening-of-multiplication-result)

    checkErr(rsvna_getViReal64ArrayWindow(instrSession,
        "CALC%ld:LIM:CONT?",
        channel,
        tempArr,
        listSize * 4,
        VI_ERROR_PARAMETER4));

    p2buf = cmd + snprintf(cmd, size, "CALC%ld:LIM:CONT ", channel);
    index = 0;
    for (i = 0; i < listSize; i++)
    {
        if (type == RSVNA_VAL_LIMIT_LOWER)
        {
            p2buf += snprintf(p2buf, size, "%.12lG,%.12lG,%.12lG,%.12lG,", tempArr[index + 0], tempArr[index + 1], startStimulus[i], stopStimulus[i]);
        }
        else
        {
            p2buf += snprintf(p2buf, size, "%.12lG,%.12lG,%.12lG,%.12lG,", startStimulus[i], stopStimulus[i], tempArr[index + 2], tempArr[index + 3]);
        }

        index += 4;
    }

    *--p2buf = '\0';
    *p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    if (tempArr)
        free(tempArr);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all limit line segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_DeleteLimitLineSegments(ViSession instrSession,
                                                ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_DELETE_ALL_SEGMENTS, NULL));

    checkErr(RsCore_ResetRepCapID(instrSession, "LimitSegment", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads a limit line definition from a specified file and
/// HIFN  assigns it to a trace with a specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Name of an existing trace in the active setup (string parameter). The
/// HIPAR traceName/imported limit line is assigned to this trace, irrespective of the
/// HIPAR traceName/trace information in the limit line file.
/// HIPAR fileName/String parameter to specify the name and directory of the limit line
/// HIPAR fileName/file to be loaded. The default extension (manual control) for limit
/// HIPAR fileName/line files is *.limit, although other extensions are allowed. If no
/// HIPAR fileName/path is specified the analyzer searches the current directory, to be
/// HIPAR fileName/queried with MMEMory:CDIRectory?.
/// HIPAR sParameter/This control selects an S-parameter from a Touchstone file.
/// HIPAR xOffset/This control sets the stimulus offset for limit lines loaded from a
/// HIPAR xOffset/Touchstone file. A 1 GHz offset shifts the limit line by 1 GHz in
/// HIPAR xOffset/(positive) horizontal direction.
/// HIPAR yOffset/This control sets the response offset for limit lines loaded from a
/// HIPAR yOffset/Touchstone file. A 1 dB offset shifts the limit line by 1 dB in
/// HIPAR yOffset/(positive) vertical direction.
/// HIPAR type/Selects the limit line type for a limit line segment.
ViStatus _VI_FUNC rsvna_RecallLimitLine(ViSession instrSession,
                                        ViString  traceName,
                                        ViString  fileName,
                                        ViString  sParameter,
                                        ViReal64  xOffset,
                                        ViReal64  yOffset,
                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSVNA_VAL_LIMIT_OFF, RSVNA_VAL_LIMIT_LOWER),
            7, "Type");

    if (strcmp(sParameter, """") == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:LIM '%s','%s'", traceName, fileName);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:LIM '%s','%s','%s',%.12lG,%.12lG,%s",
                traceName, fileName, sParameter, xOffset, yOffset, limitTypeArr[type]);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the limit lines associated to a specified trace to
/// HIFN  a limit line file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Name of an existing trace in the active setup (string parameter) for
/// HIPAR traceName/which a limit line definition exists.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/limit line file. The default extension (manual control) for limit line
/// HIPAR fileName/files is *.limit, although other extensions are allowed. If no path is
/// HIPAR fileName/specified the analyzer uses the current directory, to be queried with
/// HIPAR fileName/MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_SaveLimitLine(ViSession instrSession,
                                      ViString  traceName,
                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:LIM '%s','%s'", traceName, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function generates an lower (upper) limit line using the stimulus
/// HIFN  values of a data or memory trace and specified offset values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace. This trace provides
/// HIPAR channel/the stimulus data for the limit line unless another trace <trace_name>
/// HIPAR channel/is specified.
/// HIPAR limitLineType/Select limit line type.
/// HIPAR stimulusOffset/Stimulus offset value, used to shift all imported limit line segments
/// HIPAR stimulusOffset/in horizontal direction.
/// HIPAR responseOffset/Response offset value, used to shift all imported limit line segments
/// HIPAR responseOffset/in vertical direction.
/// HIPAR traceName/Name of the selected trace as used e.g. in
/// HIPAR traceName/CALCulate<Ch>:PARameter:SDEFine. If no trace name is specified the
/// HIPAR traceName/analyzer uses the active trace no. <Ch/Tr>.
ViStatus _VI_FUNC rsvna_ImportTraceAsLimitLine(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   limitLineType,
                                               ViReal64  stimulusOffset,
                                               ViReal64  responseOffset,
                                               ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitLineType, RSVNA_VAL_LIMIT_UPPER, RSVNA_VAL_LIMIT_LOWER),
            3, "limitLineType");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, stimulusOffset, -1000.0e9, 1000.0e9), 4, "Stimulus Offset");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, responseOffset, -10.0e12, 10.0e12), 5, "Response Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM:%sFEED %.12lG,%.12lG,'%s'", channel,
             limitCheckArr[limitLineType],
             stimulusOffset,
             responseOffset,
             traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Limit Line Formula.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR enabled/Defines whether a custom formula is used for limit line segment <Seg>
/// HIPAR enabled/of the related trace.
/// HIPAR formula/Defines a custom formula for limit line segment <Seg> of the related
/// HIPAR formula/trace.
ViStatus _VI_FUNC rsvna_ConfigureLimitLineSegmentFormula(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   segment,
                                                         ViBoolean enabled,
                                                         ViString  formula)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, segment, 1), 3, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_LINE_SEGMENT_FORMULA_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_LINE_SEGMENT_FORMULA_ENABLED,
            "Segment", segment - 1, RS_REPCAP_BUF_SIZE, repCap_segment),
            3, "Segment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_channel, repCap_segment);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_SEGMENT_FORMULA_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_SEGMENT_FORMULA, formula),
            5, "Formula");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Interpolation determines whether the limit line segment is
/// HIFN  interpolated linearly or logarithmically. The latter may be more
/// HIFN  suitable for logarithmic sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR interpolation/Interpolation determines whether the limit line segment is
/// HIPAR interpolation/interpolated linearly or logarithmically. The latter may be more
/// HIPAR interpolation/suitable for logarithmic sweeps.
ViStatus _VI_FUNC rsvna_ConfigureLimitLineSegmentInterpolation(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   segment,
                            ViInt32   interpolation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, segment, 1), 3, "Segment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld", channel, segment);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_SEGMENT_INTERPOLATION, interpolation),
            4, "Interpolation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns those sweep points that have caused a limit violation of the
/// HIFN  given <LimitFailType> for trace <TraceName>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Trace name, uniquely identifying the related trace.
/// HIPAR limitFailType/Limit (fail) type.
/// HIPAR arraySize/Size of results array.
/// HIPAR results/Returns those sweep points that have caused a limit violation of the
/// HIPAR results/given <LimitFailType> for trace <TraceName>.
/// HIPAR returnedValues/Number of returned values.
ViStatus _VI_FUNC rsvna_QueryLimitLineFailData(ViSession instrSession,
                                               ViString  traceName,
                                               ViInt32   limitFailType,
                                               ViInt32   arraySize,
                                               ViReal64  results[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   full_cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  limitFailString;
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 4, "Array size");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngLimitFailType, limitFailType, &limitFailString),
                3, "Limit Fail Type");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSVNA_ATTR_HIDDEN_QUERY_LIMIT_LINE_FAIL_DATA, repCap, &cmd));

    snprintf(full_cmd, RS_MAX_MESSAGE_BUF_SIZE, "%s '%s',%s", cmd, traceName, limitFailString);

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, full_cmd, arraySize, results, returnedValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the TTL pass/fail signals on or off. The
/// HIFN  signals are applied to the USER CONTROL connector as long as the active
/// HIFN  trace <Chn> is within limits, including the ripple limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR outputNo/Select TTL output.
/// HIPAR ttlOutput/Sets TTL output signal on or off.
ViStatus _VI_FUNC rsvna_ConfigureLimitLineTTLPass(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   outputNo,
                                                  ViBoolean ttlOutput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_output[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outputNo, 1, 2),
            3, "Output No");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_LINE_TTL_PASS,
            "OutputNumber", outputNo -1, RS_REPCAP_BUF_SIZE, repCap_output),
            3, "Output No");

    strcat(repCap, ",");
    strcat(repCap, repCap_output);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_TTL_PASS, ttlOutput),
            4, "TTL Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the default values of RSVNA_ATTR_LIMIT_LINE_TTL_PASS for new
/// HIFN  traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputNo/Selects TTL output.
/// HIPAR ttlPassDefault/Defines the default values of RSVNA_ATTR_LIMIT_LINE_TTL_PASS for new
/// HIPAR ttlPassDefault/traces.
ViStatus _VI_FUNC rsvna_ConfigureLimitLineTTLPassDefault(ViSession instrSession,
                                                         ViInt32   outputNo,
                                                         ViBoolean ttlPassDefault)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  outputNoCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "OutputNumber", &outputNoCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, outputNo, RSVNA_REPCAP_OUTPUTNUMBER_TTL1, outputNoCount),
            2, "Output No");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_LINE_TTL_PASS,
            "OutputNumber", outputNo - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Output No");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LIMIT_LINE_TTL_PASS_DEFAULT, ttlPassDefault),
            3, "TTL Pass Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows or hides the display line associated to the active
/// HIFN  trace in a Cartesian diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR displayLine/Switches the horizontal line on or off.
/// HIPAR position/Defines the position (response value) of the horizontal line.
ViStatus _VI_FUNC rsvna_ConfigureHorizontalLine(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean displayLine,
                                                ViReal64  position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_HORIZONTAL_LINE_STATE, displayLine),
            3, "Display Line");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_HORIZONTAL_LINE_POSITION, position),
            4, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads the effective lower/upper limit points from a limit line test
/// HIFN  for an arbitrary (not necessarily the active) trace, referenced by its
/// HIFN  name <TraceName>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Select limit line type.
/// HIPAR traceName/String parameter containing the trace name.
/// HIPAR data/Reads the effective lower/upper limit points from a limit line test for an
/// HIPAR data/arbitrary (not necessarily the active) trace, referenced by its name <TraceName>.
ViStatus _VI_FUNC rsvna_ConfigureLimitCheckPoints(ViSession instrSession,
                                                  ViInt32   type,
                                                  ViString  traceName,
                                                  ViChar    data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViAttr   attr[3] = {0,RSVNA_ATTR_HIDDEN_LIMIT_CHECK_UPPER_POINTS, RSVNA_ATTR_HIDDEN_LIMIT_CHECK_LOWER_POINTS};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSVNA_VAL_LIMIT_UPPER, RSVNA_VAL_LIMIT_LOWER),
            2, "Type");

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s'", traceName);

    viCheckParm(RsCore_GetAttributeViStringWithQparamUnknownLen(instrSession, NULL, attr[type], params, &data), 4, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays or hides all ripple limit lines (including all
/// HIFN  ranges) associated to the active trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR showRippleLimits/Switches the display line on or off.
ViStatus _VI_FUNC rsvna_ShowRippleLimits(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean showRippleLimits)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SHOW_RIPPLE_LIMITS, showRippleLimits),
            3, "Show Ripple Limits");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the global ripple limit check on or off. This
/// HIFN  check covers all traces in the active setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR rippleCheck/Switches the limit check on or off.
/// HIPAR horizontalPosition/Selects the horizontal position of the info field.
/// HIPAR verticalPosition/Selects the vertical position of the info field.
ViStatus _VI_FUNC rsvna_ConfigureRippleCheck(ViSession instrSession,
                                             ViInt32   channel,
                                             ViBoolean rippleCheck,
                                             ViInt32   horizontalPosition,
                                             ViInt32   verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizontalPosition, RSVNA_VAL_HORIZONTAL_LEFT, RSVNA_VAL_HORIZONTAL_RIGHT),
            4, "Horizontal Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalPosition, RSVNA_VAL_VERTICAL_TOP, RSVNA_VAL_VERTICAL_BOTTOM),
            5, "Vertical Position");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_CHECK, rippleCheck),
            3, "Ripple Check");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:RIPP:STAT:AREA %s,%s", channel,
             horizontalPositionArr[horizontalPosition],
             verticalPositionArr[verticalPosition]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a 0 or 1, to indicate whether or not the global
/// HIFN  ripple limit check has failed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR rippleLimitCheckResult/Returns a 0 or 1, to indicate whether or not the global ripple limit
/// HIPAR rippleLimitCheckResult/check has failed.
ViStatus _VI_FUNC rsvna_QueryRippleLimitGlobalCheckResult(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32*  rippleLimitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_GLOBAL_CHECK_RESULT, rippleLimitCheckResult),
            3, "Ripple Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clear the ripple limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_ClearRippleLimitGlobalCheckResult(ViSession instrSession,
                                                          ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_CHECK_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the limit check in the ripple limit
/// HIFN  range no. <Seg>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR limitCheck/Switches the limit check on or off.
ViStatus _VI_FUNC rsvna_ConfigureRippleLimitRangeSegmentCheck(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   segment,
                            ViBoolean limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",RippleSeg%ld", channel, segment);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_RANGE_SEGMENT_CHECK, limitCheck),
            4, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the result of the ripple limit check in the
/// HIFN  previously defined limit range no. <Seg>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR fail/Returns a 0 or 1, to indicate whether the LIMit test has failed or
/// HIPAR fail/not.
/// HIPAR limitCheckResult/Measured ripple in the limit range. A result is returned even if the
/// HIPAR limitCheckResult/limit check in the range no. <Seg> is disabled.
ViStatus _VI_FUNC rsvna_QueryRippleLimitCheckSegmentResult(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   segment,
                                                           ViInt32*  fail,
                                                           ViReal64* limitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64 value;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:RIPP:SEGM%ld:RES?", channel, segment);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, &value, limitCheckResult, NULL, NULL));
    *fail = (ViInt32)value;

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds and enables/disables an arbitrary number of ripple
/// HIFN  limit ranges, assigning the stimulus values and the ripple limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR noofValues/Set number of values for controls Type, Start Stimulus, Stop Stimulus,
/// HIPAR noofValues/Limit.
/// HIPAR type/Identifier for the type of the ripple limit range.
/// HIPAR startStimulus/Start Stimulus is the stimulus (x-axis) value of the first point of
/// HIPAR startStimulus/the segment (not necessarily smaller than Stop Stimulus).
/// HIPAR stopStimulus/Stop Stimulus is the stimulus (x-axis) value of the last point of the
/// HIPAR stopStimulus/segment (not necessarily larger than Start Stimulus).
/// HIPAR limit/Limit value, to be defined in accordance with the selected format
/// HIPAR limit/(CALCulate<Chn>:RIPPle:RDOMain:FORMat).
ViStatus _VI_FUNC rsvna_AddRippleLimitLineRangesSegment(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   noofValues,
                                                        ViInt32   type[],
                                                        ViReal64  startStimulus[],
                                                        ViReal64  stopStimulus[],
                                                        ViReal64  limit[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;
    ViInt32  i = 0;
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  count = 0, rc_segment = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "RippleSegment", &count));

    rc_segment = count + 1;

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "RippleSeg%ld", rc_segment);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", rc_segment);

    checkErr(RsCore_AddRepCap(instrSession, "RippleSegment", repCap_segment, repCap_cmd));

    p2buf += snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:RIPP:DATA", channel);

    for (i = 0; i < noofValues; i++)
    {
        p2buf += sprintf(p2buf, " %ld,%.12lG,%.12lG,%.12lG,",
                         type[i],
                         startStimulus[i],
                         stopStimulus[i],
                         limit[i]);
    }

    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function edits existing segment of the ripple limit line segments
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Segment number.
/// HIPAR startStimulus/Changes the start stimulus value (i.e. the smallest stimulus value) of
/// HIPAR startStimulus/a ripple limit range. A range must be created first to enable this
/// HIPAR startStimulus/command (e.g CALCulate<Chn>:RIPPle:DATA).
/// HIPAR stopStimulus/Changes the stop stimulus value (i.e. the largest stimulus value) of a
/// HIPAR stopStimulus/ripple limit range. A range must be created first to enable this
/// HIPAR stopStimulus/command (e.g CALCulate<Chn>:RIPPle:DATA).
/// HIPAR limit/Limit value, to be defined in accordance with the selected format
/// HIPAR limit/(CALCulate<Chn>:RIPPle:RDOMain:FORMat).
ViStatus _VI_FUNC rsvna_EditRippleLimitLineSegment(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   segment,
                                                   ViReal64  startStimulus,
                                                   ViReal64  stopStimulus,
                                                   ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_start[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Stop[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",RippleSeg%ld", channel, segment);

    strcpy(repCap_start, repCap);
    strcat(repCap_start, ",Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_start, RSVNA_ATTR_RIPPLE_LIMIT_LINE_SEGMENT_VALUES, startStimulus),
            4, "Start Stimulus");

    strcpy(repCap_Stop, repCap);
    strcat(repCap_Stop, ",Stop");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_Stop, RSVNA_ATTR_RIPPLE_LIMIT_LINE_SEGMENT_VALUES, stopStimulus),
            5, "Stop Stimulus");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_LINE_SEGMENT_LIMIT, limit),
            6, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of ripple limit ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR rippleLimitRangesCount/Returns 0 | 1 | ... - number of (enabled or disabled)) ripple limit
/// HIPAR rippleLimitRangesCount/ranges.
ViStatus _VI_FUNC rsvna_QueryRippleLimitRangesCount(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32*  rippleLimitRangesCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_RANGES_COUNT, rippleLimitRangesCount),
            3, "Ripple Limit RangesCount");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all ripple limit ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_DeleteAllRippleLimitRanges(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RIPPLE_LIMIT_DELETE_ALL_RANGES, NULL));

    checkErr(RsCore_ResetRepCapID(instrSession, "RippleSegment", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves or recalls the ripple limits associated with a
/// HIFN  specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operationtobePerformed/This control selects the type of operation to be performed.
/// HIPAR traceName/Name of an existing trace in the active setup (string parameter) for
/// HIPAR traceName/which a ripple limit definition exists.
/// HIPAR fileName/String parameter to specify the name and directory of the created
/// HIPAR fileName/ripple limit file.
ViStatus _VI_FUNC rsvna_SaveRecallRippleLimit(ViSession instrSession,
                                              ViInt32   operationtobePerformed,
                                              ViString  traceName,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (operationtobePerformed)
    {
    case RSVNA_VAL_SAVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:RIPP '%s','%s'", traceName, fileName);
        break;

    case RSVNA_VAL_RECALL:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:RIPP '%s','%s'", traceName, fileName);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operationtobePerformed), 2, "Operation to be Performed");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays or hides the circle limit line associated to
/// HIFN  the active trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR showCircleLimitLine/Switches the display line on or off.
ViStatus _VI_FUNC rsvna_ShowCircleLimits(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean showCircleLimitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SHOW_CIRCLE_LIMITS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SHOW_CIRCLE_LIMITS, showCircleLimitLine),
            3, "Show Circle Limit Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the circle limit check on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR circleCheck/Switches the limit check on or off.
ViStatus _VI_FUNC rsvna_ConfigureCircleLimitCheck(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean circleCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CIRCLE_LIMIT_CHECK,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CIRCLE_LIMIT_CHECK, circleCheck),
            3, "Ripple Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a circle limit lines by its center coordinates
/// HIFN  and its radius.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR centerX/Defines the limit circle by its center on the x-axis.
/// HIPAR centerY/Defines the limit circle by its center on the y-axis.
/// HIPAR radius/Defines the limit circle by its radius.
ViStatus _VI_FUNC rsvna_ConfigureCircleLimitLine(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  centerX,
                                                 ViReal64  centerY,
                                                 ViReal64  radius)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM:CIRC:DATA %.12lG,%.12lG,%.12lG", channel, centerX, centerY, radius);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a 0 or 1, to indicate whether or not the circle
/// HIFN  limit check has failed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR circleLimitCheckResult/Returns a 0 or 1, to indicate whether or not the circle limit check
/// HIPAR circleLimitCheckResult/has failed.
ViStatus _VI_FUNC rsvna_QueryCircleLimitCheckResult(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32*  circleLimitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CIRCLE_LIMIT_CHECK_RESULT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CIRCLE_LIMIT_CHECK_RESULT, circleLimitCheckResult),
            3, "Circle Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a 0 or 1, to indicate whether or not the circle
/// HIFN  limit check has failed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_ClearCircleLimitCheckResult(ViSession instrSession,
                                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CIRCLE_LIMIT_CHECK_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the display circle to its default configuration (unit circle;
/// HIFN  show border: off; limit to circle: off).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_CircleClear(ViSession instrSession,
                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CIRCLE_CLEAR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CIRCLE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the circle display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR limittoCircle/Sets the state of the display circle for the active trace of
/// HIPAR limittoCircle/channel <Chn>.
/// HIPAR showBorder/Sets the visibility of the display circle for the active trace
/// HIPAR showBorder/of channel <Chn>.
ViStatus _VI_FUNC rsvna_ConfigureCircleDisplay(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean limittoCircle,
                                               ViBoolean showBorder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LIMIT_TO_CIRCLE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_LIMIT_TO_CIRCLE, limittoCircle),
            3, "limittoCircle");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CIRCLE_SHOW_BORDER, showBorder),
            4, "showBorder");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the display circle for the active trace of channel <Chn>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR centerx/X position (real part) of the display circle's center.
/// HIPAR centery/Y position (imaginary part) of the display circle's center.
/// HIPAR radius/Radius of the display circle.
ViStatus _VI_FUNC rsvna_ConfigureCircleData(ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64  centerx,
                                            ViReal64  centery,
                                            ViReal64  radius)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG", centerx, centery, radius);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CIRCLE_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CIRCLE_DATA, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the marker state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerState/Creates or removes the marker.
ViStatus _VI_FUNC rsvna_ConfigureMarkerState(ViSession instrSession,
                                             ViInt32   channel_Trace,
                                             ViInt32   marker,
                                             ViBoolean markerState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_STATE,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_STATE, markerState),
            4, "Marker State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates the marker numbered <Mk> and assigns it to trace
/// HIFN  no. <Ch/Tr> and sets the stimulus (in Cartesian diagrams: x-axis)
/// HIFN  value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerState/Creates or removes the marker.
/// HIPAR markerStimulus/Stimulus value of marker no. <Mk>.
ViStatus _VI_FUNC rsvna_ConfigureMarker(ViSession instrSession,
                                        ViInt32   channel_Trace,
                                        ViInt32   marker,
                                        ViBoolean markerState,
                                        ViReal64  markerStimulus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_STATE,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_STATE, markerState),
            4, "Marker State");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_STIMULUS, markerStimulus),
            5, "Marker Stimulus");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the output format for the (complex) value of marker <Mk> on
/// HIFN  trace no. <Ch/Tr>.
/// HIFN  marker <Mk> on trace no. <Ch/Tr>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerFormat/This control defines the output format for the (complex) value of
/// HIPAR markerFormat/marker <Mk> on trace no. <Ch/Tr>.
ViStatus _VI_FUNC rsvna_ConfigureMarkerFormat(ViSession instrSession,
                                              ViInt32   channel_Trace,
                                              ViInt32   marker,
                                              ViInt32   markerFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_FORMAT,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel_Trace, repCap_marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_MARKER_FORMAT, markerFormat),
            4, "Marker Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the default marker format of the selected channel's active
/// HIFN  trace. New markers will be formatted with the default marker format;
/// HIFN  previously existing markers will be reformatted if (and only if) their
/// HIFN  marker format is set to (Trace) DEFault (using
/// HIFN  RSVNA_ATTR_MARKER_FORMAT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR markerDefaultFormat/Defines the default marker format of the selected channel's active
/// HIPAR markerDefaultFormat/trace. New markers will be formatted with the default marker format;
/// HIPAR markerDefaultFormat/previously existing markers will be reformatted if (and only if) their
/// HIPAR markerDefaultFormat/marker format is set to (Trace) DEFault (using
/// HIPAR markerDefaultFormat/RSVNA_ATTR_MARKER_FORMAT).
ViStatus _VI_FUNC rsvna_ConfigureMarkerDefaultFormat(ViSession instrSession,
                                                     ViInt32   channel_Trace,
                                                     ViInt32   markerDefaultFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_MARKER_DEFAULT_FORMAT, markerDefaultFormat),
            3, "Integer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the response (in Cartesian diagrams: y-axis)
/// HIFN  value of marker no. <Mk>. The marker must be created before using
/// HIFN  CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerResponse/Response value(s) of marker no. <Mk>. Unit is depending on the marker
/// HIPAR markerResponse/format; see CALCulate<Ch/Tr>:MARKer<Mk>:FORMat.
ViStatus _VI_FUNC rsvna_QueryMarkerResponse(ViSession instrSession,
                                            ViInt32   channel_Trace,
                                            ViInt32   marker,
                                            ViReal64  markerResponse[])
{
    ViStatus error = VI_SUCCESS;
    ViReal64 result[2] = {0.0, 0.0};
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:Y?", channel_Trace, marker);

    result[0] = result[1] = 0.0;

    checkErr(rsvna_getViReal64Array(instrSession,
        cmd,
        result,
        2,
        VI_ERROR_PARAMETER4));

    if (markerResponse)
    {
        markerResponse[0] = result[0];
        markerResponse[1] = result[1];
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates the reference marker and assigns it to trace no.
/// HIFN  <Ch/Tr> and defines the stimulus (in Cartesian diagrams: x-axis) value
/// HIFN  of the reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR referenceMarkerState/Creates or removes the reference marker.
/// HIPAR referenceMarkerStimulus/Stimulus value of reference marker no. <Mk>.
ViStatus _VI_FUNC rsvna_ConfigureReferenceMarker(ViSession instrSession,
                                                 ViInt32   channel_Trace,
                                                 ViInt32   marker,
                                                 ViBoolean referenceMarkerState,
                                                 ViReal64  referenceMarkerStimulus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_REFERENCE_MARKER_STATE,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_REFERENCE_MARKER_STATE, referenceMarkerState),
            4, "Reference Marker State");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_REFERENCE_MARKER_STIMULUS, referenceMarkerStimulus),
            5, "Reference Marker Stimulus");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the response (in Cartesian diagrams: y-axis)
/// HIFN  value of the reference marker. The reference marker must be created
/// HIFN  before using  CALCulate<Ch/Tr>:MARKer<Mk>:REFerence[:STATe] ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR referenceMarkerResponse/Response value of reference marker no. <Mk>. Where unit is:
ViStatus _VI_FUNC rsvna_QueryReferenceMarkerResponse(ViSession instrSession,
                                                     ViInt32   channel_Trace,
                                                     ViInt32   marker,
                                                     ViReal64* referenceMarkerResponse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_QUERY_REFERENCE_MARKER_RESPONSE,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_QUERY_REFERENCE_MARKER_RESPONSE, referenceMarkerResponse),
            4, "Reference Marker Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the delta mode for marker <Mk> on trace no.
/// HIFN  <Ch/Tr> on or off. The marker must be created before using
/// HIFN  CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON. If the active trace contains no
/// HIFN  reference marker, the command also creates a reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR deltaMarkerState/Enables or disables the delta mode.
ViStatus _VI_FUNC rsvna_ConfigureMarkerDeltaMode(ViSession instrSession,
                                                 ViInt32   channel_Trace,
                                                 ViInt32   marker,
                                                 ViBoolean deltaMarkerState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DELTA_MARKER_STATE,
            "Marker", marker -1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DELTA_MARKER_STATE, deltaMarkerState),
            4, "Delta Marker State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the output format for the (complex) value of the
/// HIFN  reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerReferenceFormat/Sets the output format for the (complex) value of the
/// HIPAR markerReferenceFormat/reference marker.
ViStatus _VI_FUNC rsvna_ConfigureMarkerReferenceFormat(ViSession instrSession,
                                                       ViInt32   channel_Trace,
                                                       ViInt32   marker,
                                                       ViInt32   markerReferenceFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel_Trace, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_REFERENCE_FORMAT,
            "Channel", channel_Trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_REFERENCE_FORMAT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCap_marker, RS_REPCAP_BUF_SIZE - strlen(repCap));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_REFERENCE_FORMAT, markerReferenceFormat),
            3, "Marker Reference Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes all markers from all traces of the active setup.
/// HIFN  The removed markers remember their properties (stimulus value, format,
/// HIFN  delta mode, number) when they are restored (CALC<Ch/Tr>:MARK<Mk> ON).
/// HIFN  The marker properties are definitely lost if the associated trace is
/// HIFN  deleted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel_Trace/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_SetAllMarkersOff(ViSession instrSession,
                                         ViInt32   channel_Trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel_Trace);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_DISABLE_ALL, VI_FALSE));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function moves the marker info field for the active trace <Chn>
/// HIFN  to one of nine predefined positions in the active diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR horizontalPosition/Selects the horizontal position of the info field.
/// HIPAR verticalPosition/Selects the vertical position of the info field.
ViStatus _VI_FUNC rsvna_ConfigureMarkerInfoFieldPosition(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   horizontalPosition,
                                                         ViInt32   verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizontalPosition, RSVNA_VAL_HORIZONTAL_LEFT, RSVNA_VAL_HORIZONTAL_RIGHT),
            3, "Horizontal Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalPosition, RSVNA_VAL_VERTICAL_TOP, RSVNA_VAL_VERTICAL_BOTTOM),
            4, "Vertical Position");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:STAT:AREA %s,%s", channel,
             horizontalPositionArr[horizontalPosition],
             verticalPositionArr[verticalPosition]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the properies of a marker created previously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerType/This control selects the type of a marker to be used.
/// HIPAR markerName/Defines a name for (reference) marker no. <Mk>.
/// HIPAR discreteMode/Sets marker no. <Mk> to continuous or discrete mode.
/// HIPAR fixedMarker/Sets marker no. <Mk> to normal or fixed mode.
ViStatus _VI_FUNC rsvna_ConfigureMarkerProperties(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   marker,
                                                  ViInt32   markerType,
                                                  ViString  markerName,
                                                  ViInt32   discreteMode,
                                                  ViInt32   fixedMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, markerType, RSVNA_VAL_MARKER, RSVNA_VAL_REFERENCE_MARKER),
            4, "Marker Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_NAME,
            "MarkerType", markerType , RS_REPCAP_BUF_SIZE, repCap_type),
            4, "Marker Type");

    strcat(repCap, ",");
    strcat(repCap, repCap_type);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_NAME, markerName),
            5, "Marker Name");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_MODE, discreteMode),
            6, "Discrete Mode");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_TYPE, fixedMarker),
            7, "Fixed Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the format in which the "Target Value"
/// HIFN  shall be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR traceOutputFormat/This control selects the search format for the target value of the marker.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTargetFormat(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   marker,
                                                    ViInt32   targetFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_TARGET_FORMAT, targetFormat),
            4, "Trace Output Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the format in which the target value shall be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR targetFormat/Selects the format in which the target value shall be specified.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTargetFormatReference(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   targetFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

   snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_TARGET_FORMAT_REFERENCE, targetFormat),
            3, "Target Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a search mode for the related marker (see Marker addressing),
/// HIFN  and initiate a search for the next valid peak to the left, the next
/// HIFN  highest or lowest value among the valid peaks, and the next valid peak
/// HIFN  to the right.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerTargetFormatReferenceSearchLeft(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SEARCH_LEFT_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_LEFT_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a search mode for the related marker (see Marker addressing),
/// HIFN  and initiate a search for the next valid peak to the left, the next
/// HIFN  highest or lowest value among the valid peaks, and the next valid peak
/// HIFN  to the right.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerTargetFormatReferenceSearchNext(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SEARCH_NEXT_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_NEXT_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a search mode for the related marker (see Marker addressing),
/// HIFN  and initiate a search for the next valid peak to the left, the next
/// HIFN  highest or lowest value among the valid peaks, and the next valid peak
/// HIFN  to the right.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerTargetFormatReferenceSearchRight(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SEARCH_RIGHT_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RIGHT_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a search for the related marker
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerTargetFormatReferenceSearchImmediate(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SEARCH_IMMEDIATE_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_IMMEDIATE_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the marker tracking mode for marker no. <Mk>.
/// HIFN  Tracking mode causes the active minimum/maximum or target search of the
/// HIFN  active marker to be repeated after each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables or disables the marker tracking mode for marker no. <Mk>.
/// HIPAR enabled/Tracking mode causes the active minimum/maximum or target search of the
/// HIPAR enabled/active marker to be repeated after each sweep.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTrackingReference(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_TRACKING_REFERENCE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a marker-specific threshold value for (single) peak searches.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR enabled/Activates or deactivates the marker-specific threshold for (single)
/// HIPAR enabled/peak searches.
/// HIPAR threshold/Defines a marker-specific threshold value for (single) peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerThreshold(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   marker,
                                                 ViBoolean enabled,
                                                 ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_THRESHOLD_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_THRESHOLD_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCap_marker, RS_REPCAP_BUF_SIZE - strlen(repCap));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_THRESHOLD_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_THRESHOLD, threshold),
            5, "threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables marker coupling to the active trace of the active channel or
/// HIFN  disables it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupledMarkers/Enables marker coupling to the active trace of the active channel or
/// HIPAR coupledMarkers/disables it.
ViStatus _VI_FUNC rsvna_ConfigureMarkerCoupledMarkers(ViSession instrSession,
                                                      ViBoolean coupledMarkers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_MARKER_COUPLED_MARKERS, coupledMarkers),
            2, "Coupled Markers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the marker coupling type to be used in the current recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the marker coupling type to be used in the current recall set.
ViStatus _VI_FUNC rsvna_ConfigureMarkerCouplingType(ViSession instrSession,
                                                    ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_MARKER_COUPLING_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a marker-specific minimum excursion value for peak searches.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR excursionMinimum/Defines a marker-specific minimum excursion value for peak searches.
/// HIPAR enabled/Activates or deactivates the marker-specific minimum excursion for
/// HIPAR enabled/peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerExcursion(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   marker,
                                                 ViBoolean enabled,
                                                 ViReal64  excursionMinimum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_EXCURSION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_EXCURSION_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCap_marker, RS_REPCAP_BUF_SIZE - strlen(repCap));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_EXCURSION_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_EXCURSION_MINIMUM, excursionMinimum),
            5, "Excursion Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For marker <Mk> of channel <Chn>'s active trace, this event does the
/// HIFN  following:
/// HIFN     - Create a channel
/// HIFN     - Set its center to marker <Mk>'s position
/// HIFN     - Set its span to 10 times channel <Chn>'s BW
/// HIFN     - Switch to spectrum mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
ViStatus _VI_FUNC rsvna_MarkerSpectrumEqualsMarker(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SPECTRUM_EQUALS_MARKER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SPECTRUM_EQUALS_MARKER,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Marker");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_SPECTRUM_EQUALS_MARKER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a minimum excursion value for multiple peak searches.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates or deactivates the minimum excursion for multiple peak
/// HIPAR enabled/searches.
/// HIPAR excursionMinimum/Defines a minimum excursion value for multiple peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerMultiplePeakExcursion(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViBoolean enabled,
                                                             ViReal64  excursionMinimum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_EXCURSION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_EXCURSION_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_EXCURSION_MINIMUM, excursionMinimum),
            4, "Excursion Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a threshold value for multiple peak searches.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates or deactivates the threshold for multiple peak searches.
/// HIPAR threshold/Defines a threshold value for multiple peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerMultiplePeakThreshold(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViBoolean enabled,
                                                             ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_THRESHOLD_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_THRESHOLD_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_MULTIPLE_PEAK_THRESHOLD, threshold),
            4, "threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the properies of a marker created previously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR enabled/Activates or deactivates the marker-specific minimum excursion for
/// HIPAR enabled/peak searches.
/// HIPAR reference/Defines a marker-specific minimum excursion value for peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerReferenceExcursion(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   marker,
                                                          ViBoolean enabled,
                                                          ViReal64  reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_EXCURSION_REFERENCE_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_EXCURSION_REFERENCE, reference),
            5, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select a search mode for the related marker (see Marker addressing)
/// HIFN  and initiate a maximum or minimum search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerReferenceMaximum(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_MAXIMUM_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_MAXIMUM_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select a search mode for the related marker (see Marker addressing)
/// HIFN  and initiate a maximum or minimum search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerReferenceMinimum(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_MINIMUM_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MARKER_MINIMUM_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the properies of a marker created previously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates or deactivates the marker-specific minimum excursion for
/// HIPAR enabled/peak searches.
/// HIPAR reference/Defines a marker-specific threshold value for (single) peak searches.
ViStatus _VI_FUNC rsvna_ConfigureMarkerReferenceThreshold(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean enabled,
                                                          ViReal64  reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_THRESHOLD_ENABLED_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_THRESHOLD_ENABLED_REFERENCE, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_THRESHOLD_REFERENCE, reference),
            4, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates search with the active marker for specific
/// HIFN  points on the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR search/Sets the active marker to specific points on the trace.
ViStatus _VI_FUNC rsvna_MarkerSearch(ViSession instrSession,
                                     ViInt32   channel,
                                     ViInt32   marker,
                                     ViInt32   search)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH, search),
            4, "Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a search mode for marker no. <Mk> and initiates the search.
/// HIFN  The marker must be created before.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The marker must be created before using function
/// HIFN  rsvna_ConfigureMarker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR search/Selects a search mode for marker no. <Mk> and initiates the search.
/// HIPAR search/The marker must be created before.
ViStatus _VI_FUNC rsvna_MarkerSearchReference(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   marker,
                                              ViInt32   search)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_REFERENCE, search),
            4, "Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates search with the active marker for target value
/// HIFN  on the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR search/Sets the active marker to specific points on the trace.
ViStatus _VI_FUNC rsvna_MarkerTargetSearch(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   marker,
                                           ViInt32   search)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH, search),
            4, "Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the target value for the target search of marker
/// HIFN  no. <Mk>, which can be activated using function
/// HIFN  rsvna_MarkerTargetSearch.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR targetValue/Defines the target value for the target search.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTargetValue(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   marker,
                                                   ViReal64  targetValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_TARGET_VALUE, targetValue),
            4, "Target Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the target value for the target search of marker
/// HIFN  no. <Mk>, which can be activated using function
/// HIFN  rsvna_MarkerTargetSearch.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR targetValue/Defines the target value for the target search.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTargetValueReference(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViReal64  targetValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_TARGET_VALUE_REFERENCE, targetValue),
            3, "Target Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates a bandfilter search with the active marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerBandfilterSearch(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk1", channel);

    checkErr(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH, RSVNA_VAL_MARKER_BFILTER));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the bandfilter search mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR searchMode/This control selects the bandfilter search mode.
ViStatus _VI_FUNC rsvna_ConfigureMarkerBandfilterSearchMode(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   searchMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_BANDFILTER_SEARCH_MODE, searchMode),
            3, "Search Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the bandfilter level, i.e. the minimum excursion
/// HIFN  for the bandpass and bandstop peaks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR xdBBandwidth/Defines x dB Bandwidth parameter. The x dB Bandwidth is the difference
/// HIPAR xdBBandwidth/between the band edges and the center response value of a bandfilter
/// HIPAR xdBBandwidth/peak; it must be negative for a bandpass search and positive for a
/// HIPAR xdBBandwidth/bandstop search.
ViStatus _VI_FUNC rsvna_ConfigureMarkerxdBBandwidth(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViReal64  xdBBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_X_DB_BANDWIDTH, xdBBandwidth),
            3, "x dB Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the results of a bandpass or a bandstop filter
/// HIFN  search. Result is only available after a bandfilter search has been
/// HIFN  executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR bandwidth/Returns n-dB bandwidth of the bandpass/bandstop region, where n is the
/// HIPAR bandwidth/selected x dB Bandwidth (setting command
/// HIPAR bandwidth/CALCulate<Ch/Tr>:MARKer<Mk>:BWIDth <x dB Bandwidth>).
/// HIPAR centerStimulus/Returns stimulus frequency at the center of the bandpass/bandstop
/// HIPAR centerStimulus/region (the stimulus value of marker no. 4).
/// HIPAR q/Returns quality factor, i.e. the ratio between the center frequency
/// HIPAR q/and the 3-dB bandwidth.
/// HIPAR loss/Returns loss at the center of the bandpass/bandstop region (the
/// HIPAR loss/response value of marker no. 4 at the time of the bandfilter search).
/// HIPAR lbe/Returns lower band edge.
/// HIPAR ube/Returns upper band edge.
ViStatus _VI_FUNC rsvna_QueryMarkerBandfilterResults(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViReal64* bandwidth,
                                                     ViReal64* centerStimulus,
                                                     ViReal64* q,
                                                     ViReal64* loss,
                                                     ViReal64* lowerEdge,
                                                     ViReal64* upperEdge)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 result[6];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_getViReal64ArrayWindow(instrSession,
        "CALC%ld:MARK:BWID?",
        channel,
        result,
        6,
        VI_ERROR_PARAMETER1));

    if (bandwidth)
        *bandwidth = result[0];
    if (centerStimulus)
        *centerStimulus = result[1];
    if (q)
        *q = result[2];
    if (loss)
        *loss = result[3];
    if (lowerEdge)
        *lowerEdge = result[4];
    if (upperEdge)
        *upperEdge = result[5];

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows or hides the bandfilter search results in the
/// HIFN  diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR searchResults/Shows or hides the bandfilter search results in the diagram area.
/// HIPAR horizontalPosition/Selects the horizontal position of the info field.
/// HIPAR verticalPosition/Selects the vertical position of the info field.
ViStatus _VI_FUNC rsvna_ConfigureMarkerBandfilterSearchResultState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean searchResults,
                            ViInt32   horizontalPosition,
                            ViInt32   verticalPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, horizontalPosition, RSVNA_VAL_HORIZONTAL_LEFT, RSVNA_VAL_HORIZONTAL_RIGHT),
            5, "Horizontal Position");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, verticalPosition, RSVNA_VAL_VERTICAL_TOP, RSVNA_VAL_VERTICAL_BOTTOM),
            6, "Vertical Position");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_BANDFILTER_SEARCH_RESULT_STATE, searchResults),
            3, "Search Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:SEAR:BFIL:RES:STAT:AREA %s,%s", channel,
             horizontalPositionArr[horizontalPosition],
             verticalPositionArr[verticalPosition]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the marker tracking mode for marker
/// HIFN  no. <Mk>. Tracking mode causes the active minimum/maximum or target
/// HIFN  search of the active marker to be repeated after each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerTracking/Enables or disables the marker tracking mode.
ViStatus _VI_FUNC rsvna_ConfigureMarkerTracking(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   marker,
                                                ViBoolean markerTracking)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_TRACKING, markerTracking),
            4, "Marker Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines search ranges for the maximum/minimum or target
/// HIFN  search. Assigns a search range to marker and define the start and stop
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR searchRange/Selects one out of 10 search ranges.
/// HIPAR start/Defines the start value of the selected search range.
/// HIPAR stop/Defines the stop value of the selected search range.
ViStatus _VI_FUNC rsvna_ConfigureMarkerSearchRange(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   marker,
                                                   ViInt32   searchRange,
                                                   ViReal64  start,
                                                   ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE, searchRange),
            4, "Search Range");

    if (searchRange != RSVNA_VAL_FULL_SPAN)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_START, start),
                5, "Start");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_STOP, stop),
                6, "Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines search ranges for the maximum/minimum or target
/// HIFN  search. Assigns a search range to marker and define the start and stop
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR searchRange/Assigns a search range to marker no <Mk> and selects the search range,
/// HIPAR searchRange/e.g. in order to display range limit lines or define the start and stop
/// HIPAR searchRange/values.
/// HIPAR start/Defines the start value of the search range.
/// HIPAR stop/Defines the stop value of the selected search range.
ViStatus _VI_FUNC rsvna_ConfigureMarkerSearchRangeReference(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   searchRange,
                                                            ViReal64  start,
                                                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_REFERENCE, searchRange),
            4, "Search Range");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_START_REFERENCE, start),
            5, "Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_STOP_REFERENCE, stop),
            6, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Returns the result (stimulus and response value) of a
/// HIFN  search started by means of function rsvna_MarkerSearch. The search must
/// HIFN  be executed before the command is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR markerStimulus/Stimulus value of marker no. <Mk>. Where unit is:
/// HIPAR markerResponse/Response value(s) of marker no. <Mk>. Unit is depending on the marker
/// HIPAR markerResponse/format; see CALCulate<Ch/Tr>:MARKer<Mk>:FORMat.
ViStatus _VI_FUNC rsvna_QueryMarkerSearchResults(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   marker,
                                                 ViReal64* markerStimulus,
                                                 ViReal64  markerResponse[])
{
    ViStatus error = VI_SUCCESS;
    ViReal64 result[3];
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:RES?", channel, marker);

    checkErr(rsvna_getViReal64Array(instrSession,
        cmd,
        result,
        3,
        VI_ERROR_PARAMETER5));

    if (markerStimulus)
        *markerStimulus = result[0];

    if (markerResponse)
    {
        markerResponse[0] = result[1];
        markerResponse[1] = result[2];
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the beginning (start), end (stop), span and center
/// HIFN  of the sweep range equal to the stimulus value of the marker <Mk> on
/// HIFN  trace no. <Chn>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR marker/Marker number.
/// HIPAR setByMarker/Sets the beginning (start), end (stop), span and center of the sweep
/// HIPAR setByMarker/range equal to the stimulus value of the marker <Mk> on trace no. <Chn>
ViStatus _VI_FUNC rsvna_ConfigureSweepRangeToMarker(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   marker,
                                                    ViInt32   setByMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 10),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setByMarker, RSVNA_VAL_SET_BY_MARKER_START, RSVNA_VAL_SET_BY_MARKER_SPAN),
            4, "Set By Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Mk%ld", channel, marker);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_RANGE_TO_MARKER,
            "StartStopCenterSpan", setByMarker , RS_REPCAP_BUF_SIZE, repCap_marker),
            4, "Set By Marker");

    strcat(repCap, ",");
    strcat(repCap, repCap_marker);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_RANGE_TO_MARKER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the beginning (start), end (stop), span and center
/// HIFN  of the sweep range equal to the stimulus value of the marker <Mk> on
/// HIFN  trace no. <Chn>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR setByMarker/Sets the beginning (start), end (stop) of the sweep range equal to the
/// HIPAR setByMarker/stimulus value of the marker <Mk> on trace no. <Chn>
ViStatus _VI_FUNC rsvna_ConfigureSweepRangeToMarkerReference(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   setByMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSVNA_ATTR_START_SWEEP_RANGE_TO_MARKER_REFERENCE,RSVNA_ATTR_STOP_SWEEP_RANGE_TO_MARKER_REFERENCE};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setByMarker, RSVNA_VAL_SET_BY_MARKER_START, RSVNA_VAL_SET_BY_MARKER_SPAN),
            3, "Set By Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, attr[setByMarker], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the center of the sweep range equal to the stimulus value of the
/// HIFN  marker <Mk> on trace no. <Chn>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_MarkerCenterSweepRangeToMarkerReference(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CENTER_SWEEP_RANGE_TO_MARKER_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CENTER_SWEEP_RANGE_TO_MARKER_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays or hides range limit lines for the search range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Displays or hides range limit lines for the search range.
ViStatus _VI_FUNC rsvna_ConfigureMarkerSearchRangeShowLimitLinesReference(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MARKER_SEARCH_RANGE_SHOW_LIMIT_LINES_REFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MARKER_SEARCH_RANGE_SHOW_LIMIT_LINES_REFERENCE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the loaded custom (external) DLL files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR externalDLLList/Returns a comma-separated list of the loaded custom (external) DLL files.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_ExternalDLLList(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    externalDLLList[],
                                        ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_EXTERNAL_DLL_LIST,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, externalDLLList, responseLength),
                3, "External DLL List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the tasks implemented by the specified DLL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dllName/Name of a loaded DLL (see RSVNA_ATTR_EXTERNAL_DLL_LIST). Lowercase
/// HIPAR dllName/string, without dll extension.
/// HIPAR bufferSize/Size of output array.
/// HIPAR tasks/Returns the tasks implemented by the specified DLL.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_ExternalDLLTaskList(ViSession instrSession,
                                            ViString  dllName,
                                            ViInt32   bufferSize,
                                            ViChar    tasks[],
                                            ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   qParam[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(qParam, RS_PARAM_BUF_SIZE, "'%s'", dllName);

    viCheckParm(RsCore_GetAttributeViStringWithQparam(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_TASK_LIST,
                    qParam, bufferSize, tasks, responseLength),
                4, "Tasks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates and configures an external DLL that has already been loaded
/// HIFN  into the instrument's firmware using rsvna_AddExternalDLL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dllName/Name of a loaded DLL (see RSVNA_ATTR_EXTERNAL_DLL_LIST). Lowercase
/// HIPAR dllName/string, without dll extension.
/// HIPAR active/Defines the active state of <DllName>.dll in the specified channel.
/// HIPAR permanent/Defines whether <DllName>.dll shall operate in permanent mode in the
/// HIPAR permanent/specified channel, if the DLL is active.
/// HIPAR task/Defines the type of task to be run, if custom <DllName>.dll is set to
/// HIPAR task/permanent mode in the specified channel.
/// HIPAR additional/Defines additional input for the task to be run, if custom
/// HIPAR additional/<DllName>.dll is set to permanent mode. This string is processed by the
/// HIPAR additional/external DLL, with DLL-specific syntax and semantics. If it is
/// HIPAR additional/malformed or inappropriate for some reason, the firmware logs and
/// HIPAR additional/displays the error messages that are returned by the DLL. See the docs
/// HIPAR additional/of your DLL for details.
/// HIPAR address/Defines the address of an external device which <DllName>.dll requires
/// HIPAR address/to complete the specified task in the specified channel. The address
/// HIPAR address/syntax is DLL-specific: the DLL manages the connection to the external
/// HIPAR address/device. See the docs of your DLL for details.
ViStatus _VI_FUNC rsvna_ConfigureExternalDLL(ViSession instrSession,
                                             ViInt32   channel,
                                             ViString  dllName,
                                             ViBoolean active,
                                             ViBoolean permanent,
                                             ViString  task,
                                             ViString  additional,
                                             ViString  address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dllParam[RS_PARAM_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);
    snprintf(dllParam, RS_PARAM_BUF_SIZE, "'%s'", dllName);

    snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", dllParam, offOnArr[active]);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_ACTIVE, RS_VAL_DIRECT_USER_CALL,
                    param),
                4, "Active");

    if (active)
    {
        snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", dllParam, offOnArr[permanent]);

        viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                        RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT, RS_VAL_DIRECT_USER_CALL,
                        param),
                    5, "Permanent");

        if (permanent)
        {
            snprintf(param, RS_PARAM_BUF_SIZE, "%s,'%s'", dllParam, task);

            viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                            RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT_TASK, RS_VAL_DIRECT_USER_CALL,
                            param),
                        6, "Task");

            snprintf(param, RS_PARAM_BUF_SIZE, "%s,'%s'", dllParam, additional);

            viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                            RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT_TASK_ADDITIONAL_INPUT, RS_VAL_DIRECT_USER_CALL,
                            param),
                        7, "Additional");
        }

        snprintf(param, RS_PARAM_BUF_SIZE, "%s,'%s'", dllParam, address);

        viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                        RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_TASK_ADDRESS, RS_VAL_DIRECT_USER_CALL,
                        param),
                    8, "Address");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the configuration of an external DLL that has been loaded into
/// HIFN  the instrument's firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dllName/Name of a loaded DLL (see RSVNA_ATTR_EXTERNAL_DLL_LIST). Lowercase
/// HIPAR dllName/string, without dll extension.
/// HIPAR active/Returns the active state of <DllName>.dll in the specified channel.
/// HIPAR permanent/Returns whether <DllName>.dll is operating in permanent mode in the
/// HIPAR permanent/specified channel, if the DLL is active.
/// HIPAR buffersSize/Sizes of the Task, Addional, and Address output arrays.
/// HIPAR task/Returns the type of task running, if custom <DllName>.dll is set to
/// HIPAR task/permanent mode in the specified channel.
/// HIPAR additional/Returns additional input for the running task, if custom <DllName>.dll
/// HIPAR additional/is set to permanent mode.
/// HIPAR address/Returns the address of an external device which <DllName>.dll requires
/// HIPAR address/to complete the current task in the specified channel.
ViStatus _VI_FUNC rsvna_QueryExternalDLLConfiguration(ViSession  instrSession,
                                                      ViInt32    channel,
                                                      ViString   dllName,
                                                      ViBoolean* active,
                                                      ViBoolean* permanent,
                                                      ViInt32    buffersSize,
                                                      ViChar     task[],
                                                      ViChar     additional[],
                                                      ViChar     address[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dllParam[RS_PARAM_BUF_SIZE] = "";
    ViChar   readBuffer[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);
    snprintf(dllParam, RS_PARAM_BUF_SIZE, "'%s'", dllName);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_ACTIVE,
                    dllParam, RS_MAX_MESSAGE_LEN, readBuffer, NULL),
                4, "Active");

    *active = RsCore_Convert_String_To_Boolean(readBuffer);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT,
                    dllParam, RS_MAX_MESSAGE_LEN, readBuffer, NULL),
                5, "Permanent");

    *permanent = RsCore_Convert_String_To_Boolean(readBuffer);

    if (buffersSize > 0)
    {
        viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                        RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT_TASK, dllParam,
                        buffersSize, task, NULL),
                    6, "Task");
        viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                        RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_PERMANENT_TASK_ADDITIONAL_INPUT, dllParam,
                        buffersSize, additional, NULL),
                    7, "Additional");
        viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                        RSVNA_ATTR_HIDDEN_EXTERNAL_DLL_TASK_ADDRESS, dllParam,
                        buffersSize, address, NULL),
                    8, "Address");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the analyzer firmware to load a custom (external) DLL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dllPath/Specifies the path to the zip archive containing the DLL.
ViStatus _VI_FUNC rsvna_AddExternalDLL(ViSession instrSession,
                                       ViString  dllPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_ADD_EXTERNAL_DLL, dllPath),
            2, "Dll Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the analyzer firmware to remove a custom (external) DLL that was
/// HIFN  previously added using rsvna_AddExternalDLL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dllName/Specifies the name of the DLL to remove.
ViStatus _VI_FUNC rsvna_RemoveExternalDLL(ViSession instrSession,
                                          ViString  dllName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_REMOVE_EXTERNAL_DLL, dllName),
            2, "Dll Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Traces calculated using external DLL <ext_dll_name>.dll
/// HIFN
/// HIFN     Set with the name of a loaded DLL (see rsvna_ExternalDLLList).
/// HIFN  Lowercase string, without dll extension.
/// HIFN
/// HIFN     Note that adding/removing custom DLLs is only possible if the firmware
/// HIFN  is run as administrator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR traceName/Define trace name.
/// HIPAR port/Sets the port.
/// HIPAR dllName/Specifies the name of the DLL to remove.
/// HIPAR taskType/Selects the task that is responsible for calculating this kind of
/// HIPAR taskType/trace.
/// HIPAR additionalInput/For some task types, the basic information can be supplemented with
/// HIPAR additionalInput/Additional Input. This string is processed by the external DLL, with
/// HIPAR additionalInput/DLL-specific syntax and semantics.
ViStatus _VI_FUNC rsvna_SelectExternalDLL(ViSession instrSession,
                                          ViInt32   channel,
                                          ViString  traceName,
                                          ViInt32   port,
                                          ViString  dllName,
                                          ViString  taskType,
                                          ViString  additionalInput)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  offset;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_traceCommandSelect(instrSession, channel, traceName, cmd, &offset));

    sprintf(cmd + offset, "'%s','Cu(P%ld)%s/%s/%s'", traceName, port, dllName, taskType, additionalInput);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Linear Deviation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR mode/Applies /discards the correction factors or re-calculates them for
/// HIPAR mode/each trace (Tracking).
/// HIPAR constant/Defines the constant value for the linearity deviation calculation.
/// HIPAR constant/The range depends on the trace format.
/// HIPAR slope/Defines the slope of the regression line for the linearity deviation
/// HIPAR slope/calculation.The range depends on the trace format.
ViStatus _VI_FUNC rsvna_ConfigureLinearityDeviation(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   mode,
                                                    ViReal64  constant,
                                                    ViReal64  slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LINEARITY_DEVIATION_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LINEARITY_DEVIATION_MODE, mode),
            3, "Mode");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_LINEARITY_DEVIATION_CONSTANT, constant),
            4, "Constant");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_LINEARITY_DEVIATION_SLOPE, slope),
            5, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Linear Deviation electrical length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR electricalLength/Defines the electrical length for the linearity deviation calculation.
/// HIPAR electricalLength/This control is available if the active trace format is Phase or
/// HIPAR electricalLength/Unwrapped Phase, or if a mixer delay is measured.
ViStatus _VI_FUNC rsvna_ConfigureLinearityDeviationElectricalLength(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  electricalLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LINEARITY_DEVIATION_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_LINEARITY_DEVIATION_ELECTRICAL_LENGTH, electricalLength),
            3, "Electrical Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a (re-)calculation of the linearity deviation correction
/// HIFN  factors and applies them to the active trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_RecalculateLinearityDeviation(ViSession instrSession,
                                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_RECALCULATE_LINEARITY_DEVIATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RECALCULATE_LINEARITY_DEVIATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the start and stop frequencies for a frequency
/// HIFN sweep which is equal to the left and right edges of a Cartesian diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR startFrequency/defines the start frequency for a frequency sweep which is equal to the left edge of a Cartesian diagram.
/// HIPAR stopFrequency/defines the stop frequency for a frequency sweep which is equal to the right edge of a Cartesian diagram.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyStartStop(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViReal64  startFrequency,
                                                    ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_START, startFrequency),
            3, "Start Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_STOP, stopFrequency),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function defines the center and width of the measurement
/// HIFN and display range for a frequency sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR centerFrequency/This control defines the center of the measurement and display range for a frequency sweep.
/// HIPAR span/This control defines the width of the measurement and display range for a frequency sweep.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyCenterSpan(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViReal64  centerFrequency,
                                                     ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_CENTER, centerFrequency),
            3, "Center Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_SPAN, span),
            4, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the power of the internal signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR frequency/Selects start/stop frequency.
/// HIPAR type/Selects the max/min/default frequency.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyMinMax(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   frequency,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    switch (frequency)
    {
    case RSVNA_VAL_FREQ_START:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_START_MIN_MAX_DEFAULT, type),
                4, "Type");
        break;
    case RSVNA_VAL_FREQ_STOP:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_STOP_MIN_MAX_DEFAULT, type),
                4, "Type");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, frequency), 3, "Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the power of the internal signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR power/This control defines the power of the internal signal source.
ViStatus _VI_FUNC rsvna_ConfigurePower(ViSession instrSession,
                                       ViInt32   channel,
                                       ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_INTERNAL_SOURCE_POWER, power),
            3, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the start and stop power for a power sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR startPower/This control defines the start power for a power sweep which is
/// HIPAR startPower/equal to the left edge of a Cartesian diagram.
/// HIPAR stopPower/This control defines the stop power for a power sweep which is
/// HIPAR stopPower/equal to the right edge of a Cartesian diagram.
ViStatus _VI_FUNC rsvna_ConfigurePowerStartStop(ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  startPower,
                                                ViReal64  stopPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_start[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_stop[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap_start, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Start", channel);
    snprintf(repCap_stop, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Stop", channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_start, RSVNA_ATTR_STIMULUS_POWER_START_STOP, startPower),
            3, "Start Power");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap_stop, RSVNA_ATTR_STIMULUS_POWER_START_STOP, stopPower),
            4, "Stop Power");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function defines the fixed (Continuous Wave, CW) frequency for all
/// HIFN sweep types operating at fixed frequency (power sweep, time sweep, CW mode sweep).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR CWFrequency/defines the fixed frequency for all sweep types operating at fixed frequency.
ViStatus _VI_FUNC rsvna_ConfigureCWFrequency(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  CWFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_CW, CWFrequency),
            3, "CW Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a source port for the stimulus signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR sourcePort/defines the test port number of the analyzer.
ViStatus _VI_FUNC rsvna_ConfigureSourcePort(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_STIMULUS_SOURCE_PORT, sourcePort),
            3, "Source Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the minimum and maximum frequency of the network analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects if querry maximum or minimum frequency range.
/// HIPAR frequency/Return minimum or maximum frequency of instrument.
ViStatus _VI_FUNC rsvna_QueryFrequencyRange(ViSession instrSession,
                                            ViInt32   range,
                                            ViReal64* frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_STIMULUS_FREQUENCY_RANGE,
            "MinMax", range , RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_STIMULUS_FREQUENCY_RANGE, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the internal preamplifier located in the
/// HIFN  measurement receiver path of port <Pt> = 1 or 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Enables or disables the internal preamplifier located in the
/// HIPAR enabled/measurement receiver path of port <Pt> = 1 or 2.
ViStatus _VI_FUNC rsvna_ConfigurePreamplifierEnabled(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   physicalPort,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREAMPLIFIER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREAMPLIFIER_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PREAMPLIFIER_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the gain of the 'Internal Low Noise Preamplifier' located in the
/// HIFN  measurement receiver path of port 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gain/Sets the gain of the 'Internal Low Noise Preamplifier' located in the
/// HIPAR gain/measurement receiver path of port 2.
ViStatus _VI_FUNC rsvna_ConfigurePreamplifierGain(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREAMPLIFIER_GAIN,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PREAMPLIFIER_GAIN, gain),
            3, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the start and stop times of the diagram in time
/// HIFN  domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR startTime/This control defines start time of the diagram in time domain.
/// HIPAR stopTime/This control defines stop time of the diagram in time domain.
ViStatus _VI_FUNC rsvna_ConfigureTimeDomainStartStopTime(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViReal64  startTime,
                                                         ViReal64  stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS_START_TIME, startTime),
            3, "Start Time");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS_STOP_TIME, stopTime),
            4, "Stop Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the center and span time of the diagram in time
/// HIFN  domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR centerTime/This control defines center time of the diagram in time domain.
/// HIPAR timeSpan/This control defines time span of the diagram in time domain.
ViStatus _VI_FUNC rsvna_ConfigureTimeDomainCenterSpanTime(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  centerTime,
                                                          ViReal64  timeSpan)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS_CENTER_TIME, centerTime),
            3, "Center Time");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS_TIME_SPAN, timeSpan),
            4, "Time Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches over between the x-axis scaling in time units
/// HIFN  or distance units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR xAxisScaling/This control switches over between the x-axis scaling in time units or
/// HIPAR xAxisScaling/distance units.
ViStatus _VI_FUNC rsvna_ConfigureTimeDomainXAxisScaling(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   xAxisScaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TIME_DOMAIN_STIMULUS_X_AXIS_SCALING, xAxisScaling),
            3, "X Axis Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables automatic IF bandwidth setting for the ALC path in
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR automaticIFBandwidth/Enables or disables automatic IF bandwidth setting for the ALC path in
/// HIPAR automaticIFBandwidth/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCAutomaticIFBandwidth(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean automaticIFBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_AUTOMATIC_IF_BANDWIDTH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_AUTOMATIC_IF_BANDWIDTH, automaticIFBandwidth),
            3, "Automatic IF Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the bandwidth in the ALC control loop in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR bandwidth/Selects the bandwidth in the ALC control loop in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCBandwidth(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_BANDWIDTH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_BANDWIDTH, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables Automatic Level Control (ALC) for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enables or disables Automatic Level Control (ALC) for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCEnabled(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enforces identical ALC settings for the individual ports in channel
/// HIFN  <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR portCouple/Enforces identical ALC settings for the individual ports in channel
/// HIPAR portCouple/<Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortCouple(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean portCouple)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_COUPLE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_COUPLE, portCouple),
            3, "Port Couple");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the ALC Settling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR autoSettling/Enables or disables Auto Settling of the ALC control loop.
/// HIPAR controlTime/Defines the (maximum) settling time of the ALC control loop, per sweep
/// HIPAR controlTime/point.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCSettling(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean autoSettling,
                                                  ViReal64  controlTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_AUTO_SETTLING,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_AUTO_SETTLING, autoSettling),
            3, "Auto Settling");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_CONTROL_TIME, controlTime),
            4, "Control Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the variation of the ALC-controlled source signal level
/// HIFN  (Settling Tolerance in ALC Config dialog) in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR settlingTolerance/Defines the variation of the ALC-controlled source signal level
/// HIPAR settlingTolerance/(Settling Tolerance in ALC Config dialog) in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCSettlingTolerance(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViReal64  settlingTolerance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_SETTLING_TOLERANCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_SETTLING_TOLERANCE, settlingTolerance),
            3, "Settling Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the ALC Optimize.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR portOptimize/Sets the ALC Optimize.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortOptimize(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   physicalPort,
                                                      ViInt32   portOptimize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_OPTIMIZE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_OPTIMIZE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_OPTIMIZE, portOptimize),
            4, "Port Optimize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PI parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR portPI/Enables automatic or manual setting of the ALC PI controller
/// HIPAR portPI/parameters at source port <Pt> in channel <Ch>.
/// HIPAR piGain/Defines the proportional gain of the ALC PI controller at source port
/// HIPAR piGain/<Pt> in channel <Ch>.
/// HIPAR piIntegrationTime/Defines the integration time of the ALC PI controller at source port
/// HIPAR piIntegrationTime/<Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortPI(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   physicalPort,
                                                ViInt32   portPI,
                                                ViReal64  piGain,
                                                ViReal64  piIntegrationTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_PI,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_PI,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_PI, portPI),
            4, "Port PI");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_PI_GAIN, piGain),
            5, "PI Gain");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_PI_INTEGRATION_TIME, piIntegrationTime),
            6, "PI Integration Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum change of the source signal level due to the ALC
/// HIFN  for physical port <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR portControlRange/Defines the maximum change of the source signal level due to the ALC
/// HIPAR portControlRange/for physical port <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortControlRange(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   physicalPort,
                                                          ViReal64  portControlRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_CONTROL_RANGE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_CONTROL_RANGE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_CONTROL_RANGE, portControlRange),
            4, "Port Control Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the receiver to be used for ALC at port <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR receiver/Defines the receiver to be used for ALC at port <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortReceiver(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   physicalPort,
                                                      ViString  receiver)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_RECEIVER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_RECEIVER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_RECEIVER, receiver),
            4, "Receiver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Increases or decreases the signal level before the ALC loop is started
/// HIFN  for port <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR startOffset/Increases or decreases the signal level before the ALC loop is started
/// HIPAR startOffset/for port <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortStartOffset(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViReal64  startOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_START_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_START_OFFSET,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_START_OFFSET, startOffset),
            4, "Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables ALC (Automatic Level Control) at source port <Pt>
/// HIFN  in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR tableAreaOn/Enables or disables ALC (Automatic Level Control) at source port <Pt>
/// HIPAR tableAreaOn/in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortTableAreaOn(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViBoolean tableAreaOn)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_TABLE_AREA_ON, tableAreaOn),
            4, "Table Area On");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the target power value the ALC for port <Pt> in channel <Ch>
/// HIFN  stabilizes at the selected measurement receiver
/// HIFN  (SOURce<Ch>:POWer<Pt>:ALC:RECeiver).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR targetPowerDefault/Defines whether the ALC target power is taken from the port
/// HIPAR targetPowerDefault/configuration or can be set using SOURce<Ch>:POWer<Pt>:ALC:TPOWer.
/// HIPAR targetPower/Defines the target power value the ALC for port <Pt> in channel <Ch>
/// HIPAR targetPower/stabilizes at the selected measurement receiver
/// HIPAR targetPower/(SOURce<Ch>:POWer<Pt>:ALC:RECeiver).
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortTargetPower(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViBoolean targetPowerDefault,
                                                         ViReal64  targetPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_TARGET_POWER_DEFAULT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_TARGET_POWER_DEFAULT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_TARGET_POWER_DEFAULT, targetPowerDefault),
            4, "Target Power Default");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_TARGET_POWER, targetPower),
            5, "Target Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to use the configured offset or single-ended deembedding
/// HIFN  network for ALC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR deembedingNetworkUsage/Allows you to use the configured offset or single-ended deembedding
/// HIPAR deembedingNetworkUsage/network for ALC.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCPortDeembedingNetworkUsage(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   deembedingNetworkUsage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_DEEMBEDING_NETWORK_USAGE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_PORT_DEEMBEDING_NETWORK_USAGE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_POWER_ALC_PORT_DEEMBEDING_NETWORK_USAGE, deembedingNetworkUsage),
            4, "Deembeding Network Usage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Suspends the ALC mechanism at physical port <Pt> in channel <Ch> while
/// HIFN  the analyzer acquires measurement data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR clamp/Suspends the ALC mechanism at physical port <Pt> in channel <Ch> while
/// HIPAR clamp/the analyzer acquires measurement data.
ViStatus _VI_FUNC rsvna_ConfigurePowerALCClamp(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   physicalPort,
                                               ViBoolean clamp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_CLAMP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_ALC_CLAMP,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_ALC_CLAMP, clamp),
            4, "Clamp");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the internal source power at all ports and the power of all external generators on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR RFState/turns the internal source power at all ports and the power of all external generators on or off.
ViStatus _VI_FUNC rsvna_ConfigureRFState(ViSession instrSession,
                                         ViBoolean RFState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_CHANNEL_RF_STATE, RFState),
            2, "RF State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the measurement bandwidth and the shape of the IF filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR bandwidth/This control defines the resolution bandwidth of the analyzer (Meas. Bandwidth).
/// HIPAR filterSelectivity/This control defines the selectivity of the IF filter for an unsegmented sweep.
ViStatus _VI_FUNC rsvna_ConfigureBandwidth(ViSession instrSession,
                                           ViInt32   channel,
                                           ViReal64  bandwidth,
                                           ViInt32   filterSelectivity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_CHANNEL_RESOLUTION_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    if (RsCore_IsInstrumentModel(instrSession, "ZNB|ZNA"))
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CHANNEL_FILTER_SELECTIVITY, filterSelectivity),
                4, "Filter Selectivity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures averaging algorithms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR mode/Selects one of the following averaging algorithms.
ViStatus _VI_FUNC rsvna_ConfigureAverageMode(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CHANNEL_SWEEP_AVERAGE_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the number of consecutive sweeps to be averaged and enables/disables the sweep average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR averageState/enables or disables the sweep average.
/// HIPAR averageFactor/defines the number of consecutive sweeps to be combined for the sweep average.
ViStatus _VI_FUNC rsvna_ConfigureAverage(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean averageState,
                                         ViInt32   averageFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CHANNEL_SWEEP_AVERAGE_STATE, averageState),
            3, "Average State");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CHANNEL_SWEEP_AVERAGE_FACTOR, averageFactor),
            4, "Average Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a new average cycle, clearing all previous
/// HIFN results and thus eliminating their effect on the new cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_RestartAverage(ViSession instrSession,
                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CHANNEL_SWEEP_AVERAGE_RESTART, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables dynamic IF bandwidth reduction at low frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR bwatLowFrequencies/Enables or disables dynamic IF bandwidth reduction at low frequencies.
ViStatus _VI_FUNC rsvna_ConfigureChannelDynamicBwatLowFrequencies(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean bwatLowFrequencies)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CHANNEL_DYNAMIC_BW_AT_LOW_FREQUENCIES,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CHANNEL_DYNAMIC_BW_AT_LOW_FREQUENCIES, bwatLowFrequencies),
            3, "BW at low Frequencies");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the analog IF signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR ifFilter/Selects the analog IF signal path.
ViStatus _VI_FUNC rsvna_ConfigureChannelAnalogIFFilter(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   ifFilter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CHANNEL_ANALOG_IF_FILTER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CHANNEL_ANALOG_IF_FILTER, ifFilter),
            3, "IF Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the total number of measurement points per sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR numberOfPoints/defines the total number of measurement points per sweep.
ViStatus _VI_FUNC rsvna_ConfigureSweepNumberOfPoints(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   numberOfPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_POINTS_COUNT, numberOfPoints),
            3, "Number Of Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the distance between two consecutive sweep points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR stepSize/This control defines the stimulus step size.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyStepSize(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_FREQUENCY_STEP_SIZE, stepSize),
            3, "Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the measurement time for a sweep or delay the start of each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR autoSweepTime/When enabled, the (minimum) sweep duration is calculated
/// HIPAR autoSweepTime/internally using the other channel settings and zero delay.
/// HIPAR sweepTime/Sets the duration of the sweep (Sweep Time).
ViStatus _VI_FUNC rsvna_ConfigureSweepTime(ViSession instrSession,
                                           ViInt32   channel,
                                           ViBoolean autoSweepTime,
                                           ViReal64  sweepTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_TIME_AUTO, autoSweepTime),
            3, "Auto Sweep Time");
    if (autoSweepTime == VI_FALSE)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_TIME, sweepTime),
                4, "Sweep Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the Meas. Delay time for each partial measurement. Setting
/// HIFN a delay disables the automatic calculation of the (minimum) sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR measDelay/Meas. delay sets a delay time allowing the DUT to settle before the hardware
/// HIPAR measDelay/settings of the analyzer are changed and a new partial measurement is started.
ViStatus _VI_FUNC rsvna_ConfigureSweepMeasDelay(ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  measDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_MEASUREMENT_DELAY, measDelay),
            3, "Meas Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether the measurement delay (previously
/// HIFN  defined via function rsvna_ConfigureSweepMeasDelay is inserted before
/// HIFN  all partial measurements or before the first partial measurement only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR insertionPoint/This control defines whether the measurement delay is inserted before
/// HIPAR insertionPoint/all partial measurements or before the first partial measurement only.
ViStatus _VI_FUNC rsvna_ConfigureSweepMeasDelayInsertionPoint(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   insertionPoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MEASUREMENT_DELAY_INSERTION_POINT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_MEASUREMENT_DELAY_INSERTION_POINT, insertionPoint),
            3, "Insertion Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the sweep type, i.e. the sweep variable(frequency/power/time)
/// HIFN and the position of the sweep points across the sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/Selects the sweep type, i.e. the sweep variable(frequency/power/time)
/// HIPAR sweepType/and the position of the sweep points across the sweep range.
ViStatus _VI_FUNC rsvna_ConfigureSweepType(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_TYPE, sweepType),
            3, "Sweep Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects either the channel base frequency/power or one of the port
/// HIFN frequencies/powers as the stimulus axis in all diagrams of the active recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR sweepAxisType/Selects sweep axis type.
/// HIPAR scale/This control selects the channel base power/frequency or one of
/// HIPAR scale/the port powers/frequencies as the stimulus axis in all diagrams
/// HIPAR scale/of the active recall set.
ViStatus _VI_FUNC rsvna_ConfigureSweepStimulusAxis(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   sweepAxisType,
                                                   ViString  scale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepAxisType, 0, 1),
            3, "Sweep Axis Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, sweepAxisTypeArr[sweepAxisType]);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_STIMULUS_AXIS, scale),
            3, "scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the acquisition attributes of the network analyzer.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Sets the timeout for the triggering routine to be finished.
/// HIPAR continuousSweepScope/This control selects the scope of the continuous sweep sequence.
/// HIPAR sweepModeContinuous/Turns on/off continuous sweeping.
/// HIPAR sweepCount/Defines the number of consecutive sweeps to be measured.
/// HIPAR singleSweepScope/This control selects the scope of the single sweep sequence.
ViStatus _VI_FUNC rsvna_ConfigureAcquisition(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   continuousSweepScope,
                                             ViBoolean sweepModeContinuous,
                                             ViInt32   sweepCount,
                                             ViInt32   singleSweepScope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, continuousSweepScope, 0, 1),
            3, "Continuous Sweep Scope");

    switch (continuousSweepScope)
    {
    case RSVNA_VAL_SWEEP_SINGLE_CHAN:
        viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MODE_CONTINUOUS, sweepModeContinuous),
                4, "sweepModeContinuous");
        break;
    case RSVNA_VAL_SWEEP_ALL_CHAN:
        viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_SWEEP_MODE_CONTINUOUS_ALL_CHANNEL, sweepModeContinuous),
                4, "sweepModeContinuous");
        break;
    }

    switch (singleSweepScope)
    {
    case RSVNA_VAL_SWEEP_SINGLE_CHAN:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_COUNT, sweepCount),
                5, "sweepCount");
        break;
    case RSVNA_VAL_SWEEP_ALL_CHAN:
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SWEEP_COUNT_ALL_CHANNELS, sweepCount),
                5, "sweepCount");
        break;
    }

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_SCOPE, singleSweepScope),
            6, "singleSweepScope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the sweep in all or single channel
/// HIFN  of the active recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR channelScope/This control selects the scope of the channel.
/// HIPAR enableSweep/Enables or disables the sweep in all or single channel of the active
/// HIPAR enableSweep/recall set.
ViStatus _VI_FUNC rsvna_ConfigureMeasurementInChannel(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   channelScope,
                                                      ViBoolean enableSweep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   channelScopeArr[] =
             {
             RSVNA_ATTR_SWEEP_IN_SINGLE_CHANNEL,
             RSVNA_ATTR_SWEEP_IN_ALL_CHANNELS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelScope, RSVNA_VAL_SWEEP_SINGLE_CHAN, RSVNA_VAL_SWEEP_ALL_CHAN),
            3, "Channel Scope");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, channelScopeArr[channelScope], enableSweep),
            4, "Enable Sweep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the firmware to automatically optimize the channel switching
/// HIFN  times for a particular channel sweep order. The instrument will
/// HIFN  optimize the switching times for the channel order that is
/// HIFN  automatically used with RSVNA_ATTR_SWEEP_START_ALL (increasing order of
/// HIFN  channel numbers).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_SweepOptimizeChannelSwitchingAuto(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_SWEEP_OPTIMIZE_CHANNEL_SWITCHING_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the firmware to manually optimize the channel switching times
/// HIFN  for the specified channel sweep order. To get the optimization to work,
/// HIFN  you have to sweep the channels manually, and in the specified order:
/// HIFN  INIT<Ch1>, INIT<Ch2>, ...
/// HIFN     Note that the optimization can only be applied, if all channels in the
/// HIFN  current setup are in single sweep mode
/// HIFN  (RSVNA_ATTR_SWEEP_MODE_CONTINUOUS OFF), and if there is no ongoing
/// HIFN  sweep sequence. Any change in the current setup or a wrong sweep
/// HIFN  sequence lets the analyzer fall back to unoptimized channel switching.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/The length of the Channel List array.
/// HIPAR channelList/The channel sweep order for optimization. All specified channels must
/// HIPAR channelList/be in single sweep mode, and you have to sweep the channels manually,
/// HIPAR channelList/and in the specified order, with RSVNA_ATTR_SWEEP_START.
ViStatus _VI_FUNC rsvna_SweepOptimizeChannelSwitchingManual(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32   channelList[])
{
    ViStatus error = VI_SUCCESS;
    ViString arrayParam = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Convert_ViInt32array_To_String(instrSession, channelList, arraySize, NULL, &arrayParam));

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_SWEEP_OPTIMIZE_CHANNEL_SWITCHING_MANUAL,
                    RS_VAL_DIRECT_USER_CALL, arrayParam),
                3, "Channel List");

Error:
    if (arrayParam)
        free(arrayParam);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function triggers all actions waiting for a trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_SendTrigger(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_INIT_SW_TRIGGER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function triggers all actions waiting for a trigger event in the selected
/// HIFN window and waits for operation completed (OPC) before returning the status code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR timeout/Document parameter here.
ViStatus _VI_FUNC rsvna_SendTriggerWaitOPC(ViSession instrSession,
                                           ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, "*TRG", timeout));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  In controlled timing mode (INITiate<Ch>:CTIMingON, this command allows
/// HIFN  you to set the wait time manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR waitTimeperPoint/In controlled timing mode (INITiate<Ch>:CTIMingON, this command allows
/// HIPAR waitTimeperPoint/you to set the wait time manually.
ViStatus _VI_FUNC rsvna_ConfigureSweepWaitTimeperPoint(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64  waitTimeperPoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_WAIT_TIME_PER_POINT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_WAIT_TIME_PER_POINT, waitTimeperPoint),
            3, "Wait Time per Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates/deactivates the controlled timing mode.
/// HIFN     Can only be executed if the current setup contains a single channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR modeControlledTiming/Activates/deactivates the controlled timing mode.
ViStatus _VI_FUNC rsvna_ConfigureSweepModeControlledTiming(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean modeControlledTiming)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MODE_CONTROLLED_TIMING,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MODE_CONTROLLED_TIMING, modeControlledTiming),
            3, "Mode Controlled Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables/disables user-defined wait times per measurement point for the
/// HIFN  controlled timing mode (INITiate<Ch>:CTIMingON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR timeMode/Enables/disables user-defined wait times per measurement point for the
/// HIPAR timeMode/controlled timing mode (INITiate<Ch>:CTIMingON.
ViStatus _VI_FUNC rsvna_ConfigureSweepTimeMode(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   timeMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_TIME_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_TIME_MODE, timeMode),
            3, "Time Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the power sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR fastPowerSweep/Selects the power sweep mode.
ViStatus _VI_FUNC rsvna_ConfigureSweepFastPowerSweep(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   fastPowerSweep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_FAST_POWER_SWEEP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_FAST_POWER_SWEEP, fastPowerSweep),
            3, "Fast Power Sweep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reverses the direction of the sweep in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR reverse/Reverses the direction of the sweep in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepReverse(ViSession instrSession,
                                              ViInt32   channel,
                                              ViBoolean reverse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_REVERSE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_REVERSE, reverse),
            3, "Reverse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the firmware calculates the 'hardware settling wait times'
/// HIFN  in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR timerMode/Defines how the firmware calculates the 'hardware settling wait times'
/// HIPAR timerMode/in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepTimerMode(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   timerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_TIMER_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_TIMER_MODE, timerMode),
            3, "Timer Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If set to ON (1), only the active channel is measured. All other
/// HIFN  channels are switched off temporarily, i.e. until
/// HIFN  RSVNA_ATTR_SWEEP_ONLY_ACTIVE_CHANNEL_ON is set to OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepOnlyActiveChannelOn/If set to ON (1), only the active channel is measured. All other
/// HIPAR sweepOnlyActiveChannelOn/channels are switched off temporarily, i.e. until
/// HIPAR sweepOnlyActiveChannelOn/RSVNA_ATTR_SWEEP_ONLY_ACTIVE_CHANNEL_ON is set to OFF.
ViStatus _VI_FUNC rsvna_ConfigureSweepOnlyActiveChannelOn(ViSession instrSession,
                                                          ViBoolean sweepOnlyActiveChannelOn)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_SWEEP_ONLY_ACTIVE_CHANNEL_ON, sweepOnlyActiveChannelOn),
            2, "Sweep Only Active Channel On");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the minimum wait time for the initialization of channel
/// HIFN  <Ch>, in case it is the first channel to be swept, or the same as the
/// HIFN  previously swept channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual wait times for the initialization of
/// HIPAR enabled/channel <Ch>, in case it is the first channel to be swept, or the same
/// HIPAR enabled/as the previously swept channel.
/// HIPAR minimumWaitTime/Sets/gets the minimum wait time for the initialization of channel
/// HIPAR minimumWaitTime/<Ch>, in case it is the first channel to be swept, or the same as the
/// HIPAR minimumWaitTime/previously swept channel.
ViStatus _VI_FUNC rsvna_ConfigureSweepMinimumWaitTime(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled,
                                                      ViReal64  minimumWaitTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME, minimumWaitTime),
            4, "Minimum Wait Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the wait time for the first point of each 2D gain
/// HIFN  compression (power) subsweep in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual wait times for the first point of each
/// HIPAR enabled/2D gain compression (power) subsweep in channel <Ch>
/// HIPAR fixedWaitTime/Sets/gets the wait time for the first point of each 2D gain
/// HIPAR fixedWaitTime/compression (power) subsweep in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepFixedWaitTime(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean enabled,
                                                    ViReal64  fixedWaitTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_FIXED_WAIT_TIME_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_FIXED_WAIT_TIME_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_FIXED_WAIT_TIME, fixedWaitTime),
            4, "Fixed Wait Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates manual wait time settings for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual wait time settings for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepManualWaitTimeEnabled(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MANUAL_WAIT_TIME_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MANUAL_WAIT_TIME_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the minimum wait time for the initialization of channel
/// HIFN  <Ch>, in case it is different from the previously swept one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual wait times for the initialization of
/// HIPAR enabled/channel <Ch>, in case it is different from the previously swept one.
/// HIPAR timeInitializationChannel/Sets/gets the minimum wait time for the initialization of channel
/// HIPAR timeInitializationChannel/<Ch>, in case it is different from the previously swept one.
ViStatus _VI_FUNC rsvna_ConfigureSweepMinimumWaitTimeInitializationChannel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled,
                            ViReal64  timeInitializationChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_INITIALIZATION_CHANNEL_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_INITIALIZATION_CHANNEL_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_INITIALIZATION_CHANNEL, timeInitializationChannel),
            4, "Time Initialization Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the maximum wait time for the first point of channel <Ch>,
/// HIFN  and the first point after a drive port change in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual maximum wait times for the first point
/// HIPAR enabled/of channel <Ch>, and the first point after a drive port change in
/// HIPAR enabled/channel <Ch>.
/// HIPAR maximumWaitTimePort/Sets/gets the maximum wait time for the first point of channel <Ch>,
/// HIPAR maximumWaitTimePort/and the first point after a drive port change in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepMaximumWaitTimePort(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean enabled,
                                                          ViReal64  maximumWaitTimePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MAXIMUM_WAIT_TIME_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MAXIMUM_WAIT_TIME_PORT_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_MAXIMUM_WAIT_TIME_PORT, maximumWaitTimePort),
            4, "MaximumWait Time Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the minimum wait time for the first point of channel <Ch>,
/// HIFN  and the first point after a drive port change in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual minimum wait times for the first point
/// HIPAR enabled/of channel <Ch>, and the first point after a drive port change in
/// HIPAR enabled/channel <Ch>.
/// HIPAR minimumWaitTimePort/Sets/gets the minimum wait time for the first point of channel <Ch>,
/// HIPAR minimumWaitTimePort/and the first point after a drive port change in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepMinimumWaitTimePort(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean enabled,
                                                          ViReal64  minimumWaitTimePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_PORT_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_MINIMUM_WAIT_TIME_PORT, minimumWaitTimePort),
            4, "Minimum Wait Time Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a new single sweep sequence.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Sets the timeout for the triggering routine to be finished.
/// HIPAR sweepScope/selects the scope of the sweep sequence.
ViStatus _VI_FUNC rsvna_RestartSweep(ViSession instrSession,
                                     ViInt32   channel,
                                     ViInt32   sweepScope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepScope, 0, 1),
            3, "Sweep Scope");

    switch (sweepScope)
    {
    case RSVNA_VAL_SWEEP_SINGLE_CHAN:
        checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_START, NULL));
        break;

    case RSVNA_VAL_SWEEP_ALL_CHAN:
        checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_SWEEP_START_ALL, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function inserts a new sweep segment with specific channel settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
/// HIPAR startFrequency/Defines the Start frequency.
/// HIPAR stopFrequency/Defines the Stop frequency.
/// HIPAR numberOfPoints/This control defines the total number of measurement Points in sweep segment no. <Segment>.
/// HIPAR power/This control defines the power of the internal signal source in segment.
/// HIPAR sweepTimeSelect/This control defines whether the sweep time of a new segment
/// HIPAR sweepTimeSelect/is entered as a segment sweep time or as a meas. delay.
/// HIPAR time/This control sets the duration of the sweep in sweep segment no. <Segment>.
/// HIPAR pointDelay/This control defines the delay time for each partial measurement in sweep segment no. <Segment>.
/// HIPAR measBandwidth/This control defines the resolution bandwidth of the analyzer (Meas. Bandwidth).
/// HIPAR localOscillatorFrequency/This control selects the position of the local oscillator frequency LO relative to the RF frequency (Spur Avoid).
/// HIPAR selectivity/Defines the Selectivity in the sweep segment.
/// HIPAR freqSweepMode/Defines the fequency sweep mode in the sweep segment.
ViStatus _VI_FUNC rsvna_InsertNewSegment(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   segment,
                                         ViReal64  startFrequency,
                                         ViReal64  stopFrequency,
                                         ViInt32   numberOfPoints,
                                         ViReal64  power,
                                         ViInt32   sweepTimeSelect,
                                         ViReal64  time,
                                         ViReal64  pointDelay,
                                         ViReal64  measBandwidth,
                                         ViInt32   localOscillatorFrequency,
                                         ViInt32   selectivity,
                                         ViInt32   freqSweepMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViString list[] = {"NEG", "POS", "AUTO",NULL};
    ViChar*  p2Selectivity;
    ViChar*  p2FreqSweepMode;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_SEGMENT_INSERT_NEW_SEGMENT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, localOscillatorFrequency, RSVNA_VAL_LOCAL_OSCILLATOR_NEGATIVE, RSVNA_VAL_LOCAL_OSCILLATOR_AUTO),
            12, "Local Oscillator Frequency");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSweepSegmentSelectivity, selectivity, &p2Selectivity),
                13, "Selectivity");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngFreqSweepMode, freqSweepMode, &p2FreqSweepMode),
                14, "Freq Sweep Mode");

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", segment);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", segment);

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_channel, repCap_segment);

    p2buf = parameter + snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%ld,%.12lG,",
                                 startFrequency,
                                 stopFrequency,
                                 numberOfPoints,
                                 power);

    switch (sweepTimeSelect)
    {
    case RSVNA_VAL_SEG_TIME:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, time, 0, 1000),
                9, "Time");

        p2buf += sprintf(p2buf, "%.12lG", time);

        break;
    case RSVNA_VAL_SEG_POINT:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, pointDelay, 0, 2.5e+3), 10, "Power");

        p2buf += sprintf(p2buf, "%.12lG", pointDelay);
        break;
    }

    if (RsCore_IsInstrumentModel(instrSession, "ZNA"))
    {
        sprintf(p2buf, ",0,%.12lG,%s,%s,AUTO", measBandwidth, list[localOscillatorFrequency], p2Selectivity);
    }
    else
    {
        sprintf(p2buf, ",0,%.12lG,%s,%s,AUTO,%s", measBandwidth, list[localOscillatorFrequency], p2Selectivity, p2FreqSweepMode);
    }

    checkErr(RsCore_AddRepCap(instrSession, "Segment", repCap_segment, repCap_cmd));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_INSERT_TIME_MODE, sweepTimeSelect),
            8, "Sweep Time Select");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_INSERT_NEW_SEGMENT, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function re-defines a sweep segment with specific channel
/// HIFN  settings (Insert New Segment). The segment replaces an existing segment
/// HIFN  <Segment> in the segment list. The modified segment must not overlap
/// HIFN  with any of the existing segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
/// HIPAR startFrequency/Defines the Start frequency.
/// HIPAR stopFrequency/Defines the Stop frequency.
/// HIPAR numberOfPoints/This control defines the total number of measurement Points in sweep segment no. <Segment>.
/// HIPAR power/This control defines the power of the internal signal source in segment.
/// HIPAR sweepTimeSelect/This control defines whether the sweep time of a new segment
/// HIPAR sweepTimeSelect/is entered as a segment sweep time or as a meas. delay.
/// HIPAR time/This control sets the duration of the sweep in sweep segment no. <Segment>.
/// HIPAR pointDelay/This control defines the delay time for each partial measurement in sweep segment no. <Segment>.
/// HIPAR measBandwidth/This control defines the resolution bandwidth of the analyzer (Meas. Bandwidth).
/// HIPAR localOscillatorFrequency/This control selects the position of the local oscillator frequency LO relative to the RF frequency (Spur Avoid).
/// HIPAR selectivity/Defines the Selectivity in the sweep segment.
/// HIPAR freqSweepMode/Defines the fequency sweep mode in the sweep segment.
ViStatus _VI_FUNC rsvna_RedefineSegment(ViSession instrSession,
                                        ViInt32   channel,
                                        ViInt32   segment,
                                        ViReal64  startFrequency,
                                        ViReal64  stopFrequency,
                                        ViInt32   numberOfPoints,
                                        ViReal64  power,
                                        ViInt32   sweepTimeSelect,
                                        ViReal64  time,
                                        ViReal64  pointDelay,
                                        ViReal64  measBandwidth,
                                        ViInt32   localOscillatorFrequency,
                                        ViInt32   selectivity,
                                        ViInt32   freqSweepMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViString list[] = {"POS", "NEG", "AUTO",NULL};
    ViChar*  p2Selectivity;
    ViChar*  p2FreqSweepMode;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_SEGMENT_REDEFINE_SEGMENT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_channel),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, localOscillatorFrequency, RSVNA_VAL_LOCAL_OSCILLATOR_NEGATIVE, RSVNA_VAL_LOCAL_OSCILLATOR_AUTO),
            12, "Local Oscillator Frequency");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSweepSegmentSelectivity, selectivity, &p2Selectivity),
                13, "Selectivity");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngFreqSweepMode, freqSweepMode, &p2FreqSweepMode),
                14, "Freq Sweep Mode");

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", segment);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", segment);

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_channel, repCap_segment);

    p2buf = parameter + snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%ld,%.12lG,",
                                 startFrequency,
                                 stopFrequency,
                                 numberOfPoints,
                                 power);

    switch (sweepTimeSelect)
    {
    case RSVNA_VAL_SEG_TIME:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, time, 0, 1000),
                9, "Time");

        p2buf += sprintf(p2buf, "%.12lG", time);

        break;
    case RSVNA_VAL_SEG_POINT:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, pointDelay, 0, 2.5e+3), 10, "Power");

        p2buf += sprintf(p2buf, "%.12lG", pointDelay);
        break;
    }

    if (RsCore_IsInstrumentModel(instrSession, "ZNA"))
    {
        sprintf(p2buf, ",0,%.12lG,%s,%s,AUTO", measBandwidth, list[localOscillatorFrequency], p2Selectivity);
    }
    else
    {
        sprintf(p2buf, ",0,%.12lG,%s,%s,AUTO,%s", measBandwidth, list[localOscillatorFrequency], p2Selectivity, p2FreqSweepMode);
    }

    checkErr(RsCore_AddRepCap(instrSession, "Segment", repCap_segment, repCap_cmd));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_REDEFINE_TIME_MODE, sweepTimeSelect),
            8, "Sweep Time Select");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_REDEFINE_SEGMENT, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function inserts a new sweep segment using default channel settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
ViStatus _VI_FUNC rsvna_AddNewSegment(ViSession instrSession,
                                      ViInt32   channel,
                                      ViInt32   segment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", segment);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", segment);

    checkErr(RsCore_AddRepCap(instrSession, "Segment", repCap_segment, repCap_cmd));

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_segment);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_ADD, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of sweep segments in the channel including
/// HIFN all segments that are switched off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR numberOfSweepSegments/This control returns of sweep segments in the channel including
/// HIFN  all segments that are switched off.
ViStatus _VI_FUNC rsvna_QuerySweepSegmentNumber(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32*  numberOfSweepSegments)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUERY_SWEEP_SEGMENT_COUNT, numberOfSweepSegments),
            3, "Number Of Sweep Segments");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes the specified (single) sweep segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
ViStatus _VI_FUNC rsvna_DeleteSelectedSegment(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   segment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", segment);

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_segment);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_DELETE, NULL));

    checkErr(RsCore_RemoveRepCap(instrSession, "Segment", repCap_segment));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes all sweep segments in the channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteAllSegments(ViSession instrSession,
                                          ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_DELETE_ALL, NULL));

    checkErr(RsCore_ResetRepCapID(instrSession, "Segment", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates or deactivates the sweep segment <Segment>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
/// HIPAR state/activates or deactivates the sweep segment <Segment>.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentState(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   segment,
                                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", segment);

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_segment);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects either frequency based or point based x-axis for segmented sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR XAxis/Selects either frequency based or point based x-axis for segmented sweeps.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentXAxis(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   XAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_X_AXIS, XAxis),
            4, "X-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function saves the sweep segment definition of a specified channel to a sweep segment file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR fileName/Specifies the name and directory of the created sweep segment file.
ViStatus _VI_FUNC rsvna_SaveSegment(ViSession instrSession,
                                    ViInt32   channel,
                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:SEGM %ld,'%s'", channel, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN loads a sweep segment definition from a specified file and assigns it to a specified channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR fileName/Specifies the name and directory of the sweep segment file to be loaded.
ViStatus _VI_FUNC rsvna_LoadSegment(ViSession instrSession,
                                    ViInt32   channel,
                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:SEGM %ld,'%s'", channel, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables a port-specific power level (base power) for port <PortId> in
/// HIFN  segment <Seg> of channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Sweep segment number.
/// HIPAR port/Port number.
/// HIPAR enabled/Enables a port-specific power level (base power) for port <PortId> in
/// HIPAR enabled/segment <Seg> of channel <Ch>.
/// HIPAR powerLevel/Defines a port-specific power level for port <PortId> in sweep segment
/// HIPAR powerLevel/<Seg> of channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentPortLevel(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   segment,
                                                       ViInt32   port,
                                                       ViBoolean enabled,
                                                       ViReal64  powerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, segment, 1), 3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, 1, 24),
            4, "Port");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld,Pt%ld", channel, segment, port);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_PORT_LEVEL_ENABLED, enabled),
            5, "Enabled");
    if (enabled)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_PORT_POWER_LEVEL, powerLevel),
                6, "Power Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets/gets the wait time for the first point of each segmented
/// HIFN  frequency sweep in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates manual wait times for the first point of each
/// HIPAR enabled/segmented frequency sweep in channel <Ch>.
/// HIPAR waitTimeFixed/Sets/gets the wait time for the first point of each segmented
/// HIPAR waitTimeFixed/frequency sweep in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentWaitTimeFixed(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean enabled,
                                                           ViReal64  waitTimeFixed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED, waitTimeFixed),
            4, "Wait Time Fixed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets a the maximum wait time per measurement point in channel
/// HIFN  <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR waitTimeFixedMaximum/Sets/gets a the maximum wait time per measurement point in channel
/// HIPAR waitTimeFixedMaximum/<Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentWaitTimeFixedMaximum(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  waitTimeFixedMaximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_MAXIMUM,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_MAXIMUM, waitTimeFixedMaximum),
            3, "Wait Time Fixed Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets a the minimum wait time per measurement point in channel
/// HIFN  <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR waitTimeFixedMinimum/Sets/gets a the minimum wait time per measurement point in channel
/// HIPAR waitTimeFixedMinimum/<Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepSegmentWaitTimeFixedMinimum(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  waitTimeFixedMinimum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_MINIMUM,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SWEEP_SEGMENT_WAIT_TIME_FIXED_MINIMUM, waitTimeFixedMinimum),
            3, "Wait Time Fixed Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates the sweep timing info box for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates the sweep timing info box for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureSweepTimingInfoBoxEnabled(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_TIMING_INFO_BOX_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SWEEP_TIMING_INFO_BOX_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets/gets the sweep mode for linear frequency sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR mode/Sets/gets the sweep mode for linear frequency sweeps.
ViStatus _VI_FUNC rsvna_ConfigureSweepGenerationMode(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SWEEP_GENERATION_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_GENERATION_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a one-port, two-port, three-port or four-port
/// HIFN  calibration type at arbitrary analyzer ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationName/This control defines the name of the calibration (string parameter).
/// HIPAR calibrationName/The name serves as a reference to delete a particular set of system
/// HIPAR calibrationName/correction data.
/// HIPAR parameters/This control selects a calibration type.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
/// HIPAR port3/Port number.
/// HIPAR port4/Port number.
ViStatus _VI_FUNC rsvna_SelectCalibrationType(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  calibrationName,
                                              ViInt32   parameters,
                                              ViInt32   port1,
                                              ViInt32   port2,
                                              ViInt32   port3,
                                              ViInt32   port4)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameters, 2, 34),
            4, "Parameters");

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:METH:DEF '%s',%s,%ld", channel,
                           calibrationName, calTypeArr[parameters], port1);

    if (port2)
    {
        p2buf += sprintf(p2buf, ",%ld", port2);
    }
    if (port3)
    {
        p2buf += sprintf(p2buf, ",%ld", port3);
    }
    if (port4)
    {
        sprintf(p2buf, ",%ld", port4);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a one-port or two-port calibration type for channel <Ch> at
/// HIFN  ports 1/2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationType/Selects a one-port or two-port calibration type for channel <Ch> at
/// HIPAR calibrationType/ports 1/2. Use the generalized command rsvna_SelectCalibrationType to
/// HIPAR calibrationType/select the calibration type for arbitrary analyzer ports or a multiport
/// HIPAR calibrationType/calibration type.
ViStatus _VI_FUNC rsvna_SelectCalibrationTypeSimple(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   calibrationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SELECT_CALIBRATION_TYPE_SIMPLE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SELECT_CALIBRATION_TYPE_SIMPLE, calibrationType),
            3, "Calibration Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a one-port, two-port, three-port or four-port
/// HIFN  calibration type at arbitrary analyzer ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationName/This control defines the name of the calibration (string parameter).
/// HIPAR calibrationName/The name serves as a reference to delete a particular set of system
/// HIPAR calibrationName/correction data.
/// HIPAR parameters/This control selects a calibration type.
/// HIPAR ports/Port numbers. The number of elements should correspond with Array Size
/// HIPAR ports/number.
/// HIPAR arraySize/Sets the number of elements of the Ports array.
/// HIPAR adapterCalKit/This control sets a adapter cal kit file name, it will be overwritten
ViStatus _VI_FUNC rsvna_SelectCalibrationTypeExtended(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViString  calibrationName,
                                                      ViInt32   parameters,
                                                      ViInt32   ports[],
                                                      ViInt32   arraySize,
                                                      ViString  adapterCalKit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameters, 2, 34),
            4, "Parameters");

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:METH:DEF '%s',%s", channel,
                           calibrationName, calTypeArr[parameters]);

    for (i = 0; i < arraySize; i++)
    {
        if ((parameters == RSVNA_VAL_CALTYPE_ART || parameters == RSVNA_VAL_CALTYPE_ARPT) &&
            i == 2 && adapterCalKit != NULL)
            p2buf += sprintf(p2buf, ",'%s'", adapterCalKit);
        else
            p2buf += sprintf(p2buf, ",%ld", ports[i]);
    }

    sprintf(p2buf, "\n");

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the detector type during the specified manual calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationName/Name of a manual calibration previously created e.g. using
/// HIPAR calibrationName/[SENSe<Ch>:]CORRection:COLLect:METHod:DEFine.
/// HIPAR detectorType/Selects the detector type.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationDetectorType(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViString  calibrationName,
                                                         ViInt32   detectorType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViPChar  p2type;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngCalDetectorType, detectorType, &p2type),
                4, "Detector Type");

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s',%s", calibrationName, p2type);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_DETECTOR_TYPE,
                    RS_VAL_DIRECT_USER_CALL, param),
                4, "Detector Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a calibration measurement in order to acquire
/// HIFN  measurement data for the selected standards. The
/// HIFN  standards are reflection or transmission standards and can be
/// HIFN  connected to arbitrary analyzer ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR standard/This control selects a standard from the selected calibration kit.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
ViStatus _VI_FUNC rsvna_StartCalibration(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   standard,
                                         ViInt32   port1,
                                         ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationGeneralStandard, standard, &p2standard),
                3, "Standard");

    switch (standard)
    {
    case RSVNA_VAL_CALCOLLSTDGEN_OPEN:
    case RSVNA_VAL_CALCOLLSTDGEN_SHORT:
    case RSVNA_VAL_CALCOLLSTDGEN_MATCH:
    case RSVNA_VAL_CALCOLLSTDGEN_REFL:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH_2:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH_3:
    case RSVNA_VAL_CALCOLLSTDGEN_SLID:
    case RSVNA_VAL_CALCOLLSTDGEN_POWER:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SEL %s,%ld", channel, p2standard, port1);
        break;
    default:
        if (port2 <= 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SEL %s,%ld", channel, p2standard, port1);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SEL %s,%ld,%ld", channel, p2standard, port1, port2);
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a calibration measurement in order to acquire
/// HIFN  measurement data for the selected standards. The
/// HIFN  standards are reflection or transmission standards and can be
/// HIFN  connected to arbitrary analyzer ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR standard/This control selects a standard from the selected calibration kit.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
/// HIPAR dispersion/This control selects if unknown through standard is non-dispersive or
/// HIPAR dispersion/dispersive.
/// HIPAR delayPhase/This controle selects automatic or manual settings of delay time and
/// HIPAR delayPhase/phase.
/// HIPAR delayPhaseValue/This control sets the delay time or phase, depending on the Dispersion
/// HIPAR delayPhaseValue/settings.
ViStatus _VI_FUNC rsvna_StartCalibrationWithOptions(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   standard,
                                                    ViInt32   port1,
                                                    ViInt32   port2,
                                                    ViBoolean dispersion,
                                                    ViInt32   delayPhase,
                                                    ViReal64  delayPhaseValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (standard != RSVNA_VAL_CALCOLLSTDGEN_UTHR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 3, "Standard");
    }

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delayPhase, RSVNA_VAL_UTHR_AUTO, RSVNA_VAL_UTHR_MAN),
            7, "Delay Phase");

    if (delayPhase == RSVNA_VAL_UTHR_AUTO)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SEL %s,%ld,%ld,%s,AUTO",
                 channel, calibrationStandardArr[standard],
                 port1, port2, offOnArr[dispersion]);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SEL %s,%ld,%ld,%s,%lf",
                channel, calibrationStandardArr[standard], port1, port2, offOnArr[dispersion], delayPhaseValue);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the system error correction for
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR correctionState/This control enables (ON) or disables (OFF) the correction
ViStatus _VI_FUNC rsvna_ConfigureSystemErrorCorrectionState(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViBoolean correctionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SYSTEM_ERROR_CORRECTION_STATE, correctionState),
            3, "Correction State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reloads a set of previously acquired calibration data
/// HIFN  for a particular standard from a file in the cal pool. The loaded data
/// HIFN  may be combined with new calibration measurement data (function
/// HIFN  rsvna_StartCalibration) in order to simplify and speed up the new
/// HIFN  calibration procedure. The channel settings for loaded and new
/// HIFN  calibrations (e.g. the number of sweep points) must be identical.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calPoolFile/
/// HIPAR standard/This control selects a standard from the selected calibration kit.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
ViStatus _VI_FUNC rsvna_LoadCalibrationDataFromCalGroupFile(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViString  calPoolFile,
                                                            ViInt32   standard,
                                                            ViInt32   port1,
                                                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationGeneralStandard, standard, &p2standard),
                3, "Standard");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, port1, 1),
                4, "Port 1");

    switch (standard)
    {
    case RSVNA_VAL_CALCOLLSTDGEN_THR:
    case RSVNA_VAL_CALCOLLSTDGEN_NET:
    case RSVNA_VAL_CALCOLLSTDGEN_ATT:
    case RSVNA_VAL_CALCOLLSTDGEN_LINE:
    case RSVNA_VAL_CALCOLLSTDGEN_LINE_2:
    case RSVNA_VAL_CALCOLLSTDGEN_LINE_3:
    case RSVNA_VAL_CALCOLLSTDGEN_ISOLATION:
    case RSVNA_VAL_CALCOLLSTDGEN_UTHROUGH:
    case RSVNA_VAL_CALCOLLSTDGEN_MIX:
    case RSVNA_VAL_CALCOLLSTDGEN_TERM:
    case RSVNA_VAL_CALCOLLSTDGEN_EATT:
    case RSVNA_VAL_CALCOLLSTDGEN_MDEL:
    default:
        if (port2 <= 0)
            snprintf(param, RS_PARAM_BUF_SIZE, "'%s',%s,%ld", calPoolFile, p2standard, port1);
        else
            snprintf(param, RS_PARAM_BUF_SIZE, "'%s',%s,%ld,%ld", calPoolFile, p2standard, port1, port2);
        break;

    case RSVNA_VAL_CALCOLLSTDGEN_OPEN:
    case RSVNA_VAL_CALCOLLSTDGEN_SHORT:
    case RSVNA_VAL_CALCOLLSTDGEN_MATCH:
    case RSVNA_VAL_CALCOLLSTDGEN_REFL:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH_2:
    case RSVNA_VAL_CALCOLLSTDGEN_OSH_3:
    case RSVNA_VAL_CALCOLLSTDGEN_SLID:
    case RSVNA_VAL_CALCOLLSTDGEN_POWER:
        snprintf(param, RS_PARAM_BUF_SIZE, "'%s',%s,%ld", calPoolFile, p2standard, port1);
        break;
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, repCap,
        RSVNA_ATTR_HIDDEN_LOAD_CALIBRATION_DATA_FROM_CAL_GROUP_FILE,
        RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activate power calibrations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR activate/This control selects a calibration state.
ViStatus _VI_FUNC rsvna_ActivatePowerCalibrations(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   activate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_ACTIVATE_POWER_CALIBRATION, activate),
            2, "Activate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function provides operations which allows to store system error
/// HIFN  correction data to the cal pool and to assign stored correction data to
/// HIFN  channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR operationToBePerformed/This control selects the type of operation to be performed.
/// HIPAR fileName/This control specifies the file name for the file operation to be
/// HIPAR fileName/performed. Cal group files must have the extension *.cal. The file
/// HIPAR fileName/extensions must be specified as part of the string parameters. In
/// HIPAR fileName/contrast the directory path must not be specified; the analyzer always
/// HIPAR fileName/uses the default path C:/Documents and Settings/All
/// HIPAR fileName/Users/Documents/Rohde-Schwarz/Vna/CALibration/DATA.
/// HIPAR secondFileName/This control is used for merge operation only
ViStatus _VI_FUNC rsvna_CalibrationManager(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   operationToBePerformed,
                                           ViString  fileName,
                                           ViString  secondFileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operationToBePerformed, RSVNA_VAL_CORR_MAN_COPY, RSVNA_VAL_CORR_MAN_DELETE),
            3, "Operation To Be Performed");

    switch (operationToBePerformed)
    {
    case RSVNA_VAL_CORR_MAN_COPY:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:CORR %ld,'%s'", channel, fileName);
        break;
    case RSVNA_VAL_CORR_MAN_APPLY:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CORR %ld,'%s'", channel, fileName);
        break;
    case RSVNA_VAL_CORR_MAN_RESOLVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CORR:RES %ld,'%s'", channel, fileName);
        break;
    case RSVNA_VAL_CORR_MAN_MERGE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CORR:MERG %ld,'%s','%s'", channel, fileName, secondFileName);
        break;
    case RSVNA_VAL_CORR_MAN_DELETE:
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DELETE_CALIBRATION_GROUP_FILE, fileName),
                4, "File Name");
        break;
    }

    if (operationToBePerformed != RSVNA_VAL_CORR_MAN_DELETE)
    {
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsvna_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects and initiates a one-port, two-port, three-port
/// HIFN  or four-port automatic calibration at arbitrary analyzer and
/// HIFN  calibration unit ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR calibrationKitName/automatic calibration:
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR calUnitPort1/Port number of the cal unit.
/// HIPAR calUnitPort2/Port number of the cal unit.
/// HIPAR calUnitPort3/Port number of the cal unit.
/// HIPAR calUnitPort4/Port number of the cal unit.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_CalibrationAuto(ViSession instrSession,
                                        ViInt32   channel,
                                        ViString  calibrationKitName,
                                        ViInt32   analyzerPort1,
                                        ViInt32   analyzerPort2,
                                        ViInt32   analyzerPort3,
                                        ViInt32   analyzerPort4,
                                        ViInt32   calUnitPort1,
                                        ViInt32   calUnitPort2,
                                        ViInt32   calUnitPort3,
                                        ViInt32   calUnitPort4,
                                        ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:PORT '%s',%ld,%ld",
                           channel,
                           calibrationKitName,
                           analyzerPort1,
                           calUnitPort1);

    if (analyzerPort2 && calUnitPort2)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort2, calUnitPort2);
    }

    if (analyzerPort3 && calUnitPort3)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort3, calUnitPort3);
    }

    if (analyzerPort4 && calUnitPort4)
    {
        sprintf(p2buf, ",%ld,%ld", analyzerPort4, calUnitPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the power calibration label of the active trace in channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR bufferSize/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR state/Returns the state of the ports.
ViStatus _VI_FUNC rsvna_QueryCalibrationPortsState(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   bufferSize,
                                                   ViChar    state[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:PST?", channel);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, state));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects and initiates a one-port, two-port, three-port
/// HIFN  or four-port automatic calibration at arbitrary analyzer ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR calibrationKitName/automatic calibration:
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_CalibrationAutoSimplified(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  calibrationKitName,
                                                  ViInt32   analyzerPort1,
                                                  ViInt32   analyzerPort2,
                                                  ViInt32   analyzerPort3,
                                                  ViInt32   analyzerPort4,
                                                  ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO '%s',%ld",
                           channel,
                           calibrationKitName,
                           analyzerPort1);

    if (analyzerPort2)
    {
        p2buf += sprintf(p2buf, ",%ld", analyzerPort2);
    }

    if (analyzerPort3)
    {
        p2buf += sprintf(p2buf, ",%ld", analyzerPort3);
    }

    if (analyzerPort4)
    {
        sprintf(p2buf, ",%ld", analyzerPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects and initiates a one-port, two-port, three-port
/// HIFN  or four-port automatic calibration at arbitrary analyzer and
/// HIFN  calibration unit ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR parameters/This control selects a calibration type.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR calibrationKitName/automatic calibration:
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR calUnitPort1/Port number of the cal unit.
/// HIPAR calUnitPort2/Port number of the cal unit.
/// HIPAR calUnitPort3/Port number of the cal unit.
/// HIPAR calUnitPort4/Port number of the cal unit.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_CalibrationAutoType(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   parameters,
                                            ViString  calibrationKitName,
                                            ViInt32   analyzerPort1,
                                            ViInt32   analyzerPort2,
                                            ViInt32   analyzerPort3,
                                            ViInt32   analyzerPort4,
                                            ViInt32   calUnitPort1,
                                            ViInt32   calUnitPort2,
                                            ViInt32   calUnitPort3,
                                            ViInt32   calUnitPort4,
                                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameters, 0, 34),
            3, "Parameters");
    if (strlen(calAutoTypeArr[parameters]) == 0)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameters),
            3, "Parameters");
    }

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:PORT:TYPE %s,'%s',%ld,%ld",
                           channel,
                           calAutoTypeArr[parameters],
                           calibrationKitName,
                           analyzerPort1,
                           calUnitPort1);

    if (analyzerPort2 && calUnitPort2)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort2, calUnitPort2);
    }

    if (analyzerPort3 && calUnitPort3)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort3, calUnitPort3);
    }

    if (analyzerPort4 && calUnitPort4)
    {
        sprintf(p2buf, ",%ld,%ld", analyzerPort4, calUnitPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Selects and initiates a one-port, two-port, three-port
/// HIFN  or four-port automatic calibration at arbitrary analyzer and cal unit
/// HIFN  ports. This command also selects the calibration type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR parameters/This control selects a calibration type.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR calibrationKitName/automatic calibration:
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_CalibrationAutoTypeSimplified(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   parameters,
                                                      ViString  calibrationKitName,
                                                      ViInt32   analyzerPort1,
                                                      ViInt32   analyzerPort2,
                                                      ViInt32   analyzerPort3,
                                                      ViInt32   analyzerPort4,
                                                      ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameters, 0, 34),
            3, "Parameters");
    if (strlen(calAutoTypeArr[parameters]) == 0)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameters),
            3, "Parameters");
    }

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:TYPE %s,'%s',%ld",
                           channel,
                           calAutoTypeArr[parameters],
                           calibrationKitName,
                           analyzerPort1);

    if (analyzerPort2)
    {
        p2buf += sprintf(p2buf, ",%ld", analyzerPort2);
    }

    if (analyzerPort3)
    {
        p2buf += sprintf(p2buf, ",%ld", analyzerPort3);
    }

    if (analyzerPort4)
    {
        sprintf(p2buf, ",%ld", analyzerPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures an external generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generatorNumber/Generator number.
/// HIPAR generatorName/This control sets the name of the external generator.
/// HIPAR generatorType/This control sets the generator type.
/// HIPAR interfaceType/This control sets the interface type
/// HIPAR interfaceAddress/This control sets the interface address, depending on the interface type.
/// HIPAR fastSweepMode/This control enables or disables the fast sweep mode.
/// HIPAR referenceFrequency/This control sets the analyzer to internal or external reference frequency.
ViStatus _VI_FUNC rsvna_ConfigureExternalGenerator(ViSession instrSession,
                                                   ViInt32   generatorNumber,
                                                   ViString  generatorName,
                                                   ViString  generatorType,
                                                   ViString  interfaceType,
                                                   ViString  interfaceAddress,
                                                   ViBoolean fastSweepMode,
                                                   ViBoolean referenceFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:GEN%ld:DEF '%s','%s','%s','%s',%s,%s", generatorNumber, generatorName,
             generatorType, interfaceType,
             interfaceAddress, offOnArr[fastSweepMode],
             offOnArr[referenceFrequency]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This queries configures an external generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generatorNumber/Generator number.
/// HIPAR generatorName/This control returns the name of the external generator.
/// HIPAR generatorType/This control returns the generator type.
/// HIPAR interfaceType/This control returns the interface type
/// HIPAR interfaceAddress/This control returns the interface address, depending on the interface type.
/// HIPAR fastSweepMode/This control returns the fast sweep mode.
/// HIPAR referenceFrequency/This control returns the analyzer to internal (OFF) or external
/// HIPAR referenceFrequency/(ON) reference frequency.
ViStatus _VI_FUNC rsvna_QueryExternalGenerator(ViSession  instrSession,
                                               ViInt32    generatorNumber,
                                               ViChar     generatorName[],
                                               ViChar     generatorType[],
                                               ViChar     interfaceType[],
                                               ViChar     interfaceAddress[],
                                               ViBoolean* fastSweepMode,
                                               ViBoolean* referenceFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  buffer;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:GEN%ld:DEF?", generatorNumber);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &buffer));

    p2buf = strtok(buffer, ",");
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", generatorName);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", generatorType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceAddress);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        *fastSweepMode = (ViBoolean)RsCore_FindStringIndex(offOnArr, p2buf);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        *referenceFrequency = (ViBoolean)RsCore_FindStringIndex(offOnArr, p2buf);
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the external generator power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generatorNumber/Generator number
/// HIPAR mode/Specifies the external generator power control after the sweep end.
/// HIPAR power/This control specifies the external generator power value after the end of a sweep.
ViStatus _VI_FUNC rsvna_ConfigureExternalGeneratorPower(ViSession instrSession,
                                                        ViInt32   generatorNumber,
                                                        ViInt32   mode,
                                                        ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Gen%ld", generatorNumber);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_EXTERNAL_GENERATOR_POWER_MODE, mode),
            3, "Mode");

    if (mode == RSVNA_VAL_EXTERNAL_GENERATOR_MODE_USER)
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_EXTERNAL_GENERATOR_POWER, power),
                4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of configured external generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generatorCount/Number of generators.
ViStatus _VI_FUNC rsvna_QueryExternalGeneratorCount(ViSession instrSession,
                                                    ViInt32*  generatorCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_EXTERNAL_GENERATORS_COUNT, generatorCount),
            3, "Generator Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the numbers of all configured external generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Array size.
/// HIPAR generatorNumbers/Returns a comma-separated list of generator numbers.
ViStatus _VI_FUNC rsvna_QueryExternalGeneratorNumbers(ViSession instrSession,
                                                      ViInt32   arraySize,
                                                      ViChar    generatorNumbers[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_EXTERNAL_GENERATORS_NUMBERS,
            arraySize, generatorNumbers),
            4, "Generator Numbers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function clears the configuration table for external generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_DeleteExternalGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_EXTERNAL_GENERATOR_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an external generator on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generator/Selects repcap Generator.
/// HIPAR state/Turns an external generator on or off.
ViStatus _VI_FUNC rsvna_ConfigureExternalGeneratorState(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   generator,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_gen[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  genCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Generator", &genCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, generator, RSVNA_REPCAP_PORT_PT1, genCount),
            3, "Generator");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_EXTERNAL_GENERATOR_STATE,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap_gen),
            3, "Generator");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, repCap_gen);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_EXTERNAL_GENERATOR_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the external generator power is permanently on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generator/Selects repcap Generator.
/// HIPAR state/Defines whether the external generator power is permanently on.
ViStatus _VI_FUNC rsvna_ConfigurePermanentSignalGeneratorState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   generator,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_gen[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  genCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Generator", &genCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, generator, RSVNA_REPCAP_PORT_PT1, genCount),
            3, "Generator");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PERMANENT_SIGNAL_GENERATOR_STATE,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap_gen),
            3, "Generator");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, repCap_gen);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PERMANENT_SIGNAL_GENERATOR_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures an external power meter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeterNumber/Number of external power meter.
/// HIPAR powerMeterName/This control sets the name of the external power meter.
/// HIPAR powerMeterType/This control sets the power meter type.
/// HIPAR interfaceType/This control sets the interface type
/// HIPAR interfaceAddress/This control sets the interface address
ViStatus _VI_FUNC rsvna_ConfigureExternalPowerMeter(ViSession instrSession,
                                                    ViInt32   powerMeterNumber,
                                                    ViString  powerMeterName,
                                                    ViString  powerMeterType,
                                                    ViString  interfaceType,
                                                    ViString  interfaceAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:PMET%ld:DEF '%s','%s','%s','%s'", powerMeterNumber, powerMeterName,
             powerMeterType, interfaceType,
             interfaceAddress);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the switch matrix on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR switchMatrixState/This controls configures the state of the switch matrix on or off.
ViStatus _VI_FUNC rsvna_ConfigureSwitchMatrixState(ViSession instrSession,
                                                   ViBoolean switchMatrixState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_SWITCH_MATRIX_STATE, switchMatrixState),
            2, "Switch Matrix State");

    checkErr(RsCore_ResetRepCapID(instrSession, "Matrix", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a switch matrix and adds it to the list of
/// HIFN  available switch matrixes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR switchMatrixName/This control sets the name of the switch matrix. An empty string means
/// HIPAR switchMatrixName/that no particular name is assigned to the switch matrix.
/// HIPAR switchMatrixType/This control sets the switch matrix type. The switch matrix type is
/// HIPAR switchMatrixType/identical with the name of the switch matrix driver file (*.matrix)
/// HIPAR switchMatrixType/stored in the resources\\extdev subdirectory of the analyzer's program
/// HIPAR switchMatrixType/directory.
/// HIPAR interfaceType/This control sets the interface type, either "USB" or "LAN".
/// HIPAR interfaceAddress/This control sets the interface address, depending on the interface
/// HIPAR interfaceAddress/type. For the LAN interface this is the IP address of the switch
/// HIPAR interfaceAddress/matrix.
ViStatus _VI_FUNC rsvna_ConfigureSwitchMatrix(ViSession instrSession,
                                              ViInt32   switchMatrixNumber,
                                              ViString  switchMatrixName,
                                              ViString  switchMatrixType,
                                              ViString  interfaceType,
                                              ViString  interfaceAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap_matrix[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:DEF '%s','%s','%s','%s'", switchMatrixNumber, switchMatrixName,
             switchMatrixType, interfaceType,
             interfaceAddress);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap_matrix, RS_REPCAP_BUF_SIZE, "Matrix%ld", switchMatrixNumber);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", switchMatrixNumber);

    checkErr(RsCore_AddRepCap(instrSession, "Matrix", repCap_matrix, repCap_cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries a switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR switchMatrixName/This control returns the name of the switch matrix. An empty string
/// HIPAR switchMatrixName/means that no particular name is assigned to the switch matrix.
/// HIPAR switchMatrixType/This control returns the switch matrix type. The switch matrix type is
/// HIPAR switchMatrixType/identical with the name of the switch matrix driver file (*.matrix)
/// HIPAR switchMatrixType/stored in the resources/extdev subdirectory of the analyzer's program
/// HIPAR switchMatrixType/directory.
/// HIPAR interfaceType/This control returns the interface type, e.g. LAN.
/// HIPAR interfaceAddress/This control returns the interface address, depending on the interface
/// HIPAR interfaceAddress/type.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrix(ViSession instrSession,
                                          ViInt32   switchMatrixNumber,
                                          ViChar    switchMatrixName[],
                                          ViChar    switchMatrixType[],
                                          ViChar    interfaceType[],
                                          ViChar    interfaceAddress[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  response = NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:DEF?", switchMatrixNumber);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    p2buf = strtok(response, ",");
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", switchMatrixName);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", switchMatrixType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceAddress);
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (response)
        free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function removes all switch matrixes from the pool of external
/// HIFN  devices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_DeteleAllSwitchMatrixes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ResetRepCapID(instrSession, "Matrix", rsvna_RsCoreRepCapTable));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DELETE_SWITCH_MATRIX, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of configured switch matrixes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixCount/Queries the number of configured switch matrixes.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixCount(ViSession instrSession,
                                               ViInt32*  switchMatrixCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_SWITCH_MATRIX_COUNT, switchMatrixCount),
            2, "Switch Matrix Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the numbers of all configured switch matrixes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/This control defines size of buffer in 'Switch Matrix Number'
/// HIPAR bufferSize/argument.
/// HIPAR switchMatrixNumbers/Returns a comma-separated list of switch matrix numbers.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixNumbers(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    switchMatrixNumbers[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_SWITCH_MATRIX_NUMBERS,
            bufferSize, switchMatrixNumbers),
            3, "Switch Matrix Numbers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the count of switch matrix test ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR testPortsCount/Returns the count of switch matrix test ports.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixTestPortsCount(ViSession instrSession,
                                                        ViInt32*  testPortsCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_SWITCH_MATRIX_TEST_PORTS_COUNT, testPortsCount),
            2, "Test Ports Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects an operation to configure the switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects an operation to configure the switch matrix.
ViStatus _VI_FUNC rsvna_SwitchMatrixConfigurationOperation(ViSession instrSession,
                                                           ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[3] =
             {
             RSVNA_ATTR_SWITCH_MATRIX_START_CONFIGURATION,
             RSVNA_ATTR_SWITCH_MATRIX_ABORT_CONFIGURATION,
             RSVNA_ATTR_SWITCH_MATRIX_END_CONFIGURATION
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSVNA_VAL_SWITCH_MATRIX_START, RSVNA_VAL_SWITCH_MATRIX_END),
            2, "Operation");

    checkErr(rsvna_SetAttributeViString(instrSession, "", attr[operation], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which VNA port is connected to which matrix
/// HIFN  VNA port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR numberofPorts/This control defines number of ports to be configured with arrays
/// HIPAR numberofPorts/'Matrix VNA Ports' and 'VNA Ports'.
/// HIPAR matrixVNAPorts/This contol sets which matrix VNA ports are connected to which VNA
/// HIPAR matrixVNAPorts/ports.
/// HIPAR vnaPorts/This contol sets which matrix VNA ports are connected to which VNA
/// HIPAR vnaPorts/ports.
ViStatus _VI_FUNC rsvna_ConfigureVNAPortToMatrixVNAPort(ViSession instrSession,
                                                        ViInt32   switchMatrixNumber,
                                                        ViInt32   numberofPorts,
                                                        ViInt32   matrixVNAPorts[],
                                                        ViInt32   vnaPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:CONF:MVNA ", switchMatrixNumber);

    for (i = 0; i < numberofPorts; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", matrixVNAPorts[i], vnaPorts[i]);
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which VNA port is connected to which matrix
/// HIFN  VNA port addressed by their label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR numberofPorts/This control defines number of ports to be configured with arrays
/// HIPAR numberofPorts/'Matrix VNA Port Labels' and 'VNA Ports'.
/// HIPAR matrixVNAPortLabels/This contol sets which VNA port is connected to which matrix VNA port
/// HIPAR matrixVNAPortLabels/addressed by their label.
/// HIPAR vnaPorts/This contol sets which VNA port is connected to which matrix VNA port
/// HIPAR vnaPorts/addressed by their label.
ViStatus _VI_FUNC rsvna_ConfigureVNAPortToMatrixVNAPortLabel(ViSession instrSession,
                                                             ViInt32   switchMatrixNumber,
                                                             ViInt32   numberofPorts,
                                                             ViChar    matrixVNAPortLabels[],
                                                             ViInt32   vnaPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViChar*  p2label;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:CONF:MLVN ", switchMatrixNumber);

    p2label = strtok(matrixVNAPortLabels, ",");
    for (i = 0; i < numberofPorts; i++)
    {
        p2buf += sprintf(p2buf, "'%s',%ld,", p2label, vnaPorts[i]);
        p2label = strtok(NULL, ",");
    }
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which matrix test port will be assigned to
/// HIFN  which test port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR numberofPorts/This control defines number of ports to be configured with arrays
/// HIPAR numberofPorts/'Matrix Test Ports' and 'Test Ports'.
/// HIPAR matrixTestPorts/This contol sets which matrix test port will be assigned to which test
/// HIPAR matrixTestPorts/port.
/// HIPAR testPorts/This contol sets which matrix test port will be assigned to which test
/// HIPAR testPorts/port.
ViStatus _VI_FUNC rsvna_ConfigureMatrixVNAPortToTestPort(ViSession instrSession,
                                                         ViInt32   switchMatrixNumber,
                                                         ViInt32   numberofPorts,
                                                         ViInt32   matrixTestPorts[],
                                                         ViInt32   testPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:CONF:MTES ", switchMatrixNumber);

    for (i = 0; i < numberofPorts; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", matrixTestPorts[i], testPorts[i]);
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which matrix test port addressed by a label
/// HIFN  will be assigned to which test port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR numberofPorts/This control defines number of ports to be configured with arrays
/// HIPAR numberofPorts/'Matrix Test Port Lables' and 'Test Ports'.
/// HIPAR matrixTestPortLabels/This contol sets which matrix test port addressed by a label will be
/// HIPAR matrixTestPortLabels/assigned to which test port.
/// HIPAR testPorts/This contol sets which matrix test port addressed by a label will be
/// HIPAR testPorts/assigned to which test port.
ViStatus _VI_FUNC rsvna_ConfigureMatrixVNAPortToTestPortLabel(
                            ViSession instrSession,
                            ViInt32   switchMatrixNumber,
                            ViInt32   numberofPorts,
                            ViChar    matrixTestPortLabels[],
                            ViInt32   testPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViChar*  p2label;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:CONF:MLT ", switchMatrixNumber);

    p2label = strtok(matrixTestPortLabels, ",");
    for (i = 0; i < numberofPorts; i++)
    {
        p2buf += sprintf(p2buf, "'%s',%ld,", p2label, testPorts[i]);
        p2label = strtok(NULL, ",");
    }
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which VNA port will be connected directly to
/// HIFN  a test port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR switchMatrixNumber/This control sets the number of the configured switch matrix. Switch
/// HIPAR switchMatrixNumber/matrixes must be numbered in ascending order, starting with 1. If a
/// HIPAR switchMatrixNumber/number is re-used, the previous switch matrix configuration is
/// HIPAR switchMatrixNumber/overwritten.
/// HIPAR numberofPorts/This control defines number of ports to be configured with arrays
/// HIPAR numberofPorts/'Test Ports' and 'VNA Ports'.
/// HIPAR testPorts/This contol sets which VNA port will be connected directly to a test
/// HIPAR testPorts/port.
/// HIPAR vnaPorts/This contol sets which VNA port will be connected directly to a test
/// HIPAR vnaPorts/port.
ViStatus _VI_FUNC rsvna_ConfigureVNAPortToTestPort(ViSession instrSession,
                                                   ViInt32   switchMatrixNumber,
                                                   ViInt32   numberofPorts,
                                                   ViInt32   testPorts[],
                                                   ViInt32   vnaPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:SMAT%ld:CONF:TVNA ", switchMatrixNumber);

    for (i = 0; i < numberofPorts; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", testPorts[i], vnaPorts[i]);
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the switch matrix route selection to either a
/// HIFN  matrix preserving or a measurement optimizing algorithm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optimization/Set the switch matrix route selection to either a matrix preserving or
/// HIPAR optimization/a measurement optimizing algorithm.
ViStatus _VI_FUNC rsvna_ConfigureSwitchMatrixOptimization(ViSession instrSession,
                                                          ViInt32   optimization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SWITCH_MATRIX_OPTIMIZATION, optimization),
            2, "Optimization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and disables the power meter S parameter group correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeterNumber/sets the number of the configured power meter.
/// HIPAR state/enables and disables the power meter S parameter group correction.
ViStatus _VI_FUNC rsvna_ConfigureExternalPowerSParameterCorrection(
                            ViSession instrSession,
                            ViInt32   powerMeterNumber,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "PM%ld", powerMeterNumber);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_EXTERNAL_POWER_METER_S_PARAMETER_CORRECTION, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries an external power meter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeterNumber/Number of external power meter.
/// HIPAR powerMeterName/This control returns the name of the external power meter.
/// HIPAR powerMeterType/This control returns the power meter type.
/// HIPAR interfaceType/This control returns the interface type
/// HIPAR interfaceAddress/This control returns the interface address
ViStatus _VI_FUNC rsvna_QueryExternalPowerMeter(ViSession instrSession,
                                                ViInt32   powerMeterNumber,
                                                ViChar    powerMeterName[],
                                                ViChar    powerMeterType[],
                                                ViChar    interfaceType[],
                                                ViChar    interfaceAddress[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  response = NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:PMET%ld:DEF?", powerMeterNumber);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    p2buf = strtok(response, ",");
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", powerMeterName);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", powerMeterType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceType);
        p2buf = strtok(NULL, ",");
    }
    if (p2buf != NULL)
    {
        (void)sscanf(p2buf, "'%[^']", interfaceAddress);
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (response)
        free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the number of configured external power meters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR powerMeterCount/Number of power meters.
ViStatus _VI_FUNC rsvna_QueryExternalPowerMeterCount(ViSession instrSession,
                                                     ViInt32*  powerMeterCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_EXTERNAL_POWER_METER_COUNT, powerMeterCount),
            3, "Power Meter Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the numbers of all configured external power meters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR bufferSize/Buffer size.
/// HIPAR powerMeterNumbers/Returns a comma-separated list of power meter numbers.
ViStatus _VI_FUNC rsvna_QueryExternalPowerMeterNumbers(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    powerMeterNumbers[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_EXTERNAL_POWER_METER_NUMBERS,
            bufferSize, powerMeterNumbers),
            4, "Power Meter Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function clears the configuration table for external power meters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
ViStatus _VI_FUNC rsvna_DeleteExternalPowerMeter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_EXTERNAL_POWER_METER_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure Port Power Limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Enables or disables the limit for the source power at port no.
/// HIPAR enabled/<PhyPt>. The limit is defined using
/// HIPAR enabled/SOURce<Ch>:POWer<PhyPt>[:LEVel][:IMMediate]:LLIMit:VALue.
/// HIPAR portPowerLimit/Defines a limit for the source power at port no. <PhyPt>. The limit
/// HIPAR portPowerLimit/must be enabled explicitly; see example.
ViStatus _VI_FUNC rsvna_ConfigurePortPowerLimitEnabled(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   physicalPort,
                                                       ViBoolean enabled,
                                                       ViReal64  portPowerLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_POWER_LIMIT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_POWER_LIMIT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PORT_POWER_LIMIT_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PORT_POWER_LIMIT, portPowerLimit),
            5, "Port Power Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the External Generator Limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR enabled/Enables or disables the limit for the source power of an external
/// HIPAR enabled/generator no. <Gen>. The limit is defined using
/// HIPAR enabled/SOURce<Ch>:POWer<PhyPt>:GENerator<Gen>:LLIMit:VALue.
/// HIPAR externalGeneratorLimit/Defines a limit for the source power of an external generator no.
/// HIPAR externalGeneratorLimit/<Gen>. The limit must be enabled explicitly using
/// HIPAR externalGeneratorLimit/SOURce<Ch>:POWer<PhyPt>:GENerator<Gen>:LLIMit[:STATe].
ViStatus _VI_FUNC rsvna_ConfigureExternalGeneratorLimit(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   generatorNumber,
                                                        ViBoolean enabled,
                                                        ViReal64  externalGeneratorLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_EXTERNAL_GENERATOR_LIMIT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_EXTERNAL_GENERATOR_LIMIT_ENABLED,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_EXTERNAL_GENERATOR_LIMIT_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_EXTERNAL_GENERATOR_LIMIT, externalGeneratorLimit),
            5, "External Generator Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects an external power meter for the SMARTerCal and assigns it to an analyzer port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeterNumber/Number of external power meter.
/// HIPAR VNAPort/VNA port number.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationExternalPowerMeter(
                            ViSession instrSession,
                            ViInt32   powerMeterNumber,
                            ViInt32   VNAPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:COLL:PMET:ID %ld,%ld", powerMeterNumber, VNAPort);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function initiates a receiver power calibration at the analyzer port <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR analyzerPort/Port number of the analyzer.
ViStatus _VI_FUNC rsvna_CalibrationAutoPower(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   analyzerPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_AUTO_POWER, analyzerPort),
            4, "Analyzer Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a port assignment numbered <Assignment>.
/// HIFN It is particularly relevant for multiple port assignments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR assignment/Current number of the port assignment.
/// HIPAR arraySize/Defines size of arrays in 'Analyzer Ports' and 'Cal Unit Ports' arguments.
/// HIPAR testPorts/Defines test port numbers corresponding to cal unit ports.
/// HIPAR calUnitPorts/Defines cal unit port numbers corresponding to test ports.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoAssignment(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   assignment,
                                                           ViInt32   arraySize,
                                                           ViInt32   testPorts[],
                                                           ViInt32   calUnitPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:ASS%ld:DEF ", channel, assignment);

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", testPorts[i], calUnitPorts[i]);
    *p2buf = '\0';
    *--p2buf = '\n';

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Assign%ld", assignment);
    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", assignment);
    checkErr(RsCore_AddRepCap(instrSession, "Assignment", repCap, repCap_cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a port assignment numbered <Assignment>.
/// HIFN It is particularly relevant for multiple port assignments. It
/// HIFN automatically detects cal unit's ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR assignment/Current number of the port assignment.
/// HIPAR arraySize/Defines size of arrays in 'Analyzer Ports' and 'Cal Unit Ports' arguments.
/// HIPAR testPorts/Defines test port numbers corresponding to cal unit ports.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoAssignmentWithDetection(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   assignment,
                            ViInt32   arraySize,
                            ViInt32   testPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:ASS%ld:DEF:TPOR ", channel, assignment);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, testPorts, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Assign%ld", assignment);
    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", assignment);
    checkErr(RsCore_AddRepCap(instrSession, "Assignment", repCap, repCap_cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a default port assignment. It is
/// HIFN  particularly relevant for multiple port assignments. It
/// HIFN  automatically detects cal unit's ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR arraySize/Defines size of arrays in 'Test Ports' argument.
/// HIPAR testPorts/Defines test port numbers.
ViStatus _VI_FUNC rsvna_GenerateDefaultCalibrationAutoAssignmentWithDetection(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   arraySize,
                            ViInt32   testPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:ASS:DEF:TPOR:DEF ", channel);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, testPorts, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a calibration type and a cal unit
/// HIFN characterization (cal kit file) for an automatic calibration
/// HIFN with multiple port assignments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationType/This control selects a calibration type.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the automatic calibration
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoAssignmentType(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   calibrationType,
                            ViString  calibrationKitName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, calibrationType, 0, 34),
            3, "Calibration Type");
    if (strlen(calAutoAssignmentArr[calibrationType]) == 0)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, calibrationType),
            3, "Calibration Type");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:CONF %s,'%s'", channel,
             calAutoAssignmentArr[calibrationType], calibrationKitName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Calculates the error terms, based on the previously acquired
/// HIFN calibration sweeps for multiple port assignments, saves the data
/// HIFN and applies the calibration to the active channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_SaveCalibrationAutoAssignment(ViSession instrSession,
                                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SAVE_CALIBRATION_AUTO_ASSIGNMENT, NULL),
            1, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Initiates an automatic calibration for a previously defined port assignment numbered <Assignment>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR assignment/Current number of the port assignment.
ViStatus _VI_FUNC rsvna_InitiateCalibrationAutoAssignment(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   assignment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Assign%ld", channel, assignment);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_INITIATE_CALIBRATION_AUTO_ASSIGNMENT, NULL),
            1, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Deletes all port assignments
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteAllCalibrationAutoAssignments(ViSession instrSession,
                                                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(RsCore_ResetRepCapID(instrSession, "Assignment", rsvna_RsCoreRepCapTable));

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_ALL_CALIBRATION_AUTO_ASSIGNMENTS, NULL),
            1, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Generates a set of default system error correction data for the
/// HIFN selected auto assignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR arraySize/Defines size of arrays in 'Test Ports' argument.
/// HIPAR testPorts/Defines test port numbers.
ViStatus _VI_FUNC rsvna_GenerateDefaultCalibrationAutoAssignment(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   arraySize,
                            ViInt32   testPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:AUTO:ASS:DEF:DEF ", channel);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, testPorts, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries number of configured assignments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR count/Returns number of configured assignments.
ViStatus _VI_FUNC rsvna_QueryCalibrationAutoAssignmentCount(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_AUTO_ASSIGNMENT_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the total number of port assignments of all calibrations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR count/Returns number of all configured assignments.
ViStatus _VI_FUNC rsvna_QueryAllCalibrationAutoAssignmentCount(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_AUTO_ASSIGNMENT_ALL_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the assignment between the network analyzer
/// HIFN  ports and the ports of the connected automatic calibration unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR analyzerPort1/Returns the port number of the cal unit that is connected to analyzer
/// HIPAR analyzerPort1/port number 1.
/// HIPAR analyzerPort2/Returns the port number of the cal unit that is connected to analyzer
/// HIPAR analyzerPort2/port number 2.
/// HIPAR analyzerPort3/Returns the port number of the cal unit that is connected to analyzer
/// HIPAR analyzerPort3/port number 3.
/// HIPAR analyzerPort4/Returns the port number of the cal unit that is connected to analyzer
/// HIPAR analyzerPort4/port number 4.
ViStatus _VI_FUNC rsvna_QueryCalibrationConnection(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32*  analyzerPort1,
                                                   ViInt32*  analyzerPort2,
                                                   ViInt32*  analyzerPort3,
                                                   ViInt32*  analyzerPort4)
{
    ViStatus error = VI_SUCCESS;
             channel++;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViInt32(instrSession, "SENS:CORR:COLL:AUTO:PORT:CONN?", analyzerPort1, analyzerPort2, analyzerPort3, analyzerPort4));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Toggles the Multiple Calibration TYpes mode that allows you to
/// HIFN  calibrate a subset of the available channels using channel-specific
/// HIFN  ports and calibration types (see 'Parallel Calibration of Multiple
/// HIFN  Channels').
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Toggles the Multiple Calibration TYpes mode that allows you to
/// HIPAR type/calibrate a subset of the available channels using channel-specific
/// HIPAR type/ports and calibration types (see 'Parallel Calibration of Multiple
/// HIPAR type/Channels').
ViStatus _VI_FUNC rsvna_ConfigureMulticalibrationType(ViSession instrSession,
                                                      ViBoolean type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_MULTICALIBRATION_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the Reduced Through logic for every port
/// HIFN  assignment of an automatic calibration.
/// HIFN     Note that for this to work, the port assignments must have a star
/// HIFN  topology, with a Common Port set using
/// HIFN  [SENSe<Ch>:]CORRection:COLLect:AUTO:CPORt.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMinimumConnect/Enables or disables the Reduced Through logic for every port
/// HIPAR autoMinimumConnect/assignment of an automatic calibration.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoMinimumConnect(
                            ViSession instrSession,
                            ViBoolean autoMinimumConnect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_CALIBRATION_AUTO_MINIMUM_CONNECT, autoMinimumConnect),
            2, "Auto Minimum Connect");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Common Port of an automatic calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Parameter ignored.
/// HIPAR commonPort/Defines the Common Port of an automatic calibration.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoCommonPort(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   commonPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_AUTO_COMMON_PORT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_AUTO_COMMON_PORT, commonPort),
            3, "Common Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Terminates a system error correction, discarding the acquired data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Parameter ignored.
ViStatus _VI_FUNC rsvna_CalibrationTerminateSystemErrorCorrection(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TERMINATE_SYSTEM_ERROR_CORRECTION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TERMINATE_SYSTEM_ERROR_CORRECTION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Cleans the channel's existing calibration data and also removes links
/// HIFN  to the calibration pool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Parameter ignored.
ViStatus _VI_FUNC rsvna_CalibrationClean(ViSession instrSession,
                                         ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_CLEAN,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_CLEAN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the wave correction logic for wave quantities
/// HIFN  and ratios.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Parameter ignored.
/// HIPAR enabled/Activates or deactivates the wave correction logic for wave quantities
/// HIPAR enabled/and ratios.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationWaveCorrectionEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_WAVE_CORRECTION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_WAVE_CORRECTION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the position of the offset calculation in the Offset Embed
/// HIFN  calculation chain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Changes the position of the offset calculation in the Offset Embed
/// HIPAR enabled/calculation chain.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationCalculateAfterDeembed(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_CALCULATE_AFTER_DEEMBED_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries date and time of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Kit Name'.
/// HIPAR calibrationDate/This control returns the data and time, e.g. '03/20/06,18:30:39'.
ViStatus _VI_FUNC rsvna_QueryCalibrationDate(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   bufferSize,
                                             ViChar    calibrationDate[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_DATE_AND_TIME,
            bufferSize, calibrationDate),
            4, "Calibration Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries state of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationState/This control returns the state of the channel's calibration.
ViStatus _VI_FUNC rsvna_QueryCalibrationState(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32*  calibrationState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_STATUS, calibrationState),
            3, "Calibration State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the calibration unit with given address as active
/// HIFN  calibration unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationUnit/This control sets the active calibration unit.
ViStatus _VI_FUNC rsvna_ConfigureActiveCalibrationUnit(ViSession instrSession,
                                                       ViString  calibrationUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_SELECT_CALIBRATION_UNIT, calibrationUnit),
            2, "Calibration Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the USB addresses of all calibration units
/// HIFN  connected to the VNA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Kit Name'.
/// HIPAR calibrationUnits/This control returns the USB addresses of all calibration units
/// HIPAR calibrationUnits/connected to the VNA.
ViStatus _VI_FUNC rsvna_QueryAllCalibrationUnits(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    calibrationUnits[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_CALIBRATION_UNIT_LIST,
            bufferSize, calibrationUnits),
            3, "Calibration Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines calibration unit standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR standard/This control selects a calibration unit standard
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationUnitStandard(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   port1,
                                                         ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationGeneralStandard, standard, &p2standard),
                3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:AKAL:CONN %s,%ld", p2standard, port1);

    if (port2 > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%s,%ld", cmd, port2);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries all characterizations (cal kit files) which are
/// HIFN  stored on the connected calibration unit R&S ZV-Z5x. A possible
/// HIFN  response is 'Factory,ZV-Z53_custom,Throughs'. The factory
/// HIFN  characterization is always available; an empty string denotes that no
/// HIFN  calibration unit is connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Cal Unit
/// HIPAR bufferSize/Characterizations'.
/// HIPAR calUnitCharacterizations/Query all characterizations stored on the connected calibration unit.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizations(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    calUnitCharacterizations[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_CALIBRATION_UNIT_CHARACTERIZATIONS_LIST,
            bufferSize, calUnitCharacterizations),
            3, "Cal Unit Characterizations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries all calibration standards of a characterization (cal kit file)
/// HIFN that is stored in a connected calibration unit R&S ZV-Z5x.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calUnitCharacterization/The name of a cal unit characterization. 'Factory' denotes the factory
/// HIPAR calUnitCharacterization/characterization; an empty string ' ' denotes the last referenced
/// HIPAR calUnitCharacterization/characterization.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Cal Unit
/// HIPAR bufferSize/Characterizations'.
/// HIPAR calUnitCharacterizations/Query all calibration standards of a characterization stored on
/// HIPAR calUnitCharacterizations/the connected calibration unit.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizationsStandards(
                            ViSession instrSession,
                            ViString  calUnitCharacterization,
                            ViInt32   bufferSize,
                            ViChar    calUnitCharacterizations[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:CKIT:STAN:CAT? '%s'", calUnitCharacterization);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, calUnitCharacterizations));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the creation date and time of a cal unit
/// HIFN  characterization (cal kit file). A possible response is 'Friday, May
/// HIFN  26, 2011, 10:13:40'. An empty string is returned if no calibration unit
/// HIFN  is connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calUnitCharacterization/The name of a cal unit characterization. 'Factory' denotes the factory
/// HIPAR calUnitCharacterization/characterization; an empty string ' ' denotes the last referenced
/// HIPAR calUnitCharacterization/characterization.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Cal Unit
/// HIPAR bufferSize/Characterization Date'.
/// HIPAR calUnitCharacterizationDate/Queries the creation date and time of a cal unit characterization (cal
/// HIPAR calUnitCharacterizationDate/kit file). A possible response is 'Friday, May 26, 2011, 10:13:40'. An
/// HIPAR calUnitCharacterizationDate/empty string is returned if no calibration unit is connected.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizationDate(
                            ViSession instrSession,
                            ViString  calUnitCharacterization,
                            ViInt32   bufferSize,
                            ViChar    calUnitCharacterizationDate[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:DATE? '%s'", calUnitCharacterization);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, calUnitCharacterizationDate));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the frequency range of a cal unit
/// HIFN  characterization (cal kit file). The response consists of the start and
/// HIFN  stop frequencies in Hz, separated by a comma. 0,0 is returned if no
/// HIFN  calibration unit is connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calUnitCharacterization/The name of a cal unit characterization. 'Factory' denotes the factory
/// HIPAR calUnitCharacterization/characterization; an empty string ' ' denotes the last referenced
/// HIPAR calUnitCharacterization/characterization.
/// HIPAR startFrequency/Returns the start frequency of the cal unit characterization.
/// HIPAR stopFrequency/Returns the stop frequency of the cal unit characterization.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizationFrequencyRange(
                            ViSession instrSession,
                            ViString  calUnitCharacterization,
                            ViReal64* startFrequency,
                            ViReal64* stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:FRAN? '%s'", calUnitCharacterization);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, startFrequency, stopFrequency, NULL, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of ports of a cal unit
/// HIFN  characterization (cal kit file), the assigned connector types, and
/// HIFN  their gender. A possible response for a two-port calibration is '1,N 50
/// HIFN  Ohm,MALE,2,N 50 Ohm,MALE'. An empty string is returned if no
/// HIFN  calibration unit is connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calUnitCharacterization/The name of a cal unit characterization. 'Factory' denotes the factory
/// HIPAR calUnitCharacterization/characterization; an empty string ' ' denotes the last referenced
/// HIPAR calUnitCharacterization/characterization.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Cal Unit
/// HIPAR bufferSize/Characterization Ports'.
/// HIPAR calUnitCharacterizationPorts/Queries the number of ports of a cal unit characterization (cal kit
/// HIPAR calUnitCharacterizationPorts/file), the assigned connector types, and their gender. A possible
/// HIPAR calUnitCharacterizationPorts/response for a two-port calibration is '1,N 50 Ohm,MALE,2,N 50
/// HIPAR calUnitCharacterizationPorts/Ohm,MALE'. An empty string is returned if no calibration unit is
/// HIPAR calUnitCharacterizationPorts/connected.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizationPorts(
                            ViSession instrSession,
                            ViString  calUnitCharacterization,
                            ViInt32   bufferSize,
                            ViChar    calUnitCharacterizationPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:PORT? '%s'", calUnitCharacterization);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, calUnitCharacterizationPorts));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the calibration data for a particular standard
/// HIFN  from a cal unit characterization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calUnitCharacterization/The name of a cal unit characterization. 'Factory' denotes the factory
/// HIPAR calUnitCharacterization/characterization; an empty string ' ' denotes the last referenced
/// HIPAR calUnitCharacterization/characterization.
/// HIPAR standard/This control selects a standard.
/// HIPAR sParameter/Sets the S-parameter of the standard.
/// HIPAR firstPortNumber/First port number (sufficient for one-port standards).
/// HIPAR secondPortNumber/Second port number, for two-port standards.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Cal Unit
/// HIPAR bufferSize/Characterization Ports'.
/// HIPAR calibrationData/Returns the calibration data for a particular standard from a cal unit
/// HIPAR calibrationData/characterization.
/// HIPAR noofValues/Returns number of Cal Unit Characterization Ports values.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitCharacterizationData(
                            ViSession instrSession,
                            ViString  calUnitCharacterization,
                            ViInt32   standard,
                            ViInt32   sParameter,
                            ViInt32   firstPortNumber,
                            ViInt32   secondPortNumber,
                            ViInt32   bufferSize,
                            ViReal64  calibrationData[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSVNA_VAL_CALSTD_MMTH, RSVNA_VAL_CALSTD_FSM),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sParameter, RSVNA_VAL_S11, RSVNA_VAL_S22),
            4, "S Parameter");

    if (secondPortNumber == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:SDAT? '%s',%s,%s,%ld", calUnitCharacterization,
                 loadCalKitStandardArr[standard],
                 sParameterArr[sParameter],
                 firstPortNumber);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:AKAL:SDAT? '%s',%s,%s,%ld,%ld", calUnitCharacterization,
                 loadCalKitStandardArr[standard],
                 sParameterArr[sParameter],
                 firstPortNumber, secondPortNumber);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, calibrationData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the warmup state of the connected calibration
/// HIFN  unit R&S ZV_Z5x. If several cal units are USB-connected to the
/// HIFN  analyzer, the function queries the cal unit selected via
/// HIFN  rsvna_ConfigureActiveCalibrationUnit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR warmupState/Queries the warmup state of the connected calibration unit R&S ZV_Z5x.
ViStatus _VI_FUNC rsvna_QueryCalibrationUnitWarmupState(ViSession  instrSession,
                                                        ViBoolean* warmupState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViBoolean(instrSession, "", RSVNA_ATTR_CALIBRATION_UNIT_WARMUP_STATE, warmupState),
            2, "Warmup State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Acquire calibration data for the calibration mixer in an automatic
/// HIFN  vector mixer calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR port1/VNA port connected to the RF port of the calibration mixer.
/// HIPAR port2/VNA port connected to the IF port of the calibration mixer.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutomaticVectorMixer(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   port1,
                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CALIBRATION_AUTOMATIC_VECTOR_MIXER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,%ld", port1, port2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_CALIBRATION_AUTOMATIC_VECTOR_MIXER, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Acquires receiver noise data in Channel <Ch>, with a matched load
/// HIFN  connected to <ReceivingPort>. This is part of a noise figure
/// HIFN  calibration using a calibration unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR receivingPort/Acquires receiver noise data in Channel <Ch>, with a matched load
/// HIPAR receivingPort/connected to <ReceivingPort>. This is part of a noise figure
/// HIPAR receivingPort/calibration using a calibration unit.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationNoiseDataReceivingPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   receivingPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_NOISE_DATA_RECEIVING_PORT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_NOISE_DATA_RECEIVING_PORT, receivingPort),
            3, "Receiving Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the Same Sweep Setup for All Standards logic for the current
/// HIFN  channel (instead globally). This setting is only available if
/// HIFN  RSVNA_ATTR_CALIBRATION_SWEEP_SETUP is not activated for the entire
/// HIFN  device.
/// HIFN
/// HIFN     OFF (0)
/// HIFN     Individual sweep setups for all standards (faster for some
/// HIFN  configurations)
/// HIFN     ON (1)
/// HIFN     Common sweep setup for all standards (no preparation phase for each
/// HIFN  standard)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sameSweepSetup/Enables the Same Sweep Setup for All Standards logic for the current
/// HIPAR sameSweepSetup/channel (instead globally). This setting is only available if
/// HIPAR sameSweepSetup/RSVNA_ATTR_CALIBRATION_SWEEP_SETUP is not activated for the entire
/// HIPAR sameSweepSetup/device.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationSameSweepSetup(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean sameSweepSetup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_SAME_SWEEP_SETUP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CALIBRATION_SAME_SWEEP_SETUP, sameSweepSetup),
            3, "Same Sweep Setup");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how calibration sweeps are prepared and performed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepSetup/Defines how calibration sweeps are prepared and performed.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationSweepSetup(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViBoolean sweepSetup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_SWEEP_SETUP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CALIBRATION_SWEEP_SETUP, sweepSetup),
            3, "Sweep Setup");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the source step attenuation the R&S ZNA uses during the first
/// HIFN  calibration sweep of the source power calibration ('Reference Receiver
/// HIFN  Cal Power'). This attenuation value is ignored if the R&S ZNA is set to
/// HIFN  use the calculated port power result
/// HIFN     (SOURce<Ch>:POWer<PhyPt>:CORRection:PSELect CPOW).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines the source step attenuation the R&S ZNA uses during the first
/// HIPAR attenuation/calibration sweep of the source power calibration ('Reference Receiver
/// HIPAR attenuation/Cal Power'). This attenuation value is ignored if the R&S ZNA is set to
/// HIPAR attenuation/use the calculated port power result
ViStatus _VI_FUNC rsvna_ConfigureCalibrationReferenceReceiverAttenuation(
                            ViSession instrSession,
                            ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_CALIBRATION_REFERENCE_RECEIVER_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the fast interpolation method for intermodulation
/// HIFN  calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR imdMethod/Enables or disables the fast interpolation method for intermodulation
/// HIPAR imdMethod/calibration.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationIMDMethod(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean imdMethod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_IMD_METHOD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CALIBRATION_IMD_METHOD, imdMethod),
            3, "IMD Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables Fast Multiport Correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastMultiportCorrection/Enables or disables Fast Multiport Correction.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationFastMultiportCorrection(
                            ViSession instrSession,
                            ViBoolean fastMultiportCorrection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_CALIBRATION_FAST_MULTIPORT_CORRECTION, fastMultiportCorrection),
            2, "Fast Multiport Correction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the algorithm for the interpolation of user system error
/// HIFN  corrections.
/// HIFN
/// HIFN     For the factory system error correction data, the analyzer always uses
/// HIFN  linear interpolation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interpolationMethod/Selects the algorithm for the interpolation of user system error
/// HIPAR interpolationMethod/corrections.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationInterpolationMethod(
                            ViSession instrSession,
                            ViInt32   interpolationMethod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_CALIBRATION_INTERPOLATION_METHOD, interpolationMethod),
            2, "Interpolation Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the receiver calibration at physical port <PhyPt>in channel
/// HIFN  <Ch>at harmonic orders [SENSe<Ch>:]CORRection:ADVanced:HARMonic:ORDer?
/// HIFN
/// HIFN     Then stores and applies the calibration data.
/// HIFN
/// HIFN     Note: The receiver calibration relies on the source power calibration
/// HIFN  acquired using SOURce<Ch>:POWer<PhyPt>:CORRection:HARMonic[:ACQuire].
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
ViStatus _VI_FUNC rsvna_StartHarmonicReceiverCalibration(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_START_HARMONIC_RECEIVER_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_START_HARMONIC_RECEIVER_CALIBRATION,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_START_HARMONIC_RECEIVER_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the source calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
ViStatus _VI_FUNC rsvna_StartHarmonicSourceCalibration(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   physicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_START_HARMONIC_SOURCE_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_START_HARMONIC_SOURCE_CALIBRATION,
        "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
        3, "Physical Port");
    strcat(repCap, ",");
    strcat(repCap, repCap2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_START_HARMONIC_SOURCE_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Use this attribute to reuse measurement data of the active automatic
/// HIFN  calibration (if available) for the next automatic calibration. This
/// HIFN  data reuse allows an automatic calibration to be repeated without
/// HIFN  repeating all measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_CalibrationICSRepeat(ViSession instrSession,
                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_ICS_REPEAT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_ICS_REPEAT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Available if the calibration unit selected using
/// HIFN  SYSTem:COMMunicate:RDEVice:AKAL:ADDRess is an inline calibration system
/// HIFN  (ICS) only. Applies to all inline calibration units (ICUs) that are
/// HIFN  connected to the selected ICS and that are used with factory
/// HIFN  characterization data .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR temperatureCompensation/Available if the calibration unit selected using
/// HIPAR temperatureCompensation/SYSTem:COMMunicate:RDEVice:AKAL:ADDRess is an inline calibration system
/// HIPAR temperatureCompensation/(ICS) only. Applies to all inline calibration units (ICUs) that are
/// HIPAR temperatureCompensation/connected to the selected ICS and that are used with factory
/// HIPAR temperatureCompensation/characterization data.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationICSTemperatureCompensation(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean temperatureCompensation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_ICS_TEMEPRATURE_COMPENSATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CALIBRATION_ICS_TEMEPRATURE_COMPENSATION, temperatureCompensation),
            3, "Temperature Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  With this command, you can perform the Unknown Through measurement
/// HIFN  during an automatic UOSM calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR port1/VNA test port 1 number.
/// HIPAR port2/VNA test port 2 number.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationICSUnknownThrough(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   port1,
                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CALIBRATION_ICS_UNKNOWN_THROUGH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,%ld", port1, port2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_CALIBRATION_ICS_UNKNOWN_THROUGH, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Tries to join the two calibrations.
/// HIFN
/// HIFN     If successful, it stores the resulting calibration as <OutputCal> in
/// HIFN  <LocationOutputCal>. Note: If (<OutputCal>,<LocationOutputCal>) matches
/// HIFN  one of the input calibrations, this input calibration is replaced.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firstCalibration/String parameter with the channel name (for
/// HIPAR firstCalibration/<LocationFirstCal>=CHANnel) or the calibration file name (for
/// HIPAR firstCalibration/<LocationFirstCal>=POOL) of the first calibration.
/// HIPAR locationFirstCalibration/First calibration location.
/// HIPAR secondCalibration/String parameter with the channel name (for
/// HIPAR secondCalibration/<LocationFirstCal>=CHANnel) or the calibration file name (for
/// HIPAR secondCalibration/<LocationFirstCal>=POOL) of the second calibration.
/// HIPAR locationSecondCalibration/Second calibration location.
/// HIPAR outputCalibration/String parameter with the channel name (for
/// HIPAR outputCalibration/<LocationFirstCal>=CHANnel) or the calibration file name (for
/// HIPAR outputCalibration/<LocationFirstCal>=POOL) of the output calibration.
/// HIPAR locationOutputCalibration/Output calibration location.
ViStatus _VI_FUNC rsvna_JoinCalibrations(ViSession instrSession,
                                         ViString  firstCalibration,
                                         ViInt32   locationFirstCalibration,
                                         ViString  secondCalibration,
                                         ViInt32   locationSecondCalibration,
                                         ViString  outputCalibration,
                                         ViInt32   locationOutputCalibration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2locationFirstCalibration;
    ViChar*  p2locationSecondCalibration;
    ViChar*  p2locationOutputCalibration;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalLocation, locationFirstCalibration, &p2locationFirstCalibration),
                3, "Location First Calibration");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalLocation, locationSecondCalibration, &p2locationSecondCalibration),
                5, "Location Second Calibration");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalLocation, locationOutputCalibration, &p2locationOutputCalibration),
                7, "Location Output Calibration");

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%s,'%s',%s,'%s',%s", firstCalibration, p2locationFirstCalibration, secondCalibration, p2locationSecondCalibration, outputCalibration, p2locationOutputCalibration);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_JOIN_CALLIBRATIONS, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs the delay mixer calibration step as part of an automatic
/// HIFN  calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rfport/RF port.
/// HIPAR ifport/IF Port.
ViStatus _VI_FUNC rsvna_CalibrationDelayMixer(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   rfport,
                                              ViInt32   ifport)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CALIBRATION_DELAY_MIXER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%ld", rfport, ifport);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_DELAY_MIXER,
                    RS_VAL_DIRECT_USER_CALL,
                    param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Can only be used on a 2-port R&S ZNAxx with second internal source and
/// HIFN  internal combiner.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Can only be used on a 2-port R&S ZNAxx with second internal source and
/// HIPAR enabled/internal combiner.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationInternalCombinerEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_INTERNAL_COMBINER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CALIBRATION_INTERNAL_COMBINER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the power of a millimeter-wave converter, or its power offset
/// HIFN  relative to the channel power.
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce<Ch>:POWer<PhyPt>:CONVerter:OFFSet
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR value/Port-specific power or power offset, depending on the offset <Type>.
/// HIPAR type/Type
ViStatus _VI_FUNC rsvna_CalibrationConverterOffset(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  value,
                                                   ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_CALIBRATION_CONVERTER_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationConverterOffsetType, type, &p2string),
        4, "Type");

    snprintf(param, RS_PARAM_BUF_SIZE, "%.12lG,%s", value, p2string);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_CONVERTER_OFFSET,
                    RS_VAL_DIRECT_USER_CALL,
                    param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a connector type at a specified port <Port> and
/// HIFN  its gender.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port numbers of the analyzer.
/// HIPAR connector/Selects a connector type and gender.
ViStatus _VI_FUNC rsvna_ConfigureConnector(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   port,
                                           ViInt32   connector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_CONNECTOR,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");

    strcpy(repCap, repCap_channel);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_CALIBRATION_CONNECTOR, connector),
            8, "Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the user-defined connector types.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR connectorName/Name of the user-defined connectors.
/// HIPAR propagationMode/This control selects a propagation mode.
/// HIPAR connectorType/This control selects a connector type.
/// HIPAR relativePermittivity/This control sets the relative permittivity.
/// HIPAR impedance/This control sets the reference impedance or cutoff frequency.
ViStatus _VI_FUNC rsvna_ConfigureUserConnectorType(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViString  connectorName,
                                                   ViInt32   propagationMode,
                                                   ViInt32   connectorType,
                                                   ViReal64  relativePermittivity,
                                                   ViReal64  impedance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, propagationMode, RSVNA_VAL_CONNECTION_MODE_TEM, RSVNA_VAL_CONNECTION_MODE_WGUIDE),
            4, "Propagation Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, connectorType, RSVNA_VAL_CONNECTION_CONNECTOR_GENDER, RSVNA_VAL_CONNECTION_CONNECTOR_NGENDER),
            5, "Connector Type");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, relativePermittivity, 0.0000000001, 1000), 6, "Relative Permittivity");

    if (propagationMode == RSVNA_VAL_CONNECTION_MODE_TEM)
    {
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, impedance, 1.0e-6, 1000.0e6), 7, "Impedance");
    }

    else if (propagationMode == RSVNA_VAL_CONNECTION_MODE_WGUIDE)
    {
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, impedance, 0.0, 1000.0e9), 7, "Impedance");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:CONN '%s',%s,%s,%.12lG,%.12lG",
            channel, connectorName, propagationModeArr[propagationMode], connectorTypeArr[connectorType], relativePermittivity, impedance);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a user-defined connector type at a specified
/// HIFN  port <Port> and its gender.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port numbers of the analyzer.
/// HIPAR connector/This control selects a connector type at a specified port <Port> and
/// HIPAR connector/gender.
/// HIPAR connectorGender/This control selects the connector gender.
ViStatus _VI_FUNC rsvna_ConfigureUserConnector(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   port,
                                               ViString  connector,
                                               ViInt32   connectorGender)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:SCON%ld '%s',%s", channel, port, connector, connectorGenderArr[connectorGender]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a user-defined connector type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR connectorName/Name of the user-defined connector.
ViStatus _VI_FUNC rsvna_DeleteUserConnectorType(ViSession instrSession,
                                                ViInt32   channel,
                                                ViString  connectorName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_DELETE_USER_CONNECTOR_TYPE, connectorName),
            3, "Connector Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Qualifies whether the connector types at the test ports (but not
/// HIFN  their gender) are equal or independent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR connectorType/This control selects a connector type.
ViStatus _VI_FUNC rsvna_ConfigurePortConnectorTypes(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   connectorType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PORT_CONNECTOR_TYPE, connectorType),
            3, "Connector Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a list of the connector types of all calibration
/// HIFN  kits in use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR catalog/Response is a string parameter with comma-separated list of the
/// HIPAR catalog/connector types of all calibration kits in use.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_ConnectorTypesCatalog(ViSession instrSession,
                                              ViChar    catalog[],
                                              ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, "", RSVNA_ATTR_CALIBRATION_CONNECTOR_TYPE_CATALOG,
            bufferSize, catalog),
            2, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function calculates the system error correction data from the
/// HIFN  acquired one or two-port measurement results (rszvb_StartCalibration),
/// HIFN  stores them and applies them to the calibrated channel. To avoid
/// HIFN  incompatibilities, older system error correction data is deleted unless
/// HIFN  it has been transferred into a cal pool (MMEMory:STORe:CORRection <Ch>,
/// HIFN  '<file_name>').
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_SaveCalibrationData(ViSession instrSession,
                                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SAVE_CALIBRATION_DATA, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function generates a set of default system error correction data
/// HIFN  for the selected ports and calibration type. The default data set
/// HIFN  corresponds to a test setup which does not introduce any systematic
/// HIFN  errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_GenerateDefaultCalibrationData(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_GENERATE_DEFAULT_CALIBRATION_DATA, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes system error correction data generated and
/// HIFN  stored previously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationName/This control defines the name of the calibration (string parameter)
/// HIPAR calibrationName/defined together with calibration type.
ViStatus _VI_FUNC rsvna_DeleteCalibrationData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  calibrationName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_CALIBRATION_DATA, calibrationName),
            3, "Calibration Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries for a given port pair the currently available
/// HIFN  correction buffers. Thus if a buffer name does not appear as a
/// HIFN  response, a query with rsvna_ReadCalibrationData will fail.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR catalog/Catalog number.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
/// HIPAR arraySize/This control defines the size of array passed to argument 'Catalog'.
/// HIPAR catalog/This control returns currently available correction buffers.
/// HIPAR numberofValues/Returns number of values. If the number of values is higher than list
/// HIPAR numberofValues/of buffers in Catalog, increase the Array Size.
ViStatus _VI_FUNC rsvna_QueryAvailableCalibrationDataCatalog(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   catalog,
                                                             ViInt32   port1,
                                                             ViInt32   port2,
                                                             ViInt32   arraySize,
                                                             ViChar    availableBuffers[],
                                                             ViInt32*  numberofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = rd_buffer;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, catalog, 1, 2),
            3, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:CDAT:CAT%ld? %ld,%ld", channel, catalog, port1, port2);
    checkErr(rsvna_QueryViString(instrSession, cmd, arraySize, ext_buf));

    p2buf = strtok(ext_buf, ",");

    *numberofValues = atoi(p2buf);

    if (*numberofValues != 0)
    {
        p2buf = strtok(NULL, "\n");

        strcpy(availableBuffers, p2buf);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads system error correction data for a specific
/// HIFN  channel, calibration method, and port combination <SourcePort>,
/// HIFN  <LoadPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR errorTermParameters/Selects the error term parameters.
/// HIPAR sourcePort/Source port number.
/// HIPAR loadPort/Load port number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Data'.
/// HIPAR calibrationData/This control returns the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
/// HIPAR noofValues/Returns number of Calibration Data values.
ViStatus _VI_FUNC rsvna_ReadCalibrationData(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   errorTermParameters,
                                            ViInt32   sourcePort,
                                            ViInt32   loadPort,
                                            ViInt32   bufferSize,
                                            ViReal64  calibrationData[],
                                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                3, "Error Term Parameters");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS%ld:CORR:CDAT? %s,%ld,%ld", channel,
             p2ErrTerm, sourcePort, loadPort);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, calibrationData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes system error correction data for a specific
/// HIFN  channel, calibration method, and port combination <SourcePort>,
/// HIFN  <LoadPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR errorTermParameters/Selects the error term parameters.
/// HIPAR sourcePort/Source port number.
/// HIPAR loadPort/Load port number.
/// HIPAR calibrationData/This control defines the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
ViStatus _VI_FUNC rsvna_WriteCalibrationData(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   errorTermParameters,
                                             ViInt32   sourcePort,
                                             ViInt32   loadPort,
                                             ViReal64  calibrationData[])
{
    ViStatus         error = VI_SUCCESS;
    ViChar           cmd[RS_MAX_MESSAGE_BUF_SIZE];
    RsCoreSessionPtr rsSession;
    ViInt32          dataSize = 0;
    ViInt32          pointsCount;
    ViByte*          data = NULL;
    ViChar           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*          p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_POINTS_COUNT, &pointsCount));

    checkErr(RsCore_InvalidViInt32Range(instrSession, pointsCount, 1, 250000));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                3, "Error Term Parameters");

    if (rsSession->sessionType == RS_INTF_ASRL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORMAT ASC;:SENS%ld:CORR:CDAT %s,%ld,%ld,", channel, p2ErrTerm, sourcePort, loadPort);
        checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, calibrationData, pointsCount));
    }
    else
    {
        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER EOI"));
        }

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS%ld:CORR:CDAT %s,%ld,%ld,", channel, p2ErrTerm, sourcePort, loadPort);
        checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, calibrationData, pointsCount, &data, &dataSize, VI_FALSE));
        checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, data, dataSize));

        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER LFEOI"));
        }
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (data)
        free(data);
    RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries for a given port pair the currently available
/// HIFN  correction buffers. Thus if a buffer name does not appear as a
/// HIFN  response, a query with rsvna_ReadCalibrationDataPort will fail.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR catalog/Catalog number.
/// HIPAR port1/Port number.
/// HIPAR port2/Port number.
/// HIPAR arraySize/This control defines the size of array passed to argument 'Catalog'.
/// HIPAR catalog/This control returns currently available correction buffers.
/// HIPAR numberofValues/Returns number of values. If the number of values is higher than list
/// HIPAR numberofValues/of buffers in Catalog, increase the Array Size.
ViStatus _VI_FUNC rsvna_QueryAvailableCalibrationDataCatalogPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   catalog,
                            ViInt32   port1,
                            ViInt32   port2,
                            ViInt32   arraySize,
                            ViChar    availableBuffers[],
                            ViInt32*  numberofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = rd_buffer;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, catalog, 1, 2),
            4, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:CDAT:PORT%ld:CAT%ld? %ld,%ld", channel, physicalPort, catalog, port1, port2);
    checkErr(rsvna_QueryViString(instrSession, cmd, arraySize, ext_buf));

    p2buf = strtok(ext_buf, ",");

    *numberofValues = atoi(p2buf);

    if (*numberofValues != 0)
    {
        p2buf = strtok(NULL, "\n");

        strcpy(availableBuffers, p2buf);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads system error correction data for a specific
/// HIFN  channel, calibration method, and port combination <SourcePort>,
/// HIFN  <LoadPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in
/// HIPAR physicalPort/arbitrary mode.
/// HIPAR errorTermParameters/Selects the error term parameters.
/// HIPAR sourcePort/Source port number.
/// HIPAR loadPort/Load port number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Data'.
/// HIPAR calibrationData/This control returns the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
/// HIPAR noofValues/Returns number of Calibration Data values.
ViStatus _VI_FUNC rsvna_ReadCalibrationDataPort(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   physicalPort,
                                                ViInt32   errorTermParameters,
                                                ViInt32   sourcePort,
                                                ViInt32   loadPort,
                                                ViInt32   bufferSize,
                                                ViReal64  calibrationData[],
                                                ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                4, "Error Term Parameters");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS%ld:CORR:CDAT:PORT%ld? %s,%ld,%ld", channel, physicalPort,
             p2ErrTerm, sourcePort, loadPort);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, calibrationData, noOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes system error correction data for a specific
/// HIFN  channel, calibration method, and port combination <SourcePort>,
/// HIFN  <LoadPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in
/// HIPAR physicalPort/arbitrary mode.
/// HIPAR errorTermParameters/Selects the error term parameters.
/// HIPAR sourcePort/Source port number.
/// HIPAR loadPort/Load port number.
/// HIPAR calibrationData/This control defines the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
ViStatus _VI_FUNC rsvna_WriteCalibrationDataPort(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   physicalPort,
                                                 ViInt32   errorTermParameters,
                                                 ViInt32   sourcePort,
                                                 ViInt32   loadPort,
                                                 ViReal64  calibrationData[])
{
    ViStatus         error = VI_SUCCESS;
    ViChar           cmd[RS_MAX_MESSAGE_BUF_SIZE];
    RsCoreSessionPtr rsSession;
    ViInt32          dataSize = 0;
    ViInt32          pointsCount;
    ViByte*          data = NULL;
    ViChar           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*          p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_POINTS_COUNT, &pointsCount));

    checkErr(RsCore_InvalidViInt32Range(instrSession, pointsCount, 1, 250000));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                4, "Error Term Parameters");

    if (rsSession->sessionType == RS_INTF_ASRL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORMAT ASC;:SENS%ld:CORR:CDAT:PORT%ld %s,%ld,%ld,",
                 channel, physicalPort, p2ErrTerm, sourcePort, loadPort);
        checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, calibrationData, pointsCount));
    }
    else
    {
        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER EOI"));
        }

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS%ld:CORR:CDAT:PORT%ld %s,%ld,%ld,",
                 channel, physicalPort, p2ErrTerm, sourcePort, loadPort);
        checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, calibrationData, pointsCount, &data, &dataSize, VI_FALSE));
        checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, data, dataSize));

        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER LFEOI"));
        }
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (data)
        free(data);
    RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries for a given port pair the currently available
/// HIFN  correction buffers. Thus if a buffer name does not appear as a
/// HIFN  response, a query with rsvna_ReadSwitchMatrixCalibrationData will fail.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR catalog/Catalog number.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR arraySize/This control defines the size of array passed to argument 'Available
/// HIPAR arraySize/Buffers'.
/// HIPAR availableBuffers/This control returns currently available correction buffers.
/// HIPAR numberofValues/Returns number of values. If the number of values is higher than
/// HIPAR numberofValues/number of values in Available Buffers, increase the Array Size.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixAvailableCalibrationDataCatalog(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   catalog,
                            ViInt32   sourceTestPort,
                            ViInt32   loadTestPort,
                            ViInt32   arraySize,
                            ViChar    availableBuffers[],
                            ViInt32*  numberofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = rd_buffer;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, catalog, 1, 2),
            3, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:SMAT:CDAT:CAT%ld? %ld,%ld", channel, catalog, sourceTestPort, loadTestPort);
    checkErr(rsvna_QueryViString(instrSession, cmd, arraySize, ext_buf));

    p2buf = strtok(ext_buf, ",");

    *numberofValues = atoi(p2buf);

    if (*numberofValues != 0)
    {
        p2buf = strtok(NULL, "\n");

        strcpy(availableBuffers, p2buf);
    }
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads switch matrix system error correction data for a
/// HIFN  specific channel, calibration method, and port combination
/// HIFN  <SourceTestPort>, <LoadTestPort>,<SourceVNAPort>, <LoadVNAPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR errorTermParameters/This control sets the error terms parameters.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR sourceVNAPort/Source VNA Port number.
/// HIPAR loadVNAPort/Load VNA Port number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Data'.
/// HIPAR calibrationData/This control returns the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
/// HIPAR noofValues/Returns number of Calibration Data values.
ViStatus _VI_FUNC rsvna_ReadSwitchMatrixCalibrationData(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   errorTermParameters,
                                                        ViInt32   sourceTestPort,
                                                        ViInt32   loadTestPort,
                                                        ViInt32   sourceVNAPort,
                                                        ViInt32   loadVNAPort,
                                                        ViInt32   bufferSize,
                                                        ViReal64  calibrationData[],
                                                        ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                3, "Error Term Parameters");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS%ld:CORR:SMAT:CDAT? %s,%ld,%ld,%ld,%ld",
            channel, p2ErrTerm, sourceTestPort, loadTestPort, sourceVNAPort, loadVNAPort);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, calibrationData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes switch matrix system error correction data for a
/// HIFN  specific channel, calibration method, and port combination
/// HIFN  <SourceTestPort>, <LoadTestPort>,<SourceVNAPort>, <LoadVNAPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR errorTermParameters/This control sets the error terms parameters.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR sourceVNAPort/Source VNA Port number.
/// HIPAR loadVNAPort/Load VNA Port number.
/// HIPAR calibrationData/This control defines the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
ViStatus _VI_FUNC rsvna_WriteSwitchMatrixCalibrationData(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   errorTermParameters,
                                                         ViInt32   sourceTestPort,
                                                         ViInt32   loadTestPort,
                                                         ViInt32   sourceVNAPort,
                                                         ViInt32   loadVNAPort,
                                                         ViReal64  calibrationData[])
{
    ViStatus         error = VI_SUCCESS;
    ViChar           cmd[RS_MAX_MESSAGE_BUF_SIZE];
    RsCoreSessionPtr rsSession;
    ViInt32          dataSize = 0;
    ViInt32          pointsCount;
    ViByte*          data = NULL;
    ViChar           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*          p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_POINTS_COUNT, &pointsCount));

    checkErr(RsCore_InvalidViInt32Range(instrSession, pointsCount, 1, 250000));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                3, "Error Term Parameters");

    if (rsSession->sessionType == RS_INTF_ASRL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORMAT ASC;:SENS%ld:CORR:SMAT:CDAT %s,%ld,%ld,%ld,%ld,",
                channel, p2ErrTerm, sourceTestPort, loadTestPort, sourceVNAPort, loadVNAPort);
        checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, calibrationData, pointsCount));
    }
    else
    {
        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER EOI"));
        }

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS%ld:CORR:SMAT:CDAT %s,%ld,%ld,%ld,%ld,",
                channel, p2ErrTerm, sourceTestPort, loadTestPort, sourceVNAPort, loadVNAPort);
        checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, calibrationData, pointsCount, &data, &dataSize, VI_FALSE));
        checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, data, dataSize));

        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER LFEOI"));
        }
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (data)
        free(data);
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries for a given port pair the currently available
/// HIFN  correction buffers. Thus if a buffer name does not appear as a
/// HIFN  response, a query with rsvna_ReadSwitchMatrixCalibrationDataPort will
/// HIFN  fail.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR catalog/Catalog number.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR arraySize/This control defines the size of array passed to argument 'Available
/// HIPAR arraySize/Buffers'.
/// HIPAR availableBuffers/This control returns currently available correction buffers.
/// HIPAR numberofValues/Returns number of values. If the number of values is higher than
/// HIPAR numberofValues/number of values in Available Buffers, increase the Array Size.
ViStatus _VI_FUNC rsvna_QuerySwitchMatrixAvailableCalibrationDataCatalogPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   catalog,
                            ViInt32   sourceTestPort,
                            ViInt32   loadTestPort,
                            ViInt32   arraySize,
                            ViChar    availableBuffers[],
                            ViInt32*  numberofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   rd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = rd_buffer;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, catalog, 1, 2),
            4, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:SMAT:CDAT:PORT%ld:CAT%ld? %ld,%ld", channel, physicalPort, catalog, sourceTestPort, loadTestPort);
    checkErr(rsvna_QueryViString(instrSession, cmd, arraySize, ext_buf));

    p2buf = strtok(ext_buf, ",");

    *numberofValues = atoi(p2buf);

    if (*numberofValues != 0)
    {
        p2buf = strtok(NULL, "\n");

        strcpy(availableBuffers, p2buf);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads switch matrix system error correction data for a
/// HIFN  specific channel, calibration method, and port combination
/// HIFN  <SourceTestPort>, <LoadTestPort>, <SourceVNAPort>, <LoadVNAPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in
/// HIPAR physicalPort/arbitrary mode.
/// HIPAR errorTermParameters/This control sets the error terms parameters.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR sourceVNAPort/Source VNA Port number.
/// HIPAR loadVNAPort/Load VNA Port number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Calibration
/// HIPAR bufferSize/Data'.
/// HIPAR calibrationData/This control returns the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
/// HIPAR noofValues/Returns number of Calibration Data values.
ViStatus _VI_FUNC rsvna_ReadSwitchMatrixCalibrationDataPort(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   physicalPort,
                                                            ViInt32   errorTermParameters,
                                                            ViInt32   sourceTestPort,
                                                            ViInt32   loadTestPort,
                                                            ViInt32   sourceVNAPort,
                                                            ViInt32   loadVNAPort,
                                                            ViInt32   bufferSize,
                                                            ViReal64  calibrationData[],
                                                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                4, "Error Term Parameters");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS%ld:CORR:SMAT:CDAT:PORT%ld? %s,%ld,%ld,%ld,%ld", channel, physicalPort,
             p2ErrTerm,
             sourceTestPort, loadTestPort,
             sourceVNAPort, loadVNAPort);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, calibrationData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes switch matrix system error correction data for a
/// HIFN  specific channel, calibration method, and port combination
/// HIFN  <SourceTestPort>, <LoadTestPort>, <SourceVNAPort>, <LoadVNAPort>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in
/// HIPAR physicalPort/arbitrary mode.
/// HIPAR errorTermParameters/This control sets the error terms parameters.
/// HIPAR sourceTestPort/Source Test Port number.
/// HIPAR loadTestPort/Load Test Port number.
/// HIPAR sourceVNAPort/Source VNA Port number.
/// HIPAR loadVNAPort/Load VNA Port number.
/// HIPAR calibrationData/This control defines the calibration data. For each sweep point (set
/// HIPAR calibrationData/by rsvna_ConfigureSweepNumberOfPoints) real and imaginary part should
/// HIPAR calibrationData/be provided.
ViStatus _VI_FUNC rsvna_WriteSwitchMatrixCalibrationDataPort(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   physicalPort,
                                                             ViInt32   errorTermParameters,
                                                             ViInt32   sourceTestPort,
                                                             ViInt32   loadTestPort,
                                                             ViInt32   sourceVNAPort,
                                                             ViInt32   loadVNAPort,
                                                             ViReal64  calibrationData[])
{
    ViStatus         error = VI_SUCCESS;
    ViChar           cmd[RS_MAX_MESSAGE_BUF_SIZE];
    RsCoreSessionPtr rsSession;
    ViInt32          dataSize = 0;
    ViInt32          pointsCount;
    ViByte*          data = NULL;
    ViChar           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*          p2ErrTerm;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SWEEP_POINTS_COUNT, &pointsCount));

    checkErr(RsCore_InvalidViInt32Range(instrSession, pointsCount, 1, 250000));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngErrorTermParameter, errorTermParameters, &p2ErrTerm),
                4, "Error Term Parameters");

    if (rsSession->sessionType == RS_INTF_ASRL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORMAT ASC;:SENS%ld:CORR:SMAT:CDAT:PORT%ld %s,%ld,%ld,%ld,%ld,", channel, physicalPort, p2ErrTerm, sourceTestPort, loadTestPort, sourceVNAPort, loadVNAPort);
        checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, calibrationData, pointsCount));
    }
    else
    {
        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER EOI"));
        }

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS%ld:CORR:SMAT:CDAT:PORT%ld %s,%ld,%ld,%ld,%ld,",
                 channel, physicalPort, p2ErrTerm, sourceTestPort, loadTestPort, sourceVNAPort, loadVNAPort);
        checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, calibrationData, pointsCount, &data, &dataSize, VI_FALSE));
        checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, data, dataSize));

        if (rsSession->sessionType == RS_INTF_GPIB)
        {
            checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER LFEOI"));
        }
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (data)
        free(data);
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function exports the factory calibration data of the standards in
/// HIFN  the active calibration unit in Touchstone format into the given
/// HIFN  directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control sets the directory.
ViStatus _VI_FUNC rsvna_ExportFactoryCalibrationDataToTouchstoneFile(
                            ViSession instrSession,
                            ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_EXPORT_FACTORY_CALIBRATION_DATA_TO_TOUCHSTONE_FILE, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function exports an arbitrary (e.g. user-defined) set of
/// HIFN  calibration data of the standards in the active calibration unit in
/// HIFN  Touchstone format into the given directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control sets the directory.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the
/// HIPAR calibrationKitName/automatic calibration:
ViStatus _VI_FUNC rsvna_ExportUserDefinedCalibrationDataToTouchstoneFile(
                            ViSession instrSession,
                            ViString  directory,
                            ViString  calibrationKitName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (strncmp(calibrationKitName, """", 2) != 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:AKAL:USER:CONV '%s','%s'", directory, calibrationKitName);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:AKAL:USER:CONV '%s'", directory);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the stimulus values of the active calibration. A
/// HIFN  calibration must be selected before the function is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Stimulus
/// HIPAR bufferSize/Frequencies'.
/// HIPAR stimulusFrequencies/This control returns the stimulus frequencies.
/// HIPAR noOfValues/Returns number of stimulus frequencies.
ViStatus _VI_FUNC rsvna_QueryCalibrationStimulusFrequencies(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   bufferSize,
                                                            ViReal64  stimulusFrequencies[],
                                                            ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:STIM?", channel);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, stimulusFrequencies, noOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the stimulus values of the active calibration. A
/// HIFN  calibration must be selected before the function is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR bufferSize/This control defines the size of array passed to argument 'Stimulus
/// HIPAR bufferSize/Frequencies'.
/// HIPAR stimulusFrequencies/This control returns the stimulus frequencies.
/// HIPAR noOfValues/Returns number of stimulus frequencies.
ViStatus _VI_FUNC rsvna_QueryCalibrationStimulusFrequenciesPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   bufferSize,
                            ViReal64  stimulusFrequencies[],
                            ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:STIM:PORT%ld?", channel, physicalPort);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, stimulusFrequencies, noOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the parameters of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR frequencyStart/This control returns Frequency Start value.
/// HIPAR frequencyStop/This control returns Frequency Stop value.
/// HIPAR numberOfPoints/This control returns Number Of Points.
/// HIPAR internalSignalSourcePower/This control returns Initial Signal Source Power value.
/// HIPAR sweepType/This control returns Sweep Type.
ViStatus _VI_FUNC rsvna_QueryCalibrationDataParameters(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64* frequencyStart,
                                                       ViReal64* frequencyStop,
                                                       ViInt32*  numberOfPoints,
                                                       ViReal64* internalSignalSourcePower,
                                                       ViInt32*  sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  ext_buf = NULL;
    ViChar*  p2buf;
    ViString sweepTypeArr[] = {"LIN", "LOG", "SEGM", NULL};

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR?", channel);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2buf = strtok(ext_buf, ",");
    *frequencyStart = atof(p2buf);
    p2buf = strtok(NULL, ",");
    *frequencyStop = atof(p2buf);
    p2buf = strtok(NULL, ",");
    *numberOfPoints = atoi(p2buf);
    p2buf = strtok(NULL, ",");
    *internalSignalSourcePower = atof(p2buf);
    p2buf = strtok(NULL, ",\n\r");
    *sweepType = RsCore_FindStringIndex(sweepTypeArr, p2buf);

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (ext_buf)
        free(ext_buf);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the parameters of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR systemErrorCorrection/Channel number.
/// HIPAR type/Selects the type.
/// HIPAR index/If one or more external switch matrices were used during calibration,
/// HIPAR index/this refers to the index of a particular switch matrix.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR info/Returns the calibration informations.
ViStatus _VI_FUNC rsvna_QueryCalibrationDataParametersSpecific(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   systemErrorCorrection,
                            ViInt32   type,
                            ViInt32   index,
                            ViInt32   bufferSize,
                            ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2Type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationDataParameter, type, &p2Type),
                4, "Type");

    if (type == RSVNA_VAL_CAL_PARAM_MVNA || type == RSVNA_VAL_CAL_PARAM_MTEST)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR%ld? %s,%ld", channel, systemErrorCorrection, p2Type, index);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR%ld? %s", channel, systemErrorCorrection, p2Type);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the calibration data parameters count.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR parametersCount/Returns the calibration data parameters count.
ViStatus _VI_FUNC rsvna_QueryCalibrationDataParametersCount(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32*  parametersCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUERY_CALIBRATION_DATA_PARAMETERS_COUNT, parametersCount),
            3, "Parameters Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the parameters of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR frequencyStart/This control returns Frequency Start value.
/// HIPAR frequencyStop/This control returns Frequency Stop value.
/// HIPAR numberOfPoints/This control returns Number Of Points.
/// HIPAR internalSignalSourcePower/This control returns Initial Signal Source Power value.
/// HIPAR sweepType/This control returns Sweep Type.
ViStatus _VI_FUNC rsvna_QueryCalibrationDataParametersPort(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   physicalPort,
                                                           ViReal64* frequencyStart,
                                                           ViReal64* frequencyStop,
                                                           ViInt32*  numberOfPoints,
                                                           ViReal64* internalSignalSourcePower,
                                                           ViInt32*  sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  ext_buf = NULL;
    ViChar*  p2buf;
    ViString sweepTypeArr[] = {"LIN", "LOG", "SEGM", NULL};

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR:PORT%ld?", channel, physicalPort);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2buf = strtok(ext_buf, ",");
    *frequencyStart = atof(p2buf);
    p2buf = strtok(NULL, ",");
    *frequencyStop = atof(p2buf);
    p2buf = strtok(NULL, ",");
    *numberOfPoints = atoi(p2buf);
    p2buf = strtok(NULL, ",");
    *internalSignalSourcePower = atof(p2buf);
    p2buf = strtok(NULL, ",\n\r");
    *sweepType = RsCore_FindStringIndex(sweepTypeArr, p2buf);

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the parameters of the channel's calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR systemErrorCorrection/Channel number.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in
/// HIPAR physicalPort/arbitrary mode.
/// HIPAR type/Selects the type.
/// HIPAR index/If one or more external switch matrices were used during calibration,
/// HIPAR index/this refers to the index of a particular switch matrix.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR info/Returns the calibration informations.
ViStatus _VI_FUNC rsvna_QueryCalibrationDataParametersPortSpecific(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   systemErrorCorrection,
                            ViInt32   physicalPort,
                            ViInt32   type,
                            ViInt32   index,
                            ViInt32   bufferSize,
                            ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2Type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngCalibrationDataParameter, type, &p2Type),
                5, "Type");

    if (type == RSVNA_VAL_CAL_PARAM_MVNA || type == RSVNA_VAL_CAL_PARAM_MTEST)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR%ld:PORT%ld? %s,%ld", channel, systemErrorCorrection, physicalPort,
                 p2Type, index);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:DATA:PAR%ld:PORT%ld? %s", channel, systemErrorCorrection, physicalPort,
                 p2Type);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the calibration kit to be used for a specified
/// HIFN  connector type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connector/This control selects a connector type.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKit(ViSession instrSession,
                                                ViInt32   connector,
                                                ViString  calibrationKitName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSVNA_VAL_CONNECTOR_KIT_N50, RSVNA_VAL_CONNECTOR_KIT_SMA),
            2, "Connector");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SELECT_CALIBRATION_KIT,
            "ConnType", connector , RS_REPCAP_BUF_SIZE, repCap),
            2, "Connector");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SELECT_CALIBRATION_KIT, calibrationKitName),
            3, "Calibration Kit Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects and initiates a one-port, two-port, three-port or four-port
/// HIFN automatic calibration at arbitrary analyzer and calibration unit ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationKitName/Name and (possibly) directory of the cal kit file to be used for the automatic calibration.
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR calUnitPort1/Port number of the cal unit.
/// HIPAR calUnitPort2/Port number of the cal unit.
/// HIPAR calUnitPort3/Port number of the cal unit.
/// HIPAR calUnitPort4/Port number of the cal unit.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitAuto(ViSession instrSession,
                                                    ViString  calibrationKitName,
                                                    ViInt32   analyzerPort1,
                                                    ViInt32   analyzerPort2,
                                                    ViInt32   analyzerPort3,
                                                    ViInt32   analyzerPort4,
                                                    ViInt32   calUnitPort1,
                                                    ViInt32   calUnitPort2,
                                                    ViInt32   calUnitPort3,
                                                    ViInt32   calUnitPort4,
                                                    ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENSe:CORR:COLL:AUTO:CKIT:PORT '%s',%ld,%ld",
                           calibrationKitName,
                           analyzerPort1,
                           calUnitPort1);

    if (analyzerPort2 && calUnitPort2)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort2, calUnitPort2);
    }

    if (analyzerPort3 && calUnitPort3)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort3, calUnitPort3);
    }

    if (analyzerPort4 && calUnitPort4)
    {
        sprintf(p2buf, ",%ld,%ld", analyzerPort4, calUnitPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function extends an existing characterization of the active calibration
/// HIFN  unit (rsvna_ConfigureActiveCalibrationUnit) with additional,
/// HIFN  previously uncharacterized cal unit ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR characterization/Name and (possibly) directory of an existing characterization.
/// HIPAR analyzerPort1/Port number.
/// HIPAR analyzerPort2/Port number.
/// HIPAR analyzerPort3/Port number.
/// HIPAR analyzerPort4/Port number.
/// HIPAR calUnitPort1/Port number of the cal unit.
/// HIPAR calUnitPort2/Port number of the cal unit.
/// HIPAR calUnitPort3/Port number of the cal unit.
/// HIPAR calUnitPort4/Port number of the cal unit.
/// HIPAR timeout/This control sets the timeout for the operation to finish.
ViStatus _VI_FUNC rsvna_AddCalibrationKitAuto(ViSession instrSession,
                                              ViString  characterization,
                                              ViInt32   analyzerPort1,
                                              ViInt32   analyzerPort2,
                                              ViInt32   analyzerPort3,
                                              ViInt32   analyzerPort4,
                                              ViInt32   calUnitPort1,
                                              ViInt32   calUnitPort2,
                                              ViInt32   calUnitPort3,
                                              ViInt32   calUnitPort4,
                                              ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENSe:CORR:COLL:AUTO:CKIT:PORT:ADD '%s',%ld,%ld",
                           characterization,
                           analyzerPort1,
                           calUnitPort1);

    if (analyzerPort2 && calUnitPort2)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort2, calUnitPort2);
    }

    if (analyzerPort3 && calUnitPort3)
    {
        p2buf += sprintf(p2buf, ",%ld,%ld", analyzerPort3, calUnitPort3);
    }

    if (analyzerPort4 && calUnitPort4)
    {
        sprintf(p2buf, ",%ld,%ld", analyzerPort4, calUnitPort4);
    }

    checkErr(rsvna_GetVISATimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetVISATimeout(instrSession, timeout));

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (old_timeout != 0)
        checkErr(rsvna_SetVISATimeout(instrSession, old_timeout));
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the calibration kit to be used for a specified
/// HIFN  connector type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connector/This control sets a user-defined connector type.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitUserConnectorType(
                            ViSession instrSession,
                            ViString  connector,
                            ViString  calibrationKitName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:SEL '%s','%s'", connector, calibrationKitName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a list of all cal kits for a given connector
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connectorName/This control sets the name of the connector type. Use function
/// HIPAR connectorName/rszvb_CalibrationConnectorCatalog to query connector names.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
/// HIPAR catalog/Response is a string parameter with comma-separated list of all cal
/// HIPAR catalog/kits for a given connector type.
ViStatus _VI_FUNC rsvna_CalibrationKitCatalog(ViSession instrSession,
                                              ViString  connectorName,
                                              ViInt32   bufferSize,
                                              ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:CAT? '%s'", connectorName);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the parameters of a (possibly non-ideal) 1 port or 2-port
/// HIFN  calibration standard <StandardType> within a particular cal kit.
/// HIFN  Depending on the standard type, only a subset of the parameters may be
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standardType/Selects a 1- or 2-port calibration standard within a particular cal
/// HIPAR standardType/kit. Depending on the standard type, only a subset of the parameters
/// HIPAR standardType/may be used.
/// HIPAR connector/Selects a connector type. Required for all standard types.
/// HIPAR calKitName/This control specifies calibration kit name. If the specified cal kit
/// HIPAR calKitName/does not exist, it is created with the specified calibration standard.
/// HIPAR standardLabel/Additional string label for the standard, typically the standard's
/// HIPAR standardLabel/serial number. May be an empty string.
/// HIPAR minFreq/Min Freq and Max Freq define the rated frequency range in Hz of the
/// HIPAR minFreq/standard. During calibration, the analyzer checks whether the sweep
/// HIPAR minFreq/range is within the validity range of all measured standards and
/// HIPAR minFreq/possibly generates a warning.
/// HIPAR maxFreq/Min Freq and Max Freq define the rated frequency range in Hz of the
/// HIPAR maxFreq/standard. During calibration, the analyzer checks whether the sweep
/// HIPAR maxFreq/range is within the validity range of all measured standards and
/// HIPAR maxFreq/possibly generates a warning.
/// HIPAR delayParameter/Depending on the cal kit's model type (selected using
/// HIPAR delayParameter/rsvna_ConfigureCalibrationDelayMode) this parameter is interpreted as:
/// HIPAR loss/Loss (offset parameter) of the standard in dB. Not used for sliding
/// HIPAR loss/match or attenuation standards, required for all others.
/// HIPAR z0/Reference impedance in Ohms. Not used for sliding match or attenuation
/// HIPAR z0/standards, required for all others.
/// HIPAR capacitances/C0...C3: Polynomial coefficients for the fringing capacitance of the
/// HIPAR capacitances/standard (load parameter). Unit of <C_i> is fF / (GHz)^i.
/// HIPAR residualInductances/L0...L3: Polynomial coefficients for the residual inductance of the
/// HIPAR residualInductances/standard (load parameter). Unit of <L_i> is pH / (GHz)^i.
/// HIPAR loadCircuitModel/A load circuit model generally consists of a capacitance (modelled by
/// HIPAR loadCircuitModel/<C0>,...,<C3>) which is connected in parallel to an inductance
/// HIPAR loadCircuitModel/(modelled by <L0>,...,<L3>) and a resistance, both connected in series.
/// HIPAR loadCircuitModel/Open | Short | Match indicates a simplified modelling as an Open or
/// HIPAR loadCircuitModel/Short or Match standard. Resistance indicates the general load circuit
/// HIPAR loadCircuitModel/model, defined in the Resistance parameter.
/// HIPAR resistance/If Load Circuit Model is set to Resistance, this parameter defines the
/// HIPAR resistance/resistance of the general load circuit model.
/// HIPAR port1/Optional port restriction. Provide one port number for one port
/// HIPAR port1/standards, two for two port standards.
/// HIPAR port2/Optional port restriction. Defines the second port restriction for two
/// HIPAR port2/port standards.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationStandard(ViSession instrSession,
                                                     ViInt32   standardType,
                                                     ViInt32   connector,
                                                     ViString  calKitName,
                                                     ViString  standardLabel,
                                                     ViReal64  minFreq,
                                                     ViReal64  maxFreq,
                                                     ViReal64  delayParameter,
                                                     ViReal64  loss,
                                                     ViReal64  z0,
                                                     ViReal64  capacitances[],
                                                     ViReal64  residualInductances[],
                                                     ViInt32   loadCircuitModel,
                                                     ViReal64  resistance,
                                                     ViInt32   port1,
                                                     ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2Connector;
    ViChar*  p2CircuitModel;
    ViChar   cat_str[RS_MAX_MESSAGE_LEN] = "";
    ViChar   inductances_str[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONFIGURE_CALIBRATION_STANDARD,
            "StandardType", standardType , RS_REPCAP_BUF_SIZE, repCap),
            2, "Standard Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngCalibrationConnectorKitString, connector, &p2Connector),
                3, "Connector");
    viCheckParm(RsCore_InvalidStringLength(instrSession, calKitName, 1, -1),
                4, "Cal Kit Name");

    // Begin with mandatory parameters
    snprintf(value, RS_MAX_MESSAGE_LEN, "%s,'%s','%s',%.12lG,%.12lG",
             p2Connector,
             calKitName,
             standardLabel,
             minFreq,
             maxFreq);

    // Append parameters up to load circuit model and resistance
    switch (standardType)
    {
    case RSVNA_REPCAP_STANDARDTYPE_MSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_FSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_MMATTEN:
    case RSVNA_REPCAP_STANDARDTYPE_MFATTEN:
    case RSVNA_REPCAP_STANDARDTYPE_FFATTEN:
        break;
    default:
        snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG,%.12lG,%.12lG",
                 delayParameter,
                 loss,
                 z0);
        strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));

        if ((RSVNA_REPCAP_STANDARDTYPE_MMTHROUGH <= standardType) &&
            (standardType <= RSVNA_REPCAP_STANDARDTYPE_FFLINE3))
            break;

        // Capacitance and inductance arrays
        strcpy(inductances_str, "");
        for (ViInt32 i = 0; i < 4; i++)
        {
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", capacitances[i]);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", residualInductances[i]);
            strncat(inductances_str, cat_str, RS_MAX_MESSAGE_LEN - strlen(inductances_str));
        }
        strncat(value, inductances_str, RS_MAX_MESSAGE_LEN - strlen(value));

        // Load circuit model and resistance
        switch (standardType)
        {
        case RSVNA_REPCAP_STANDARDTYPE_MREFLECT:
        case RSVNA_REPCAP_STANDARDTYPE_FREFLECT:
        case RSVNA_REPCAP_STANDARDTYPE_MMSYMMNETWORK:
        case RSVNA_REPCAP_STANDARDTYPE_MFSYMMNETWORK:
        case RSVNA_REPCAP_STANDARDTYPE_FFSYMMNETWORK:
            if ((loadCircuitModel == RSVNA_VAL_CAL_CIRCUIT_NONE) ||
                (loadCircuitModel == RSVNA_VAL_CAL_CIRCUIT_MATCH))
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, loadCircuitModel), 13, "Load Circuit Model");
            break;
        }

        switch (loadCircuitModel)
        {
        case RSVNA_VAL_CAL_CIRCUIT_RESISTANCE:
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", resistance);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            break;
        default:
            viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                            &rsvna_rngCalibrationLoadCircuitModel, loadCircuitModel, &p2CircuitModel),
                        14, "Load Circuit Model");
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%s", p2CircuitModel);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            break;
        }
    }

    // Append port restrictions
    switch (standardType)
    {
    case RSVNA_REPCAP_STANDARDTYPE_MSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_FSLIDINGMATCH:
        break;
    default:
        if (port1 > 0)
        {
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%ld", port1);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));

            // For 2-port standards
            if (RSVNA_REPCAP_STANDARDTYPE_MMTHROUGH <= standardType)
            {
                if (port2 > port1)
                {
                    snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%ld", port2);
                    strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
                }
                else
                    viCheckParm(RsCore_InvalidViInt32Value(instrSession, port2), 16, "Port 2");
            }
        }
        break;
    }

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CONFIGURE_CALIBRATION_STANDARD, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the parameters of a calibration standard
/// HIFN  <std_type> for a specified connector type <conn_type>. A particular
/// HIFN  physical standard can be selected by specifying the name of the
/// HIFN  calibration kit and its serial number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connector/This control selects a connector type.
/// HIPAR standard/This control selects a standard from the selected calibration
/// HIPAR standard/kit.(N50, N75, SMA, PC7, PC3.5)
/// HIPAR kit/This control specifies calibration kit name.
/// HIPAR serialNumber/This control specifies serial number of the standard.
/// HIPAR min_Freq_Hz/This control specifies min. frequency of the standard.
/// HIPAR maxFreqHz/This control specifies max. frequency of the standard.
/// HIPAR length_mm/This control specifies electrical length of the standard.
/// HIPAR loss/This control specifies losses in dB/GHz.
/// HIPAR z0/This control specifies reference impedance
/// HIPAR approximation/This control specifies approximate modeling of the standard.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationStandardExt(ViSession instrSession,
                                                        ViInt32   connector,
                                                        ViInt32   standard,
                                                        ViString  kit,
                                                        ViString  serialNumber,
                                                        ViReal64  minFreqHz,
                                                        ViReal64  maxFreqHz,
                                                        ViReal64  lengthmm,
                                                        ViReal64  loss,
                                                        ViReal64  z0,
                                                        ViReal64  capacitances[],
                                                        ViReal64  residualInductances[],
                                                        ViInt32   approximation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSVNA_VAL_CONNECTOR_KIT_N50, RSVNA_VAL_CONNECTOR_KIT_SMA),
            2, "connector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, approximation, RSVNA_VAL_OPEN, RSVNA_VAL_MATCH),
            15, "Approximation");

    switch (standard)
    {
    case RSVNA_VAL_CALSTD_MMTH:
    case RSVNA_VAL_CALSTD_FFTH:
    case RSVNA_VAL_CALSTD_MFTH:
    case RSVNA_VAL_CALSTD_MMLI:
    case RSVNA_VAL_CALSTD_FFLI:
    case RSVNA_VAL_CALSTD_MFLI:
    case RSVNA_VAL_CALSTD_MMLI_2:
    case RSVNA_VAL_CALSTD_FFLI_2:
    case RSVNA_VAL_CALSTD_MFLI_2:
    case RSVNA_VAL_CALSTD_MMLI_3:
    case RSVNA_VAL_CALSTD_FFLI_3:
    case RSVNA_VAL_CALSTD_MFLI_3:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG,%.12lG,%.12lG,%.12lG",
                 connectorKitArr[connector],
                 loadCalKitStandardArr[standard],
                 kit,
                 serialNumber,
                 minFreqHz,
                 maxFreqHz,
                 lengthmm,
                 loss,
                 z0);
        break;
        //case RSVNA_VAL_CALSTD_OSH:
    case RSVNA_VAL_CALSTD_MOSH:
    case RSVNA_VAL_CALSTD_FOSH:
    case RSVNA_VAL_CALSTD_MOP:
    case RSVNA_VAL_CALSTD_FOP:
    case RSVNA_VAL_CALSTD_MSH:
    case RSVNA_VAL_CALSTD_FSH:
    case RSVNA_VAL_CALSTD_MOSH_2:
    case RSVNA_VAL_CALSTD_FOSH_2:
    case RSVNA_VAL_CALSTD_MOSH_3:
    case RSVNA_VAL_CALSTD_FOSH_3:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE,
                 "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG",
                 connectorKitArr[connector],
                 loadCalKitStandardArr[standard],
                 kit,
                 serialNumber,
                 minFreqHz,
                 maxFreqHz,
                 lengthmm,
                 loss,
                 z0,
                 capacitances[0],
                 capacitances[1],
                 capacitances[2],
                 capacitances[3],
                 residualInductances[0],
                 residualInductances[1],
                 residualInductances[2],
                 residualInductances[3]);
        break;
    case RSVNA_VAL_CALSTD_MMAT:
    case RSVNA_VAL_CALSTD_FFAT:
    case RSVNA_VAL_CALSTD_MFAT:
    case RSVNA_VAL_CALSTD_MSM:
    case RSVNA_VAL_CALSTD_FSM:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG",
                connectorKitArr[connector], loadCalKitStandardArr[standard], kit, serialNumber, minFreqHz, maxFreqHz);
        break;
    case RSVNA_VAL_CALSTD_MMTC:
    case RSVNA_VAL_CALSTD_FMTC:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE,
                 "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG",
                 connectorKitArr[connector],
                 loadCalKitStandardArr[standard],
                 kit,
                 serialNumber,
                 minFreqHz,
                 maxFreqHz,
                 loss,
                 z0,
                 capacitances[0],
                 capacitances[1],
                 capacitances[2],
                 capacitances[3],
                 residualInductances[0],
                 residualInductances[1],
                 residualInductances[2],
                 residualInductances[3]);
        break;
    case RSVNA_VAL_CALSTD_MREF:
    case RSVNA_VAL_CALSTD_FREF:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE,
                 "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%s",
                 connectorKitArr[connector],
                 loadCalKitStandardArr[standard],
                 kit,
                 serialNumber,
                 minFreqHz,
                 maxFreqHz,
                 lengthmm,
                 loss,
                 z0,
                 capacitances[0],
                 capacitances[1],
                 capacitances[2],
                 capacitances[3],
                 residualInductances[0],
                 residualInductances[1],
                 residualInductances[2],
                 residualInductances[3],
                 approxArr[approximation]);
        break;
    case RSVNA_VAL_CALSTD_MMSN:
    case RSVNA_VAL_CALSTD_FFSN:
    case RSVNA_VAL_CALSTD_MFSN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE,
                 "SENS:CORR:CKIT:%s:%s '%s','%s',%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG,%s",
                 connectorKitArr[connector],
                 loadCalKitStandardArr[standard],
                 kit,
                 serialNumber,
                 minFreqHz,
                 maxFreqHz,
                 loss,
                 z0,
                 capacitances[0],
                 capacitances[1],
                 capacitances[2],
                 capacitances[3],
                 residualInductances[0],
                 residualInductances[1],
                 residualInductances[2],
                 residualInductances[3],
                 approxArr[approximation]);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 3, "Standard");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a list of all standards in a given calibration
/// HIFN  kit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationKitName/This control sets the name of the cal kit. Use function
/// HIPAR calibrationKitName/rszvb_CalibrationKitCatalog to query cal kit names.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
/// HIPAR catalog/Response is a string parameter with comma-separated list of all
/// HIPAR catalog/standards in a given calibration kit.
ViStatus _VI_FUNC rsvna_CalibrationStandardsCatalog(ViSession instrSession,
                                                    ViString  calibrationKitName,
                                                    ViInt32   bufferSize,
                                                    ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:STAN:CAT? '%s'", calibrationKitName);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a label to an imported or user-defined
/// HIFN  calibration kit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationKitName/This control defines the name of the imported or user-defined
/// HIPAR calibrationKitName/calibration kit.
/// HIPAR label/This control defines the label of the calibration kit.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitLabel(ViSession instrSession,
                                                     ViString  calibrationKitName,
                                                     ViString  label)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LAB '%s','%s'", calibrationKitName, label);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a password to enable a single password-protected action.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR password/This control defines the password.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitPassword(ViSession instrSession,
                                                        ViString  password)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_CALIBRATION_KIT_PASSWORD, password),
            2, "password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the delay mode for the related cal kit (identified
/// HIFN  by connector type, name and label), i.e allows to toggle between ZVR
/// HIFN  compatible and Agilent modelling. Subsequent standard definitions
/// HIFN  interpret the specified <DelayParam> accordingly.
/// HIFN     If a cal kit with the given connector type, name and label is not
/// HIFN  available on the analyzer, it is created automatically. Cal kits that
/// HIFN  are not created/modified using this function use the ZVR compatible
/// HIFN  modelling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectorName/This control specifies the name of the connector type. Use
/// HIPAR connectorName/rsvna_ConnectorTypesCatalog to query connector names.
/// HIPAR calibrationKitName/This control specifies the name of a calibration kit available on the
/// HIPAR calibrationKitName/analyzer.
/// HIPAR calibrationKitLabel/This control a label for the cal kit data. An empty string means that
/// HIPAR calibrationKitLabel/no label is defined.
/// HIPAR delayMode/Sets the delay mode for the related cal kit.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationDelayMode(ViSession instrSession,
                                                      ViString  connectorName,
                                                      ViString  calibrationKitName,
                                                      ViString  calibrationKitLabel,
                                                      ViInt32   delayMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delayMode, RSVNA_VAL_AGILENT, RSVNA_VAL_ZVR),
            5, "Delay Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:DMOD '%s','%s','%s',%s", connectorName,
             calibrationKitName, calibrationKitLabel, delayModeArr[delayMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function generates a characterization (cal kit file) with the specified
/// HIFN name containing the cal kit data of the active calibration unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Name and (possibly) directory of the created characterization/cal kit file.
ViStatus _VI_FUNC rsvna_SaveCalibrationKit(ViSession instrSession,
                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_SAVE_CALIBRATION_KIT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes an imported or user-defined cal kit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
ViStatus _VI_FUNC rsvna_DeleteCalibrationKit(ViSession instrSession,
                                             ViString  calibrationKitName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DELETE_CALIBRATION_KIT, calibrationKitName),
            2, "Calibration Kit Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads cal kit data for a specific connector type, cal
/// HIFN  kit, and calibration standard from a specified Touchstone file,
/// HIFN  assigning a label for the cal data. A restriction on the port
/// HIFN  assignment may be defined in addition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connectorName/This control specifies the name of the connector type.
/// HIPAR calibrationKitName/This control specifies the name of a calibration kit available on the
/// HIPAR calibrationKitName/analyzer.
/// HIPAR standard/This control selects a calibration standard from a specified
/// HIPAR standard/Touchstone file.
/// HIPAR calibrationKitLabel/This control a label for the cal kit data. An empty string means that
/// HIPAR calibrationKitLabel/no label is defined.
/// HIPAR fileName/This control specifies the name and directory of the Touchstone file
/// HIPAR fileName/to be loaded. A *.s1p file must be used for one-port standards, a *.s2p
/// HIPAR fileName/file for two-port standards. If no path is specified the analyzer
/// HIPAR fileName/searches the current directory, to be queried with MMEMory:CDIRectory?.
/// HIPAR portNumber1/This control specifies the port assignment: One port number for
/// HIPAR portNumber1/one-port standards, two port number for two-port standards. If the port
/// HIPAR portNumber1/numbers are omitted, the cal kit data is valid for all ports.
/// HIPAR portNumber2/This control specifies the port assignment: One port number for
/// HIPAR portNumber2/one-port standards, two port number for two-port standards. If the port
/// HIPAR portNumber2/numbers are omitted, the cal kit data is valid for all ports.
ViStatus _VI_FUNC rsvna_LoadCalibrationKit(ViSession instrSession,
                                           ViString  connectorName,
                                           ViString  calibrationKitName,
                                           ViInt32   standard,
                                           ViString  calibrationKitLabel,
                                           ViString  fileName,
                                           ViInt32   portNumber1,
                                           ViInt32   portNumber2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (portNumber2 != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CKIT:SDAT '%s','%s',%s,'%s','%s',%ld,%ld",
                 connectorName, calibrationKitName, loadCalKitStandardArr[standard],
                 calibrationKitLabel, fileName, portNumber1, portNumber2);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CKIT:SDAT '%s','%s',%s,'%s','%s',%ld",
                connectorName, calibrationKitName, loadCalKitStandardArr[standard], calibrationKitLabel, fileName, portNumber1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows to load cal kit data by gender from the given
/// HIFN  file and to the given ports or all ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationKitFile/This control sets the path to the cal kit file, either absolute or
/// HIPAR calibrationKitFile/relative to the current directory (see MMEMory:CDIRectory)
/// HIPAR gender/This control selects a gender.
/// HIPAR portSelector/This control selects whether cal kit is loaded for all ports or given
/// HIPAR portSelector/ports.
/// HIPAR arraySize/Sets the number of elements of the Ports array.
/// HIPAR ports/This control specifies the ports for cal kit. The number of elements
/// HIPAR ports/should correspond to Array Size number.
ViStatus _VI_FUNC rsvna_LoadCalibrationKitByGenderFromFile(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViString  calibrationKitFile,
                                                           ViInt32   gender,
                                                           ViInt32   portSelector,
                                                           ViInt32   arraySize,
                                                           ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gender, RSVNA_VAL_GENDER_NGENDER, RSVNA_VAL_GENDER_FEMALE),
            4, "Gender");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portSelector, RSVNA_VAL_ALL_PORTS, RSVNA_VAL_SELECTED_PORTS),
            5, "Port Selector");

    if (portSelector == RSVNA_VAL_ALL_PORTS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:CKIT:INST '%s',%s,ALL", channel,
                 calibrationKitFile, genderArr[gender]);
    }
    else
    {
        p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:CKIT:INST '%s',%s", channel,
                               calibrationKitFile, genderArr[gender]);

        for (i = 0; i < arraySize; i++)
        {
            p2buf += sprintf(p2buf, ",%ld", ports[i]);
        }

        sprintf(p2buf, "\n");
    }
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows to load cal kit data by gender from the given
/// HIFN  file and to the given ports or all ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR calibrationKitName/This control defines the name of the calibration kit.
/// HIPAR label/This control defines the label of the calibration kit.
/// HIPAR gender/This control selects a gender.
/// HIPAR portSelector/This control selects whether cal kit is loaded for all ports or given
/// HIPAR portSelector/ports.
/// HIPAR arraySize/Sets the number of elements of the Ports array.
/// HIPAR ports/This control specifies the ports for cal kit. The number of elements
/// HIPAR ports/should correspond with Array Size number.
ViStatus _VI_FUNC rsvna_LoadCalibrationKitByGenderFromPool(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViString  calibrationKitName,
                                                           ViString  label,
                                                           ViInt32   gender,
                                                           ViInt32   portSelector,
                                                           ViInt32   arraySize,
                                                           ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gender, RSVNA_VAL_GENDER_NGENDER, RSVNA_VAL_GENDER_FEMALE),
            5, "Gender");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portSelector, RSVNA_VAL_ALL_PORTS, RSVNA_VAL_SELECTED_PORTS),
            6, "Port Selector");

    if (portSelector == RSVNA_VAL_ALL_PORTS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:CKIT:LOAD '%s',%s,%s,ALL", channel,
                 calibrationKitName, label, genderArr[gender]);
    }
    else
    {
        p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:CKIT:LOAD '%s',%s,%s", channel,
                               calibrationKitName, label, genderArr[gender]);

        for (i = 0; i < arraySize; i++)
        {
            p2buf += sprintf(p2buf, ",%ld", ports[i]);
        }

        sprintf(p2buf, "\n");
    }
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the cal kit data assigned to the given port via
/// HIFN  rsvna_LoadCalibrationKitByGenderFromFile or
/// HIFN  rsvna_LoadCalibrationKitByGenderFromPool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR detail/This control selects a data to be queried.
/// HIPAR result/This control returns the value of the cal kit depending on the
/// HIPAR result/selected detail.
/// HIPAR port/Port number.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Result' parameter.
ViStatus _VI_FUNC rsvna_QueryCalibrationKitData(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   port,
                                                ViInt32   detail,
                                                ViInt32   bufferSize,
                                                ViChar    result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detail, RSVNA_VAL_CONNECTOR, RSVNA_VAL_GENDER),
            4, "Detail");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:CKIT:PORT%ld? %s", channel, port, detailArr[detail]);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads cal kit data from a specified NVA cal kit file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be loaded. If no path is specified the analyzer searches the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_ImportKit(ViSession instrSession,
                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_IMPORT_CAL_KIT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the data of a calibration kit to a specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR kitName/This control specifies the name of a user-defined calibration kit
/// HIPAR kitName/available on the analyzer. If no path is specified the analyzer uses
/// HIPAR kitName/the current directory, to be queried with MMEMory:CDIRectory?.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be created. The file is a NWA-specific cal kit file with the extension
/// HIPAR fileName/*.calkit.
ViStatus _VI_FUNC rsvna_ExportKit(ViSession instrSession,
                                  ViString  kitName,
                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:CKIT '%s','%s'", kitName, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries type, name, label and gender of a cal kit
/// HIFN  defined in the specified cal kit file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calKitFile/This control sets the path to the cal kit file, either absolute or
/// HIPAR calKitFile/relative to the current directory (see MMEMory:CDIRectory)
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 'Cal
/// HIPAR bufferSize/Kit Name' and 'Cal Kit Label' parameter.
/// HIPAR connectorType/This control returns a connector type.
/// HIPAR calKitName/This control returns the name of a calibration kit.
/// HIPAR calKitLabel/This control defines the label of the calibration kit.
/// HIPAR calKitGender/This control returns a cal kit gender.
ViStatus _VI_FUNC rsvna_QueryCalibrationKitInfo(ViSession instrSession,
                                                ViString  calKitFile,
                                                ViInt32   bufferSize,
                                                ViChar    connectorType[],
                                                ViChar    calKitName[],
                                                ViChar    calKitLabel[],
                                                ViInt32*  calKitGender)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  result = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CKIT:INFO? '%s'", calKitFile);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &result));
    checkErr(RsCore_GetViStringAtCsvIndex(result, 0, VI_TRUE, bufferSize, connectorType));
    checkErr(RsCore_GetViStringAtCsvIndex(result, 1, VI_TRUE, bufferSize, calKitName));
    checkErr(RsCore_GetViStringAtCsvIndex(result, 2, VI_TRUE, bufferSize, calKitLabel));
    checkErr(RsCore_GetViInt32AtCsvIndex(result, 3, calKitGender));

Error:
    if (result)
        free(result);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the calibration kit to be used for a specified
/// HIFN  connector type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/This control selects a connector type.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
/// HIPAR calibrationKitLabel/This control defines the label of a calibration kit available
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitWithLabel(ViSession instrSession,
                                                         ViInt32   connector,
                                                         ViString  calibrationKitName,
                                                         ViString  calibrationKitLabel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:LSEL '%s','%s'", connectorKitArr[connector], calibrationKitName, calibrationKitLabel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the calibration kit to be used for a specified
/// HIFN  connector type.
/// HIFN     Note(s):
/// HIFN     (1) The function is suitable for connector types with arbitrary,
/// HIFN  user-defined names. For standard connector types you can use the
/// HIFN  command [SENSe<Ch>:]CORRection:CKIT:<conn_type>:LSELect (function
/// HIFN  rsvna_ConfigureCalibrationKitWithLabel)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/This control sets a user-defined connector type.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
/// HIPAR calibrationKitLabel/This control defines the label of a calibration kit available
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitUserConnectorTypeWithLabel(
                            ViSession instrSession,
                            ViString  connector,
                            ViString  calibrationKitName,
                            ViString  calibrationKitLabel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LSEL '%s','%s','%s'", connector, calibrationKitName, calibrationKitLabel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a list of all cal kits and their labels for a
/// HIFN  given connector type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectorName/This control sets the name of the connector type. Use function
/// HIPAR connectorName/rsvna_ConnectorTypesCatalog to query connector names.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
/// HIPAR catalog/Response is a string parameter with comma-separated list of all cal
/// HIPAR catalog/kits and their labels for a given connector type.
ViStatus _VI_FUNC rsvna_CalibrationKitCatalogWithLabel(ViSession instrSession,
                                                       ViString  connectorName,
                                                       ViInt32   bufferSize,
                                                       ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LCAT? '%s'", connectorName);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the parameters of a non-ideal 1 port or 2-port calibration
/// HIFN  standard <StandardType>, where a particular cal kit can be addressed by
/// HIFN  name and label. Depending on the standard type, only a subset of the
/// HIFN  parameters may be used.
/// HIFN     Apart from the additional <CalKitLabel> parameter, the syntax and
/// HIFN  semantics of this command is identical to
/// HIFN  rsvna_ConfigureCalibrationStandard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standardType/Selects a 1- or 2-port calibration standard within a particular cal
/// HIPAR standardType/kit. Depending on the standard type, only a subset of the parameters
/// HIPAR standardType/may be used.
/// HIPAR connector/Selects a connector type. Required for all standard types.
/// HIPAR calKitName/This control specifies calibration kit name. If the specified cal kit
/// HIPAR calKitName/does not exist, it is created with the specified calibration standard.
/// HIPAR calKitLabel/A label which identifies the calibration kit in combination with the
/// HIPAR calKitLabel/<CalKitName>. Typically, the serial number of the calibration kit
/// HIPAR calKitLabel/serves as a calibration kit label.
/// HIPAR standardLabel/Additional string label for the standard, typically the standard's
/// HIPAR standardLabel/serial number. May be an empty string.
/// HIPAR minFreq/Min Freq and Max Freq define the rated frequency range in Hz of the
/// HIPAR minFreq/standard. During calibration, the analyzer checks whether the sweep
/// HIPAR minFreq/range is within the validity range of all measured standards and
/// HIPAR minFreq/possibly generates a warning.
/// HIPAR maxFreq/Min Freq and Max Freq define the rated frequency range in Hz of the
/// HIPAR maxFreq/standard. During calibration, the analyzer checks whether the sweep
/// HIPAR maxFreq/range is within the validity range of all measured standards and
/// HIPAR maxFreq/possibly generates a warning.
/// HIPAR delayParameter/Depending on the cal kit's model type (selected using
/// HIPAR delayParameter/rsvna_ConfigureCalibrationDelayMode) this parameter is interpreted as:
/// HIPAR loss/Loss (offset parameter) of the standard in dB. Not used for sliding
/// HIPAR loss/match or attenuation standards, required for all others.
/// HIPAR z0/Reference impedance in Ohms. Not used for sliding match or attenuation
/// HIPAR z0/standards, required for all others.
/// HIPAR capacitances/C0...C3: Polynomial coefficients for the fringing capacitance of the
/// HIPAR capacitances/standard (load parameter). Unit of <C_i> is fF / (GHz)^i. The array
/// HIPAR capacitances/must contain a total of 4 values {C0,C1,C2,C3}.
/// HIPAR residualInductances/L0...L3: Polynomial coefficients for the residual inductance of the
/// HIPAR residualInductances/standard (load parameter). Unit of <L_i> is pH / (GHz)^i. The array
/// HIPAR residualInductances/must contain a total of 4 values {L0,L1,L2,L3}.
/// HIPAR loadCircuitModel/A load circuit model generally consists of a capacitance (modelled by
/// HIPAR loadCircuitModel/<C0>,...,<C3>) which is connected in parallel to an inductance
/// HIPAR loadCircuitModel/(modelled by <L0>,...,<L3>) and a resistance, both connected in series.
/// HIPAR loadCircuitModel/Open | Short | Match indicates a simplified modelling as an Open or
/// HIPAR loadCircuitModel/Short or Match standard. Resistance indicates the general load circuit
/// HIPAR loadCircuitModel/model, defined in the <Resistance> parameter.
/// HIPAR resistance/If Load Circuit Model is set to Resistance, this parameter defines the
/// HIPAR resistance/resistance of the general load circuit model.
/// HIPAR port1/Optional port restriction. Provide one port number for one port
/// HIPAR port1/standards, two for two port standards. Set this to 0 for no port
/// HIPAR port1/restriction.
/// HIPAR port2/Optional port restriction. Defines the second port restriction for two
/// HIPAR port2/port standards. Must be greater than the value provided for <Port 1>.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationStandardWithLabel(
                            ViSession instrSession,
                            ViInt32   standardType,
                            ViInt32   connector,
                            ViString  calKitName,
                            ViString  calKitLabel,
                            ViString  standardLabel,
                            ViReal64  minFreq,
                            ViReal64  maxFreq,
                            ViReal64  delayParameter,
                            ViReal64  loss,
                            ViReal64  z0,
                            ViReal64  capacitances[],
                            ViReal64  residualInductances[],
                            ViInt32   loadCircuitModel,
                            ViReal64  resistance,
                            ViInt32   port1,
                            ViInt32   port2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2Connector;
    ViChar*  p2CircuitModel;
    ViChar   cat_str[RS_MAX_MESSAGE_LEN] = "";
    ViChar   inductances_str[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (calKitLabel == NULL)
    {
        checkErr(rsvna_ConfigureCalibrationStandard(instrSession, standardType,
            connector, calKitName, /*calKitLabel,*/ standardLabel, minFreq, maxFreq,
            delayParameter, loss, z0, capacitances, residualInductances,
            loadCircuitModel, resistance, port1, port2));
        goto Error;
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CONFIGURE_CALIBRATION_STANDARD_WITH_LABEL,
            "StandardType", standardType , RS_REPCAP_BUF_SIZE, repCap),
            2, "Standard Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngCalibrationConnectorKitString, connector, &p2Connector),
                3, "Connector");
    viCheckParm(RsCore_InvalidStringLength(instrSession, calKitName, 1, -1),
                4, "Cal Kit Name");

    // Begin with mandatory parameters
    snprintf(value, RS_MAX_MESSAGE_LEN, "%s,'%s','%s','%s',%.12lG,%.12lG",
             p2Connector,
             calKitName,
             calKitLabel,
             standardLabel,
             minFreq,
             maxFreq);

    // Append parameters up to load circuit model and resistance
    switch (standardType)
    {
    case RSVNA_REPCAP_STANDARDTYPE_MSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_FSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_MMATTEN:
    case RSVNA_REPCAP_STANDARDTYPE_MFATTEN:
    case RSVNA_REPCAP_STANDARDTYPE_FFATTEN:
        break;
    default:
        snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG,%.12lG,%.12lG",
                 delayParameter,
                 loss,
                 z0);
        strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));

        if ((RSVNA_REPCAP_STANDARDTYPE_MMTHROUGH <= standardType) &&
            (standardType <= RSVNA_REPCAP_STANDARDTYPE_FFLINE3))
            break;

        // Capacitance and inductance arrays
        strcpy(inductances_str, "");
        for (ViInt32 i = 0; i < 4; i++)
        {
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", capacitances[i]);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", residualInductances[i]);
            strncat(inductances_str, cat_str, RS_MAX_MESSAGE_LEN - strlen(inductances_str));
        }
        strncat(value, inductances_str, RS_MAX_MESSAGE_LEN - strlen(value));

        // Load circuit model and resistance
        switch (standardType)
        {
        case RSVNA_REPCAP_STANDARDTYPE_MREFLECT:
        case RSVNA_REPCAP_STANDARDTYPE_FREFLECT:
        case RSVNA_REPCAP_STANDARDTYPE_MMSYMMNETWORK:
        case RSVNA_REPCAP_STANDARDTYPE_MFSYMMNETWORK:
        case RSVNA_REPCAP_STANDARDTYPE_FFSYMMNETWORK:
            if ((loadCircuitModel == RSVNA_VAL_CAL_CIRCUIT_NONE) || (loadCircuitModel == RSVNA_VAL_CAL_CIRCUIT_MATCH))
            {
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, loadCircuitModel), 14, "Load Circuit Model");
            }
            break;
        }

        switch (loadCircuitModel)
        {
        case RSVNA_VAL_CAL_CIRCUIT_RESISTANCE:
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%.12lG", resistance);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            break;
        default:
            viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                            &rsvna_rngCalibrationLoadCircuitModel, loadCircuitModel, &p2CircuitModel),
                        14, "Load Circuit Model");
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%s", p2CircuitModel);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
            break;
        }
    }

    // Append port restrictions
    switch (standardType)
    {
    case RSVNA_REPCAP_STANDARDTYPE_MSLIDINGMATCH:
    case RSVNA_REPCAP_STANDARDTYPE_FSLIDINGMATCH:
        break;
    default:
        if (port1 > 0)
        {
            snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%ld", port1);
            strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));

            // For 2-port standards
            if (RSVNA_REPCAP_STANDARDTYPE_MMTHROUGH <= standardType)
            {
                if (port2 > port1)
                {
                    snprintf(cat_str, RS_MAX_MESSAGE_LEN, ",%ld", port2);
                    strncat(value, cat_str, RS_MAX_MESSAGE_LEN - strlen(value));
                }
                else
                {
                    viCheckParm(RsCore_InvalidViInt32Value(instrSession, port2), 17, "Port 2");
                }
            }
        }
        break;
    }

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CONFIGURE_CALIBRATION_STANDARD_WITH_LABEL, value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns a list of all standards in a given calibration
/// HIFN  kit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationKitName/This control sets the name of the cal kit. Use function
/// HIPAR calibrationKitName/rsvna_CalibrationKitCatalogWithLabel to query cal kit names.
/// HIPAR calibrationKitLabel/This control defines the label of a calibration kit available
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
/// HIPAR catalog/Response is a string parameter with comma-separated list of all
/// HIPAR catalog/standards in a given calibration kit.
ViStatus _VI_FUNC rsvna_CalibrationStandardsCatalogWithLabel(ViSession instrSession,
                                                             ViString  calibrationKitName,
                                                             ViString  calibrationKitLabel,
                                                             ViInt32   bufferSize,
                                                             ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:STAN:LCAT? '%s','%s'", calibrationKitName, calibrationKitLabel);
    checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a calibration kit label - usually the serial
/// HIFN  number - to an imported or user-defined calibration kit or renames an
/// HIFN  existing calibration kit label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationKitName/This control defines the name of the imported or user-defined
/// HIPAR calibrationKitName/calibration kit.
/// HIPAR calibrationKitLabel/This control defines the current label, usually the serial number of
/// HIPAR calibrationKitLabel/the calibration kit.
/// HIPAR newLabel/This control defines the new calibration kit label.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationKitLabelWithLabel(
                            ViSession instrSession,
                            ViString  calibrationKitName,
                            ViString  calibrationKitLabel,
                            ViString  newLabel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (newLabel == NULL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LLAB '%s','%s'", calibrationKitName, calibrationKitLabel);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LLAB '%s','%s','%s'", calibrationKitName, calibrationKitLabel, newLabel);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes an imported or user-defined cal kit including
/// HIFN  label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationKitName/This control defines the name of calibration kit.
/// HIPAR calibrationKitLabel/This control defines the label of an imported or user-defined
/// HIPAR calibrationKitLabel/calibration kit available on the analyzer.
ViStatus _VI_FUNC rsvna_DeleteCalibrationKitWithLabel(ViSession instrSession,
                                                      ViString  calibrationKitName,
                                                      ViString  calibrationKitLabel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:LDEL '%s','%s'", calibrationKitName, calibrationKitLabel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads cal kit data for a specific connector type, cal
/// HIFN  kit, and calibration standard from a specified Touchstone file,
/// HIFN  assigning a label for the cal data. A restriction on the port
/// HIFN  assignment may be defined in addition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectorName/This control specifies the name of the connector type.
/// HIPAR calibrationKitName/This control specifies the name of a calibration kit available on the
/// HIPAR calibrationKitName/analyzer.
/// HIPAR calibrationKitLabel/This control defines the label of a calibration kit available
/// HIPAR standard/This control selects a calibration standard from a specified
/// HIPAR standard/Touchstone file.
/// HIPAR standardLabel/String parameter addressing a particular calibration standard by
/// HIPAR fileName/This control specifies the name and directory of the Touchstone file
/// HIPAR fileName/to be loaded. A *.s1p file must be used for one-port standards, a *.s2p
/// HIPAR fileName/file for two-port standards. If no path is specified the analyzer
/// HIPAR fileName/searches the current directory, to be queried with MMEMory:CDIRectory?.
/// HIPAR portNumber1/This control specifies the port assignment: One port number for
/// HIPAR portNumber1/one-port standards, two port number for two-port standards. If the port
/// HIPAR portNumber1/numbers are omitted, the cal kit data is valid for all ports.
/// HIPAR portNumber2/This control specifies the port assignment: One port number for
/// HIPAR portNumber2/one-port standards, two port number for two-port standards. If the port
/// HIPAR portNumber2/numbers are omitted, the cal kit data is valid for all ports.
ViStatus _VI_FUNC rsvna_LoadCalibrationKitWithLabel(ViSession instrSession,
                                                    ViString  connectorName,
                                                    ViString  calibrationKitName,
                                                    ViString  calibrationKitLabel,
                                                    ViInt32   standard,
                                                    ViString  standardLabel,
                                                    ViString  fileName,
                                                    ViInt32   portNumber1,
                                                    ViInt32   portNumber2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (portNumber2 != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CKIT:SDAT:WLAB '%s','%s','%s',%s,'%s','%s',%ld,%ld",
                 connectorName, calibrationKitName, calibrationKitLabel,
                 loadCalKitStandardArr[standard], standardLabel,
                 fileName, portNumber1, portNumber2);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:CKIT:SDAT:WLAB '%s','%s',%s,'%s','%s','%s',%ld", connectorName, calibrationKitName, calibrationKitLabel,
                 loadCalKitStandardArr[standard], standardLabel,
                 fileName, portNumber1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the S-parameter data for a particular cal kit
/// HIFN  standard previously loaded from Touchstone file using
/// HIFN  rsvna_LoadCalibrationKit or rsvna_LoadCalibrationKitWithLabel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/This control selects a calibration standard from a specified
/// HIPAR standard/Touchstone file.
/// HIPAR connectorName/This control specifies the name of the connector type.
/// HIPAR calibrationKitName/This control specifies the name of a calibration kit available on the
/// HIPAR calibrationKitName/analyzer.
/// HIPAR calibrationKitLabel/This control defines the label of a calibration kit available
/// HIPAR sParameter/S-parameter of the 2-port standard. Two-port standards are either MM =
/// HIPAR sParameter/connector/connector, FF = socket/socket or MF = connector/socket.
/// HIPAR portNumber1/Numbers of the physical ports for which the S-parameter data is
/// HIPAR portNumber2/Numbers of the physical ports for which the S-parameter data is
/// HIPAR arraySize/This control sets the number of values returned in array. The array
/// HIPAR arraySize/size has to be at least equal to this number.
/// HIPAR traceData/Returns array of the S-parameter data.
/// HIPAR noofValues/Returns number of taken trace data values.
ViStatus _VI_FUNC rsvna_ReadCalibrationKitWithLabelSParameterData(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  connectorName,
                            ViString  calibrationKitName,
                            ViString  calibrationKitLabel,
                            ViInt32   sParameter,
                            ViInt32   portNumber1,
                            ViInt32   portNumber2,
                            ViInt32   arraySize,
                            ViReal64  traceData[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (standard)
    {
    case RSVNA_VAL_CALSTD_MMTH:
    case RSVNA_VAL_CALSTD_FFTH:
    case RSVNA_VAL_CALSTD_MFTH:
    case RSVNA_VAL_CALSTD_MMLI:
    case RSVNA_VAL_CALSTD_FFLI:
    case RSVNA_VAL_CALSTD_MFLI:
    case RSVNA_VAL_CALSTD_MMLI_2:
    case RSVNA_VAL_CALSTD_FFLI_2:
    case RSVNA_VAL_CALSTD_MFLI_2:
    case RSVNA_VAL_CALSTD_MMLI_3:
    case RSVNA_VAL_CALSTD_FFLI_3:
    case RSVNA_VAL_CALSTD_MFLI_3:
    case RSVNA_VAL_CALSTD_MMAT:
    case RSVNA_VAL_CALSTD_FFAT:
    case RSVNA_VAL_CALSTD_MFAT:
    case RSVNA_VAL_CALSTD_MMSN:
    case RSVNA_VAL_CALSTD_FFSN:
    case RSVNA_VAL_CALSTD_MFSN:
        if (portNumber1 != 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:WLAB:SDAT? '%s','%s','%s',%s,%ld,%ld",
                     loadCalKitStandardArr[standard], connectorName, calibrationKitName,
                     calibrationKitLabel, sParameterArr[sParameter], portNumber1, portNumber2);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:WLAB:SDAT? '%s','%s','%s',%s",
                     loadCalKitStandardArr[standard], connectorName, calibrationKitName,
                     calibrationKitLabel, sParameterArr[sParameter]);
        break;

    case RSVNA_VAL_CALSTD_MOSH:
    case RSVNA_VAL_CALSTD_FOSH:
    case RSVNA_VAL_CALSTD_MOSH_2:
    case RSVNA_VAL_CALSTD_FOSH_2:
    case RSVNA_VAL_CALSTD_MOSH_3:
    case RSVNA_VAL_CALSTD_FOSH_3:
    case RSVNA_VAL_CALSTD_MOP:
    case RSVNA_VAL_CALSTD_FOP:
    case RSVNA_VAL_CALSTD_MSH:
    case RSVNA_VAL_CALSTD_FSH:
    case RSVNA_VAL_CALSTD_MREF:
    case RSVNA_VAL_CALSTD_FREF:
    case RSVNA_VAL_CALSTD_MMTC:
    case RSVNA_VAL_CALSTD_FMTC:
    case RSVNA_VAL_CALSTD_MSM:
    case RSVNA_VAL_CALSTD_FSM:
        if (portNumber1 != 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:WLAB:SDAT? '%s','%s','%s',%ld",
                     loadCalKitStandardArr[standard], connectorName, calibrationKitName,
                     calibrationKitLabel, portNumber1);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:CKIT:%s:WLAB:SDAT? '%s','%s','%s'",
                     loadCalKitStandardArr[standard], connectorName, calibrationKitName,
                     calibrationKitLabel);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 2, "Standard");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, traceData, noofValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the data of a calibration kit to a specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR kitName/This control specifies the name of a user-defined calibration kit
/// HIPAR kitName/available on the analyzer. If no path is specified the analyzer uses
/// HIPAR kitName/the current directory, to be queried with MMEMory:CDIRectory?.
/// HIPAR calkitLabel/This control specifies the label (e.g. the serial number) of the
/// HIPAR calkitLabel/calibration kit.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be created. The file is a NWA-specific cal kit file with the extension
/// HIPAR fileName/*.calkit.
ViStatus _VI_FUNC rsvna_ExportKitWithLabel(ViSession instrSession,
                                           ViString  kitName,
                                           ViString  calkitLabel,
                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:CKIT:WLAB '%s','%s','%s'", kitName, calkitLabel, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source for the source power calibration, starts and
/// HIFN  applies the source power calibration.
/// HIFN     The function cannot be used unless a power meter is connected via GPIB
/// HIFN  bus, USB or LAN interface and configured in the "External Power
/// HIFN  Meters".
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_SOURCE_POWER_CALIBRATION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce<Ch>:POWer<PhyPt>:CORRection[:ACQuire]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR sourceType/Selects the source type
/// HIPAR sourcePort/Analyzer port or generator port number, if 'Port' or 'Generator' is
/// HIPAR sourcePort/the first parameter.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibration(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   sourceType,
                                                        ViInt32   sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", sourceTypeArr[sourceType], sourcePort);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the source power calibration. The attribute is
/// HIFN  valid for all channels, ports and external generators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR state/Enables or disables the source power calibration. The control is valid
/// HIPAR state/for all channels, ports and external generators.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerFlatnessCalibrationState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_PORT_FLATNESS_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_PORT_FLATNESS_CALIBRATION, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the source calibration power settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR totalReadings/Sets a limit for the number of calibration sweeps in the source power calibration.
/// HIPAR tolerance/This control sets the tolerance value.
/// HIPAR convergence/Specifies the convergence factor for a source power calibration.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerFlatnessCalibration(
                            ViSession instrSession,
                            ViInt32   totalReadings,
                            ViReal64  tolerance,
                            ViReal64  convergence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_NUMBER_OF_READINGS, totalReadings),
            2, "Total Readings");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_TOLERANCE, tolerance),
            3, "Tolerance");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_CONVERGENCE, convergence),
            4, "Convergence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source power calibration method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR method/Selects the source power calibration method.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerFlatnessCalibrationMethod(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   method)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_CALIBRATION_METHOD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_POWER_CALIBRATION_METHOD, method),
            3, "Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects an external power meter for the source power calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR externalPowerMeterNumber/Selects an external power meter for the source power calibration.
/// HIPAR reserved/Reserved for future use.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationPowerMeter(
                            ViSession instrSession,
                            ViInt32   externalPowerMeterNumber,
                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_POWER_METER, externalPowerMeterNumber),
            2, "External Power Meter Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the source flatness cal uses the configured offset
/// HIFN  de-/embedding to get the power values at the DUT (instead of the
/// HIFN  calibrated reference plane).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR networkType/Selects the offset or de-/embedding type to be used, if configured.
/// HIPAR embedding/Selects between embedding and deembedding for the selected network
/// HIPAR embedding/type. Must be specified for network types that can be used for
/// HIPAR embedding/embedding and deembedding.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationPowerDeembedding(
                            ViSession instrSession,
                            ViInt32   networkType,
                            ViInt32   embedding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViPChar  p2nwtype;
    ViPChar  p2embtype;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngPowerDeEmbNwType, networkType, &p2nwtype),
                2, "Network Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &rsvna_rngPowerDeEmbType, embedding, &p2embtype),
                3, "Embedding");

    snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", p2nwtype, p2embtype);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL,
        RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION_POWER_DEEMBEDDING,
        RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the source power calibration data parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number
/// HIPAR waveNumber/Port number assignment.The input (stimulus) or output (response) port number is
/// HIPAR waveNumber/selected. The range of output and input port numbers depends on the analyzer model.
/// HIPAR settingType/Sets the power calibration setting type.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR info/Returns the calibration informations.
ViStatus _VI_FUNC rsvna_QuerySourcePowerCalibrationDataParameters(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   waveNumber,
                            ViInt32   settingType,
                            ViInt32   bufferSize,
                            ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Wave[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2SettingType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION_DATA_PARAMETERS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION_DATA_PARAMETERS,
            "Wave", waveNumber - 1, RS_REPCAP_BUF_SIZE, repCap_Wave),
            3, "Wave");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Wave);

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSourcePowerCalibrationData, settingType, &p2SettingType),
                4, "Setting Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 2),
                5, "Buffer Size");
    viCheckParm(RsCore_GetAttributeViStringWithQparam(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_SOURCE_POWER_CALIBRATION_DATA_PARAMETERS, p2SettingType, bufferSize, info, NULL),
                6, "Info");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the source power calibration data parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number
/// HIPAR waveNumber/Port number assignment.The input (stimulus) or output (response) port number is
/// HIPAR waveNumber/selected. The range of output and input port numbers depends on the analyzer model.
/// HIPAR settingType/Sets the power calibration setting type.
/// HIPAR index/If one or more external switch matrices were used during calibration,
/// HIPAR index/this refers to the index of a particular switch matrix.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR info/Returns the calibration informations.
ViStatus _VI_FUNC rsvna_QuerySourcePowerCalibrationDataParametersSwitchingMatrix(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   waveNumber,
                            ViInt32   settingType,
                            ViInt32   index,
                            ViInt32   bufferSize,
                            ViChar    info[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, settingType, RSVNA_VAL_SETTING_TYPE_TVNA, RSVNA_VAL_SETTING_TYPE_MTEST),
            4, "Setting Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngSourcePowerCalibrationData, settingType, &p2string),
        4, "Setting Type");

    if (settingType == RSVNA_VAL_CAL_PARAM_MVNA || settingType == RSVNA_VAL_CAL_PARAM_MTEST)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA:PAR%ld? %s,%ld", channel, waveNumber, p2string, index);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA:PAR%ld? %s", channel, waveNumber, p2string);
        checkErr(rsvna_QueryViString(instrSession, cmd, bufferSize, info));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the source power calibration data parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in arbitrary mode.
/// HIPAR waveNumber/Port number assignment.The input (stimulus) or output (response) port number is
/// HIPAR waveNumber/selected. The range of output and input port numbers depends on the analyzer model.
/// HIPAR parametersCount/Returns source power calibration data parameters count.
ViStatus _VI_FUNC rsvna_QuerySourcePowerCalibrationDataParametersCount(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   waveNumber,
                            ViInt32*  parametersCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Pt%ld,Wave%ld", channel, physicalPort, waveNumber);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUERY_SOURCE_POWER_CALIBRATION_DATA_PARAMETERS_COUNT, parametersCount),
            5, "Parameters Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the source for the source power calibration,
/// HIFN starts and applies the source power calibration.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number
/// HIPAR source/This control toggle between the analyzer port and external generator..
/// HIPAR portNumber/This control sets the analyzer port number of generator number.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationSource(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   source,
                            ViInt32   portNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, 0, 7),
            3, "Source");

    switch (source)
    {
    case RSVNA_VAL_PWR_CAL_PORT:
    case RSVNA_VAL_PWR_CAL_GEN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:ACQ %s,%ld", channel,
                 calSourceArr[source], portNumber);
        break;

    case RSVNA_VAL_PWR_CAL_A1:
    case RSVNA_VAL_PWR_CAL_A2:
    case RSVNA_VAL_PWR_CAL_A3:
    case RSVNA_VAL_PWR_CAL_A4:
    case RSVNA_VAL_PWR_CAL_ESRC1:
    case RSVNA_VAL_PWR_CAL_ESRC2:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:ACQ %s", channel, calSourceArr[source]);
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the source calibration power offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR portNumber/This control sets the calibrated port number.
/// HIPAR portPowerOffset/This control sets the port-specific power offset.
/// HIPAR offsetParameter/This control sets the offset parameter.
/// HIPAR calibrationPowerOffset/This control sets the calibration power offset.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationOffset(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   portNumber,
                            ViReal64  portPowerOffset,
                            ViInt32   offsetParameter,
                            ViReal64  calibrationPowerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Pt%ld", channel, portNumber);

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, portPowerOffset, -300.0, 300.0), 4, "Port Power Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, offsetParameter, 0, 1),
            5, "Offset Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW%ld:OFFS %.12lG,%s", channel, portNumber, portPowerOffset,
             offsetParameterArr[offsetParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_POWER_CALIBRATION_OFFSET, calibrationPowerOffset),
            6, "Calibration Power Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the source power calibration generator offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR generator/This control sets the generator number.
/// HIPAR portPowerOffset/This control sets the port-specific power offset.
/// HIPAR offsetParameter/This control sets the offset parameter.
/// HIPAR calibrationPowerOffset/This control sets the calibration power offset.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationGeneratorOffset(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   generator,
                            ViReal64  portPowerOffset,
                            ViInt32   offsetParameter,
                            ViReal64  calibrationPowerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Gen%ld", channel, generator);

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, portPowerOffset, -300.0, 300.0), 4, "Port Power Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, offsetParameter, 0, 1),
            5, "Offset Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:GEN%ld:OFFS %G,%s", channel, generator, portPowerOffset, offsetParameterArr[offsetParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_POWER_CALIBRATION_GENERATOR_OFFSET, calibrationPowerOffset),
            6, "Calibration Power Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the source power calibration Reference Receiver Calibration Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerSelect/Qualifies how to define the source power which the R&S ZNB uses to perform
/// HIPAR powerSelect/the first calibration sweep of the source power calibration.
/// HIPAR referencePower/Defines the source power which the R&S ZNB uses to perform the
/// HIPAR referencePower/first calibration sweep of the source power calibration.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationPower(ViSession instrSession,
                                                             ViInt32   powerSelect,
                                                             ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_POWER_SELECT, powerSelect),
            2, "Power Select");

    if (powerSelect == RSVNA_VAL_PWR_CAL_POWER_USER)
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_SOURCE_POWER_CALIBRATION_REFERENCE_POWER, referencePower),
                2, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the electronic source attenuation at port <Pt> in channel
/// HIFN  <Ch>. This setting will only take effect if
/// HIFN  SOURce<Ch>:POWer<Pt>:EATTenuation:MODE is set to MANual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR mode/Defines how the electronic source attenuation of port <Pt> is selected
/// HIPAR mode/in channel <Ch>.
/// HIPAR attenuation/Defines the electronic source attenuation at port <Pt> in channel
/// HIPAR attenuation/<Ch>. This setting will only take effect if
/// HIPAR attenuation/SOURce<Ch>:POWer<Pt>:EATTenuation:MODE is set to MANual.
ViStatus _VI_FUNC rsvna_ConfigureElectronicSourceAttenuation(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   physicalPort,
                                                             ViInt32   mode,
                                                             ViInt32   attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ELECTRONIC_SOURCE_ATTENUATION_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ELECTRONIC_SOURCE_ATTENUATION_MODE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_ELECTRONIC_SOURCE_ATTENUATION_MODE, mode),
            4, "Mode");

    if (mode != RSVNA_VAL_ATT_MODE_AUTO)
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_ELECTRONIC_SOURCE_ATTENUATION, attenuation),
                5, "Attenuation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the analyzer to create a set of default source power
/// HIFN  calibration data. The analyzer uses the reference channel power to
/// HIFN  acquire the default calibration data. No external power meter is
/// HIFN  required.
/// HIFN
/// HIFN     Enables (ON) or disables (OFF) the dummy source power calibration for
/// HIFN  port number <PhyPt>. OFF means that a real source power calibration is
/// HIFN  performed, for which an external power meter must be connected to the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enables the analyzer to create a set of default source power
/// HIPAR enabled/calibration data. The analyzer uses the reference channel power to
/// HIPAR enabled/acquire the default calibration data. No external power meter is
/// HIPAR enabled/required.
ViStatus _VI_FUNC rsvna_ConfigureDummySourcePowerCalibrationEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DUMMY_SOURCE_POWER_CALIBRATION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DUMMY_SOURCE_POWER_CALIBRATION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables a fast source power calibration, where the
/// HIFN  external power meter is used for the first calibration sweep only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enables or disables a fast source power calibration, where the
/// HIPAR enabled/external power meter is used for the first calibration sweep only.
ViStatus _VI_FUNC rsvna_ConfigureFastSourcePowerCalibrationEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FAST_SOURCE_POWER_CALIBRATION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_FAST_SOURCE_POWER_CALIBRATION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If external generators are used in the measurement, this defines the
/// HIFN  minimum settling delay before the external generators are considered to
/// HIFN  be ready, i.e. the time between setting the generator to the requested
/// HIFN  frequency and power and continuing with the measurement. This is a
/// HIFN  global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimumSettlingDelay/If external generators are used in the measurement, this defines the
/// HIPAR minimumSettlingDelay/minimum settling delay before the external generators are considered to
/// HIPAR minimumSettlingDelay/be ready, i.e. the time between setting the generator to the requested
/// HIPAR minimumSettlingDelay/frequency and power and continuing with the measurement. This is a
/// HIPAR minimumSettlingDelay/global setting.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationGeneratorMinimumSettlingDelay(
                            ViSession instrSession,
                            ViReal64  minimumSettlingDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_EXTERNAL_GENERATOR_MINIMUM_SETTLING_DELAY, minimumSettlingDelay),
            2, "Minimum Settling Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the result of the last verification sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calibrationPassed/Calibration passed, i.e. the cal. power offset is below the
/// HIPAR calibrationPassed/tolerance (1), or failed (0).
/// HIPAR maxOffset/This control returns maximum power offset between the measured power at
/// HIPAR maxOffset/the reference plane and the Cal Power during the verification sweep in dB.
ViStatus _VI_FUNC rsvna_QuerySourcePowerCalibrationVerificationResults(
                            ViSession  instrSession,
                            ViChar     calibratedChannel[],
                            ViBoolean* calibrationPassed,
                            ViReal64*  maxOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  buffer = NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SOUR:POW:CORR:VER:RES?", &buffer));

    if (strstr(buffer, ","))
    {
        p2buf = strtok(buffer, " ");
        strcpy(calibratedChannel, p2buf);
        p2buf = strtok(NULL, ",");
        *calibrationPassed = RsCore_Convert_String_To_Boolean(p2buf);
        p2buf = strtok(NULL, ",");
        *maxOffset = RsCore_Convert_String_To_ViReal64(p2buf);
    }
    else
    {
        *calibrationPassed = VI_FALSE;
        *maxOffset = RS_VAL_NAN_VI_REAL64;
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (buffer)
        free(buffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control sets the number of power correction values.
/// HIPAR powerCorrectionValues/This control sets the power correction values.
ViStatus _VI_FUNC rsvna_WriteSourcePowerCalibrationCorrectionData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViString  calibratedWave,
                            ViInt32   numberOfValues,
                            ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA '%s',", channel, calibratedWave);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR arraySize/Array size.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control displays the number of power correction values.
/// HIPAR powerCorrectionValues/This control displays the power correction values.
ViStatus _VI_FUNC rsvna_ReadSourcePowerCalibrationCorrectionData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   arraySize,
                            ViString  calibratedWave,
                            ViInt32*  numberOfValues,
                            ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA? '%s'", channel, calibratedWave);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, powerCorrectionValues,
        numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control sets the number of power correction values.
/// HIPAR powerCorrectionValues/This control sets the power correction values.
ViStatus _VI_FUNC rsvna_WriteSourcePowerCalibrationCorrectionDataPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViString  calibratedWave,
                            ViInt32   numberOfValues,
                            ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA:PORT%ld '%s',", channel, physicalPort, calibratedWave);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR arraySize/Array size.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control displays the number of power correction values.
/// HIPAR powerCorrectionValues/This control displays the power correction values.
ViStatus _VI_FUNC rsvna_ReadSourcePowerCalibrationCorrectionDataPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   arraySize,
                            ViString  calibratedWave,
                            ViInt32*  numberOfValues,
                            ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:POW:CORR:DATA:PORT%ld? '%s'", channel, physicalPort, calibratedWave);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the power mode at sweep end. This is a global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the power mode at sweep end.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerSweepEndMode(ViSession instrSession,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_POWER_SWEEP_END_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If sweep end mode REDuce or KEEP is active (see
/// HIFN  SOURce:POWer:SWEepend:MODE) this defines the settling delay at
/// HIFN  subsequent sweep starts, i.e. the time between power reset and sweep
/// HIFN  start when a new sweep is requested.
/// HIFN     This is a global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR delay/If sweep end mode REDuce or KEEP is active (see
/// HIPAR delay/SOURce:POWer:SWEepend:MODE) this defines the settling delay at
/// HIPAR delay/subsequent sweep starts, i.e. the time between power reset and sweep
/// HIPAR delay/start when a new sweep is requested.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerSweepEndSettlingDelay(
                            ViSession instrSession,
                            ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_POWER_SWEEP_END_SETTLING_DELAY, delay),
            2, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the wave quantity and the source for the receiver power
/// HIFN calibration, starts the calibration sweep, and applies the receiver power correction.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR wave/This control selects the wave.
/// HIPAR portNumber/This control sets the calibrated port number.
/// HIPAR source/This control toggle between the analyzer port and external generator.
ViStatus _VI_FUNC rsvna_ConfigureReceiverPowerCalibration(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   wave,
                                                          ViInt32   portNumber,
                                                          ViInt32   source,
                                                          ViInt32   sourceNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wave, 0, 5),
            3, "Wave");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, 0, 1),
            5, "Source");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, 0, 1),
            6, "Source");

    switch (wave)
    {
    case RSVNA_VAL_PWR_CAL_AWAVE:
    case RSVNA_VAL_PWR_CAL_BWAVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:ACQ %s,%ld,%s,%ld", channel,
                 receiverPowerCalArr[wave], portNumber,
                 calSourceArr[source], sourceNumber);
        break;
    case RSVNA_VAL_PWR_CAL_B1:
    case RSVNA_VAL_PWR_CAL_B2:
    case RSVNA_VAL_PWR_CAL_B3:
    case RSVNA_VAL_PWR_CAL_B4:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:ACQ %s", channel, receiverPowerCalArr[wave]);
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sets the attenuation for the received waves.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR physicalPort/Physical port number, used to select a specific frequency axis in arbitrary mode.
/// HIPAR attenuation/Sets the attenuation for the received waves.
ViStatus _VI_FUNC rsvna_ConfigureReceiverPowerAttenuation(ViSession instrSession,
                                                          ViInt32   physicalPort,
                                                          ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Pt%ld", physicalPort);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RECEIVER_POWER_ATTENUATION, attenuation),
            3, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables calibration of the reference receiver together
/// HIFN  with the source power calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enables or disables calibration of the reference receiver together
/// HIPAR enabled/with the source power calibration.
ViStatus _VI_FUNC rsvna_ConfigureReferenceReceiverEnabled(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_CALIBRATION_REFERENCE_RECEIVER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_POWER_CALIBRATION_REFERENCE_RECEIVER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control sets the number of power correction values.
/// HIPAR powerCorrectionValues/This control sets the power correction values.
ViStatus _VI_FUNC rsvna_WriteReceiverPowerCorrectionData(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViString  calibratedWave,
                                                         ViInt32   numberOfValues,
                                                         ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:DATA '%s',", channel, calibratedWave);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads receiver power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR arraySize/Array size.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control displays the number of power correction values.
/// HIPAR powerCorrectionValues/This control displays the power correction values.
ViStatus _VI_FUNC rsvna_ReadReceiverPowerCorrectionData(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   arraySize,
                                                        ViString  calibratedWave,
                                                        ViInt32*  numberOfValues,
                                                        ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:DATA? '%s'", channel, calibratedWave);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function writes source power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control sets the number of power correction values.
/// HIPAR powerCorrectionValues/This control sets the power correction values.
ViStatus _VI_FUNC rsvna_WriteReceiverPowerCorrectionDataPort(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   physicalPort,
                                                             ViString  calibratedWave,
                                                             ViInt32   numberOfValues,
                                                             ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:DATA:PORT%ld '%s',", channel, physicalPort, calibratedWave);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, powerCorrectionValues, numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads receiver power correction data sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR arraySize/Array size.
/// HIPAR calibratedWave/This control sets the identifier for the source of the calibrated wave.
/// HIPAR numberOfValues/This control displays the number of power correction values.
/// HIPAR powerCorrectionValues/This control displays the power correction values.
ViStatus _VI_FUNC rsvna_ReadReceiverPowerCorrectionDataPort(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   physicalPort,
                                                            ViInt32   arraySize,
                                                            ViString  calibratedWave,
                                                            ViInt32*  numberOfValues,
                                                            ViReal64  powerCorrectionValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:POW:DATA:PORT%ld? '%s'", channel, physicalPort, calibratedWave);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, powerCorrectionValues,
        numberOfValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures power meter test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR testSetup/This control select the test setup.
ViStatus _VI_FUNC rsvna_ConfigurePowerMeterTestSetup(ViSession instrSession,
                                                     ViInt32   testSetup,
                                                     ViInt32   deEmbeddingPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2DeEmbedding;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeEmbeddingPosition, deEmbeddingPosition, &p2DeEmbedding),
                2, "DeEmbedding Position");

    switch (testSetup)
    {
    case RSVNA_VAL_POWER_METER_NO_COEFFICIENTS:
        snprintf (params, RS_PARAM_BUF_SIZE, "OFF,%s", p2DeEmbedding);
        checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_POWER_METER_TRANSMISSION_COEFFICIENTS_STATE, params));

        break;
    case RSVNA_VAL_POWER_METER_TWO_PORT_DUT:
        snprintf (params, RS_PARAM_BUF_SIZE, "ON,%s", p2DeEmbedding);

        checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_POWER_METER_TRANSMISSION_COEFFICIENTS_STATE, params));
        checkErr(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_POWER_METER_TWO_PORT_POSITION, VI_FALSE));

        break;
    case RSVNA_VAL_POWER_METER_TWO_PORT_POWER_METER:
        snprintf (params, RS_PARAM_BUF_SIZE, "ON,%s", p2DeEmbedding);

        checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_POWER_METER_TRANSMISSION_COEFFICIENTS_STATE, params));
        checkErr(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_POWER_METER_TWO_PORT_POSITION, VI_TRUE));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Adds a new frequency and transmission coefficient to the end of the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/This control specifies frequency value.
/// HIPAR transmissionCoeficient/This control specifies the transmission coefficient.
ViStatus _VI_FUNC rsvna_PowerMeterDefineCoefficient(ViSession instrSession,
                                                    ViReal64  frequency,
                                                    ViReal64  transmissionCoeficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, transmissionCoeficient, -300.0, 200.0), 3, "Transmission Coeficient");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:POW:CORR:TCO:DEF %lG,%lG", frequency, transmissionCoeficient);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Inserts a new frequency and transmission coefficient at position no. <listNumber> in the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR listNumber/Position of the coefficient in the list.
/// HIPAR frequency/This control specifies frequency value.
/// HIPAR transmissionCoeficient/This control specifies the transmission coefficient.
/// HIPAR deEmbeddingPosition/Determines the two-port the command.
ViStatus _VI_FUNC rsvna_PowerMeterInsertCoefficient(ViSession instrSession,
                                                    ViInt32   listNumber,
                                                    ViReal64  frequency,
                                                    ViReal64  transmissionCoeficient,
                                                    ViInt32   deEmbeddingPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2DeEmbedding;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, transmissionCoeficient, -300.0, 200.0), 3, "Transmission Coeficient");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeEmbeddingPosition, deEmbeddingPosition, &p2DeEmbedding),
                2, "DeEmbedding Position");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:POW:CORR:TCO:INS%ld %lG,%lG,%s", listNumber, frequency, transmissionCoeficient, p2DeEmbedding);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Deletes a single point no. <List Number> in the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR listNumber/Position of the coefficient in the list.
/// HIPAR deEmbeddingPosition/Determines the two-port the command.
ViStatus _VI_FUNC rsvna_PowerMeterDeleteCoefficient(ViSession instrSession,
                                                    ViInt32   listNumber,
                                                    ViInt32   deEmbeddingPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "LN%ld", listNumber);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_POWER_METER_DELETE_COEFFICIENT, deEmbeddingPosition),
            3, "DeEmbedding Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Clears the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_PowerMeterDeleteAllCoefficients(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_POWER_METER_DELETE_ALL_COEFFICIENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a trace which provides the points for the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects a trace which provides the points for the power loss list.
/// HIPAR deEmbeddingPosition/Determines the two-port the command.
ViStatus _VI_FUNC rsvna_PowerMeterTraceToList(ViSession instrSession,
                                              ViString  trace,
                                              ViInt32   deEmbeddingPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2DeEmbedding;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeEmbeddingPosition, deEmbeddingPosition, &p2DeEmbedding),
                2, "DeEmbedding Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "'%s',%s", trace, p2DeEmbedding);

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_POWER_METER_TRACE_TO_LIST, params),
            2, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the number of frequency values and transmission coefficients
/// HIFN in the power loss list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfPoints/Returns the number of frequency values and transmission
/// HIPAR numberOfPoints/coefficients in the power loss list.
ViStatus _VI_FUNC rsvna_QueryPowerMeterNumberOfPoints(ViSession instrSession,
                                                      ViInt32   deEmbeddingPosition,
                                                      ViInt32*  numberOfPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2DeEmbedding;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeEmbeddingPosition, deEmbeddingPosition, &p2DeEmbedding),
                2, "DeEmbedding Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s", p2DeEmbedding);

    viCheckParm(RsCore_GetAttributeViInt32WithQparam(instrSession, NULL, RSVNA_ATTR_HIDDEN_POWER_METER_NUMBER_OF_POINTS, params, numberOfPoints),
            3, "Number Of Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the frequency and transmission coefficient no. <List Number>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR listNumber/Position of the coefficient in the list.
/// HIPAR frequency/This control returns the transmission coefficient.
/// HIPAR transmissionCoefficient/This control returns the transmission coefficient.
ViStatus _VI_FUNC rsvna_QueryPowerMeterPointValues(ViSession instrSession,
                                                   ViInt32   listNumber,
                                                   ViInt32   deEmbeddingPosition,
                                                   ViReal64* frequency,
                                                   ViReal64* transmissionCoefficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2DeEmbedding;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeEmbeddingPosition, deEmbeddingPosition, &p2DeEmbedding),
                2, "DeEmbedding Position");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:POW:CORR:TCO:DEF%ld? %s", listNumber, p2DeEmbedding);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, frequency, transmissionCoefficient, NULL, NULL));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function saves the current power loss list to a specified power meter correction list file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceFile/Specifies the name and directory of the created power meter correction
/// HIPAR traceFile/list file. The file extension *.pmcl is mandatory.
ViStatus _VI_FUNC rsvna_PowerMeterSavePowerLossList(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViString  traceFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_METER_SAVE_POWER_LOSS_LIST,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_POWER_METER_SAVE_POWER_LOSS_LIST, traceFile),
            3, "Trace File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads trace data from a specified power meter correction file or trace
/// HIFN  file and assigns it to a trace with a specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR traceFile/Specifies the name and directory of the loaded power meter correction
/// HIPAR traceFile/list file. If no path is specified the analyzer searches the current
/// HIPAR traceFile/directory. The file extensions *.s<n>p, *csv, and *.pmcl for
/// HIPAR traceFile/Touchstone, ASCII, and power meter correction list files are mandatory.
/// HIPAR traceFile/
/// HIPAR trace/This control specifies optional string parameter: For imported
/// HIPAR trace/Touchstone files for more than one port (*.s2p, *.s3p, *.s4p), the
/// HIPAR trace/parameter denotes the imported S-parameter ('S11', 'S12', ...). For
/// HIPAR trace/ASCII (*.csv) files, the parameter references a trace name in the file
/// HIPAR trace/(case sensitive). If the parameter is omitted, the first trace in the
/// HIPAR trace/specified file is imported. The parameter is not used for power meter
/// HIPAR trace/correction list files (*.pmcl).
ViStatus _VI_FUNC rsvna_PowerMeterLoadPowerLossList(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViString  traceFile,
                                                    ViString  trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_POWER_METER_LOAD_POWER_LOSS_LIST,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    if ((trace != NULL) && (strlen(trace) > 0))
        snprintf(value, RS_MAX_MESSAGE_LEN, "'%s','%s'", traceFile, trace);
    else
        snprintf(value, RS_MAX_MESSAGE_LEN, "'%s'", traceFile);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_POWER_METER_LOAD_POWER_LOSS_LIST, value),
            3, "Trace File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the source calibration, stores and applies the calibration
/// HIFN  data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR timeout/Sets the timeout for the operation to complete.
/// HIPAR calibrate/Select calibartion step.
ViStatus _VI_FUNC rsvna_StartCalibrationIntermodulation(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   timeout,
                                                        ViInt32   calibrate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_GetOPCTimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetOPCTimeout(instrSession, timeout));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    if (calibrate == RSVNA_VAL_INTERMODULATION_RECEIVER)
    {
        checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_START_CALIBRATION_INTERMODULATION_RECEIVER, NULL));
    }
    else
    {
        strcat(repCap, calibrate ? ",Upper" : ",Lower");
        checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_START_CALIBRATION_INTERMODULATION_SOURCE, NULL));
    }

Error:
    checkErr(rsvna_SetOPCTimeout(instrSession, old_timeout));

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Runs the cal validation with its configured settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_RunCalibrationValidation(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_RUN_CALIBRATION_VALIDATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the CalU standard to be used for cal validation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/Selects the CalU standard to be used for cal validation.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationStandard(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_STANDARD, standard),
            2, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the CalU characterization to be used for the cal validation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR characterization/Selects the CalU characterization to be used for the cal validation.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationCharacterization(
                            ViSession instrSession,
                            ViString  characterization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_CHARACTERIZATION, characterization),
            2, "Characterization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum magnitude deviations in dB from the reference
/// HIFN  characterization S-parameters that are allowed during cal validation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects a port.
/// HIPAR lowerDeviation/The maximum lower magnitude deviation from the reference
/// HIPAR lowerDeviation/characterization.
/// HIPAR upperDeviation/The maximum upper magnitude deviation from the reference
/// HIPAR upperDeviation/characterization.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationMagnitudeDeviation(
                            ViSession instrSession,
                            ViInt32   port,
                            ViReal64  lowerDeviation,
                            ViReal64  upperDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, lowerDeviation);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_MAGNITUDE_LOWER_DEVIATION,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Lower Deviation");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, upperDeviation);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_MAGNITUDE_UPPER_DEVIATION,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                4, "Upper Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum phase deviations in degrees from the reference
/// HIFN  characterization S-parameters that are allowed during cal validation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects a port.
/// HIPAR lowerDeviation/The maximum lower phase deviation from the reference characterization.
/// HIPAR upperDeviation/The maximum upper phase deviation from the reference characterization.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationPhaseDeviation(
                            ViSession instrSession,
                            ViInt32   port,
                            ViInt32   lowerDeviation,
                            ViInt32   upperDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%ld", port, lowerDeviation);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_PHASE_LOWER_DEVIATION,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Lower Deviation");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%ld", port, upperDeviation);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_PHASE_UPPER_DEVIATION,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                4, "Upper Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Calibration Validation Average Enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Calibration Validation Average Enabled.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationAverageEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_AVERAGE_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Calibration Validation Format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationValidationFormat/Sets the Calibration Validation Format.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationValidationFormat(ViSession instrSession,
                                                             ViInt32   calibrationValidationFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_FORMAT, calibrationValidationFormat),
            2, "Calibration Validation Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Presets Validation Calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_CalibrationValidationPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays the result (PASS/FAIL) of the preceding cal validation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR result/Displays the result (PASS/FAIL) of the preceding cal validation.
ViStatus _VI_FUNC rsvna_QueryCalibrationValidationResult(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    result[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_CALIBRATION_VALIDATION_RESULT,
            bufferSize, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Calbration Validation Imaginary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Sets the Calibration Imaginary Lower.
/// HIPAR upper/Sets the Calibration Imaginary Upper.
ViStatus _VI_FUNC rsvna_CalibrationValidationImaginary(ViSession instrSession,
                                                       ViInt32   port,
                                                       ViReal64  lower,
                                                       ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, lower);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_IMAGINARY_LOWER,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Lower Deviation");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, upper);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_IMAGINARY_UPPER,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                4, "Upper Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Calbration Validation Real.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generator/This control sets the generator number.
/// HIPAR lower/Sets the Calibration Imaginary Lower.
/// HIPAR upper/Sets the Calibration Imaginary Upper.
ViStatus _VI_FUNC rsvna_CalibrationValidationReal(ViSession instrSession,
                                                  ViInt32   port,
                                                  ViReal64  lower,
                                                  ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, lower);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_REAL_LOWER,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Lower Deviation");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%.12lG", port, upper);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, NULL,
                    RSVNA_ATTR_HIDDEN_CALIBRATION_VALIDATION_REAL_UPPER,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                4, "Upper Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  After a successful METAS calibration, this command activates or
/// HIFN  deactivates the uncertainty calculation for the related channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR metasCalibrationActive/After a successful METAS calibration, this command activates or
/// HIPAR metasCalibrationActive/deactivates the uncertainty calculation for the related channel.
ViStatus _VI_FUNC rsvna_ConfigureMETASCalibrationActive(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean metasCalibrationActive)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_METAS_CALIBRATION_ACTIVE, metasCalibrationActive),
            2, "METAS Calibration Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Lists the available METAS cable models, identified by their file names
/// HIFN  (without *.cable extension) in the local METAS VNA Tools II database.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR catalog/Lists the available METAS cable models, identified by their file names
/// HIPAR catalog/(without *.cable extension) in the local METAS VNA Tools II database.
/// HIPAR returnedBytes/The length of the response received from the instrument. If this is
/// HIPAR returnedBytes/longer than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryMETASCableCatalog(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    catalog[],
                                               ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_METAS_CABLE_CATALOG,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, catalog, returnedBytes),
                3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the METAS cable type that is used at the respective ports.
/// HIFN  Providing an array of length 0 will allocate the cable to all ports.
/// HIFN     Use rsvna_QueryMETASCableCatalog to list the available cable types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR cableNameFile/Name of a METAS cable model file (without *.cable extension) in the
/// HIPAR cableNameFile/database of the local METAS VNA Tools II installation. Use
/// HIPAR cableNameFile/RSVNA_ATTR_METAS_CABLE_CATALOG to list the available cable types.
/// HIPAR arrayLength/Length of the provided ports array. Providing an array of length 0
/// HIPAR arrayLength/will allocate the cable to all ports.
/// HIPAR ports/Any array of port numbers of size 'Array Length' to which the selected
/// HIPAR ports/cable will be allocated.
ViStatus _VI_FUNC rsvna_ConfigureMETASCableToPorts(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViString  cableNameFile,
                                                   ViInt32   arrayLength,
                                                   ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar   portBuf[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s'", cableNameFile);
    if (arrayLength > 0)
    {
        for (int i = 0; i < arrayLength; i++)
        {
            snprintf(portBuf, 10, ",%ld", ports[i]);
            strncat(param, portBuf, RS_MAX_MESSAGE_LEN - strlen(param));
        }
    }

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_METAS_SELECT_CABLE,
                    RS_VAL_DIRECT_USER_CALL,
                    param),
                3, "Cable Name File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  While a METAS calibration is active for the related channel (see
/// HIFN  RSVNA_ATTR_METAS_CALIBRATION_ACTIVE), this command notifies the METAS
/// HIFN  VNA Tools II about additional connector reconnections at the specified
/// HIFN  ports.
/// HIFN     Providing an array of length 0 will notify the METAS VNA Tools II that
/// HIFN  all ports were reconnected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR arrayLength/Length of the provided ports array. Providing an array of length 0
/// HIPAR arrayLength/will notify the METAS VNA Tools II that all ports were reconnected.
/// HIPAR ports/An array of reconnected ports.
ViStatus _VI_FUNC rsvna_METASCableReconnection(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   arrayLength,
                                               ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = {'\0'};
    ViPChar  p2param = param;
    ViChar   portBuf[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    if (arrayLength > 0)
    {
        for (int i = 0; i < arrayLength; i++)
        {
            snprintf(portBuf, 10, ",%ld", ports[i]);
            strncat(param, portBuf, RS_MAX_MESSAGE_LEN - strlen(param));
        }
        p2param++; //Skip leading comma
    }
    else
    {
        p2param = "ALL";
    }

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_METAS_CABLE_RECONNECTION,
                    RS_VAL_DIRECT_USER_CALL,
                    p2param),
                4, "Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the channel connector reconnection history in the METAS
/// HIFN  project.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_METASClearCableReconnectionHistory(ViSession instrSession,
                                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_METAS_CLEAR_CABLE_RECONNECTION_HISTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  While a METAS calibration is active for the related channel (see
/// HIFN  RSVNA_ATTR_METAS_CALIBRATION_ACTIVE), this command notifies the METAS
/// HIFN  VNA Tools II about additional cable movements at the specified ports.
/// HIFN     Providing an array of length 0 will notify the METAS VNA Tools II that
/// HIFN  all ports were moved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR arrayLength/Length of the provided ports array. Providing an array of length 0
/// HIPAR arrayLength/will notify the METAS VNA Tools II that all ports were reconnected.
/// HIPAR ports/An array of reconnected ports.
ViStatus _VI_FUNC rsvna_METASCableMovement(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   arrayLength,
                                           ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = {'\0'};
    ViPChar  p2param = param;
    ViChar   portBuf[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    if (arrayLength > 0)
    {
        for (int i = 0; i < arrayLength; i++)
        {
            snprintf(portBuf, 10, ",%ld", ports[i]);
            strncat(param, portBuf, RS_MAX_MESSAGE_LEN - strlen(param));
        }
        p2param++; //Skip leading comma
    }
    else
    {
        p2param = "ALL";
    }

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                    RSVNA_ATTR_HIDDEN_METAS_CABLE_MOVEMENT,
                    RS_VAL_DIRECT_USER_CALL,
                    p2param),
                4, "Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the channel connector movement history in the METAS project.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_METASClearCableMovementHistory(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_METAS_CLEAR_CABLE_MOVEMENT_HISTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the width of the uncertainty band, expressed as multiples of
/// HIFN  the standard deviation.
/// HIFN
/// HIFN     Default is 2 (95% confidence interval).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR uncertainlyBand/Defines the width of the uncertainty band, expressed as multiples of
/// HIPAR uncertainlyBand/the standard deviation.
ViStatus _VI_FUNC rsvna_ConfigureMETASUncertainlyBand(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64  uncertainlyBand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_METAS_UNCERTAINLY_BAND,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_METAS_UNCERTAINLY_BAND, uncertainlyBand),
            3, "Uncertainly Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the list of harmonic orders at which the source and receiver
/// HIFN  power calibration shall be performed for harmonics measurements in
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR bufferSize/Size of output array.
/// HIPAR orders/Returns the list of harmonic orders at which the source and receiver
/// HIPAR orders/power calibration shall be performed for harmonics measurements in
/// HIPAR orders/channel <Ch>.
ViStatus _VI_FUNC rsvna_QueryAdvancedHarmonicOrders(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   bufferSize,
                                                    ViChar    orders[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_HARMONIC_ORDERS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_ADVANCED_HARMONIC_ORDERS,
            bufferSize, orders),
            4, "Orders");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds a harmonic order to the list of harmonic orders at which source
/// HIFN  and receiver calibrations shall be performed for harmonics measurements
/// HIFN  in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR add/Adds a harmonic order to the list of harmonic orders at which source
/// HIPAR add/and receiver calibrations shall be performed for harmonics measurements
/// HIPAR add/in channel <Ch>.
ViStatus _VI_FUNC rsvna_AddAdvancedHarmonicOrder(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   add)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_HARMONIC_ORDER_ADD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_ADVANCED_HARMONIC_ORDER_ADD, add),
            3, "Add");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes a harmonic order from the list of harmonic orders at which
/// HIFN  source and receiver calibrations shall be performed for harmonics
/// HIFN  measurements in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_RemoveAdvancedHarmonicOrder(ViSession instrSession,
                                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_HARMONIC_ORDER_REMOVE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_ADVANCED_HARMONIC_ORDER_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the advanced power.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_ADVANCED_POWER
/// HIFN     RSVNA_ATTR_ADVANCED_POWER_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe<Ch>:]CORRection:ADVanced:POWer<PhyPt>
/// HIFN     [SENSe<Ch>:]CORRection:ADVanced:POWer<PhyPt>:STATe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR enabled/Correction Advanced Power Enabled.
/// HIPAR power/Sets the power.
ViStatus _VI_FUNC rsvna_ConfigureAdvancedPower(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   physicalPort,
                                               ViBoolean enabled,
                                               ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER, power),
            5, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the power attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR enabled/Correction Advanced Power Attenuator Enabled.
/// HIPAR attenuator/Correction Advanced Power Attenuator.
ViStatus _VI_FUNC rsvna_ConfigureAdvancedPowerAttenuator(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViBoolean enabled,
                                                         ViReal64  attenuator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_ATTENUATOR_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_ATTENUATOR_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_ATTENUATOR_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_ATTENUATOR, attenuator),
            5, "Attenuator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the power source attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR enabled/Correction Advanced Power Source Attenuator Enabled.
/// HIPAR attenuator/Correction Advanced Power Source Attenuator.
ViStatus _VI_FUNC rsvna_ConfigureAdvancedPowerSourceAttenuator(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViBoolean enabled,
                            ViReal64  attenuator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_SOURCE_ATTENUATOR_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_SOURCE_ATTENUATOR_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_SOURCE_ATTENUATOR_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_SOURCE_ATTENUATOR, attenuator),
            5, "Attenuator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Advanced Power Preamplifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number. Number of the redefined physical port.
/// HIPAR enabled/Correction Advanced Power Preamplifier Enabled.
/// HIPAR preamplifier/Correction Advanced Power Preamplifier.
ViStatus _VI_FUNC rsvna_ConfigureAdvancedPowerPreamplifier(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   physicalPort,
                                                           ViBoolean enabled,
                                                           ViReal64  preamplifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_PREAMPLIFIER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ADVANCED_POWER_PREAMPLIFIER_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_PREAMPLIFIER_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ADVANCED_POWER_PREAMPLIFIER, preamplifier),
            5, "Preamplifier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Terminates a noise figure calibration, discarding the acquired
/// HIFN  calibration data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_NoiseFigureCalibrationTerminate(ViSession instrSession,
                                                        ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_TERMINATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_TERMINATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Completes a noise figure calibration, storing and applying the
/// HIFN  acquired calibration data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_SaveNoiseFigureCalibration(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SAVE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SAVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a noise figure calibration for the related channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR dutOutput/Number of the physical VNA port connected to the DUT output.
/// HIPAR dutInput/Number of the physical VNA port connected to the DUT input.
/// HIPAR extAtt/Boolean parameter, indicating whether an external attenuator is used.
/// HIPAR attenuationNoise/Attenuation of the source level during the source noise calibration.
/// HIPAR attenuationMeas/Attenuation of the source level during the measurement (i.e. after
/// HIPAR attenuationMeas/calibration).
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationStart(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   dutOutput,
                                                             ViInt32   dutInput,
                                                             ViBoolean extAtt,
                                                             ViReal64  attenuationNoise,
                                                             ViReal64  attenuationMeas)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_NOISE_FIGURE_CALIBRATION_START,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,%ld,%s,%.12lG,%.12lG", dutOutput, dutInput, extAtt ? "OFF" : "ON", attenuationNoise, attenuationMeas);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_NOISE_FIGURE_CALIBRATION_START, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts one of the three noise calibration sweeps that are required for
/// HIFN  a noise figure calibration of the related channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweep/Starts one of the three noise calibration sweeps that are required for
/// HIPAR sweep/a noise figure calibration of the related channel.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationSweep(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   sweep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SWEEP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SWEEP, sweep),
            3, "Sweep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Disables or enables the Noise Figure Calibration for the related
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Disables or enables the Noise Figure Calibration for the related
/// HIPAR enabled/channel.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configure Noise Figure Time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR noiseTime/Defines the detector time of the RMS detector which is used for the
/// HIPAR noiseTime/noise measurement during a NF calibration in channel <Ch>.
/// HIPAR gainTime/Defines the detector time of the AVG detector which is used for the
/// HIPAR gainTime/gain measurement during a NF calibration in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationTime(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViReal64  noiseTime,
                                                            ViReal64  gainTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_NOISE_TIME,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_NOISE_TIME, noiseTime),
            3, "Noise Time");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_GAIN_TIME, gainTime),
            4, "Gain Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a noise figure measurement channel <Ch> and defines the
/// HIFN  receiving and driving port to which the DUT is connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR receiveport/Port number of the receiving port.
/// HIPAR driveport/Port number of the driving port.
ViStatus _VI_FUNC rsvna_InitiateNoiseFigureCalibrationMeasurement(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   receiveport,
                            ViInt32   driveport)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_NOISE_FIGURE_CALIBRATION_INITIATE_MEASUREMENT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,%ld", receiveport, driveport);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_NOISE_FIGURE_CALIBRATION_INITIATE_MEASUREMENT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the detector time of the AVG detector which is used for the
/// HIFN  gain measurement during a NF measurement in channel <Ch>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR avgDetectorTime/Defines the detector time of the AVG detector which is used for the
/// HIPAR avgDetectorTime/gain measurement during a NF measurement in channel <Ch>
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationAVGDetectorTime(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  avgDetectorTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_AVG_DETECTOR_TIME,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_AVG_DETECTOR_TIME, avgDetectorTime),
            3, "AVG Detector Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the sideband correction for a noise figure measurement in
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates the sideband correction for a noise figure
/// HIPAR enabled/measurement in channel <Ch>.
/// HIPAR minimumFrequency/Sets the lower|upper frequency limit to which the sideband correction
/// HIPAR minimumFrequency/is used in a noise figure measurement in channel <Ch>.
/// HIPAR frequencyLimit/Sets the lower|upper frequency limit to which the sideband correction
/// HIPAR frequencyLimit/is used in a noise figure measurement in channel <Ch>.
/// HIPAR maximumOrder/Sets the sideband order of the sideband correction for a noise figure
/// HIPAR maximumOrder/measurement in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationSidebandCorrection(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled,
                            ViReal64  minimumFrequency,
                            ViReal64  frequencyLimit,
                            ViInt32   maximumOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_SIDEBAND_CORRECTION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_SIDEBAND_CORRECTION_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SIDEBAND_CORRECTION_MINIMUM_FREQUENCY, minimumFrequency),
            4, "Minimum Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SIDEBAND_CORRECTION_FREQUENCY_LIMIT, frequencyLimit),
            5, "Frequency Limit");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SIDEBAND_CORRECTION_MAXIMUM_ORDER, maximumOrder),
            6, "Maximum Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the IF Frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Allows you to switch between automatic and manual IF selection for
/// HIPAR enabled/niose figure measurements in channel <Ch>
/// HIPAR ifFrequency/If manual selection is activated ([SENSe<Ch>:]NFIGure:IFConst:STATe
/// HIPAR ifFrequency/ON, this command allows you to select an IF for niose figure
/// HIPAR ifFrequency/measurements in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationIFFrequency(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled,
                            ViReal64  ifFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_IF_FREQUENCY_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_IF_FREQUENCY_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_IF_FREQUENCY, ifFrequency),
            4, "IF Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the detector time of the RMS detector which is used for the
/// HIFN  actual noise measurement during a NF measurement in channel <Ch>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rmsDetectorTime/Defines the detector time of the RMS detector which is used for the
/// HIPAR rmsDetectorTime/actual noise measurement during a NF measurement in channel <Ch>
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationRMSDetectorTime(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  rmsDetectorTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_RMS_DETECTOR_TIME,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_RMS_DETECTOR_TIME, rmsDetectorTime),
            3, "RMS Detector Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates the image band correction at the DUT, during
/// HIFN  a frequency converting noise figure measurement in Channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates or deactivates the image band correction at the DUT, during
/// HIPAR enabled/a frequency converting noise figure measurement in Channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationDUTCorrectionEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_DUT_CORRECTION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_DUT_CORRECTION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the ambient temperature used for proper determination of the DUT
/// HIFN  input source noise during a noise figure measurement in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR ambientTemperature/Sets the ambient temperature used for proper determination of the DUT
/// HIPAR ambientTemperature/input source noise during a noise figure measurement in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationAmbientTemperature(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   ambientTemperature)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_AMBIENT_TEMPERATURE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_AMBIENT_TEMPERATURE, ambientTemperature),
            3, "Ambient Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For segmented sweeps in noise figure channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR virtualImageRejection/For segmented sweeps in noise figure channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationVirtualImageRejection(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean virtualImageRejection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_VIRTUAL_IMAGE_REJECTION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_VIRTUAL_IMAGE_REJECTION, virtualImageRejection),
            3, "Virtual Image Rejection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For segmented sweeps in noise figure channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR segment/Select configured segment.
/// HIPAR virtualImageRejection/For segmented sweeps in noise figure channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationSegmentVirtualImageRejection(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   segment,
                            ViBoolean virtualImageRejection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SEGMENT_VIRTUAL_IMAGE_REJECTION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SEGMENT_VIRTUAL_IMAGE_REJECTION,
            "Segment", segment - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Segment");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SEGMENT_VIRTUAL_IMAGE_REJECTION, virtualImageRejection),
            4, "Virtual Image Rejection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For segmented sweeps in noise figure channels, this command defines
/// HIFN  whether virtual image rejection can be activated or deactivated per
/// HIFN  segment or per channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/For segmented sweeps in noise figure channels, this command defines
/// HIPAR enabled/whether virtual image rejection can be activated or deactivated per
/// HIPAR enabled/segment or per channel.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureCalibrationSegmentVirtualImageControlEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SEGMENT_VIRTUAL_IMAGE_CONTROL_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_CALIBRATION_SEGMENT_VIRTUAL_IMAGE_CONTROL_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Noise Figure Quickset External Attenuator.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXTERNAL_ATTENUATOR_RECEIVER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe<Ch>:]NFIGure:QSET:ATTReceiver
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR externalAttenuatorReceiver/Sets the Noise Figure Quickset External Attenuator.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetExternalAttenuatorReceiver(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  externalAttenuatorReceiver)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXTERNAL_ATTENUATOR_RECEIVER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXTERNAL_ATTENUATOR_RECEIVER, externalAttenuatorReceiver),
            3, "External Attenuator Receiver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Noise Figure Quickset External Attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR externalAttenuatorSource/Sets the Noise Figure Quickset External Attenuator.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetExternalAttenuatorSource(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  externalAttenuatorSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXTERNAL_ATTENUATOR_SOURCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXTERNAL_ATTENUATOR_SOURCE, externalAttenuatorSource),
            3, "External Attenuator Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Desired maximum Trace Noise.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR traceNoise/Sets the Desired maximum Trace Noise.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetTraceNoise(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  traceNoise)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_TRACE_NOISE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_TRACE_NOISE, traceNoise),
            3, "Trace Noise");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Quickset DUT Expected Noise Figure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dutExpectedNoiseFigure/Sets the Quickset DUT Expected Noise Figure.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetDUTExpectedNoiseFigure(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  dutExpectedNoiseFigure)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_EXPECTED_NOISE_FIGURE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_EXPECTED_NOISE_FIGURE, dutExpectedNoiseFigure),
            3, "DUT Expected Noise Figure");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Quickset DUT Expected Gain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dutExpectedGain/Sets the Quickset DUT Expected Gain.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetDUTExpectedGain(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  dutExpectedGain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_EXPECTED_GAIN,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_EXPECTED_GAIN, dutExpectedGain),
            3, "DUT Expected Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Noise Figure Quickset DUT Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dutPower/Sets the Noise Figure Quickset DUT Power.
ViStatus _VI_FUNC rsvna_ConfigureNoiseFigureQuicksetDUTPower(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViReal64  dutPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_POWER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_DUT_POWER, dutPower),
            3, "DUT Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Executes the Noise Figure Quickset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_NoiseFigureQuicksetExecute(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXECUTE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_NOISE_FIGURE_QUICKSET_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the IF receiver calibration (2nd power calibration step for
/// HIFN  scalar mixer measurements), stores and applies the calibration data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_StartMixerPhaseIFCalibration(ViSession instrSession,
                                                     ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_IF_START_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_IF_START_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the IF source calibration (2nd power calibration step for
/// HIFN  vector (not scalar) mixer measurements), stores and applies the
/// HIFN  calibration data. The external power meter is selected via
/// HIFN  SOURce<Ch>:POWer<PhyPt>:CORRection:PMETer:ID.
/// HIFN
/// HIFN     The IF source calibration is independent of the RF source and LO
/// HIFN  source calibration; see description of the mixer calibration procedure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_StartCalibrationMixerPhaseIFSecondStep(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_IF_START_SECOND_STEP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_IF_START_SECOND_STEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the RF source calibration (1st power calibration step for
/// HIFN  scalar and vector mixer measurements), stores and applies the
/// HIFN  calibration data. The external power meter used is selected via
/// HIFN  SOURce<Ch>:POWer<PhyPt>:CORRection:PMETer:ID.
/// HIFN
/// HIFN     Note: For scalar mixer measurements the command initiates two
/// HIFN  calibration steps, the first over the RF frequency range, the second
/// HIFN  over the IF frequency range (for a subsequent receiver power
/// HIFN  calibration, [SENSe<Ch>:]CORRection:POWer<PhyPt>:MIXer:IF:ACQuire). See
/// HIFN  description of the mixer calibration procedure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_StartCalibrationMixerPhaseRFCalibration(
                            ViSession instrSession,
                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_START_RF_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_START_RF_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the LO source calibration (3rd power calibration step for
/// HIFN  scalar mixer measurements, 4th calibration step for two-stage scalar
/// HIFN  mixer measurements),), stores and applies the calibration data. The
/// HIFN  external power meter is selected via
/// HIFN  SOURce<Ch>:POWer<PhyPt>:CORRection:PMETer:ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generator/Selects mixer stage.
ViStatus _VI_FUNC rsvna_StartCalibrationMixerPhaseLOCalibration(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   mixerStage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_START_LO_CALIBRATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_START_LO_CALIBRATION,
            "MixerStage", mixerStage - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Mixer Stage");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CALIBRATION_MIXER_PHASE_START_LO_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the leveling data for all physical ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_SourceLevelingClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_SOURCE_LEVELING_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the leveling data for physical port <PhyPt>, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
ViStatus _VI_FUNC rsvna_SourceLevelingClear(ViSession instrSession,
                                            ViInt32   physicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_CLEAR,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the (global) leveling dataseet of port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
ViStatus _VI_FUNC rsvna_SourceLevelingRemove(ViSession instrSession,
                                             ViInt32   physicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_REMOVE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a leveling dataset for port <PhyPt> from the specified *.lev
/// HIFN  file (R&S ZNA-proprietary, binary file format).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR sourceLevelingLoad/Loads a leveling dataset for port <PhyPt> from the specified *.lev
/// HIPAR sourceLevelingLoad/file (R&S ZNA-proprietary, binary file format).
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingLoad(ViSession instrSession,
                                                    ViInt32   physicalPort,
                                                    ViString  sourceLevelingLoad)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_LOAD,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_LOAD, sourceLevelingLoad),
            3, "Source Leveling Load");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Persists the current leveling dataset of port <PhyPt> to the specified
/// HIFN  *.lev file (R&S ZNA-proprietary, binary file format).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR sourceLevelingStore/Persists the current leveling dataset of port <PhyPt> to the specified
/// HIPAR sourceLevelingStore/*.lev file (R&S ZNA-proprietary, binary file format).
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingStore(ViSession instrSession,
                                                     ViInt32   physicalPort,
                                                     ViString  sourceLevelingStore)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_STORE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_STORE, sourceLevelingStore),
            3, "Source Leveling Store");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of frequency sweep points of the leveling grid for
/// HIFN  physical port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR numberofPoints/Defines the number of frequency sweep points of the leveling grid for
/// HIPAR numberofPoints/physical port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingNumberofPoints(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViInt32   numberofPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_NUMBER_OF_POINTS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_NUMBER_OF_POINTS, numberofPoints),
            3, "Number of Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the start and stop frequency of the leveling grid for physical
/// HIFN  port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR startFrequency/Defines the start and stop frequency of the leveling grid for physical
/// HIPAR startFrequency/port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingStartFrequency(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViReal64  startFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_START_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_START_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_START_FREQUENCY, startFrequency),
            4, "Start Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the start and stop frequency of the leveling grid for physical
/// HIFN  port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR stopFrequency/Defines the start and stop frequency of the leveling grid for physical
/// HIPAR stopFrequency/port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingStopFrequency(ViSession instrSession,
                                                             ViInt32   physicalPort,
                                                             ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_STOP_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_STOP_FREQUENCY, stopFrequency),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the start and stop power of the leveling grid for physical
/// HIFN  port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR startPower/Defines the start and stop power of the leveling grid for physical
/// HIPAR startPower/port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingStartPower(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   physicalPort,
                                                          ViReal64  startPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_START_POWER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_START_POWER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_START_POWER, startPower),
            4, "Start Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the start and stop power of the leveling grid for physical
/// HIFN  port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR stopPower/Defines the start and stop power of the leveling grid for physical
/// HIPAR stopPower/port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingStopPower(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViReal64  stopPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_STOP_POWER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_STOP_POWER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_STOP_POWER, stopPower),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If checked, the power step size of the leveling grid for physical port
/// HIFN  <PhyPt> is fixed to 1 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR alcStepSize/If checked, the power step size of the leveling grid for physical port
/// HIPAR alcStepSize/<PhyPt> is fixed to 1 dB.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingALCStepSize(ViSession instrSession,
                                                           ViInt32   physicalPort,
                                                           ViReal64  alcStepSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_ALC_STEP_SIZE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_ALC_STEP_SIZE, alcStepSize),
            3, "ALC Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the (source) power step size of the leveling grid for physical
/// HIFN  port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR powerStepSize/Defines the (source) power step size of the leveling grid for physical
/// HIPAR powerStepSize/port <PhyPt>.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingPowerStepSize(ViSession instrSession,
                                                             ViInt32   physicalPort,
                                                             ViReal64  powerStepSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_POWER_STEP_SIZE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_POWER_STEP_SIZE, powerStepSize),
            3, "Power Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries whether a leveling data set is available for physical port
/// HIFN  <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR bufferSize/Size of output array.
/// HIPAR state/Queries whether a leveling data set is available for physical port
/// HIPAR state/<PhyPt>.
ViStatus _VI_FUNC rsvna_QuerySourceLevelingState(ViSession instrSession,
                                                 ViInt32   physicalPort,
                                                 ViInt32   bufferSize,
                                                 ViChar    state[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_STATE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_STATE,
            bufferSize, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the leveling data acquisition at port <PhyPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
ViStatus _VI_FUNC rsvna_SourceLevelingStartCalSweep(ViSession instrSession,
                                                    ViInt32   physicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_START_CAL_SWEEP,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_START_CAL_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines whether converter power control is used, if a leveling dataset
/// HIFN  is available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Defines whether converter power control is used, if a leveling dataset
/// HIPAR enabled/is available.
ViStatus _VI_FUNC rsvna_ConfigureSourceLevelingPortEnabled(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   physicalPort,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_LEVELING_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SOURCE_LEVELING_PORT_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates cal premeas for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Activates/deactivates cal premeas for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementEnabled(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the firmware perfoms calibration pre-measurements in
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR mode/Defines how the firmware perfoms calibration pre-measurements in
/// HIPAR mode/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementMode(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_MODE, mode),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables cal premeas for port <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Enables cal premeas for port <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementPortEnabled(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   physicalPort,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_PORT_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the convergence factor for a cal premeas at port <Pt> in
/// HIFN  channel <Ch>.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_PREMEASUREMENT_CONVERGENCE_FACTOR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce<Ch>:POWer<Pt>:CPRemeas:CFACtor
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR convergenceFactor/Specifies the convergence factor for a cal premeas at port <Pt> in
/// HIPAR convergenceFactor/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementConvergenceFactor(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViReal64  convergenceFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_CONVERGENCE_FACTOR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_CONVERGENCE_FACTOR,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_CONVERGENCE_FACTOR, convergenceFactor),
            4, "Convergence Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the detector to be used during the calibration
/// HIFN  pre-measurement.
/// HIFN
/// HIFN     This setting is channel- and port-specific.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_PREMEASUREMENT_DETECTOR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce<Ch>:POWer<Pt>:CPRemeas:DETector
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR detector/Selects the detector to be used during the calibration
/// HIPAR detector/pre-measurement.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementDetector(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   physicalPort,
                                                        ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_DETECTOR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_DETECTOR,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_DETECTOR, detector),
            4, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum number of cal precal sweeps to be performed in
/// HIFN  once mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR maxPrecalSweeps/Defines the maximum number of cal precal sweeps to be performed in
/// HIPAR maxPrecalSweeps/once mode.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementMaxPrecalSweeps(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   maxPrecalSweeps)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_MAX_PRECAL_SWEEPS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_MAX_PRECAL_SWEEPS,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_MAX_PRECAL_SWEEPS, maxPrecalSweeps),
            4, "Max Precal Sweeps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum tolerated deviation between measured and target
/// HIFN  power during cal premeas.
/// HIFN
/// HIFN     This setting is channel- and port-specific.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR tolerance/Defines the maximum tolerated deviation between measured and target
/// HIPAR tolerance/power during cal premeas.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementTolerance(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   physicalPort,
                                                         ViReal64  tolerance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_TOLERANCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_TOLERANCE,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_TOLERANCE, tolerance),
            4, "Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the received signal to be used for cal premeas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR premeasurementReceiver/Defines the received signal to be used for cal premeas.
ViStatus _VI_FUNC rsvna_ConfigurePremeasurementReceiver(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   physicalPort,
                                                        ViString  premeasurementReceiver)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_RECEIVER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PREMEASUREMENT_RECEIVER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_PREMEASUREMENT_RECEIVER, premeasurementReceiver),
            4, "Premeasurement Receiver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates channel and selects it as the active channel.
/// HIFN  Italso assigns a name to channel number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR channelName/Define channel name.
ViStatus _VI_FUNC rsvna_ChannelAdd(ViSession instrSession,
                                   ViInt32   channel,
                                   ViString  channelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", channel);

    checkErr(RsCore_AddRepCap(instrSession, "Channel", repCap, repCap_cmd));

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CHANNEL_STATE, VI_TRUE));

    if (strncmp(channelName, """", 2) != 0)
    {
        viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CHANNEL_NAME, channelName),  // NOLINT(readability-suspicious-call-argument)
                3, "Channel Name");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new channel and a new trace, which is
/// HIFN  displayed in the selected existing diagram area. The new channel
/// HIFN  settings (including a possible channel calibration) is preset to the
/// HIFN  default settings; the trace is created with the default trace settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR channelName/Define channel name.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_ChannelAddTrace(ViSession instrSession,
                                        ViInt32   channel,
                                        ViString  channelName,
                                        ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_ChannelAdd(instrSession, channel, channelName));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:SDEF '%s','S21'", channel, traceName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new channel and a new trace, which is
/// HIFN  displayed in a new diagram area. The new channel settings (including a
/// HIFN  possible channel calibration) is preset to the default settings; the
/// HIFN  trace is created with the default trace settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR channel/Channel number.
/// HIPAR channelName/Define channel name.
/// HIPAR traceName/Define trace name.
ViStatus _VI_FUNC rsvna_ChannelAddTraceDiagramArea(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   channel,
                                                   ViString  channelName,
                                                   ViString  traceName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_ChannelAddTrace(instrSession, channel, channelName, traceName));

    snprintf(repCap_window, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", window);

    checkErr(RsCore_AddRepCap(instrSession, "Window", repCap_window, repCap_cmd));  // NOLINT(readability-suspicious-call-argument)

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap_window, RSVNA_ATTR_TRACE_DIAGRAM_STATE, VI_TRUE));

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap_window, RSVNA_ATTR_TRACE_ASSIGN_DIAGRAM, traceName),
            5, "Trace Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the current channel including all traces
/// HIFN  assigned to the channel and removes all display elements related to the
/// HIFN  channel from the diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ChannelDelete(ViSession instrSession,
                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_CHANNEL_STATE, VI_FALSE));

    checkErr(RsCore_RemoveRepCap(instrSession, "Channel", repCap));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a name to channel number. The channel must be
/// HIFN  created before. Moreover it is not possible to assign the same name to
/// HIFN  two different channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR channelName/Assigns a name to channel number.
ViStatus _VI_FUNC rsvna_ChannelRename(ViSession instrSession,
                                      ViInt32   channel,
                                      ViString  channelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_CHANNEL_NAME, channelName),  // NOLINT(readability-suspicious-call-argument)
            3, "Channel Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the numbers and names of all channels in the
/// HIFN  current setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR catalog/Returns string with comma-separated list of channel numbers and names.
/// HIPAR catalog/If all channels have been deleted the response is an empty string ("").
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_ChannelList(ViSession instrSession,
                                    ViChar    catalog[],
                                    ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_QueryViString(instrSession, "CONF:CHAN:CAT?", bufferSize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the channel number (numeric suffix) of a channel
/// HIFN  with known channel name. A channel name must be assigned before.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelName/Sets the channel name.
/// HIPAR channelNumber/Returns the channel number (numeric suffix) of a channel with known
/// HIPAR channelNumber/channel name.
ViStatus _VI_FUNC rsvna_QueryChannelNumber(ViSession instrSession,
                                           ViString  channelName,
                                           ViInt32*  channelNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:CHAN:NAME:ID? '%s'", channelName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, channelNumber));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a channel <Ch> as the active channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ConfigureActiveChannel(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_ACTIVE_CHANNEL, channel),
            2, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the voltage/current measurements on all RFFE pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_RFFETestStartAllMeasurements(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_RFFE_TEST_START_ALL_MEASUREMENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the current output voltage and current range (shunt) settings
/// HIFN  to the RFFE/GPIO pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects repcap Channel.
ViStatus _VI_FUNC rsvna_RFFETestApply(ViSession instrSession,
                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_RFFE_TEST_APPLY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the measurement time for the voltage/current measurements on the
/// HIFN  RFFE and GPIO pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR time/Sets the measurement time for the voltage/current measurements on the
/// HIPAR time/RFFE and GPIO pins.
ViStatus _VI_FUNC rsvna_ConfigureRFFETestMeasurementTime(ViSession instrSession,
                                                         ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_RFFE_TEST_MEASUREMENT_TIME, time),
            2, "Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function define the output voltages and range for the
/// HIFN  voltage/current measurements on the RFFE pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR pins/Selects the RFFE pin.
/// HIPAR voltage/These controls define the output voltages for the voltage/current
/// HIPAR voltage/measurements on the RFFE pins.
/// HIPAR range/Defines the (upper bound of the) current range for the voltage/current
/// HIPAR range/measurement on the respective RFFE pin.
ViStatus _VI_FUNC rsvna_ConfigureRFFETestData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   rffeNumber,
                                              ViInt32   pins,
                                              ViReal64  voltage,
                                              ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBus[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_RFFE_TEST_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_RFFE_TEST_DATA,
                    "RFFEBus", rffeNumber - 1, RS_REPCAP_BUF_SIZE, repCapBus), 3, "RFFE Number");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCapBus, RS_REPCAP_BUF_SIZE - strlen(repCap));

    switch (pins)
    {
    case RSVNA_RFFE_PIN_DATA:
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_DATA, voltage),
                5, "Voltage");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_DATA_RANGE, range),
                6, "Range");
        break;

    case RSVNA_RFFE_PIN_CLOCK:
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_CLOCK, voltage),
                5, "Voltage");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_CLOCK_RANGE, range),
                6, "Range");
        break;

    case RSVNA_RFFE_PIN_VIO:
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_VIO, voltage),
                5, "Voltage");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_VIO_RANGE, range),
                6, "Range");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, pins), 4, "Pins");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the shunt resistance selected by the analyzer firmware for the
/// HIFN  configured current range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR pins/Selects the RFFE pin.
/// HIPAR shunt/Returns the shunt resistance selected by the analyzer firmware for the
/// HIPAR shunt/configured current range.
ViStatus _VI_FUNC rsvna_QueryRFFETestDataShunt(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   rffeNumber,
                                               ViInt32   pins,
                                               ViReal64* shunt)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBus[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_RFFE_TEST_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_RFFE_TEST_DATA,
                    "RFFEBus", rffeNumber - 1, RS_REPCAP_BUF_SIZE, repCapBus), 3, "RFFE Number");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCapBus, RS_REPCAP_BUF_SIZE - strlen(repCap));

    switch (pins)
    {
    case RSVNA_RFFE_PIN_DATA:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_DATA_SHUNT, shunt),
                5, "Shunt");
        break;

    case RSVNA_RFFE_PIN_CLOCK:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_CLOCK_SHUNT, shunt),
                5, "Shunt");
        break;

    case RSVNA_RFFE_PIN_VIO:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_VIO_SHUNT, shunt),
                5, "Shunt");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, pins), 4, "Pins");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the voltage and current measurements on the
/// HIFN  related RFFE pin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pins/Selects the RFFE pin.
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR voltage/Returns the results of the voltage measurement on the related RFFE
/// HIPAR voltage/pin.
/// HIPAR current/Returns the results of the current measurement on the related RFFE
/// HIPAR current/pin.
ViStatus _VI_FUNC rsvna_QueryRFFETestData(ViSession instrSession,
                                          ViInt32   rffeNumber,
                                          ViInt32   pins,
                                          ViReal64* voltage,
                                          ViReal64* current)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_RFFE_TEST_DATA,
            "RFFEBus", rffeNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "RFFE Number");

    switch (pins)
    {
    case RSVNA_RFFE_PIN_DATA:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_DATA_VOLTAGE, voltage),
                4, "Voltage");

        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_DATA_CURRENT, current),
                5, "Current");
        break;

    case RSVNA_RFFE_PIN_CLOCK:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_CLOCK_VOLTAGE, voltage),
                4, "Voltage");

        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_CLOCK_CURRENT, current),
                5, "Current");
        break;

    case RSVNA_RFFE_PIN_VIO:
        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_VIO_VOLTAGE, voltage),
                4, "Voltage");

        viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_TEST_VIO_CURRENT, current),
                5, "Current");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, pins), 3, "Pins");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines a port group numbered <group_no> containing a continuous
/// HIFN range of logical ports between <Logical Port 1> and <Logical Port 2>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR group/Logical port group number.
/// HIPAR logicalPort1/First logical port number in the port group.
/// HIPAR logicalPort2/Last logical port number in the port group.
ViStatus _VI_FUNC rsvna_DefinePortGroupContinuous(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   group,
                                                  ViInt32   logicalPort1,
                                                  ViInt32   logicalPort2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap_name[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd_value[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:GRO%ld %ld,%ld", channel, group, logicalPort1, logicalPort2);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "Group%ld", group);
    sprintf(cmd_value, "%ld", group);
    checkErr(RsCore_AddRepCap(instrSession, "Group", repCap_name, cmd_value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines a port group numbered <group_no> containing an arbitrary
/// HIFN selection of logical ports. The ports do not have to be numbered continuously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR group/Logical port group number.
/// HIPAR arraySize/Defines size of array in argument 'Logical Ports'.
/// HIPAR logicalPorts/Array of logical ports.
ViStatus _VI_FUNC rsvna_DefinePortGroupArbitrary(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   group,
                                                 ViInt32   arraySize,
                                                 ViInt32   logicalPorts[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap_name[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd_value[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:GRO%ld ", channel, group);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, logicalPorts, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "Group%ld", group);
    sprintf(cmd_value, "%ld", group);
    checkErr(RsCore_AddRepCap(instrSession, "Group", repCap_name, cmd_value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Dissolves a group of measured ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR group/Logical port group number.
ViStatus _VI_FUNC rsvna_DeletePortGroup(ViSession instrSession,
                                        ViInt32   channel,
                                        ViInt32   group)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_PORT_GROUP, NULL),
            3, "Delete Port Group");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Group%ld", group);
    checkErr(RsCore_RemoveRepCap(instrSession, "Group", repCap));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Dissolves all groups of measured ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteAllPortGroups(ViSession instrSession,
                                            ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_ALL_PORT_GROUPS, NULL),
            2, "Delete All Port Groups");

    checkErr(RsCore_ResetRepCapID(instrSession, "Group", rsvna_RsCoreRepCapTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns number of created logical port groups.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR count/Returns number of created logical port groups.
ViStatus _VI_FUNC rsvna_QueryPortGroupsCount(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PORT_GROUPS_COUNT, count),
            3, "Port Groups Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines port groups. It creates port group <Grp> from physical VNA
/// HIFN  ports <pport1>, <pport2>, ....
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for R&S ZNBT!
/// HIFN     (2) Port groups must be created in ascending order, i.e. first create
/// HIFN  port group 1, then port group 2 etc.
/// HIFN     (3) By default there is a single port group, which contains all
/// HIFN  physical VNA ports ("default port group").
/// HIFN     (4) Redefinition of an existing port group is not allowed (exception:
/// HIFN  "default port group"). Use function rsvna_DeletePortGroup to dissolve
/// HIFN  existing port groups first.
/// HIFN     (5) It is not allowed to assign the same physical VNA port to multiple
/// HIFN  port groups.
/// HIFN     (6) When a port group is created, the number of ports of the
/// HIFN  corresponding DUT is set to the port group size and "aligned" port
/// HIFN  connections are etablished. The result can be modified using
/// HIFN  SOURce<Ch>:GROup<Grp>:DPORt:COUNt and
/// HIFN  SOURce<Ch>:GROup<Grp>:PPORt<PhyPort>:DPORt.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR arraySize/Sets the number of elements of the Ports array.
/// HIPAR ports/This control specifies the physical VNA port numbers. The number of
/// HIPAR ports/elements should correspond with Array Size number.
ViStatus _VI_FUNC rsvna_DefineParallelMeasurementPortGroup(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   group,
                                                           ViInt32   arraySize,
                                                           ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_name[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd_value[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "ZNBT"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:GRO%ld:PPOR ", channel, group);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, ports, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "Group%ld", group);
    snprintf(cmd_value, RS_REPCAP_BUF_SIZE, "%ld", group);
    checkErr(RsCore_AddRepCap(instrSession, "Group", repCap_name, cmd_value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the name of port group <Grp>.
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for R&S ZNBT!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR groupName/Sets the name of port group <Grp>.
ViStatus _VI_FUNC rsvna_ConfigureParallelMeasurementPortGroupName(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViString  groupName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_PARALLEL_MEASUREMENT_PORT_GROUP_NAME, groupName),
            4, "Group Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of ports on DUT <Grp>.
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for R&S ZNBT!
/// HIFN     (2) The number of DUT ports may be higher than the number of ports in
/// HIFN  the related port group, but not lower.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR numberofPorts/Sets the number of ports on DUT <Grp>.
ViStatus _VI_FUNC rsvna_ConfigureParallelMeasurementNumberOfPorts(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   numberofPorts)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PARALLEL_MEASUREMENT_NUMBER_OF_PORTS, numberofPorts),
            4, "Number of Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Connects physical VNA port <PhyPort> to DUT <Grp> port
/// HIFN  <DestinationPort>.
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for R&S ZNBT!
/// HIFN     (2) Neither physical VNA ports nor DUT ports may be connected more
/// HIFN  than once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR physicalPort/Number of a physical VNA port in port group <Grp>.
/// HIPAR dutPort/Number of a port on the related DUT. Must be smaller than the size of
/// HIPAR dutPort/the DUT <Grp>
ViStatus _VI_FUNC rsvna_ConfigureParallelMeasurementDUTPort(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   group,
                                                            ViInt32   physicalPort,
                                                            ViInt32   dutPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:GRO%ld:PPOR%ld:DPOR %ld", channel, group, physicalPort, dutPort);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the order of partial measurements and sweeps.
/// HIFN     To configure alternated blocks driving mode, see
/// HIFN  rsvna_ConfigureDrivingModeAlternatedBlocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR drivingMode/This control determines the order of partial measurements and sweeps.
ViStatus _VI_FUNC rsvna_ConfigureDrivingMode(ViSession instrSession,
                                             ViInt32   drivingMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DRIVING_MODE, drivingMode),
            2, "Driving Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Driving Mode to 'Block Alternated' and defines the number of
/// HIFN  blocks.
/// HIFN     If it is set to 1, 'Block Alternated' is equivalent to 'Alternated'
/// HIFN  mode. If it is set to the number of sweep points, 'Block Alternated' is
/// HIFN  equivalent to 'Chopped' mode. By selecting a suitable number of blocks,
/// HIFN  you can minimize the total sweep duration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alternatedBlocks/Sets the Driving Mode to 'Block Alternated' and defines the number of
/// HIPAR alternatedBlocks/blocks.
ViStatus _VI_FUNC rsvna_ConfigureDrivingModeAlternatedBlocks(ViSession instrSession,
                                                             ViInt32   alternatedBlocks)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DRIVING_MODE_ALTERNATED_BLOCKS, alternatedBlocks),
            2, "Alternated Blocks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables tracking of the (unknown) LO signal for two-tone group delay
/// HIFN  measurements (with wmbedded LO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR trackLO/Enables tracking of the (unknown) LO signal for two-tone group delay
/// HIPAR trackLO/measurements (with wmbedded LO).
ViStatus _VI_FUNC rsvna_ConfigureTrackLO(ViSession instrSession,
                                         ViInt32   channel,
                                         ViBoolean trackLO)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACK_LO,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACK_LO, trackLO),
            3, "Track LO");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If LO tracking is active in channel <Ch> (SOURce<Ch>:LOTRack ON), this
/// HIFN  query gets the frequency difference between expected and measured LO
/// HIFN  frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR frequencyDifference/If LO tracking is active in channel <Ch> (SOURce<Ch>:LOTRack ON), this
/// HIPAR frequencyDifference/query gets the frequency difference between expected and measured LO
/// HIPAR frequencyDifference/frequency.
ViStatus _VI_FUNC rsvna_QueryTrackLOFrequencyDifference(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViReal64* frequencyDifference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRACK_LO_FREQUENCY_DIFFERENCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TRACK_LO_FREQUENCY_DIFFERENCE, frequencyDifference),
            3, "Frequency Difference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the LO usage for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR loUsage/Defines the LO usage for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureLOUsage(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   loUsage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LO_USAGE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_LO_USAGE, loUsage),
            3, "LO Usage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the phase mode of channel <Ch>, i.e. de/activates coherence
/// HIFN  mode in combination with low phase noise mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR phaseMode/Defines the phase mode of channel <Ch>, i.e. de/activates coherence
/// HIPAR phaseMode/mode in combination with low phase noise mode.
ViStatus _VI_FUNC rsvna_ConfigurePhaseMode(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   phaseMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PHASE_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PHASE_MODE, phaseMode),
            3, "Phase Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Source Power Thru Meas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Source Power Thru Meas Enabled.
/// HIPAR powerThruMeas/Source Power Thru Meas.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerThruMeas(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean enabled,
                                                     ViReal64  powerThruMeas)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_POWER_THRU_MEAS_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SOURCE_POWER_THRU_MEAS_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_POWER_THRU_MEAS, powerThruMeas),
            4, "Power Thru Meas");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a logical port numbered <Logicalog Port>,
/// HIFN  combining one or two physical ports <Physical Port 1> and <Physical Port 2>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/Logical port number.
/// HIPAR physicalPort1/This control defines the port number.
/// HIPAR physicalPort2/This control defines the port number.
ViStatus _VI_FUNC rsvna_DefineLogicalPort(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   logicalPort,
                                          ViInt32   physicalPort1,
                                          ViInt32   physicalPort2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (physicalPort2 > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:LPOR%ld %ld,%ld", channel, logicalPort, physicalPort1, physicalPort2);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:LPOR%ld %ld", channel, logicalPort, physicalPort1);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function dissolves selected logical port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR logicalPort/Logical port number.
ViStatus _VI_FUNC rsvna_DeleteLogicalPort(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   logicalPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",LogPort%ld", channel, logicalPort);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_LOGICAL_PORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function dissolves all logical ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DeleteAllLogicalPorts(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DELETE_ALL_LOGICAL_PORTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the complex reference impedance for the
/// HIFN  physical port numbered <phys_port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR real/This control defines the real part of the port impedance.
/// HIPAR imaginary/This control defines the imaginary part of the port impedance.
ViStatus _VI_FUNC rsvna_ConfigurePhysicalPortReferenceImpedance(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViReal64  real,
                            ViReal64  imaginary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, real, 1.0e-3, 10.0e6), 4, "Real");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, imaginary, -10.0e6, 10.0e6), 4, "Imaginary");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:PORT%ld:ZREF %.12lG,%.12lG", channel, physicalPort, real, imaginary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the RFFE settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR rffeState/This control enables/disables RFFE bus interface <Bus> in the Sweep
/// HIPAR rffeState/Sequencer.
/// HIPAR frequency/Sets the clock frequency for for channel <Ch> and RFFE bus
/// HIPAR frequency/interface <Bus>.
/// HIPAR ioVoltage/Sets the IO voltage for channel <Ch> and RFFE bus interface
/// HIPAR ioVoltage/<Bus>.
/// HIPAR lowVoltage/Sets the Low voltage for channel <Ch> and RFFE bus interface
/// HIPAR lowVoltage/<Bus>.
/// HIPAR highVoltage/Sets the High voltage for channel <Ch> and RFFE bus interface
/// HIPAR highVoltage/<Bus>.
ViStatus _VI_FUNC rsvna_ConfigureRFFESettings(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   rffeNumber,
                                              ViBoolean rffeState,
                                              ViReal64  frequency,
                                              ViReal64  ioVoltage,
                                              ViReal64  lowVoltage,
                                              ViReal64  highVoltage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rffeNumber, 1, 2),
            3, "RFFE Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Bus%ld", channel, rffeNumber);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_RFFE_STATE, rffeState),
            4, "RFFE State");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_FREQUENCY, frequency),
            5, "Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_IO_VOLTAGE, ioVoltage),
            6, "IO Voltage");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_LOW_VOLTAGE, lowVoltage),
            7, "Low Voltage");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_RFFE_HIGH_VOLTAGE, highVoltage),
            8, "High Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures and sends the RFFE command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR command/Defines an RFFE command for channel <Ch> and RFFE bus interface <Bus>.
ViStatus _VI_FUNC rsvna_ConfigureRFFECommand(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   rffeNumber,
                                             ViString  command)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rffeNumber, 1, 2),
            3, "RFFE Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Bus%ld", channel, rffeNumber);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RFFE_COMMAND, command),
            4, "Command");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_RFFE_SEND_COMMAND, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the GPIO settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR gpioState/This control enables/disables GPIO port <port> in the Sweep Sequencer
/// HIPAR gpioState/for channel <Ch>
/// HIPAR voltage/Sets the default voltage of the respective GPIO port.
ViStatus _VI_FUNC rsvna_ConfigureGPIO(ViSession instrSession,
                                      ViInt32   channel,
                                      ViInt32   gpioPortNumber,
                                      ViBoolean gpioState,
                                      ViReal64  voltage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gpioPortNumber, 1, 10),
            3, "GPIO Port Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",GPIO%ld", channel, gpioPortNumber);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_GPIO_STATE, gpioState),
            4, "GPIO State");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GPIO_VOLTAGE, voltage),
            5, "Voltage");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_GPIO_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an upper bound of the current to be measured on the respective
/// HIFN  GPIO pin. The analyzer firmware automatically selects a suitable shunt
/// HIFN  resistance, which can be queried using rsvna_QueryGPIOShunt.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR range/Upper bound of the current to be measured on the respective GPIO pin.
ViStatus _VI_FUNC rsvna_ConfigureGPIORange(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   gpioPortNumber,
                                           ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPort[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GPIO_RANGE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_GPIO_RANGE,
                    "GPIOPort", gpioPortNumber - 1, RS_REPCAP_BUF_SIZE, repCapPort), 3, "GPIO Port Number");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCapPort, RS_REPCAP_BUF_SIZE - strlen(repCap));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GPIO_RANGE, range),
            4, "range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the shunt resistance selected by the analyzer firmware for the
/// HIFN  configured current range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR shunt/Returns the shunt resistance selected by the analyzer firmware for the
/// HIPAR shunt/configured current range.
ViStatus _VI_FUNC rsvna_QueryGPIOShunt(ViSession instrSession,
                                       ViInt32   channel,
                                       ViInt32   gpioPortNumber,
                                       ViReal64* shunt)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPort[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GPIO_SHUNT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_GPIO_SHUNT,
                    "GPIOPort", gpioPortNumber - 1, RS_REPCAP_BUF_SIZE, repCapPort), 3, "GPIO Port Number");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap));
    strncat(repCap, repCapPort, RS_REPCAP_BUF_SIZE - strlen(repCap));

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GPIO_SHUNT, shunt),
            4, "Shunt");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the voltage/current measurements on all GPIO pins.ration.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_GPIOStartAllMeasurements(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_GPIO_START_ALL_MEASUREMENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the measurement time for the voltage/current measurements on the
/// HIFN  RFFE and GPIO pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR time/Sets the measurement time for the voltage/current measurements on the
/// HIPAR time/RFFE and GPIO pins.
ViStatus _VI_FUNC rsvna_ConfigureGPIOMeasurementTime(ViSession instrSession,
                                                     ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, "", RSVNA_ATTR_GPIO_MEASUREMENT_TIME, time),
            2, "Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the sum of the currents measured at the specified GPIO ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Size of ports array.
/// HIPAR ports/List of GPIO port numbers The list must contain minimum two and
/// HIPAR ports/maximum 10 numbers between 1 and 10 in arbitrary order.
/// HIPAR result/Returns the sum of the currents measured at the specified GPIO ports.
ViStatus _VI_FUNC rsvna_QueryGPIOSum(ViSession instrSession,
                                     ViInt32   arraySize,
                                     ViInt32   ports[],
                                     ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   temp[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 2, "Array Size");
    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%ld", ports[0]);

    for (int i = 1; i < arraySize; i++)
    {
        snprintf(temp, RS_MAX_MESSAGE_BUF_SIZE, ",%ld", ports[i]);
        strncat(parameter, temp, RS_MAX_MESSAGE_BUF_SIZE - strlen(parameter));
    }

    checkErr(RsCore_GetAttributeViReal64WithQparam(instrSession, "",
        RSVNA_ATTR_HIDDEN_GPIO_SUM,
        parameter, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the current measurement on the related GPIO pin
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR measurement/Returns the results of the current measurement on the related GPIO
/// HIPAR measurement/pin.
ViStatus _VI_FUNC rsvna_GPIOCurrentMeasurement(ViSession instrSession,
                                               ViInt32   gpioPortNumber,
                                               ViReal64* measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GPIO_CURRENT_MEASUREMENT,
            "GPIOPort", gpioPortNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "GPIO Port Number");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GPIO_CURRENT_MEASUREMENT, measurement),
            3, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the current measurement on all GPIO pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Size of measurements array.
/// HIPAR results/Returns the results of the current measurement on all GPIO pins.
/// HIPAR returnedValues/Number of returned results.
ViStatus _VI_FUNC rsvna_GPIOCurrentMeasurementAll(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 2, "Array size");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSVNA_ATTR_GPIO_CURRENT_MEASUREMENT_ALL, "", &cmd));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the voltage measurement on the related GPIO
/// HIFN  pin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR measurement/Returns the results of the voltage measurement on the related GPIO
/// HIPAR measurement/pin.
ViStatus _VI_FUNC rsvna_GPIOVoltageMeasurement(ViSession instrSession,
                                               ViInt32   gpioPortNumber,
                                               ViReal64* measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GPIO_VOLTAGE_MEASUREMENT,
            "GPIOPort", gpioPortNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "GPIO Port Number");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GPIO_VOLTAGE_MEASUREMENT, measurement),
            3, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the voltage measurement on all GPIO pins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Size of measurements array.
/// HIPAR results/Returns the results of the voltage measurement on all GPIO pins.
/// HIPAR returnedValues/Number of returned results.
ViStatus _VI_FUNC rsvna_GPIOVoltageMeasurementAll(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 2, "Array size");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSVNA_ATTR_GPIO_VOLTAGE_MEASUREMENT_ALL, "", &cmd));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the length of the command sequence defined for a
/// HIFN  particular channel and sweep segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/This control selects the sweep type.
/// HIPAR sweepSegment/Segment number
/// HIPAR count/Returns the length of the command/switch sequence defined for a
/// HIPAR count/particular channel and sweep segment.
ViStatus _VI_FUNC rsvna_QuerySweepSequencerCount(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   sweepType,
                                                 ViInt32   sweepSegment,
                                                 ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_UNSEGMENTED_SWEEP, RSVNA_VAL_SEGMENTED_SWEEP),
            3, "Sweep Type");

    if (sweepType == RSVNA_VAL_UNSEGMENTED_SWEEP)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

        viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_COUNT, count),
                5, "Count");
    }
    else
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld", channel, sweepSegment);

        viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SEQUENCER_SEGMENTED_SWEEP_COUNT, count),
                5, "Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the command/switch sequence for the respective
/// HIFN  channel and sweep segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/This control selects the sweep type.
/// HIPAR sweepSegment/Segment number
ViStatus _VI_FUNC rsvna_DeleteAllSweepSequencerItems(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   sweepType,
                                                     ViInt32   sweepSegment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_UNSEGMENTED_SWEEP, RSVNA_VAL_SEGMENTED_SWEEP),
            3, "Sweep Type");

    if (sweepType == RSVNA_VAL_UNSEGMENTED_SWEEP)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

        checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_DELETE_ALL, NULL));
    }
    else
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld", channel, sweepSegment);

        checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SEQUENCER_SEGMENTED_SWEEP_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the RFFE command(s) to be executed at the start
/// HIFN  of each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/This control selects the sweep type.
/// HIPAR sweepSegment/Segment number
/// HIPAR sequenceNumber/Sequence number, defining the order in which the commands/
/// HIPAR rffeNumber/RFFE bus interface number
/// HIPAR command/Defines the RFFE command(s) to be executed at the start of each
/// HIPAR command/segment.
ViStatus _VI_FUNC rsvna_ConfigureSweepSequencerRFFECommand(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   sweepType,
                                                           ViInt32   sweepSegment,
                                                           ViInt32   sequenceNumber,
                                                           ViInt32   rffeNumber,
                                                           ViString  command)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_sequence[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bus[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  rffeRepCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_UNSEGMENTED_SWEEP, RSVNA_VAL_SEGMENTED_SWEEP),
            3, "Sweep Type");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "RFFEBus", &rffeRepCapCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rffeNumber, 1, rffeRepCapCount),
            6, "GPIO Port Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_RFFE_COMMAND,
            "RFFEBus", rffeNumber - 1, RS_REPCAP_BUF_SIZE, repCap_bus),
            6, "GPIO Port Number");

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    switch (sweepType)
    {
    case RSVNA_VAL_UNSEGMENTED_SWEEP:
        //check if sequence number already exists
        rsvna_GetOrAddSequenceRepCap(instrSession,
                                     RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_RFFE_COMMAND,
                                     sequenceNumber - 1,
                                     RS_REPCAP_BUF_SIZE,
                                     repCap_sequence);

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_channel, repCap_sequence, repCap_bus);

        viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_RFFE_COMMAND, command),
                7, "Command");
        break;

    case RSVNA_VAL_SEGMENTED_SWEEP:
        snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", sweepSegment);

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_channel, repCap_segment, repCap_bus);

        viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SEQUENCER_SEGMENTED_SWEEP_RFFE_COMMAND, command),
                7, "Command");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the GPIO voltage settings to be applied at the
/// HIFN  start of each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/This control selects the sweep type.
/// HIPAR sweepSegment/Segment number
/// HIPAR sequenceNumber/Sequence number, defining the order in which the commands/
/// HIPAR gpioPortNumber/GPIO port number
/// HIPAR gpioVoltage/Sets the GPIO voltage settings to be applied at the start of each
/// HIPAR gpioVoltage/sweep.
ViStatus _VI_FUNC rsvna_ConfigureSweepSequencerGPIOVoltage(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   sweepType,
                                                           ViInt32   sweepSegment,
                                                           ViInt32   sequenceNumber,
                                                           ViInt32   gpioPortNumber,
                                                           ViReal64  gpioVoltage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_segment[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_sequence[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_gpio[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  gpioRepCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_UNSEGMENTED_SWEEP, RSVNA_VAL_SEGMENTED_SWEEP),
            3, "Sweep Type");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "GPIOPort", &gpioRepCapCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gpioPortNumber, 1, gpioRepCapCount),
            6, "GPIO Port Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_GPIO_VOLTAGE,
            "GPIOPort", gpioPortNumber - 1, RS_REPCAP_BUF_SIZE, repCap_gpio),
            6, "GPIO Port Number");

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    switch (sweepType)
    {
    case RSVNA_VAL_UNSEGMENTED_SWEEP:
        //check if sequence number already exists
        rsvna_GetOrAddSequenceRepCap(instrSession,
                                     RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_GPIO_VOLTAGE,
                                     sequenceNumber - 1,
                                     RS_REPCAP_BUF_SIZE,
                                     repCap_sequence);

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_channel, repCap_sequence, repCap_gpio);

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_GPIO_VOLTAGE, gpioVoltage),
                7, "GPIO Voltage");
        break;

    case RSVNA_VAL_SEGMENTED_SWEEP:
        snprintf(repCap_segment, RS_REPCAP_BUF_SIZE, "Seg%ld", sweepSegment);

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_channel, repCap_segment, repCap_gpio);

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SEQUENCER_SEGMENTED_SWEEP_GPIO_VOLTAGE, gpioVoltage),
                7, "GPIO Voltage");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines delays between subsequent sequence steps and
/// HIFN  between the final sequence step and the sweep (segment) start.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sweepType/This control selects the sweep type.
/// HIPAR sweepSegment/Segment number
/// HIPAR sequenceNumber/Sequence number, defining the order in which the commands/
/// HIPAR delayTime/Sets delays between subsequent sequence steps and between the final
/// HIPAR delayTime/sequence step and the sweep (segment) start.
ViStatus _VI_FUNC rsvna_ConfigureSweepSequencerDelayTime(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   sweepType,
                                                         ViInt32   sweepSegment,
                                                         ViInt32   sequenceNumber,
                                                         ViReal64  delayTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_sequence[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_UNSEGMENTED_SWEEP, RSVNA_VAL_SEGMENTED_SWEEP),
            3, "Sweep Type");

    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    switch (sweepType)
    {
    case RSVNA_VAL_UNSEGMENTED_SWEEP:
        //check if sequence number already exists
        rsvna_GetOrAddSequenceRepCap(instrSession,
                                     RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_DELAY_TIME,
                                     sequenceNumber - 1,
                                     RS_REPCAP_BUF_SIZE,
                                     repCap_sequence);

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_channel, repCap_sequence);

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SEQUENCER_UNSEGMENTED_SWEEP_DELAY_TIME, delayTime),
                6, "Delay Time");
        break;

    case RSVNA_VAL_SEGMENTED_SWEEP:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,Seg%ld", repCap_channel, sweepSegment);

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SEQUENCER_SEGMENTED_SWEEP_DELAY_TIME, delayTime),
                6, "Delay Time");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Automatic Gain Control in the receiver
/// HIFN  path. The selected ACG mode applies to all analyzer ports, however,
/// HIFN  different settings are allowed for the measurement receiver (b-waves)
/// HIFN  and the reference receiver (a-waves).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurementChannel/This control configures the Automatic Gain Control (AGC) for the
/// HIPAR measurementChannel/measurement channel (b-waves).
/// HIPAR reserved/This control is reserved for future use.
ViStatus _VI_FUNC rsvna_ConfigureAGCMode(ViSession instrSession,
                                         ViInt32   measurementChannel,
                                         ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_AUTOMATIC_GAIN_CONTROL_FOR_MEASUREMENT_CHANNEL, measurementChannel),
            2, "Measurement Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Automatic Gain Control in the receiver
/// HIFN  path. The selected ACG mode applies to all analyzer ports, however,
/// HIFN  different settings are allowed for the measurement receiver (b-waves)
/// HIFN  and the reference receiver (a-waves).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR mode/This control configures the Automatic Gain Control (AGC).
ViStatus _VI_FUNC rsvna_ConfigureAGCGlobalMode(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_AGC_GLOBAL_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures manual AGC settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR drivePort/Select analyzer port.
/// HIPAR wave/Selects the type of the wave quantity.
/// HIPAR waveNumber/Port number assignment. The input (stimulus) or output
/// HIPAR waveNumber/(response) port number is selected. The range of output and
/// HIPAR waveNumber/input port numbers depends on the analyzer model.
/// HIPAR gainMode/This control configures the Automatic Gain Control (AGC).
ViStatus _VI_FUNC rsvna_ConfigureAGC(ViSession instrSession,
                                     ViInt32   channel,
                                     ViInt32   drivePort,
                                     ViInt32   wave,
                                     ViInt32   waveNumber,
                                     ViInt32   gainMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   wavePort[RS_MAX_MESSAGE_LEN];
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wave, RSVNA_VAL_A, RSVNA_VAL_B),
            4, "Wave");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gainMode, RSVNA_VAL_AGC_MODE_AUTO, RSVNA_VAL_AGC_MODE_LOW_DISTORTION),
            6, "Gain Mode");

    snprintf(wavePort, RS_MAX_MESSAGE_LEN, "%s%ldD%ld", wave ? "B" : "A", waveNumber, drivePort);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:POW:GAIN '%s',%s", channel, wavePort, AGCModeArr[gainMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the same manual gain control (GC) <Mode> to all a and b wave
/// HIFN  receivers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gainControl/Selects the manual gain control (GC) <Mode> to apply to all a and b
/// HIPAR gainControl/wave receivers.
ViStatus _VI_FUNC rsvna_ConfigureAGCAllAAndBWaves(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   gainControl)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_AGC_ALL_A_AND_B_WAVES_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_AGC_ALL_A_AND_B_WAVES_MODE, gainControl),
            3, "Gain Control");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the segment specific AGC settings. It is
/// HIFN  available for segmented sweep type only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enable/Enables the segment specific AGC settings.
ViStatus _VI_FUNC rsvna_ConfigureAGCSegmentedEnable(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap,
                    RSVNA_ATTR_AGC_SEGMENTED_ENABLE,
                    !enable
                ), 3, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures segment specific AGC settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Port number assignment. The input (stimulus) or output (response) port
/// HIPAR segment/number is selected. The range of output and input port numbers depends
/// HIPAR segment/on the analyzer model.
/// HIPAR drivePort/Select analyzer port.
/// HIPAR wave/Selects the type of the wave quantity.
/// HIPAR waveNumber/Port number assignment. The input (stimulus) or output
/// HIPAR waveNumber/(response) port number is selected. The range of output and
/// HIPAR waveNumber/input port numbers depends on the analyzer model.
/// HIPAR gainMode/This control configures the Automatic Gain Control (AGC).
ViStatus _VI_FUNC rsvna_ConfigureAGCSegment(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   segment,
                                            ViInt32   drivePort,
                                            ViInt32   wave,
                                            ViInt32   waveNumber,
                                            ViInt32   gainMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   wavePort[RS_MAX_MESSAGE_LEN] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  segCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUERY_SWEEP_SEGMENT_COUNT, &segCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wave, 1, segCount),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wave, RSVNA_VAL_A, RSVNA_VAL_B),
            5, "Wave");

    snprintf(wavePort, RS_MAX_MESSAGE_LEN, "%s%ldD%ld", wave ? "B" : "A", waveNumber, drivePort);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gainMode, RSVNA_VAL_AGC_MODE_AUTO, RSVNA_VAL_AGC_MODE_LOW_DISTORTION),
            7, "Gain Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:SEGM%ld:POW:GAIN '%s',%s", channel,
             segment,
             wavePort,
             AGCModeArr[gainMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the drive port order of the related DUT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR portOrder/Sets or queries the drive port order of the related DUT (= port group,
/// HIPAR portOrder/created using SOURce<Ch>:GROup<Grp>:PPORts)
ViStatus _VI_FUNC rsvna_ConfigureParallelMeasurementPortGroupsPortOrder(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViString  portOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_PORT_GROUPS_PORT_ORDER, portOrder),
            4, "Port Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command is used to activate/deactivate a frequency offset for
/// HIFN  parallel measurements on an R&S ZNB/ZNBT with Internal Second Source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR enabled/This command is used to activate/deactivate a frequency offset for
/// HIPAR enabled/parallel measurements on an R&S ZNB/ZNBT with Internal Second Source.
ViStatus _VI_FUNC rsvna_ConfigureParallelMeasurementPortGroupsWithFrequencyOffsetEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PORT_GROUPS_PARALLEL_MEASUREMENT_WITH_FREQUENCY_OFFSET_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command is used to query the operation mode of Simultaneous
/// HIFN  Measurement with Frequency Offset. The list below contains all possible
/// HIFN  return values of this query.
/// HIFN     Please note that this query will return an error if the channel
/// HIFN  specified by <Ch> does not contain any trace. This is due to the fact
/// HIFN  that without traces no measurements are performed so that no
/// HIFN  information about Simultaneous Measurement with Frequency Offset is
/// HIFN  available.
/// HIFN     This query waits until all previous commands are fully executed
/// HIFN  because these commands may affect the operation mode of Simultaneous
/// HIFN  Measurement with Frequency Offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'State Indication Warning Parameter' parameter.
/// HIPAR stateIndicationWarning/This command is used to query the operation mode of Simultaneous
/// HIPAR stateIndicationWarning/Measurement with Frequency Offset. The list below contains all possible
/// HIPAR stateIndicationWarning/return values of this query.
ViStatus _VI_FUNC rsvna_QueryParallelMeasurementPortGroupsStateIndicationWarning(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   bufferSize,
                            ViChar    stateIndicationWarning[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Group%ld", channel, group);

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_PORT_GROUPS_PORT_ORDER,
            bufferSize, stateIndicationWarning),
            5, "State Indication Warning");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the source of the trigger events that the analyzer uses
/// HIFN to start a measurement sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR mode/Selects the source of the trigger events.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyConversionMeasurementMode(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FREQUENCY_CONVERSION_MEASUREMENT_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_FREQUENCY_CONVERSION_MEASUREMENT_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the fixed frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR ifFrequency/Defines a fixed frequency and assigns it to the IF signal.
/// HIPAR loFrequency/Defines a fixed frequency and assigns it to the LO signal.
/// HIPAR rfFrequency/Defines a fixed frequency and assigns it to the RF signal.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyConversionMixer(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  ifFrequency,
                                                          ViReal64  loFrequency,
                                                          ViReal64  rfFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FREQUENCY_CONVERSION_MIXER_FIXED_IF_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_FREQUENCY_CONVERSION_MIXER_FIXED_IF_FREQUENCY, ifFrequency),
            3, "IF Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_FREQUENCY_CONVERSION_MIXER_FIXED_LO_FREQUENCY, loFrequency),
            4, "LO Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_FREQUENCY_CONVERSION_MIXER_FIXED_RF_FREQUENCY, rfFrequency),
            5, "RF Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This control defines a linear factor to modify the internal source power at port <Pt>
/// HIFN as a function of the stimulus frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR portNumber/Port number of the analyzer.
/// HIPAR slope/Defines a linear factor to modify the internal source power.
ViStatus _VI_FUNC rsvna_ConfigureSourcePowerCalibrationSlope(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   portNumber,
                                                             ViReal64  slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Pt%ld", channel, portNumber);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_POWER_CALIBRATION_SLOPE, slope),
            4, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the port-specific source frequency for frequency-converting measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR portNumber/Port number of the analyzer.
/// HIPAR numerator/This control defines the numerator.
/// HIPAR denominator/This control defines the denominator.
/// HIPAR offset/This control defines the offset.
/// HIPAR sweepType/This control select the sweep type.
ViStatus _VI_FUNC rsvna_ConfigureFrequencyConversion(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   portNumber,
                                                     ViInt32   numerator,
                                                     ViInt32   denominator,
                                                     ViReal64  offset,
                                                     ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_SWEEP_TYPE_SWEEP, RSVNA_VAL_SWEEP_TYPE_FIXED),
            7, "Sweep Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FREQ%ld:CONV:ARB:IFR %ld,%ld,%lG,%s",
            channel, portNumber, numerator, denominator, offset, sweepTypeConvArr[sweepType]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function Defines the receiver frequency of a power meter used for frequency-converting measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR powerMeterNumber/This control sets the number of the configured power meter.
/// HIPAR numerator/This control defines the numerator.
/// HIPAR denominator/This control defines the denominator.
/// HIPAR offset/This control defines the offset.
/// HIPAR sweepType/This control select the sweep type.
ViStatus _VI_FUNC rsvna_ConfigureReceiverFrequencyConversion(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   powerMeterNumber,
                                                             ViInt32   numerator,
                                                             ViInt32   denominator,
                                                             ViReal64  offset,
                                                             ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_SWEEP_TYPE_SWEEP, RSVNA_VAL_SWEEP_TYPE_FIXED),
            7, "Sweep Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:ARB:PMET%ld %ld,%ld,%lf,%s",
            channel, powerMeterNumber, numerator, denominator, offset, sweepTypeConvArr[sweepType]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines an external generator frequency for frequency-converting measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number
/// HIPAR state/This control switch the generator on or off.
/// HIPAR numerator/This control defines the numerator.
/// HIPAR denominator/This control defines the denominator.
/// HIPAR offset/This control defines the offset.
/// HIPAR sweepType/This control select the sweep type.
ViStatus _VI_FUNC rsvna_ConfigureGeneratorFrequencyConversion(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   generatorNumber,
                            ViBoolean state,
                            ViInt32   numerator,
                            ViInt32   denominator,
                            ViReal64  offset,
                            ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepType, RSVNA_VAL_SWEEP_TYPE_SWEEP, RSVNA_VAL_SWEEP_TYPE_FIXED),
            8, "Sweep Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FREQ:CONV:ARB:EFR%ld %s,%ld,%ld,%lG,%s", channel, generatorNumber, state ? "ON" : "OFF", numerator, denominator, offset, sweepTypeConvArr[sweepType]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the selected segment, this command applies the same manual gain
/// HIFN  control (GC) <Mode> to all a and b wave receivers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Select configured segment.
/// HIPAR gainControl/For the selected segment, selects the manual gain control (GC) <Mode>
/// HIPAR gainControl/to apply to all a and b wave receivers.
ViStatus _VI_FUNC rsvna_ConfigureAGCSegmentAllAAndBWaves(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   segment,
                                                         ViInt32   gainControl)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, segment, 1), 3, "Segment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld", channel, segment);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_AGC_SEGMENT_ALL_A_AND_B_WAVES_MODE, gainControl),
            4, "Gain Control");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows you to select between AGC algorithms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR method/Allows you to select between AGC algorithms.
ViStatus _VI_FUNC rsvna_ConfigureAGCGlobalMethod(ViSession instrSession,
                                                 ViInt32   method)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_AGC_GLOBAL_METHOD, method),
            2, "Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function resets the AGC properties and starts a AGC
/// HIFN learning sweep on the currently visible S-Parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_StartAGCLearningSweep(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_START_AGC_LEARNING_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function saves the results of an AGC learning and revert
/// HIFN changes from LearnSweep in channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_SaveAGCLearningResults(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SAVE_AGC_LEARNING_RESULTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the number of converter/mixer stages for scalar mixer
/// HIFN  measurements and intermodulation distortion measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR mixerStages/Sets the number of converter/mixer stages
ViStatus _VI_FUNC rsvna_ConfigureMixerNumberOfStages(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   mixerStages)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_NUMBER_OF_STAGES, mixerStages),
            3, "Mixer Stages");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the test ports and configures the mixer input signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR RFPort/Selects an analyzer port as source port for the RF signal.
/// HIPAR IFPort/Selects an analyzer port as receive port for the IF signal.
ViStatus _VI_FUNC rsvna_ConfigureMixerPorts(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   RFPort,
                                            ViInt32   IFPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_RF_PORT, RFPort),
            3, "RF Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_IF_PORT, IFPort),
            4, "IF Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the mixer LO port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR stage/Select configured mixer stage.
/// HIPAR source/Selects source of LO signal.
/// HIPAR port/Selects an analyzer port or an external generator number as a signal
/// HIPAR port/source for the LO or LO2 signal.
ViStatus _VI_FUNC rsvna_ConfigureMixerLO(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   stage,
                                         ViInt32   source,
                                         ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString sourceArr[] = {"NONE", "PORT", "GEN"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, stage, RSVNA_VAL_MIXER_STAGE1, RSVNA_VAL_MIXER_STAGE2),
            3, "Stage");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSVNA_VAL_MIXER_LO_NONE, RSVNA_VAL_MIXER_LO_GENERATOR),
            4, "Source");

    if (source == RSVNA_VAL_MIXER_LO_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:MIX:LOP%ld %s", channel, stage, sourceArr[source]);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:MIX:LOP%ld %s,%ld", channel, stage, sourceArr[source], port);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frequency of the RF and the LO signal(s) and the
/// HIFN     analyzer (IF signal) frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR type/Select configured frequency type.
/// HIPAR port/Selects port (signal) to be frequency freq or fixed frequency.
/// HIPAR fixedFrequency/Sets a fixed frequency to selected port.
ViStatus _VI_FUNC rsvna_ConfigureMixerFrequency(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   type,
                                                ViInt32   port,
                                                ViReal64  fixedFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    if (type == RSVNA_VAL_MIXER_BASE)
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_BASE_FREQUENCY_PORT, port),
                4, "Port");
    }
    else
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_FIXED_FREQUENCY_PORT, port),
                4, "Port");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:MIX:MFF %s,%le", channel, mixerPortArr[port], fixedFrequency);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsvna_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the power of the RF and the LO signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Selects configured port.
/// HIPAR fixedPower/Sets a fixed power to the RF, LO 1, LO 2, or to the IF signal.
/// HIPAR type/Sets the RF, LO 1, LO 2, or the IF signal ports to fixed power or to
/// HIPAR type/the channel base power.
ViStatus _VI_FUNC rsvna_ConfigureMixerPower(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   port,
                                            ViInt32   type,
                                            ViReal64  fixedPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString typeArr[] = {"FIX", "FUND"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_VAL_MIXER_PORT_RF, RSVNA_VAL_MIXER_PORT_IF),
            3, "Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSVNA_VAL_MIXER_FIXED, RSVNA_VAL_MIXER_BASE),
            4, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FREQ:CONV:MIX:PMOD %s,%s", channel, mixerPortArr[port], typeArr[type]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsvna_CheckStatus(instrSession));

    if (type == RSVNA_VAL_MIXER_FIXED)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FREQ:CONV:MIX:PMF %s,%le", channel, mixerPortArr[port], fixedPower);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the mixer input signal which is at the fundamental power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR inputSignal/Selects the mixer input signal.
ViStatus _VI_FUNC rsvna_ConfigureMixerInputSignal(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   inputSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_INPUT_SIGNAL, inputSignal),
            3, "Input Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frequency conversion factors for the RF and LO 1 /LO 2
/// HIFN  signals, i.e. defines an additional frequency conversion between source
/// HIFN  (port or generator) and mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Selects configured port.
/// HIPAR numerator/Sets the numerator of the conversion factor ratio.
/// HIPAR denominator/Sets the Denominator of the conversion factor ratio.
ViStatus _VI_FUNC rsvna_ConfigureMixerMultiplication(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   port,
                                                     ViInt32   numerator,
                                                     ViInt32   denominator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K4"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1),
                2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_VAL_MIXER_MULTIPLICATION_PORT_RF, RSVNA_VAL_MIXER_MULTIPLICATION_PORT_LO2),
            3, "Port");

    if (port == RSVNA_VAL_MIXER_MULTIPLICATION_PORT_RF)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:MIX:RFM %ld,%ld", channel, numerator, denominator);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:CONV:MIX:LOM%ld %ld,%ld", channel, port, numerator, denominator);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the frequency conversion mode of the IF signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR stage/Select configured mixer stage.
/// HIPAR frequencyConversionMode/This control selects the frequency conversion mode of the IF signal.
ViStatus _VI_FUNC rsvna_ConfigureMixerConversion(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   stage,
                                                 ViInt32   frequencyConversionMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",", channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_MIXER_CONVERSION_MODE,
                    "MixerStage",
                    stage-1,
                    10,
                    repCap2), 3, "Stage");
    strcat(repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_CONVERSION_MODE, frequencyConversionMode),
            4, "Frequency Conversion Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the measurement of the intermodulation spectrum
/// HIFN  without creating a new channel. If a new channel is desired, it must be
/// HIFN  created first.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR enable/Enables or disables intermodulation spectrum measurement.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationState(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_STATE, enable),
            3, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the source settings for the intermodulation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR configure/Select configured signal source.
/// HIPAR source/Selects source of signal.
/// HIPAR port/Number of the port for the internal source or of the generator.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationSourcePort(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   configure,
                                                           ViInt32   source,
                                                           ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    static   ViString sourceArr[] = {"NONE", "PORT", "GEN"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K14"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, configure, RSVNA_VAL_INTERMODULATION_LOWER_TONE, RSVNA_VAL_INTERMODULATION_UPPER_TONE),
            3, "Configure");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSVNA_VAL_SOURCE_NONE, RSVNA_VAL_SOURCE_GENERATOR),
            4, "Source");

    if (source == RSVNA_VAL_SOURCE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:IMOD:%sTON %s", channel, configure ? "U" : "L", sourceArr[source]);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:FREQ:IMOD:%sTON %s,%ld", channel, configure ? "U" : "L", sourceArr[source], port);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the receiving port for the signal from the output of the
/// HIFN  DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Selects the receiving port for the intermodulation measurement.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationReceivePort(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_RECEIVER_PORT, port),
            3, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the source port for the receiver power calibration of an
/// HIFN  intermodulation measurement.
/// HIFN     Note(s):
/// HIFN     (1) Available only with K14 option.
/// HIFN     (2) To define source and receive ports use
/// HIFN  rsvna_ConfigureIntermodulationSourcePort and
/// HIFN  rsvna_ConfigureIntermodulationReceivePort.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR sourcePort/String parameter to identify the source port.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationCalSourcePort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViString  sourcePort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_CAL_SOURCE_PORT, sourcePort),
            3, "Source Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the source for the two tone output signal for
/// HIFN  intermodulation measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR source/Selects the source for the two tone output signal for intermodulation
/// HIPAR source/measurements.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationTwoToneSignalSource(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    if (source == RSVNA_VAL_TTO_SOURCE_PORT1)
    {
        checkErr(RsCore_CheckInstrumentOptions(instrSession, "B16"));
    }

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_TWO_TONE_SIGNAL_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the order(s) of the measured intermodulation products. Each
/// HIFN  order requires an additional measurement at a different frequency,
/// HIFN  therefore the measurement time increases with the number of selected orders.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR imOrder/Order of IM products
/// HIPAR enable/Enables or disables the measurement of the intermodulation products of
/// HIPAR enable/selected IM order.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationOrderState(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   imOrder,
                                                           ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[10] = "";
    ViInt32  orderArr[] = {0, 0, 0, 0, 0, 1, 0, 2, 0, 3};

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",", channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSVNA_ATTR_INTERMODULATION_ORDER_STATE,
                    "IMOrder",
                    orderArr[imOrder],
                    10,
                    repCap2), 3, "IM Order");

    strcat(repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_ORDER_STATE, enable),
            4, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the maximum order of intermodulation products for the
/// HIFN  intermodulation spectrum measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR maximumOrder/Maximum order of IM products
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationMaximumOrder(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   maximumOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_MAXIMUM_ORDER, maximumOrder),
            3, "Maximum Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frequency difference between the upper and the lower
/// HIFN  tone. The tone distance is kept constant across the entire sweep.

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR toneDistance/Defines the tone distance (frequency offset) between the upper and the
/// HIPAR toneDistance/lower tone.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationToneDistance(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViReal64  toneDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_TONE_DISTANCE, toneDistance),
            3, "Tone Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables the intermodulation measurement and switches back to normal
/// HIFN  (non frequencyconverting) mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ResetIntermodulation(ViSession instrSession,
                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies how the lower and upper tone are combined for
/// HIFN  intermodulation and two-tone group delay measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR combiner/Specifies how the lower and upper tone are combined for
/// HIPAR combiner/intermodulation and two-tone group delay measurements.
ViStatus _VI_FUNC rsvna_ConfigureIntermodulationCombiner(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   combiner)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INTERMODULATION_COMBINER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_COMBINER, combiner),
            3, "Combiner");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates a new Spectrum measurement channel, centered at the position
/// HIFN  of the active marker. If there is no marker in the active
/// HIFN  intermodulation trace, the center frequency of the related channel is
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_IntermodulationAddCWMode(ViSession instrSession,
                                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INTERMODULATION_ADD_CW_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_INTERMODULATION_ADD_CW_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables/disables the calculation of primed wave quantities for
/// HIFN  two-tone group delay measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR usePrimedWaves/Enables/disables the calculation of primed wave quantities for
/// HIPAR usePrimedWaves/two-tone group delay measurements.
ViStatus _VI_FUNC rsvna_ConfigurePortConfigUsePrimedWaves(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean usePrimedWaves)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_USE_PRIMED_WAVES,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PORT_CONFIG_USE_PRIMED_WAVES, usePrimedWaves),
            3, "Use Primed Waves");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the constant receiver frequency offset for primed wave
/// HIFN  quantities a' and b' relative to the common receiver frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR frequency/Defines the constant receiver frequency offset for primed wave
/// HIPAR frequency/quantities a' and b' relative to the common receiver frequency.
ViStatus _VI_FUNC rsvna_ConfigurePortConfigPrimedWavesFrequency(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_PRIMED_WAVES_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_PRIMED_WAVES_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PORT_CONFIG_PRIMED_WAVES_FREQUENCY, frequency),
            4, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the constant receiver frequency offset for unprimed wave
/// HIFN  quantities a, b relative to the common receiver frequency.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_PORT_CONFIG_WAVES_FREQUENCY_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe<Ch>:]FREQuency<Pt>:OFFSet:WAVes
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR frequencyOffset/Defines the constant receiver frequency offset for unprimed wave
/// HIPAR frequencyOffset/quantities a, b relative to the common receiver frequency.
ViStatus _VI_FUNC rsvna_ConfigurePortConfigWavesFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_WAVES_FREQUENCY_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_WAVES_FREQUENCY_OFFSET,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PORT_CONFIG_WAVES_FREQUENCY_OFFSET, frequencyOffset),
            4, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frequencies of the 'a wave receivers', i.e. the
/// HIFN  frequencies to which the receivers of port <Pt> are set while this port
/// HIFN  is the source port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR conversion/Defines the frequencies of the 'a wave receivers', i.e. the
/// HIPAR conversion/frequencies to which the receivers of port <Pt> are set while this port
/// HIPAR conversion/is the source port.
ViStatus _VI_FUNC rsvna_ConfigurePortConfigReceiverFrequencyConversion(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViInt32   conversion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_RECEIVER_FREQUENCY_CONVERSION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PORT_CONFIG_RECEIVER_FREQUENCY_CONVERSION,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PORT_CONFIG_RECEIVER_FREQUENCY_CONVERSION, conversion),
            4, "Conversion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the direct generator/receiver access path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR directReceiverInput/Defines the direct generator/receiver access path.
ViStatus _VI_FUNC rsvna_ConfigureDirectReceiverInput(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   physicalPort,
                                                     ViInt32   directReceiverInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_RECEIVER_INPUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_RECEIVER_INPUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_RECEIVER_INPUT, directReceiverInput),
            4, "Direct Receiver Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the input paths of the Direct IF Access Input / Output
/// HIFN  connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIFN
/// HIFN     This setting only takes effect if [SENSe<Ch>:]PATH<Pt>:IFSWitch is set
/// HIFN  to IN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR inputPath/Selects the input paths of the Direct IF Access Input / Output
/// HIPAR inputPath/connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureDirectRearInputPath(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   physicalPort,
                                                     ViInt32   inputPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REAR_INPUT_PATH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REAR_INPUT_PATH,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_REAR_INPUT_PATH, inputPath),
            4, "Input Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIFN  <Pt> are determined in channel <Ch>.
/// HIFN
/// HIFN     This setting only takes effect if [SENSe<Ch>:]PATH<Pt>:IFSWitch is set
/// HIFN  to OUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR autoDetermineIFOut/Defines how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIPAR autoDetermineIFOut/<Pt> are determined in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureDirectAutoDetermineIFOut(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   physicalPort,
                                                          ViBoolean autoDetermineIFOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_AUTO_DETERMINE_IF_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_AUTO_DETERMINE_IF_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DIRECT_AUTO_DETERMINE_IF_OUT, autoDetermineIFOut),
            4, "Auto Determine IF Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the freqency conversion formula for Direct IF Access Input /
/// HIFN  Output ports IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR numerator/The specified numerator is rounded to the next higher integer; zero is
/// HIPAR numerator/not allowed.
/// HIPAR denominator/The specified denominator must be positive. It is rounded to the next
/// HIPAR denominator/higher integer.
/// HIPAR offset/The specified offset value is rounded to (positive or negative)
/// HIPAR offset/multiples of 1 Hz.
/// HIPAR sweeptype/Sets the sweep type.
ViStatus _VI_FUNC rsvna_ConfigureDirectRearIFFrequency(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   physicalPort,
                                                       ViInt32   numerator,
                                                       ViInt32   denominator,
                                                       ViReal64  offset,
                                                       ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  sweepTypeOut;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_REAR_IF_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_REAR_IF_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngRearLOSweepType, sweepType, &sweepTypeOut),
                7, "Sweep Type");

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,%ld,%.12lG,%s", numerator, denominator, offset, sweepTypeOut);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_DIRECT_REAR_IF_FREQUENCY, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines if and how the Direct IF Access Input / Output ports IF
/// HIFN  Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rearOut/Defines if and how the Direct IF Access Input / Output ports IF
/// HIPAR rearOut/Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureDirectRearOut(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   physicalPort,
                                               ViInt32   rearOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REAR_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REAR_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_REAR_OUT, rearOut),
            4, "Rear Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the direct generator/receiver access path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR measurementInput/Queries the direct generator/receiver access path.
ViStatus _VI_FUNC rsvna_QueryDirectMeasurementInput(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   physicalPort,
                                                    ViInt32*  measurementInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_INPUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_INPUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_MEASUREMENT_INPUT, measurementInput),
            4, "Measurement Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the direct generator/receiver access path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR referenceInput/Queries the direct generator/receiver access path.
ViStatus _VI_FUNC rsvna_QueryDirectReferenceInput(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   physicalPort,
                                                  ViInt32*  referenceInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_INPUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_INPUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_REFERENCE_INPUT, referenceInput),
            4, "Reference Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Query for the input paths of Direct IF Access Input / Output
/// HIFN  connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rearInputPath/Query for the input paths of Direct IF Access Input / Output
/// HIPAR rearInputPath/connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_QueryDirectMeasurementRearInputPath(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   physicalPort,
                                                            ViInt32*  rearInputPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_INPUT_PATH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_INPUT_PATH,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_INPUT_PATH, rearInputPath),
            4, "Rear Input Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Query for the input paths of Direct IF Access Input / Output
/// HIFN  connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rearInputPath/Query for the input paths of Direct IF Access Input / Output
/// HIPAR rearInputPath/connectors IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
ViStatus _VI_FUNC rsvna_QueryDirectReferenceRearInputPath(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   physicalPort,
                                                          ViInt32*  rearInputPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_REAR_INPUT_PATH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_REAR_INPUT_PATH,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_REFERENCE_REAR_INPUT_PATH, rearInputPath),
            4, "Rear Input Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIFN  <Pt> are determined in channel <Ch> if [SENSe<Ch>:]PATH<Pt>:IFSWitch is
/// HIFN  set to OUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR autoDetermineIFOut/Queries how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIPAR autoDetermineIFOut/<Pt> are determined in channel <Ch> if [SENSe<Ch>:]PATH<Pt>:IFSWitch is
/// HIPAR autoDetermineIFOut/set to OUT.
ViStatus _VI_FUNC rsvna_QueryDirectMeasurementAutoDetermineIFOut(
                            ViSession  instrSession,
                            ViInt32    channel,
                            ViInt32    physicalPort,
                            ViBoolean* autoDetermineIFOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_AUTO_DETERMINE_IF_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_AUTO_DETERMINE_IF_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DIRECT_MEASUREMENT_AUTO_DETERMINE_IF_OUT, autoDetermineIFOut),
            4, "Auto Determine IF Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIFN  <Pt> are determined in channel <Ch> if [SENSe<Ch>:]PATH<Pt>:IFSWitch is
/// HIFN  set to OUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR autoDetermineIFOut/Queries how the outgoing frequencies at IF Reference <Pt> and IF Meas
/// HIPAR autoDetermineIFOut/<Pt> are determined in channel <Ch> if [SENSe<Ch>:]PATH<Pt>:IFSWitch is
/// HIPAR autoDetermineIFOut/set to OUT.
ViStatus _VI_FUNC rsvna_QueryDirectReferenceAutoDetermineIFOut(
                            ViSession  instrSession,
                            ViInt32    channel,
                            ViInt32    physicalPort,
                            ViBoolean* autoDetermineIFOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_AUTO_DETERMINE_IF_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_AUTO_DETERMINE_IF_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DIRECT_REFERENCE_AUTO_DETERMINE_IF_OUT, autoDetermineIFOut),
            4, "Auto Determine IF Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the freqency conversion formula for Direct IF Access Input /
/// HIFN  Output ports IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR numerator/Returns the numerator.
/// HIPAR denominator/Returns the denominator.
/// HIPAR offset/Returns the offset value.
/// HIPAR sweeptype/Returns the sweep type.
ViStatus _VI_FUNC rsvna_QueryDirectMeasurementIFFrequency(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   physicalPort,
                                                          ViInt32   *numerator,
                                                          ViInt32   *denominator,
                                                          ViReal64  *offset,
                                                          ViInt32   *sweepType)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar    repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar    response[RS_MAX_MESSAGE_LEN] = "";
    ViReal64* resArr = NULL;
    ViInt32   count;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_MEASUREMENT_IF_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_MEASUREMENT_IF_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");

    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_GetAttributeViString(instrSession, repCap,
                                           RSVNA_ATTR_HIDDEN_DIRECT_MEASUREMENT_IF_FREQUENCY, RS_MAX_MESSAGE_LEN,
                                           response));

    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &resArr, &count));

    if (count == 4)
    {
        *numerator = (ViInt32) resArr[0];
        *denominator = (ViInt32) resArr[1];
        *offset = resArr[2];
        *sweepType = (ViInt32) resArr[3];
    }
    else
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    }

Error:
    if (resArr)
        free(resArr);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the freqency conversion formula for Direct IF Access Input /
/// HIFN  Output ports IF Reference <Pt> and IF Meas <Pt> in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR numerator/Returns the numerator.
/// HIPAR denominator/Returns the denominator.
/// HIPAR offset/Returns the offset value.
/// HIPAR sweeptype/Returns the sweep type.
ViStatus _VI_FUNC rsvna_QueryDirectReferenceIFFrequency(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   physicalPort,
                                                        ViInt32   *numerator,
                                                        ViInt32   *denominator,
                                                        ViReal64  *offset,
                                                        ViInt32   *sweepType)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar    repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar    response[RS_MAX_MESSAGE_LEN] = "";
    ViReal64* resArr = NULL;
    ViInt32   count;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_REFERENCE_IF_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_DIRECT_REFERENCE_IF_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_GetAttributeViString(instrSession, repCap,
                                             RSVNA_ATTR_HIDDEN_DIRECT_REFERENCE_IF_FREQUENCY, RS_MAX_MESSAGE_LEN,
                                             response));

    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &resArr, &count));

    if (count == 4)
    {
        *numerator = (ViInt32) resArr[0];
        *denominator = (ViInt32) resArr[1];
        *offset = resArr[2];
        *sweepType = (ViInt32) resArr[3];
    }
    else
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    }

Error:
    if (resArr != NULL)
        free(resArr);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries if and how the Direct IF Access Input / Output ports IF
/// HIFN  Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rearOut/Queries if and how the Direct IF Access Input / Output ports IF
/// HIPAR rearOut/Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
ViStatus _VI_FUNC rsvna_QueryDirectMeasurementRearOut(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   physicalPort,
                                                      ViInt32*  rearOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_MEASUREMENT_REAR_OUT, rearOut),
            4, "Rear Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries if and how the Direct IF Access Input / Output ports IF
/// HIFN  Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rearOut/Queries if and how the Direct IF Access Input / Output ports IF
/// HIPAR rearOut/Reference <Pt> and IF Meas <Pt> are used in channel <Ch>.
ViStatus _VI_FUNC rsvna_QueryDirectReferenceRearOut(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   physicalPort,
                                                    ViInt32*  rearOut)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_REAR_OUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DIRECT_REFERENCE_REAR_OUT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_DIRECT_REFERENCE_REAR_OUT, rearOut),
            4, "Rear Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frequency formula at the (optional) rear 'LO Out' port for
/// HIFN  channel <Ch>.
/// HIFN
/// HIFN     Note: The frequency formula is applied even if the analyzer returns an
/// HIFN  error message, because the frequency is outside the allowed range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR numerator/Parameters of the port frequency formula.
/// HIPAR denominator/Parameters of the port frequency formula.
/// HIPAR offset/Parameters of the port frequency formula.
/// HIPAR sweepType/Sweep type.
ViStatus _VI_FUNC rsvna_ConfigureRearLOFrequency(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  numerator,
                                                 ViReal64  denominator,
                                                 ViReal64  offset,
                                                 ViInt32   sweepType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  sweepTypeOut;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_REAR_LO_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngRearLOSweepType, sweepType, &sweepTypeOut),
                7, "Sweep Type");

    snprintf (params, RS_PARAM_BUF_SIZE, "%.12lG,%.12lG,%.12lG,%s", numerator, denominator, offset, sweepTypeOut);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_REAR_LO_FREQUENCY, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to define the output power mode at the (optional) rear LO
/// HIFN  Out port for channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR powerMode/Allows you to define the output power mode at the (optional) rear LO
/// HIPAR powerMode/Out port for channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureRearLOOutputPowerMode(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViBoolean powerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_REAR_LO_OUTPUT_POWER_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_REAR_LO_OUTPUT_POWER_MODE, powerMode),
            3, "Power Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines whether the source power at the (optional) rear LO Out port is
/// HIFN  permanently on in channel <Ch> (generator port) .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Defines whether the source power at the (optional) rear LO Out port is
/// HIPAR enabled/permanently on in channel <Ch> (generator port) .
ViStatus _VI_FUNC rsvna_ConfigureRearLOPermanentEnabled(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_REAR_LO_PERMANENT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_REAR_LO_PERMANENT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the power offset of the the (optional) rear LO Out port for
/// HIFN  channel <Ch>.
/// HIFN
/// HIFN     If SOURce<Ch>:RLO:PABSolut is OFF, then the output power is equal to
/// HIFN  the channel base power fb plus the power offset. Otherwise the output
/// HIFN  power is set to the fixed power offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR powerOffset/Defines the power offset of the the (optional) rear LO Out port for
/// HIPAR powerOffset/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureRearLOPowerOffset(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  powerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_REAR_LO_POWER_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_REAR_LO_POWER_OFFSET, powerOffset),
            3, "Power Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source of the trigger events that the
/// HIFN  analyzer uses to start a measurement sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR triggerSource/Selects the source of the trigger events that the analyzer uses to
/// HIPAR triggerSource/start a measurement sequence.
ViStatus _VI_FUNC rsvna_ConfigureTriggerSource(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   triggerSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR triggerMeasSequence/This control selects the Triggered Meas. Sequence.
/// HIPAR triggerDelay/This control defines a delay time between the trigger event and the
/// HIPAR triggerDelay/start of the measurement.
/// HIPAR slopeLevel/This control qualifies whether the trigger event occurs on the rising
/// HIPAR slopeLevel/or on the falling edge of the external TTL trigger signal.
ViStatus _VI_FUNC rsvna_ConfigureTriggerSettings(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   triggerMeasSequence,
                                                 ViReal64  triggerDelay,
                                                 ViInt32   slopeLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_MEASUREMENT_SEQUENCE, triggerMeasSequence),
            3, "Trigger Meas Sequence");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TRIGGER_DELAY, triggerDelay),
            4, "Trigger Delay");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_SLOPE, slopeLevel),
            5, "Slope Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source of the trigger events that the
/// HIFN  analyzer uses to start a measurement sequence in multiple trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR triggerMeasSequence/This control selects the Triggered Meas. Sequence.
/// HIPAR source/Selects the source of the trigger events that the analyzer uses to
/// HIPAR source/start a measurement sequence in multiple trigger mode.
/// HIPAR triggerEvent/Distinguishes between the two trigger events that can be
/// HIPAR triggerEvent/logically combined to trigger measurement sequence.
ViStatus _VI_FUNC rsvna_ConfigureMultipleTriggerSource(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   triggerMeasSequence,
                                                       ViInt32   source,
                                                       ViInt32   triggerEvent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerMeasSequence, RSVNA_VAL_TRG_SEQ_SWEEP, RSVNA_VAL_TRG_SEQ_PARTIAL_MEAS),
            3, "Trigger Meas Sequence");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSVNA_VAL_MULTIPLE_TRIGGER_SOURCE_IMM, RSVNA_VAL_MULTIPLE_TRIGGER_SOURCE_UPORT),
            4, "Source");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerEvent, 1, 2),
            5, "Trigger Event");

    if (RsCore_IsInstrumentModel(instrSession, "ZNA"))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:SEQ:MULT:SOUR%ld %s,%s", channel,
                 triggerEvent,
                 triggerMeasSequenceArr[triggerMeasSequence],
                 multipleTriggerSourceArr[source]);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:SEQ:MULT:SOUR %s,%s", channel,
                 triggerMeasSequenceArr[triggerMeasSequence],
                 multipleTriggerSourceArr[source]);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function qualifies whether the multiple trigger events occur on
/// HIFN  the rising or on the falling edge or on the beginning of the high / low
/// HIFN  level periods of the external TTL trigger signal <Num>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR triggerSignal/Channel number.
/// HIPAR triggerMeasSequence/This control selects the Triggered Meas. Sequence.
/// HIPAR slopeLevel/Qualifies whether the multiple trigger events occur on the rising or
/// HIPAR slopeLevel/on the falling edge or on the beginning of the high / low level periods
/// HIPAR slopeLevel/of the external TTL trigger signal <Num>.
ViStatus _VI_FUNC rsvna_ConfigureMultipleTriggerSlope(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   triggerSignal,
                                                      ViInt32   triggerMeasSequence,
                                                      ViInt32   slopeLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerMeasSequence, RSVNA_VAL_TRG_SEQ_SWEEP, RSVNA_VAL_TRG_SEQ_PARTIAL_MEAS),
            3, "Trigger Meas Sequence");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slopeLevel, RSVNA_VAL_NEG, RSVNA_VAL_LOW),
            4, "Slope Level");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:SEQ:MULT:SLOP%ld %s,%s", channel, triggerSignal,
             triggerMeasSequenceArr[triggerMeasSequence],
             slopeLevelArr[slopeLevel]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets a delay time between the trigger event and the
/// HIFN  start of the measurement (Trigger Delay) in multiple trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR triggerMeasSequence/This control selects the Triggered Meas. Sequence.
/// HIPAR triggerDelay/Sets a delay time between the trigger event and the start of the
/// HIPAR triggerDelay/measurement (Trigger Delay) in multiple trigger mode.
ViStatus _VI_FUNC rsvna_ConfigureMultipleTriggerDelay(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   triggerMeasSequence,
                                                      ViReal64  triggerDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerMeasSequence, RSVNA_VAL_TRG_SEQ_SWEEP, RSVNA_VAL_TRG_SEQ_PARTIAL_MEAS),
            3, "Trigger Meas Sequence");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerDelay, 0.0, 13680.0), 4, "Trigger Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG%ld:SEQ:MULT:HOLD %s,%.12lG", channel,
             triggerMeasSequenceArr[triggerMeasSequence],
             triggerDelay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  In external trigger mode (TRIGger<Ch>[:SEQuence]:SOURce EXTernal),
/// HIFN  this command selects one of the available external trigger sources.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR externalInput/In external trigger mode (TRIGger<Ch>[:SEQuence]:SOURce EXTernal),
/// HIPAR externalInput/this command selects one of the available external trigger sources.
ViStatus _VI_FUNC rsvna_ConfigureTriggerExternalInput(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   externalInput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_EXTERNAL_INPUT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_EXTERNAL_INPUT, externalInput),
            3, "External Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures trigger output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR connector/Select rear panel connector used to send or receive signals.
/// HIPAR enable/Sets trigger output ON / OFF.
ViStatus _VI_FUNC rsvna_ConfigureAUXTriggerOutEnable(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   connector,
                                                     ViBoolean enable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_con[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",", channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_ENABLE,
            "AUXPort", connector - 1, RS_REPCAP_BUF_SIZE, repCap_con),
            3, "Connector");

    strcat(repCap, repCap_con);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_ENABLE, enable),
            4, "Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures trigger output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR connector/Select rear panel connector used to send or receive signals.
/// HIPAR polarity/Selects the polarity of the Aux Output signal.
/// HIPAR position/Specifies whether the aux trigger out signal is sent before or after
/// HIPAR position/the acquisition.
/// HIPAR interval/Specifies how often a trigger output signal is sent.
/// HIPAR duration/Sets the width of the output pulse, which is the time that the Aux
/// HIPAR duration/trigger output will be asserted.
ViStatus _VI_FUNC rsvna_ConfigureAUXTriggerOut(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   connector,
                                               ViInt32   polarity,
                                               ViInt32   position,
                                               ViInt32   interval,
                                               ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_con[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_POLARITY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_POLARITY,
            "AUXPort", connector - 1, RS_REPCAP_BUF_SIZE, repCap_con),
            3, "Connector");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_con);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_POLARITY, polarity),
            4, "Polarity");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_POSITION, position),
            5, "Position");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_INTERVAL, interval),
            6, "Interval");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_DURATION, duration),
            7, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the delay of the trigger out signal relative to the selected
/// HIFN  sweep interval.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR connector/Select rear panel connector used to send or receive signals.
/// HIPAR delay/Specifies the delay of the trigger out signal relative to the selected
/// HIPAR delay/sweep interval.
ViStatus _VI_FUNC rsvna_ConfigureAUXTriggerOutDelay(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   connector,
                                                    ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_DELAY,
            "AUXPort", connector - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Connector");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_DELAY, delay),
            4, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the selected sweep sequence is indicated at the related
/// HIFN  Trigger Out connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR connector/Select rear panel connector used to send or receive signals.
/// HIPAR signalType/Defines how the selected sweep sequence is indicated at the related
/// HIPAR signalType/Trigger Out connector.
ViStatus _VI_FUNC rsvna_ConfigureAUXTriggerOutSignalType(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   connector,
                                                         ViInt32   signalType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_SIGNAL_TYPE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TRIGGER_AUX_OUT_SIGNAL_TYPE,
            "AUXPort", connector - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Connector");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TRIGGER_AUX_OUT_SIGNAL_TYPE, signalType),
            4, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the logical combination of the trigger events defined in
/// HIFN  TRIGger<Ch>[:SEQuence]:MULTiple:SOURce<inp> that will trigger
/// HIFN  measurement sequence <Sequence>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sequence/Triggered measurement sequence.
/// HIPAR logic/Trigger Logic.
ViStatus _VI_FUNC rsvna_ConfigureTriggerLogic(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   sequence,
                                              ViInt32   logic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2sequence;
    ViChar*  p2logic;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequence, RSVNA_VAL_TRIGGER_LOGIC_SEQ_SWEEP, RSVNA_VAL_TRIGGER_LOGIC_SEQ_PPOINT),
            3, "Sequence");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, logic, RSVNA_VAL_TRIGGER_LOGIC_NONE, RSVNA_VAL_TRIGGER_LOGIC_OR),
            4, "Logic");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngTriggerLogicSeq, sequence, &p2sequence),
        3, "Sequence");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngTriggerLogic, logic, &p2logic),
        4, "Logic");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_TRIGGER_LOGIC,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%s", p2sequence, p2logic);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_TRIGGER_LOGIC, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current Ready for Trigger state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR triggerStatusReady/Queries the current Ready for Trigger state.
ViStatus _VI_FUNC rsvna_QueryTriggerStatusReady(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    triggerStatusReady[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_TRIGGER_STATUS_READY,
            bufferSize, triggerStatusReady),
            3, "Trigger Status Ready");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns 1 when the R&S ZNA is Ready for Trigger, or 0 after the
/// HIFN  specified <Timeout>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR timeout/Timeout value.
/// HIPAR output/Returns 1 when the R&S ZNA is Ready for Trigger, or 0 after the specified <Timeout>.
ViStatus _VI_FUNC rsvna_TriggerReadyTimeout(ViSession  instrSession,
                                            ViReal64   timeout,
                                            ViBoolean* output)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    param[RS_PARAM_BUF_SIZE] = "";
	
    checkErr (RsCore_LockSession (instrSession));

    snprintf (param, RS_PARAM_BUF_SIZE, "%.12lG", timeout);

    checkErr (RsCore_GetAttributeViBooleanWithQparam (instrSession, NULL, RSVNA_ATTR_HIDDEN_TRIGGER_READY_TIMEOUT,
													param, output));
    
Error:
    RsCore_UnlockSession(instrSession);    
    return error;
}


/// HIFN  Activates/deactivates tracking of lost trigger events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activates/deactivates tracking of lost trigger events.
ViStatus _VI_FUNC rsvna_ConfigureLostTriggerEnabled(ViSession instrSession,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_LOST_TRIGGER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of lost trigger events the analyzer firmware
/// HIFN  observed since the last reset of the lost trigger event log (see
/// HIFN  TRIGger[:SEQuence]:LTRigger:RESet).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Returns the number of lost trigger events the analyzer firmware
/// HIPAR count/observed since the last reset of the lost trigger event log (see
/// HIPAR count/TRIGger[:SEQuence]:LTRigger:RESet).
ViStatus _VI_FUNC rsvna_QueryLostTriggerCount(ViSession instrSession,
                                              ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_LOST_TRIGGER_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the lost trigger event log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_LostTriggerReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_LOST_TRIGGER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the offset parameter for test port <Port> as a delay time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
/// HIPAR delay/This control defines the offset parameter for test port <Port> as a
/// HIPAR delay/delay time.
ViStatus _VI_FUNC rsvna_ConfigureOffsetDelay(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   port,
                                             ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_OFFSET_DELAY_TIME,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_OFFSET_DELAY_TIME, delay),
            4, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the offset parameter for test port <Port> as an electrical length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
/// HIPAR electricalLength/This control defines the offset parameter for test port <Port> as an
/// HIPAR electricalLength/electrical length.
ViStatus _VI_FUNC rsvna_ConfigureOffsetElectricalLength(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   port,
                                                        ViReal64  electricalLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_OFFSET_DELAY_TIME,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_OFFSET_ELECTRICAL_LENGTH, electricalLength),
            4, "Electrical Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the offset parameter for test port <Port> as a mechanical length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
/// HIPAR mechanicalLength/This control defines the offset parameter for test port <Port> as a
/// HIPAR mechanicalLength/mechanical length.
ViStatus _VI_FUNC rsvna_ConfigureOffsetMechanicalLength(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   port,
                                                        ViReal64  mechanicalLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_OFFSET_DELAY_TIME,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_OFFSET_MECHANICAL_LENGTH, mechanicalLength),
            4, "Mechanical Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the permittivity for the offset correction at test port <Port>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
/// HIPAR permittivity/This control defines the permittivity for the offset correction at
/// HIPAR permittivity/test port <Port>.
ViStatus _VI_FUNC rsvna_ConfigureOffsetPermittivity(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   port,
                                                    ViReal64  permittivity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_OFFSET_DELAY_TIME,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_OFFSET_PERMITTIVITY, permittivity),
            4, "Permittivity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameters of One-way Loss.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number of the offset-corrected channel.
/// HIPAR port/Port number of the analyzer.
/// HIPAR lossAtDC/This control defines the frequency-independent part (DC value) of the
/// HIPAR lossAtDC/offset loss.
/// HIPAR lossAtFrequency/This control defines the offset loss at the reference frequency.
/// HIPAR frequencyForLoss/This control defines the reference frequency.
ViStatus _VI_FUNC rsvna_ConfigureOneWayLoss(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   port,
                                            ViReal64  lossAtDC,
                                            ViReal64  lossAtFrequency,
                                            ViReal64  frequencyForLoss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ONE_WAY_LOSS_AT_DC,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_AT_DC, lossAtDC),
            4, "Loss At DC");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_AT_FREQUENCY, lossAtFrequency),
            5, "Loss At Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_FREQUENCY_FOR_LOSS, frequencyForLoss),
            6, "Frequency For Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures a seconds frequency for one-way loss
/// HIFN  calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number of the offset-corrected channel.
/// HIPAR port/Port number of the analyzer.
/// HIPAR useSecondFrequency/Defines whether the second reference frequency f2 is used for the
/// HIPAR useSecondFrequency/offset loss approximation.
/// HIPAR loss/Defines the offset loss at the second reference frequency f2.
/// HIPAR frequency/Defines the second reference frequency f2 for the offset loss
/// HIPAR frequency/approximation.
ViStatus _VI_FUNC rsvna_ConfigureOneWayLossSecondFrequency(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   port,
                                                           ViBoolean useSecondFrequency,
                                                           ViReal64  loss,
                                                           ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            3, "Port");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ONE_WAY_LOSS_USE_SECOND_FREQUENCY,
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",%s", channel, repCap_port);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_USE_SECOND_FREQUENCY, useSecondFrequency),
            2, "Use Second Frequency");

    if (useSecondFrequency)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_AT_SECOND_FREQUENCY, loss),
                3, "Loss");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_ONE_WAY_LOSS_SECOND_FREQUENCY_FOR_LOSS, frequency),
                4, "Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the offset parameter for the active test port
/// HIFN  such that the residual delay of the active trace is minimized across
/// HIFN  the entire sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
ViStatus _VI_FUNC rsvna_AutoLength(ViSession instrSession,
                                   ViInt32   channel,
                                   ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_AUTO_LENGTH,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_AUTO_LENGTH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines all offset parameters such that the residual
/// HIFN  group delay of the active trace (defined as the negative derivative of
/// HIFN  the phase response) is minimized and the measured loss is minimized as
/// HIFN  far as possible across the entire sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number of the offset-corrected channel.
/// HIPAR port/Port number of the analyzer.
ViStatus _VI_FUNC rsvna_AutoLengthAndLoss(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_AUTO_LENGTH_AND_LOSS,
            "Port", port -1, RS_REPCAP_BUF_SIZE, repCap_port),
            3, "Port");
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_AUTO_LENGTH_AND_LOSS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the offset parameters for all test ports to zero
/// HIFN  and the reference frequency to 1 GHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ResetOffsets(ViSession instrSession,
                                     ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_RESET_OFFSET, VI_FALSE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function toggles length/loss/fixture compensation for physical
/// HIFN  port <PhyPt> ON/OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR port/Port number of the analyzer.
/// HIPAR compensation/Toggles length/loss/fixture compensation for physical port <PhyPt>
/// HIPAR compensation/ON/OFF.
ViStatus _VI_FUNC rsvna_ConfigureOffsetCompensation(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   port,
                                                    ViBoolean compensation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Pt%ld", channel, port);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_OFFSET_COMPENSATION, compensation),
            4, "Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the deembedding/embedding function
/// HIFN  for single ended ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR state/This control enables or disables the deembedding function for single
/// HIPAR state/ended ports.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkSingleEndedState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   physicalPortNumber,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_SINGLE_ENDED_STATE,
            "EmbedDeembed", functionType , RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Function Type");

    snprintf(repCap_port, RS_REPCAP_BUF_SIZE, "Pt%ld", physicalPortNumber);

    strcat(repCap, ",");
    strcat(repCap, repCap_type);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_SINGLE_ENDED_STATE, state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance, inductance or resistance
/// HIFN  values C1, C2, L1, L2 or R1, R2 in the different circuit models for
/// HIFN  single ended port deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR parameterType/Parameter type, capacitance, inductance, resistance or conductance.
/// HIPAR parameterNumber/Number of capacitances, inductances, resistances or conductances in
/// HIPAR parameterNumber/circuit model.
/// HIPAR circuitModel/This control defines the possible circuit model.
/// HIPAR parameterValue/Capacitance C<no>, Inductance L<no>, Resistance R<no> or Conductance
/// HIPAR parameterValue/G<no> for the specified circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkSingleEndedPort(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   physicalPortNumber,
                            ViInt32   parameterType,
                            ViInt32   parameterNumber,
                            ViInt32   circuitModel,
                            ViReal64  parameterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterNumber, 1, 3),
            5, "Parameter Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, circuitModel, RSVNA_VAL_SENDED_CIRCUIT_CSL, RSVNA_VAL_SENDED_CIRCUIT_SGG),
            7, "Circuit Model");

    switch (parameterType)
    {
    case RSVNA_VAL_VNET_PARAM_C:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e-3, 1.0e-3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:SEND:%s%ld:PAR:C%ld %s,%.12lG",
                channel, functionTypeArr[functionType], physicalPortNumber, parameterNumber, circuitModelSArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_L:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0, 1.0), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:SEND:%s%ld:PAR:L%ld %s,%.12lG",
                channel, functionTypeArr[functionType], physicalPortNumber, parameterNumber, circuitModelSArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_R:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -10.0e+6, 10.0e+6), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:SEND:%s%ld:PAR:R%ld %s,%.12lG",
                channel, functionTypeArr[functionType], physicalPortNumber, parameterNumber, circuitModelSArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_G:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e3, 1.0e3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:SEND:%s%ld:PAR:G%ld %s,%.12lG",
                channel, functionTypeArr[functionType], physicalPortNumber, parameterNumber, circuitModelSArr[circuitModel], parameterValue);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameterType), 5, "Parameter Type");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines an embedding network for a single-ended port
/// HIFN  based on the given S-Parameter traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR interchange/This control defines the interchange of port numbers.
/// HIPAR touchstoneFile/Content of a Touchstone file (*.s2p or *.s4p)
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkSingleEndedData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   physicalPortNumber,
                            ViInt32   interchange,
                            ViString  touchstoneFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, interchange, RSVNA_VAL_PORT_NUMBERS_NOT_INTERCHANGED, RSVNA_VAL_PORT_NUMBERS_INTERCHANGED),
            5, "Interchange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC%ld:TRAN:VNET:SEND:%s%ld:PAR:DATA %s,", channel,
             functionTypeArr[functionType],
             physicalPortNumber,
             interchangeArr[interchange]);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, touchstoneFile));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for single ended port
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkSingleEndedCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   physicalPortNumber,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_SINGLE_ENDED_CIRCUIT_MODEL,
            "EmbedDeembed", functionType , RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Function Type");

    snprintf(repCap_port, RS_REPCAP_BUF_SIZE, "Pt%ld", physicalPortNumber);

    strcat(repCap, ",");
    strcat(repCap, repCap_type);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_SINGLE_ENDED_CIRCUIT_MODEL, circuitModel),
            5, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified two-port (*.sp2) Touchstone
/// HIFN  file defining the characteristics of the current embedded/deembedded
/// HIFN  single ended port circuit model. The single ended port circuit model
/// HIFN  involving file import (FIMPort) must be selected before using this
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. If no path is specified the analyzer searches the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_LoadSingleEndedPortCircuitModelData(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   functionType,
                                                            ViInt32   physicalPortNumber,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_DATA,
            "EmbedDeembed", functionType , RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Function Type");

    snprintf(repCap_port, RS_REPCAP_BUF_SIZE, "Pt%ld", physicalPortNumber);

    strcat(repCap, ",");
    strcat(repCap, repCap_type);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_DATA, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified two-port (*.sp2) Touchstone
/// HIFN  file defining the characteristics of the current embedded/deembedded
/// HIFN  single ended port circuit model with inverted port sequence: network
/// HIFN  port 2 towards VNA, network port 1 towards DUT. The single ended port
/// HIFN  circuit model involving file import (FIMPort) must be selected before
/// HIFN  using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR physicalPortNumber/Physical port number.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. If no path is specified the analyzer searches the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_LoadSingleEndedPortCircuitModelDataInverted(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   physicalPortNumber,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   value[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2Ipor;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Port[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] =
             {
             RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_DEEMBEDDING_DATA,
             RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_EMBEDDING_DATA
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_DEEMBEDDING_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_LOAD_SINGLE_ENDED_PORT_CIRCUIT_MODEL_DEEMBEDDING_DATA,
            "Port", physicalPortNumber - 1, RS_REPCAP_BUF_SIZE, repCap_Port),
            4, "Port");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Port);

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngVnetDataInterchange,
        RSVNA_VAL_PORT_NUMBERS_INTERCHANGED, &p2Ipor));
    snprintf(value, RS_MAX_MESSAGE_LEN, "'%s',%s", fileName, p2Ipor);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, attr[functionType], value),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the deembedding/embedding function
/// HIFN  for balanced ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR state/This control enables or disables the deembedding function for balanced
/// HIPAR state/ports.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkABalancedState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_BALANCED_STATE,
            "EmbedDeembed", functionType , RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Function Type");

    snprintf(repCap_port, RS_REPCAP_BUF_SIZE, "LogPort%ld", logicalPortNumber);

    strcat(repCap, ",");
    strcat(repCap, repCap_type);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_BALANCED_STATE, state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance, inductance or resistance
/// HIFN  values C1, C2, C3, L1, L2, L3 or R1, R2, R3 in the different circuit
/// HIFN  models for balanced port deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR parameterType/Parameter type, capacitance, inductance, resistance  or conductance.
/// HIPAR parameterNumber/Number of capacitances, inductances, resistances or conductance in
/// HIPAR parameterNumber/circuit model.
/// HIPAR circuitModel/This control defines the possible circuit model.
/// HIPAR parameterValue/Capacitance C<no>, Inductance L<no>, Resistance R<no> or Conductance
/// HIPAR parameterValue/G<no> for the specified circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkBalancedPort(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   functionType,
                                                            ViInt32   logicalPortNumber,
                                                            ViInt32   parameterType,
                                                            ViInt32   parameterNumber,
                                                            ViInt32   circuitModel,
                                                            ViReal64  parameterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterNumber, 1, 3),
            5, "Parameter Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, circuitModel, RSVNA_VAL_BALANCED_CIRCUIT_STSL, RSVNA_VAL_BALANCED_CIRCUIT_SGGS),
            7, "Circuit Model");

    switch (parameterType)
    {
    case RSVNA_VAL_VNET_PARAM_C:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e-3, 1.0e-3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:BAL:%s%ld:PAR:C%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_L:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0, 1.0), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:BAL:%s%ld:PAR:L%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_R:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -10.0e+6, 10.0e+6), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:BAL:%s%ld:PAR:R%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_G:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e+3, 1.0e+3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:BAL:%s%ld:PAR:G%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameterType), 5, "Parameter Type");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a deembedding network for a balanced port based
/// HIFN  on the given S-Parameter traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR portAssignment/Port assignment for 2-port networks
/// HIPAR interchange/This control defines the interchange of port numbers.
/// HIPAR touchstoneFile/Content of a Touchstone file (*.s2p or *.s4p)
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkBalancedData(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   functionType,
                                                            ViInt32   logicalPortNumber,
                                                            ViInt32   portAssignment,
                                                            ViInt32   interchange,
                                                            ViString  touchstoneFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2Interchange;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portAssignment, 1, 2),
            5, "Port Assignment");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngVnetDataInterchange, interchange, &p2Interchange),
                6, "Interchange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC%ld:TRAN:VNET:BAL:%s%ld:PAR:DATA%ld %s,", channel,
             functionTypeArr[functionType],
             logicalPortNumber,
             portAssignment,
             p2Interchange);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, touchstoneFile));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for balanced port
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkBalancedCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_type[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_port[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_BALANCED_CIRCUIT_MODEL,
            "EmbedDeembed", functionType , RS_REPCAP_BUF_SIZE, repCap_type),
            3, "Function Type");

    snprintf(repCap_port, RS_REPCAP_BUF_SIZE, "LogPort%ld", logicalPortNumber);

    strcat(repCap, ",");
    strcat(repCap, repCap_type);
    strcat(repCap, ",");
    strcat(repCap, repCap_port);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_BALANCED_CIRCUIT_MODEL, circuitModel),
            5, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified Touchstone file defining the
/// HIFN  characteristics of the current embedded/deembedded  balanced port
/// HIFN  circuit model. A balanced port circuit model involving file import must
/// HIFN  be selected before using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. The balanced port circuit models STSL | STSC | SLST | SCST
/// HIPAR fileName/require two 2-port (*.s2p) files, to be assigned to the different ports
/// HIPAR fileName/PMAin and PSECondary; the FIMPort model requires a single 4-port
/// HIPAR fileName/(*.s4p) file but no additional port assignment.
/// HIPAR parameter/Port assignment for two 2-port (*.s2p) files.
ViStatus _VI_FUNC rsvna_LoadBalancedPortCircuitModelData(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   functionType,
                                                         ViInt32   logicalPortNumber,
                                                         ViString  fileName,
                                                         ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSVNA_VAL_VNET_PARAM_PMAIN, RSVNA_VAL_VNET_PARAM_PSECOND),
            6, "Circuit Model");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:VNET%ld:BAL:%s%ld '%s',%s", channel, functionTypeArr[functionType], logicalPortNumber,
             fileName, portCircuitParameterArr[parameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines port set <ListId> for port set
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portSet/Current number of the defined port set within the channel's overall
/// HIPAR portSet/list of port sets for deembedding/embedding.
/// HIPAR arraySize/Number of ports to be defined.
/// HIPAR ports/An array of port set, a port set consist of two, three or four
/// HIPAR ports/different ports. A port may be an element of multiple port sets.
ViStatus _VI_FUNC rsvna_DefineVirtualNetworkPortSet(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32   functionType,
                                                    ViInt32   portSet,
                                                    ViInt32   arraySize,
                                                    ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap_name[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd_value[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PSET:%s%ld:DEF ", channel, functionTypeArr[functionType], portSet);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, ports, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

    if (functionType == RSVNA_VAL_VNET_FUNC_DEEMBED)
    {
        snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "PPairDeembedding%ld", portSet);
        snprintf(cmd_value, RS_REPCAP_BUF_SIZE, "%ld", portSet);
        checkErr(RsCore_AddRepCap(instrSession, "List_idx_deembedding", repCap_name, cmd_value));
    }
    else
    {
        snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "PPairEmbedding%ld", portSet);
        snprintf(cmd_value, RS_REPCAP_BUF_SIZE, "%ld", portSet);
        checkErr(RsCore_AddRepCap(instrSession, "List_idx_embedding", repCap_name, cmd_value));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a list of port pairs for port pair
/// HIFN  embedding/deembedding. The function can be used
/// HIFN     repeatedly to extend or (partially) overwrite the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portPair/Current number of a port pair in the list.
/// HIPAR arraySize/Number of ports to be defined.
/// HIPAR ports/An array of port pairs, each one consisting of two different
ViStatus _VI_FUNC rsvna_DefineVirtualNetworkPortPair(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   functionType,
                                                     ViInt32   portPair,
                                                     ViInt32   arraySize,
                                                     ViInt32   ports[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap_name[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd_value[50] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PPA:%s%ld:DEF ", channel, functionTypeArr[functionType], portPair);
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, ports, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

    if (functionType == RSVNA_VAL_VNET_FUNC_DEEMBED)
    {
        snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "PPairDeembedding%ld", portPair);
        sprintf(cmd_value, "%ld", portPair);
        checkErr(RsCore_AddRepCap(instrSession, "List_idx_deembedding", repCap_name, cmd_value));
    }
    else
    {
        snprintf(repCap_name, RS_REPCAP_BUF_SIZE, "PPairEmbedding%ld", portPair);
        sprintf(cmd_value, "%ld", portPair);
        checkErr(RsCore_AddRepCap(instrSession, "List_idx_embedding", repCap_name, cmd_value));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the previously defined list of port pairs for port pair
/// HIFN  embedding/deembedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
ViStatus _VI_FUNC rsvna_DeleteVirtualNetworkPortPair(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   functionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_DELETE_PORT_PAIR_DEEMBEDDING,
             RSVNA_ATTR_VIRTUAL_NETWORK_DELETE_PORT_PAIR_EMBEDDING
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, attr[functionType], NULL));

    if (functionType == RSVNA_VAL_VNET_FUNC_DEEMBED)
    {
        checkErr(RsCore_ResetRepCapID(instrSession, "List_idx_deembedding", rsvna_RsCoreRepCapTable));
    }
    else
    {
        checkErr(RsCore_ResetRepCapID(instrSession, "List_idx_embedding", rsvna_RsCoreRepCapTable));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the deembedding/embedding function
/// HIFN  for for port pairs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portPair/Current number of a port pair in the list.
/// HIPAR state/This control enables or disables the deembedding function for port
/// HIPAR state/pair.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortPairState(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32   functionType,
                                                             ViInt32   portPair,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_PAIR_DEEMBEDDING_STATE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_PAIR_EMBEDDING_STATE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    if (functionType == RSVNA_VAL_VNET_FUNC_DEEMBED)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",PPairDeembedding%ld", channel, portPair);
    }
    else
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",PPairEmbedding%ld", channel, portPair);
    }

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attr[functionType], state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance, inductance or resistance
/// HIFN  values C1, C2, C3, L1, L2, L3 or R1, R2, R3 in the different circuit
/// HIFN  models for port pair deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portPair/Current number of a port pair in the list.
/// HIPAR parameterType/Parameter type, capacitance, inductance, resistance or conductance.
/// HIPAR parameterNumber/Number of capacitances, inductances, resistances or conductance in
/// HIPAR parameterNumber/circuit model.
/// HIPAR circuitModel/This control defines the possible circuit model.
/// HIPAR parameterValue/Capacitance C<no>, Inductance L<no>, Resistance R<no> or Conductance
/// HIPAR parameterValue/G<no> for the specified circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortPair(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   functionType,
                                                        ViInt32   portPair,
                                                        ViInt32   parameterType,
                                                        ViInt32   parameterNumber,
                                                        ViInt32   circuitModel,
                                                        ViReal64  parameterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterNumber, 1, 3),
            5, "Parameter Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, circuitModel, RSVNA_VAL_BALANCED_CIRCUIT_STSL, RSVNA_VAL_BALANCED_CIRCUIT_SGGS),
            7, "Circuit Model");

    switch (parameterType)
    {
    case RSVNA_VAL_VNET_PARAM_C:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e-3, 1.0e-3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PPA:%s%ld:PAR:C%ld %s,%.12lG",
                channel, functionTypeArr[functionType], portPair, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_L:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0, 1.0), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PPA:%s%ld:PAR:L%ld %s,%.12lG",
                channel, functionTypeArr[functionType], portPair, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_R:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -10.0e+6, 10.0e+6), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PPA:%s%ld:PAR:R%ld %s,%.12lG",
                channel, functionTypeArr[functionType], portPair, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_G:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e+3, 1.0e+3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:PPA:%s%ld:PAR:G%ld %s,%.12lG",
                channel, functionTypeArr[functionType], portPair, parameterNumber, circuitModelBArr[circuitModel], parameterValue);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameterType), 5, "Parameter Type");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for port pair
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portPair/Current number of a port pair in the list.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortPairCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   portPair,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_PAIR_DEEMBEDDING_CIRCUIT_MODEL,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_PAIR_EMBEDDING_CIRCUIT_MODEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    if (functionType == RSVNA_VAL_VNET_FUNC_DEEMBED)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",PPairDeembedding%ld", channel, portPair);
    }
    else
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",PPairEmbedding%ld", channel, portPair);
    }

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, attr[functionType], circuitModel),
            5, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified Touchstone file defining the
/// HIFN  characteristics of the current embedded/deembedded  port pair circuit
/// HIFN  model. A balanced port circuit model involving file import must be
/// HIFN  selected before using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR portPair/Current number of a port pair in the list.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. The balanced port circuit models STSL | STSC | SLST | SCST
/// HIPAR fileName/require two 2-port (*.s2p) files, to be assigned to the different ports
/// HIPAR fileName/PMAin and PSECondary; the FIMPort model requires a single 4-port
/// HIPAR fileName/(*.s4p) file but no additional port assignment.
/// HIPAR parameter/Port assignment for two 2-port (*.s2p) files.
/// HIPAR interchangePortNumbers/Interchange port numbers of loaded *.s2p file. If the parameter is
/// HIPAR interchangePortNumbers/omitted, the port numbers are not interchanged. The parameter must not
/// HIPAR interchangePortNumbers/be used for 4-port files.
ViStatus _VI_FUNC rsvna_LoadPortPairCircuitModelData(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   functionType,
                                                     ViInt32   portPair,
                                                     ViString  fileName,
                                                     ViInt32   parameter,
                                                     ViBoolean interchangePortNumbers)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSVNA_VAL_VNET_PARAM_PMAIN, RSVNA_VAL_VNET_PARAM_PSECOND),
            6, "Circuit Model");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:VNET%ld:PPA:%s%ld '%s',%s",
             channel, functionTypeArr[functionType], portPair,
             fileName, portCircuitParameterArr[parameter]);
    if (interchangePortNumbers)
        strcat(cmd, ",IPOR");

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the deembedding/embedding function
/// HIFN  for ground loops. It is allowed to change the circuit model and its
/// HIFN  parameters while deembedding is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR state/This control enables or disables the deembedding function for ground
/// HIPAR state/loop.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkGroundLoopState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_DEEMBEDDING_STATE,
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_EMBEDDING_STATE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attr[functionType], state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance C, inductance L or resistance
/// HIFN  R value in the different circuit models for ground loop
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR parameterType/Parameter type, capacitance, inductance or resistance.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkGroundLoop(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   functionType,
                                                          ViInt32   parameterType,
                                                          ViInt32   circuitModel,
                                                          ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, circuitModel, RSVNA_VAL_GLOOP_CIRCUIT_SL, RSVNA_VAL_GLOOP_CIRCUIT_SG),
            5, "Circuit Model");

    switch (parameterType)
    {
    case RSVNA_VAL_VNET_PARAM_C:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, value, -1.0e-3, 1.0e-3), 6, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:GLO:%s:PAR:C %s,%.12lG",
                 channel, functionTypeArr[functionType],
                 circuitModelGLArr[circuitModel], value);
        break;

    case RSVNA_VAL_VNET_PARAM_L:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, value, -1.0, 1.0), 6, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:GLO:%s:PAR:L %s,%.12lG",
                 channel, functionTypeArr[functionType],
                 circuitModelGLArr[circuitModel], value);
        break;

    case RSVNA_VAL_VNET_PARAM_R:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, value, -10.0e+6, 10.0e+6), 6, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:GLO:%s:PAR:R %s,%.12lG",
                 channel, functionTypeArr[functionType],
                 circuitModelGLArr[circuitModel], value);
        break;

    case RSVNA_VAL_VNET_PARAM_G:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, value, -1.0e+3, 1.0e+3), 6, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:GLO:%s:PAR:G %s,%.12lG",
                 channel, functionTypeArr[functionType],
                 circuitModelGLArr[circuitModel], value);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameterType), 4, "Parameter Type");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for ground loop
/// HIFN  deembedding/embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkGroundLoopCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_EMBEDDING_CIRCUIT_MODEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, attr[functionType], circuitModel),
            4, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified one-port (*.s1p) Touchstone
/// HIFN  file defining the characteristics of the current embedded/deembedded
/// HIFN  ground loop circuit model. The ground loop circuit model involving file
/// HIFN  import (FIMPort) must be selected before using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. If no path is specified the analyzer searches the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_LoadGroundLoopCircuitModelData(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   functionType,
                                                       ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_DEEMBEDDING_LOAD_CIRCUIT_MODEL,
             RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_EMBEDDING_LOAD_CIRCUIT_MODEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "functionType");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, attr[functionType], fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables differential match embedding for
/// HIFN  balanced port <LogPt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, only embedding is available at the moment.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR state/This control enables or disables differential match embedding for
/// HIPAR state/balanced port.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkDifferentialMatchState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             functionType++;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",LogPort%ld", channel, logicalPortNumber);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_DIFFERENTIAL_MATCH_EMBEDDING_STATE, state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance, inductance, resistance or
/// HIFN  conductance values in the "Shunt L, Shunt C" lumped element model for
/// HIFN  differential match embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, only embedding is available at the moment.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR parameterType/Parameter type, capacitance, inductance, resistance or conductance.
/// HIPAR parameterNumber/Number of capacitances, inductances or resistances in circuit model.
/// HIPAR circuitModel/This control defines the possible circuit model.
/// HIPAR parameterValue/Capacitance C<no>, Inductance L<no>, Resistance R<no> or Conductance
/// HIPAR parameterValue/G<no> for the specified circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkDifferentialMatch(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViInt32   parameterType,
                            ViInt32   parameterNumber,
                            ViInt32   circuitModel,
                            ViReal64  parameterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_EMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterNumber, 1, 1),
            5, "Parameter Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, circuitModel, RSVNA_VAL_CIRCUIT_SHLC, RSVNA_VAL_CIRCUIT_SHLC),
            7, "Circuit Model");

    switch (parameterType)
    {
    case RSVNA_VAL_VNET_PARAM_C:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0e-3, 1.0e-3), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:DIFF:%s%ld:PAR:C%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelDiffArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_L:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -1.0, 1.0), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:DIFF:%s%ld:PAR:L%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelDiffArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_R:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -10.0e+6, 10.0e+6), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:DIFF:%s%ld:PAR:R%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelDiffArr[circuitModel], parameterValue);
        break;

    case RSVNA_VAL_VNET_PARAM_G:
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, parameterValue, -100.0e6, 100.0e6), 8, "Value");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TRAN:VNET:DIFF:%s%ld:PAR:G%ld %s,%.12lG",
                channel, functionTypeArr[functionType], logicalPortNumber, parameterNumber, circuitModelDiffArr[circuitModel], parameterValue);
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameterType), 5, "Parameter Type");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines an differential match embedding network for a
/// HIFN  balanced port based on the given S-Parameter traces.
/// HIFN     Note(s):
/// HIFN     (1) Use rsvna_ConfigureVirtualNetworkDifferentialMatchCircuitModel
/// HIFN  (File import) to select the file import model before executing this
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, only embedding is available at the moment.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR interchange/This control defines the interchange of port numbers.
/// HIPAR touchstoneFile/Content of a Touchstone file (*.s2p or *.s4p)
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkDifferentialMatchData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViInt32   interchange,
                            ViString  touchstoneFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_EMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, interchange, RSVNA_VAL_PORT_NUMBERS_NOT_INTERCHANGED, RSVNA_VAL_PORT_NUMBERS_INTERCHANGED),
            5, "Interchange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC%ld:TRAN:VNET:DIFF:%s%ld:PAR:DATA %s,", channel,
             functionTypeArr[functionType],
             logicalPortNumber,
             interchangeArr[interchange]);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, touchstoneFile));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for differential match
/// HIFN  embedding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkDifferentialMatchCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             functionType++;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",LogPort%ld", channel, logicalPortNumber);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_DIFFERENTIAL_MATCH_EMBEDDING_CIRCUIT_MODEL, circuitModel),
            5, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified Touchstone file defining the
/// HIFN  characteristics of the current embedded differential match circuit
/// HIFN  model. A differential match circuit model involving file import must be
/// HIFN  selected before using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, only embedding is available at the moment.
/// HIPAR logicalPortNumber/Logical port number.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file.
/// HIPAR interchangePortNumbers/Interchange port numbers of loaded *.s2p file. If the parameter is
/// HIPAR interchangePortNumbers/omitted, the port numbers are not interchanged. The parameter must not
/// HIPAR interchangePortNumbers/be used for 4-port files.
ViStatus _VI_FUNC rsvna_LoadDifferentialMatchCircuitModelData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   functionType,
                            ViInt32   logicalPortNumber,
                            ViString  fileName,
                            ViBoolean interchangePortNumbers)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_EMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            3, "Function Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:VNET%ld:DIFF:%s%ld '%s'",
             channel, functionTypeArr[functionType], logicalPortNumber,
             fileName);

    if (interchangePortNumbers)
        strcat(cmd, ",IPOR");

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the firmware uses the new, wave-based de-/embedding
/// HIFN  calculation (default). Otherwise, the legacy S parameter-based
/// HIFN  calculation is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/If enabled, the firmware uses the new, wave-based de-/embedding
/// HIPAR enabled/calculation (default). Otherwise, the legacy S parameter-based
/// HIPAR enabled/calculation is used.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkWaveDeEmbedEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_VIRTUAL_NETWORK_WAVE_DE_EMBED_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If set to ON (1), each port group has its own ground loop embedding
/// HIFN  and deembedding network. Otherwise the same embedding network and
/// HIFN  deembedding network is used for all active ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR portGroup/Enables or disables each port group having its own ground loop
/// HIPAR portGroup/embedding and deembedding network.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkGroundLoopPerPortGroup(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean portGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_PER_PORT_GROUP_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_VIRTUAL_NETWORK_GROUND_LOOP_PER_PORT_GROUP_ENABLED, portGroup),
            3, "Port Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the deembedding/embedding function
/// HIFN  for ground loops in port groups. It is allowed to change the circuit
/// HIFN  model and its parameters while deembedding is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR state/This control enables or disables the deembedding function for ground
/// HIPAR state/loop.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortGroupGroundLoopState(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   functionType,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Group[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_STATE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_STATE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_STATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_STATE,
            "Group", group - 1, RS_REPCAP_BUF_SIZE, repCap_Group),
            3, "Group");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Group);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            4, "Function Type");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attr[functionType], state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the capacitance C, inductance L, resistance R
/// HIFN  or conductance G value in the different circuit models for ground loop
/// HIFN  deembedding/embedding for a port group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR parameterType/Parameter type, capacitance, inductance, resistance or conductance.
/// HIPAR circuitModel/This control defines the possible circuit model.
/// HIPAR groundLoopValue/Capacitance C, Inductance L, Resistance R or Conductance G for the
/// HIPAR groundLoopValue/specified circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortGroupGroundLoop(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   functionType,
                            ViInt32   parameterType,
                            ViInt32   circuitModel,
                            ViReal64  groundLoopValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Group[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2CircuitModel;
    ViChar   value[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViAttr   attr[2][4] =
             {
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CAPACITANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_INDUCTANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_RESISTANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CONDUCTANCE
             },
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_CAPACITANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_INDUCTANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_RESISTANCE,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_CONDUCTANCE
             }
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CAPACITANCE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CAPACITANCE,
            "Group", group - 1, RS_REPCAP_BUF_SIZE, repCap_Group),
            3, "Group");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Group);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            4, "Function Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameterType, RSVNA_VAL_VNET_PARAM_C, RSVNA_VAL_VNET_PARAM_G),
            5, "Parameter Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngVirtualNetworkGroundLoopDeembeddingCircuitModel, circuitModel, &p2CircuitModel),
                6, "Circuit Model");
    snprintf(value, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%s,%.12lG", p2CircuitModel, groundLoopValue);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, attr[functionType][parameterType], value));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the circuit model for ground loop
/// HIFN  deembedding/embedding in a port group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR group/Port group number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR circuitModel/This control defines the possible circuit model.
ViStatus _VI_FUNC rsvna_ConfigureVirtualNetworkPortGroupGroundLoopCircuitModel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   functionType,
                            ViInt32   circuitModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Group[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_CIRCUIT_MODEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
            "Group", group - 1, RS_REPCAP_BUF_SIZE, repCap_Group),
            3, "Group");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Group);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            4, "Function Type");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, attr[functionType], circuitModel),
            5, "Circuit Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads data from a specified one-port (*.s1p) Touchstone
/// HIFN  file defining the characteristics of an embedded/deembedded ground loop
/// HIFN  port group circuit model. The ground loop circuit model involving file
/// HIFN  import (FIMPort) must be selected before using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR functionType/Function type, deembedding or embedding.
/// HIPAR fileName/This control specifies the name and directory of the loaded Touchstone
/// HIPAR fileName/file. If no path is specified the analyzer searches the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?.
/// HIPAR group/Port group number.
ViStatus _VI_FUNC rsvna_LoadPortGroupGroundLoopCircuitModelData(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   group,
                            ViInt32   functionType,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Channel[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_Group[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[2] =
             {
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
             RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_EMBEDDING_CIRCUIT_MODEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_Channel),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_VIRTUAL_NETWORK_PORT_GROUP_GROUND_LOOP_DEEMBEDDING_CIRCUIT_MODEL,
            "Group", group - 1, RS_REPCAP_BUF_SIZE, repCap_Group),
            3, "Group");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_Channel, repCap_Group);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionType, RSVNA_VAL_VNET_FUNC_DEEMBED, RSVNA_VAL_VNET_FUNC_EMBED),
            4, "Function Type");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, attr[functionType], fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures offset compensation.
/// HIFN  If enabled, the names of subsequently generated Test Coupon and DUT +
/// HIFN  Test Fixture files are prefixed with the current date and time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If set to ON (1), the names of subsequently generated Test Coupon and
/// HIPAR enabled/DUT + Test Fixture files are prefixed with the current date and time.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingTimestampFilenames(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_TIMESTAMD_FILENAMES_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows you to configure the coupon type to be measured
/// HIFN  for fixture modelling. This has to be defined before measuring the test
/// HIFN  coupon (using rsvna_MeasureFixtureModelingCoupon).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
/// HIPAR couponType/Sets the coupon type to be measured for a fixture modeling with the
/// HIPAR couponType/ISD tool.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingCouponType(ViSession instrSession,
                                                           ViInt32   tool,
                                                           ViInt32   couponType)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrCouponType[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_COUPON_TYPE,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_COUPON_TYPE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, attrCouponType[tool], couponType),
            3, "Coupon Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows you to configure ports to be measured for fixture
/// HIFN  modelling. This has to be defined before measuring the test coupon
/// HIFN  (using rsvna_MeasureFixtureModelingCoupon).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
/// HIPAR port/Physical port number.
/// HIPAR active/Specifies a port to which the test coupon is connected.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingCouponPortActive(
                            ViSession instrSession,
                            ViInt32   tool,
                            ViInt32   port,
                            ViBoolean active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attrPortActive[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_PORT_ACTIVE,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_PORT_ACTIVE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attrPortActive[tool],
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attrPortActive[tool], active),
            4, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the measurement of the selected coupon at the
/// HIFN  active ports (see rsvna_ConfigureFixtureModelingCoupon).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
ViStatus _VI_FUNC rsvna_MeasureFixtureModelingCoupon(ViSession instrSession,
                                                     ViInt32   tool)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_START_COUPON_MEASUREMENT,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_START_COUPON_MEASUREMENT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, attr[tool], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For fixture modeling with the ISD tool and coupon type '1x Open, 1x
/// HIFN  Short' (see rsvna_ConfigureFixtureModelingCoupon), this event starts the
/// HIFN  measurement of the Open coupon at the active ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_MeasureFixtureModelingISDCouponOpenStep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_START_COUPON_MEASUREMENT_OPEN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For fixture modeling with the ISD tool and coupon type '1x Open, 1x
/// HIFN  Short' (see rsvna_ConfigureFixtureModelingCoupon), this event starts the
/// HIFN  measurement of the Short coupon at the active ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_MeasureFixtureModelingISDCouponShortStep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_START_COUPON_MEASUREMENT_SHORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the coupon properties from a Touchstone file. The
/// HIFN  coupon type can be selected using rsvna_ConfigureFixtureModelingCoupon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
/// HIPAR fileName/The relative or absolute path to the Touchstone file.
ViStatus _VI_FUNC rsvna_LoadFixtureModelingTouchstoneFile(ViSession instrSession,
                                                          ViInt32   tool,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_LOAD_TOUCHSTONE_FILE,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_LOAD_TOUCHSTONE_FILE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, attr[tool], fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For fixture modeling with the ISD tool and coupon type '1x Open, 1x
/// HIFN  Short' (see rsvna_ConfigureFixtureModelingCoupon), this function loads
/// HIFN  the properties of the Open and Short coupon from Touchstone files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR openCouponFileName/The relative or absolute path to the Touchstone file defining the Open
/// HIPAR openCouponFileName/coupon. A *.s2p touchstone file is required.
/// HIPAR shortCouponFileName/The relative or absolute path to the Touchstone file defining the
/// HIPAR shortCouponFileName/Short coupon. A *.s2p touchstone file is required.
ViStatus _VI_FUNC rsvna_LoadFixtureModelingISDOpenShortTouchstoneFiles(
                            ViSession instrSession,
                            ViString  openCouponFileName,
                            ViString  shortCouponFileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LOAD_TOUCHSTONE_FILE_OPEN, openCouponFileName),
            2, "Open Coupon File Name");

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LOAD_TOUCHSTONE_FILE_SHORT, shortCouponFileName),
            3, "Short Coupon File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Invalidates the file path previously set using
/// HIFN  CALCulate:FMODel:ISD:DUT:MEASure:FILename,
/// HIFN  CALCulate:FMODel:ISD:COUPon:MEASure:OPEN:FILename, or
/// HIFN  CALCulate:FMODel:ISD:COUPon:MEASure:SHORt:FILename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingISDCouponInvalidatesFilePath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_COUPON_INVALIDATES_FILE_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Invalidates the file path previously set using
/// HIFN  CALCulate:FMODel:ISD:DUT:MEASure:FILename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingISDDUTInvalidatesFilePath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_INVALIDATES_FILE_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the global default settings of the ISD tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingISDResetToDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_RESET_TO_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables/disables the impedance correction of the ISD tool.
/// HIFN
/// HIFN     This is a global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useImpedanceCorrection/Enables/disables the impedance correction of the ISD tool.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDUseImpedanceCorrection(
                            ViSession instrSession,
                            ViBoolean useImpedanceCorrection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_USE_IMPEDANCE_CORRECTION, useImpedanceCorrection),
            2, "Use Impedance Correction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows you to specify the ports to which the test
/// HIFN  fixture is connected. This has to be defined before measuring DUT +
/// HIFN  Fixture (using rsvna_MeasureFixtureModelingDUTAndTestFixture).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
/// HIPAR port/Physical port number.
/// HIPAR active/Specifies a port to which the test fixture is connected.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDUTPortActive(
                            ViSession instrSession,
                            ViInt32   tool,
                            ViInt32   port,
                            ViBoolean active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attrDUTPort[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_DUT_PORT_ACTIVE,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_PORT_ACTIVE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attrDUTPort[tool],
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attrDUTPort[tool], active),
            4, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For fixture modeling with the ISD tool, this function allows you to
/// HIFN  measure the DUT + Fixture at the active ports (see
/// HIFN  rsvna_ConfigureFixtureModelingDUTActivePorts).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
ViStatus _VI_FUNC rsvna_MeasureFixtureModelingDUTAndTestFixture(
                            ViSession instrSession,
                            ViInt32   tool)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_MEASURE_DUT,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_MEASURE_DUT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, attr[tool], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Runs the selected fixture modeling tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
ViStatus _VI_FUNC rsvna_RunFixtureModelingSimulator(ViSession instrSession,
                                                    ViInt32   tool)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_RUN_SFD_SIMULATOR,
             RSVNA_ATTR_FIXTURE_MODELING_RUN_ISD_SIMULATOR
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, attr[tool], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Use this function to  indicate the ports (physical for single-ended
/// HIFN  deembedding, logical for balanced deembedding) to which the deembedding
/// HIFN  files (generated by rsvna_RunFixtureModelingSimulator) shall be
/// HIFN  assigned (by rsvna_LoadSingleEndedPortCircuitModelData).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modelling tool to access with the function.
/// HIPAR port/Physical (for SFD) or logical (for ISD) port number.
/// HIPAR apply/Specifies the ports to which the deembedding files shall be assigned
/// HIPAR apply/after the tool has run.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingApplyToPort(ViSession instrSession,
                                                            ViInt32   tool,
                                                            ViInt32   port,
                                                            ViBoolean apply)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attrApplyToPort[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_SFD_APPLY_TO_PORT_ENABLED,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_APPLY_TO_PORT_ENABLED
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tool, RSVNA_VAL_FIXTURE_TOOL_SFD, RSVNA_VAL_FIXTURE_TOOL_ISD),
            2, "Tool");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, attrApplyToPort[tool],
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attrApplyToPort[tool], apply),
            4, "Apply");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the common working directory of the ISD, SFD and EZD fixture
/// HIFN  modeling tools.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Defines the working directory.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDirectory(ViSession instrSession,
                                                          ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all files in the common default working directory of the of
/// HIFN  the ISD, SFD, and EZD tool
/// HIFN  (C:\Users\Public\Documents\Rohde-Schwarz\Vna\Embedding).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingClearDefaultDirectory(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_CLEAR_DEFAULT_DIRECTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows you to configure advanced options for single
/// HIFN  ended fixture modeling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR _2xThruPortOrdering/Tells the SFD tool about the port ordering of the 2x Thru test coupon.
/// HIPAR totalPortOrdering/Tells the SFD tool about the port ordering of theTest Fixture.
/// HIPAR adjustImpedanceMismatch/Defines whether the SFD tool shall perform automatic impedance
/// HIPAR adjustImpedanceMismatch/adjustments.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingSFDAdvancedOptions(
                            ViSession instrSession,
                            ViInt32   _2xThruPortOrdering,
                            ViInt32   totalPortOrdering,
                            ViBoolean adjustImpedanceMismatch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_2X_THRU_PORT_ORDERING, _2xThruPortOrdering),
            2, "2x Thru Port Ordering");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_TOTAL_PORT_ORDERING, totalPortOrdering),
            3, "Total Port Ordering");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_ADJUST_IMPEDANCE_MISMATCH_ENABLED, adjustImpedanceMismatch),
            4, "Adjust Impedance Mismatch");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Invalidates the file path previously set using
/// HIFN  CALCulate:FMODel:SFD:COUPon:MEASure:FILename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingSFDCouponInvalidatesFilePath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_COUPON_INVALIDATES_FILE_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Invalidates the file path previously set using
/// HIFN  CALCulate:FMODel:SFD:DUT:MEASure:FILename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingSFDDUTInvalidatesFilePath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_DUT_INVALIDATES_FILE_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables/disables the impedance correction of the SFD tool.
/// HIFN
/// HIFN     This is a global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useImpedanceCorrection/Enables/disables the impedance correction of the SFD tool.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingSFDUseImpedanceCorrection(
                            ViSession instrSession,
                            ViBoolean useImpedanceCorrection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_USE_IMPEDANCE_CORRECTION, useImpedanceCorrection),
            2, "Use Impedance Correction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the global default settings of the SFD tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingSFDResetToDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_SFD_RESET_TO_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the linear_2x batch mode parameter of the ISD tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR insertionLoss/Sets the linear_2x batch mode parameter of the ISD tool.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDTestCouponsInsertionLoss(
                            ViSession instrSession,
                            ViInt32   insertionLoss)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_TEST_COUPONS_INSERTION_LOSS, insertionLoss),
            2, "Insertion Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures advanced options for ISD lead ins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scalingforAttenuation/Scales the test coupon's attenuation (dB). Sets the atten_scale 1
/// HIPAR scalingforAttenuation/batch mode parameter of the ISD tool.
/// HIPAR scalingforFlightTime/Overrides the lead-in's flight time in case the through-trace test
/// HIPAR scalingforFlightTime/coupon is a bit too short or too long. Sets the atten_scale 1 batch
/// HIPAR scalingforFlightTime/mode parameter of the ISD tool.
/// HIPAR flightTimeOverride/If set to true, the flight time for DUT + lead-in is specified using
/// HIPAR flightTimeOverride/the 'Flight Time' control. Otherwise, it is calculated automatically.
/// HIPAR flightTime/If 'Flight Time Override' is enabled, this control allows you to set
/// HIPAR flightTime/the flight time for DUT + Lead-ins manually. This is equivalent to
/// HIPAR flightTime/setting the leadin_dut_time batch mode parameter of the ISD tool to a
/// HIPAR flightTime/numeric value (not auto).
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDLeadIns(ViSession instrSession,
                                                           ViReal64  scalingforAttenuation,
                                                           ViReal64  scalingforFlightTime,
                                                           ViBoolean flightTimeOverride,
                                                           ViReal64  flightTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LEAD_INS_ATTENUATION_SCALING, scalingforAttenuation),
            2, "Scaling for Attenuation");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LEAD_INS_FLIGHT_TIME_SCALING, scalingforFlightTime),
            3, "Scaling for Flight Time");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LEAD_INS_FLIGHT_TIME_OVERRIDE_ENABLED, flightTimeOverride),
            4, "Flight Time Override");
    if (flightTimeOverride)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_LEAD_INS_FLIGHT_TIME_OVERRIDE, flightTime),
                5, "Flight Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ISD tool's advanced DUT With Fixture
/// HIFN  options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR portSequence/Defines how the ISD tool shall interpret the DUT + Fixture data (see
/// HIPAR portSequence/rsvna_MeasureFixtureModelingDUTAndTestFixture). Corresponds to the
/// HIPAR portSequence/port_order batch mode parameter of the ISD tool.
/// HIPAR dutType/Defines whether the DUT is active or passive. Corresponds to the
/// HIPAR dutType/active_dut batch mode parameter of the ISD tool.
/// HIPAR portstoSkip/Tells the ISD tool which ports, if any, to skip in the measured DUT +
/// HIPAR portstoSkip/Test Fixture file when the tool is run. Equivalent to setting the
/// HIPAR portstoSkip/ports_to_skip batch mode parameter of the ISD tool.
/// HIPAR manualSkipPorts/If 'Ports to Skip' is set to Manual, this control ells the ISD tool
/// HIPAR manualSkipPorts/which ports shall be skipped when the tool is run. Equivalent to using
/// HIPAR manualSkipPorts/the ports_to_skip batch mode parameter of the ISD tool with a list of
/// HIPAR manualSkipPorts/(positive) port numbers.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDDUTWithFixture(
                            ViSession instrSession,
                            ViInt32   portSequence,
                            ViInt32   dutType,
                            ViInt32   portstoSkip,
                            ViString  manualSkipPorts)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrPortsToSkip[] =
             {
             RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_PORTS_TO_SKIP_NONE,
             RSVNA_ATTR_FIXTURE_MODELING_IDS_DUT_WITH_FIXTURE_PORTS_TO_SKIP_LEFT,
             RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_PORTS_TO_SKIP_RIGHT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portstoSkip, RSVNA_VAL_FIXTURE_PORTS_TO_SKIP_NONE, RSVNA_VAL_FIXTURE_PORTS_TO_SKIP_MANUAL),
            4, "Ports to Skip");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_PORT_SEQUENCE, portSequence),
            2, "Port Sequence");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_DUT_TYPE, dutType),
            3, "DUT Type");

    if (portstoSkip == RSVNA_VAL_FIXTURE_PORTS_TO_SKIP_MANUAL)
    {
        viCheckParm(RsCore_InvalidNullPointer(instrSession, manualSkipPorts),
                    5, "Manual Skip Ports");
        if (strlen(manualSkipPorts) == 0)
        {
            viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_PORTS_TO_SKIP_NONE, NULL),
                    5, "Manual Skip Ports");
        }
        else
        {
            viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_DUT_WITH_FIXTURE_PORTS_TO_SKIP, manualSkipPorts),
                    5, "Manual Skip Ports");
        }
    }
    else
    {
        viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, attrPortsToSkip[portstoSkip], NULL),
                4, "Ports to Skip");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ISD tool's advanced Calculation options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enforcePassivity/Defines whether the ISD tool shall enforce passivity and reciprocity
/// HIPAR enforcePassivity/for the test coupons and the test fixture. Corresponds to the passive
/// HIPAR enforcePassivity/batch mode parameter of the ISD tool.
/// HIPAR traceCoupling/Tells the ISD tool about coupling among lead-in traces. Corresponds to
/// HIPAR traceCoupling/the coupling batch mode parameter of the ISD tool.
/// HIPAR operationAccuracy/Defines the execution mode of the ISD tool.
/// HIPAR maxFreqtoDeembed/Defines the maximum frequency to deembed in Hz. Corresponds to the
/// HIPAR maxFreqtoDeembed/max_frequency batch mode parameter of the ISD tool.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDCalculations(
                            ViSession instrSession,
                            ViBoolean enforcePassivity,
                            ViInt32   traceCoupling,
                            ViInt32   operationAccuracy,
                            ViReal64  maxFreqtoDeembed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_CALCULATIONS_ENFORCE_PASSIVITY, enforcePassivity),
            2, "Enforce Passivity");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_CALCULATIONS_TRACE_COUPLING, traceCoupling),
            3, "Trace Coupling");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_CALCULATIONS_OPERATION_ACCURACY, operationAccuracy),
            4, "Operation Accuracy");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_CALCULATIONS_MAX_FREQUENCY, maxFreqtoDeembed),
            5, "Max Freq to Deembed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the de-/embedding file generated by the ISD tool
/// HIFN  contains (extrapolated) DC data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dcExtrapolation/Defines whether the de-/embedding file generated by the ISD tool
/// HIPAR dcExtrapolation/contains (extrapolated) DC data. ON correspons to add_dc=1 in the
/// HIPAR dcExtrapolation/tool's batch mode.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingISDCalculationsDCExtrapolation(
                            ViSession instrSession,
                            ViBoolean dcExtrapolation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_ISD_CALCULATIONS_DC_EXTRAPOLATION, dcExtrapolation),
            2, "DC Extrapolation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For a fixture modeling with the EZD tool, this command allows to
/// HIFN  specify the ports to which the test coupon is connected.
/// HIFN     This has to be defined before measuring the test coupon (using
/// HIFN  rsvna_FixtureModelingEZDStartCouponMeasurement).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects a port.
/// HIPAR active/For a fixture modeling with the EZD tool, this command allows to
/// HIPAR active/specify the ports to which the test coupon is connected.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDPortActive(
                            ViSession instrSession,
                            ViInt32   port,
                            ViBoolean active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount),
                2, "Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FIXTURE_MODELING_EZD_PORT_ACTIVE,
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_FIXTURE_MODELING_EZD_PORT_ACTIVE, active),
            3, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For a fixture modeling with the Easy De-embedding (EZD) tool, this
/// HIFN  command starts the measurement of the symmetrical 2xThru coupon at the
/// HIFN  active ports (see rsvna_ConfigureFixtureModelingEZDPortActive).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingEZDStartCouponMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_START_COUPON_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For a fixture modeling with the EZD tool, this command loads the
/// HIFN  coupon properties from a Touchstone file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/For a fixture modeling with the EZD tool, this command loads the
/// HIPAR file/coupon properties from a Touchstone file.
ViStatus _VI_FUNC rsvna_FixtureModelingEZDLoadTouchstoneFile(ViSession instrSession,
                                                             ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_LOAD_TOUCHSTONE_FILE, file),
            2, "File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Invalidates the file path previously set using
/// HIFN  CALCulate:FMODel:EZD:DUT:MEASure:FILename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingEZDDUTInvalidatesFilePath(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_DUT_INVALIDATES_FILE_PATH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows you to specify a port to which the test fixture is connected.
/// HIFN  All DUT ports have to be defined before measuring DUT + Fixture (using
/// HIFN  rsvna_FixtureModelingEZDMeasureDUT). For single-ended deembedding 2
/// HIFN  ports must be active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects a port.
/// HIPAR active/Specifies whether the test fixture is connected to this port.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDDUTPortActive(
                            ViSession instrSession,
                            ViInt32   port,
                            ViBoolean active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount),
                2, "Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FIXTURE_MODELING_EZD_PORT_ACTIVE,
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_FIXTURE_MODELING_EZD_DUT_PORT_ACTIVE, active),
            3, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For fixture modeling with the EZD tool, this event allows you to
/// HIFN  measure the DUT + Fixture at the active ports (see
/// HIFN  rsvna_ConfigureFixtureModelingEZDDUTPortActive). This measurement is
/// HIFN  only available if impedance correction is enabled
/// HIFN  (RSVNA_ATTR_FIXTURE_MODELING_EZD_IMPEDANCE_CORRECTION_ENABLED).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingEZDMeasureDUT(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_MEASURE_DUT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Load DUT + test fixture data from file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/Load DUT + test fixture data from file.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDLoadDUTFile(
                            ViSession instrSession,
                            ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_LOAD_DUT_FILE, file),
            2, "File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Runs the EZD tool.
/// HIFN     Before executing this command, make sure that:
/// HIFN     - The test coupon measurement (using
/// HIFN  rsvna_FixtureModelingEZDStartCouponMeasurement) finished successfully,
/// HIFN  or the test coupon data were successfully loaded from file (using
/// HIFN  rsnva_FixtureModelingEZDLoadTouchstoneFile).
/// HIFN     - The measurement of DUT + test fixture (using
/// HIFN  rsvna_FixtureModelingEZDMeasureDUT) finished successfully.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingRunEZDSimulator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_RUN_EZD_SIMULATOR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For a fixture modeling with the EZD tool, this allows you to specify
/// HIFN  the ports to which the deembedding files shall be assigned after the
/// HIFN  tool has run.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects a port.
/// HIPAR applyToPort/Specify whether the deembedding files shall be assigned to this port
/// HIPAR applyToPort/after the tool has run.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDApplyToPortEnabled(
                            ViSession instrSession,
                            ViInt32   port,
                            ViBoolean applyToPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  portCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount),
                2, "Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            2, "Port");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_FIXTURE_MODELING_EZD_PORT_ACTIVE,
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_FIXTURE_MODELING_EZD_APPLY_TO_PORT_ENABLED, applyToPort),
            3, "Apply To Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the EZD tool about the port ordering of the test fixture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR order/Tells the EZD tool about the port ordering of the test fixture.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDPortOrder(ViSession instrSession,
                                                             ViInt32   order)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_PORT_ORDER, order),
            2, "Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the global default settings of the EZD tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDResetToDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_RESET_TO_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The EZD tool can create fixture deembedding data at side 1, at side 2,
/// HIFN  or at both sides.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR side1/Configures the EZD tool to generate fixture deembedding data at side
/// HIPAR side1/1.
/// HIPAR side2/Configures the EZD tool to generate fixture deembedding data at side
/// HIPAR side2/2.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDGenerateSides(
                            ViSession instrSession,
                            ViBoolean side1,
                            ViBoolean side2)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_GENERATE_SIDE_1_ENABLED, side1),
            2, "Side 1");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_GENERATE_SIDE_2_ENABLED, side2),
            3, "Side 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If not enabled (default), the touchstone data of the generated
/// HIFN  deembedding file reach down to the start frequency of the instrument
/// HIFN  sweep. If enabled, the EZD tool adds (extrapolated) S-parameter data
/// HIFN  for f = 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If not enabled (default), the touchstone data of the generated
/// HIPAR enabled/deembedding file reach down to the start frequency of the instrument
/// HIPAR enabled/sweep. If enabled, the EZD tool adds (extrapolated) S-parameter data
/// HIPAR enabled/for f = 0.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDDCExtrapolationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_DC_EXTRAPOLATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures impedance correction for the EaZy Deembedding
/// HIFN  Tool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables impedance correction.
/// HIPAR referenceImpedance/If empedance correction is enabled, provides the reference impendance
/// HIPAR referenceImpedance/in ohms.
/// HIPAR bandwidthLimit/If empedance correction is enabled, provides the bandwidth limit in
/// HIPAR bandwidthLimit/Hz.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingEZDImpedanceCorrection(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViReal64  referenceImpedance,
                            ViReal64  bandwidthLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_IMPEDANCE_CORRECTION_ENABLED, enabled),
            2, "Enabled");

    if (enabled)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_REFERENCE_IMPEDANCE, referenceImpedance),
                3, "Reference Impedance");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_EZD_BANDWIDTH_LIMIT, bandwidthLimit),
                4, "Bandwidth Limit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the DUT and fixture settings of the deembedding assistant to
/// HIFN  their respective defaults.
/// HIFN
/// HIFN     Note that a *RST does not affect these settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingDeembedAssistantReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Runs the selected demmbedding (CALCulate:FMODel:DEASsistant:TOOL) tool
/// HIFN  and applies the calculated fixture deembedding to the ports selected
/// HIFN  using CALCulate:FMODel:DEASsistant:DUT:PORT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingDeembedAssistantRun(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_RUN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the deembedding assistant, this command defines whether only the
/// HIFN  left coupon model is measured or loaded from file.
/// HIFN
/// HIFN     This simplification is allowed for two-sided DUTs using the same
/// HIFN  coupon model for the left and right side
/// HIFN  (CALCulate:FMODel:DEASsistant:COUPon:LEFT|RIGHt:TYPE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useSameCoupon/For the deembedding assistant, this command defines whether only the
/// HIPAR useSameCoupon/left coupon model is measured or loaded from file.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantUseSameCoupon(
                            ViSession instrSession,
                            ViBoolean useSameCoupon)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_USE_SAME_COUPON, useSameCoupon),
            2, "Use Same Coupon");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the IMP Correct.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR impCorrect/Sets the IMP Correct.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantIMPCorrect(
                            ViSession instrSession,
                            ViBoolean impCorrect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_IMP_CORRECT, impCorrect),
            2, "IMP Correct");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the fixture modeling tool to be used in the deembedding
/// HIFN  assistant.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tool/Selects the fixture modeling tool to be used in the deembedding
/// HIPAR tool/assistant.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantTool(
                            ViSession instrSession,
                            ViInt32   tool)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_TOOL, tool),
            2, "Tool");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to specify a model for the right side (lead-out) of the
/// HIFN  DUT, depending on the selected DUT type
/// HIFN  (RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_DUT_TYPE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deembedModel/Sets the deembed model.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantRightModel(
                            ViSession instrSession,
                            ViInt32   deembedModel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_RIGHT_MODEL, deembedModel),
            2, "Deembed Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to specify a model for the right side (lead-out) of the
/// HIFN  DUT, depending on the selected DUT type
/// HIFN  (RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_DUT_TYPE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deembedModel/Sets the deembed model.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantLeftModel(
                            ViSession instrSession,
                            ViInt32   deembedModel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_LEFT_MODEL, deembedModel),
            2, "Deembed Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the test coupon data available for the lead-in/lead-out.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingDeembedAssistantCouponLeftClear(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_LEFT_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the test coupon data available for the lead-in/lead-out.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingDeembedAssistantCouponRightClear(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_RIGHT_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the properties of a test coupon for the lead-in/lead-out of the
/// HIFN  DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
/// HIPAR path/Path to the Touchstone file, either absolute or relative to the
/// HIPAR path/selected working directory.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponLeftFile(
                            ViSession instrSession,
                            ViInt32   type,
                            ViString  path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeembedAssistantCouponType, type, &p2Type),
                2, "Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,'%s'", p2Type, path);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_LEFT_FILE, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the properties of a test coupon for the lead-in/lead-out of the
/// HIFN  DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
/// HIPAR path/Path to the Touchstone file, either absolute or relative to the
/// HIPAR path/selected working directory.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponRightFile(
                            ViSession instrSession,
                            ViInt32   type,
                            ViString  path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeembedAssistantCouponType, type, &p2Type),
                2, "Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,'%s'", p2Type, path);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_RIGHT_FILE, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Measures the test coupon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponLeftMeasure(
                            ViSession instrSession,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_LEFT_MEASURE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Measures the test coupon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponRightMeasure(
                            ViSession instrSession,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_RIGHT_MEASURE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the port or ports for measuring the selected test coupon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Sets the coupon position.
/// HIPAR portNumber/Sets the port number.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponLeftPort(
                            ViSession instrSession,
                            ViInt32   position,
                            ViInt32   portNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Position;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeembedAssistantPosition, position, &p2Position),
                2, "Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", p2Position, portNumber);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_LEFT_PORT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the port or ports for measuring the selected test coupon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Sets the coupon position.
/// HIPAR portNumber/Sets the port number.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponRightPort(
                            ViSession instrSession,
                            ViInt32   position,
                            ViInt32   portNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Position;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeembedAssistantPosition, position, &p2Position),
                2, "Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", p2Position, portNumber);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_RIGHT_PORT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For fixture modeling with the deembedding assistant, this command
/// HIFN  selects a test coupon to be measured for the lead-in/lead-out, or
/// HIFN  loaded from file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponLeftType(
                            ViSession instrSession,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_LEFT_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For fixture modeling with the deembedding assistant, this command
/// HIFN  selects a test coupon to be measured for the lead-in/lead-out, or
/// HIFN  loaded from file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the deembed assistant coupon type.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantCouponRightType(
                            ViSession instrSession,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_COUPON_RIGHT_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Measures DUT + fixture at the ports defined using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_FixtureModelingDeembedAssistantDUTMeasure(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_DUT_MEASURE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Before you can measure DUT + fixture or run the fixture deembedding
/// HIFN  tool (CALCulate:FMODel:DEASsistant:RUN), you have to define consecutive
/// HIFN  port ranges for the left/right side of the fixture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Selects DUT Position.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantDUTPort(
                            ViSession instrSession,
                            ViInt32   position,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2Position;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDeembedAssistantDUTPosition, position, &p2Position),
                2, "Position");

    snprintf (params, RS_PARAM_BUF_SIZE, "%s,%ld", p2Position, port);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_FIXTURE_MODELING_DEEMBED_ASSISTANT_DUT_PORT, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the DUT type for the Deembedding Assistant.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the DUT type for the Deembedding Assistant.
ViStatus _VI_FUNC rsvna_ConfigureFixtureModelingDeembedAssistantDUTType(
                            ViSession instrSession,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FIXTURE_MODELING_DEEMBED_ASSISTANT_DUT_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and configures the Delta-L+ characterization method to be
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR characterizationMethod/Selects the Delta-L+ characterization method to be used.
/// HIPAR oneLineDifferential/The one-length method (RSVNA_ATTR_DELTA_L_CHARACTERIZATION_METHOD M1L)
/// HIPAR oneLineDifferential/handles single-ended and balanced measurement results differently. This
/// HIPAR oneLineDifferential/selects between the two modes.
/// HIPAR measurement/Defines whether single-ended or balanced ports are measured for
/// HIPAR measurement/Delta-L characterizations.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLCharacterization(ViSession instrSession,
                                                        ViInt32   characterizationMethod,
                                                        ViBoolean oneLineDifferential,
                                                        ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DELTA_L_CHARACTERIZATION_METHOD, characterizationMethod),
            2, "Characterization Method");

    if (characterizationMethod == RSVNA_VAL_DELTA_L_1L)
    {
        viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DELTA_L_CHARACTERIZATION_M1L_DIFFERENTIAL_ENABLED, oneLineDifferential),
                3, "One Line Differential");
    }

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DELTA_L_CHARACTERIZATION_MEASUREMENT, measurement),
            4, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Declares how the DUT is connected to the measurement system. Choosing
/// HIFN  the wrong port ordering for your setup, can result in erroneous
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR portOrder/Declares how the DUT is connected to the measurement system. Choosing
/// HIPAR portOrder/the wrong port ordering for your setup, can result in erroneous
/// HIPAR portOrder/results.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLPortOrder(ViSession instrSession,
                                                 ViInt32   portOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DELTA_L_PORT_ORDER, portOrder),
            2, "Port Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Delta-L computes the uncertainty in the loss-per-inch calculation at
/// HIFN  specified frequencies.
/// HIFN     This command specifies whether to use the tool's default frequencies
/// HIFN  or custom frequencies, which can be defined using
/// HIFN  CALCulate:FMODel:DELT<Ph_pt>:FREQuencies:USER commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Delta-L computes the uncertainty in the loss-per-inch calculation at
/// HIPAR enabled/specified frequencies.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLFrequenciesUseDefaultEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DELTA_L_FREQUENCIES_USE_DEFAULT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the instrument for the length of the specified frequency list.
/// HIFN  Use rsvna_QueryDeltaLFrequencyList to query the list of frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyList/Selects a frequency list.
/// HIPAR count/The number of frequencies defined in the specified list.
ViStatus _VI_FUNC rsvna_QueryDeltaLFequencyListCount(ViSession instrSession,
                                                     ViInt32   frequencyList,
                                                     ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_FREQUENCY_COUNT_CURRENT,
             RSVNA_ATTR_DELTA_L_FREQUENCY_COUNT_DEFAULT,
             RSVNA_ATTR_DELTA_L_FREQUENCY_COUNT_USER
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyList,
                    RSVNA_VAL_DELTA_L_FREQUENCIES_CURRENT, RSVNA_VAL_DELTA_L_FREQUENCIES_USER),
                2, "Frequency List")

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, attrArr[frequencyList], count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the instrument for the specified frequency list. Use
/// HIFN  rsvna_QueryDeltaLFrequencyListCount to find the length of the list for
/// HIFN  array allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyList/Selects a frequency list.
/// HIPAR arraySize/The size of the allocated array.
/// HIPAR frequencies/The frequencies in the selected list returned by the instrument.
/// HIPAR returnedValues/The number of values returned by the instrument. If this number is
/// HIPAR returnedValues/larger than the allocated Array Size, some values were lost.
ViStatus _VI_FUNC rsvna_QueryDeltaLFrequencyList(ViSession instrSession,
                                                 ViInt32   frequencyList,
                                                 ViInt32   arraySize,
                                                 ViReal64  frequencies[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPChar  response = NULL;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_FREQUENCIES_CURRENT,
             RSVNA_ATTR_DELTA_L_FREQUENCIES_DEFAULT,
             RSVNA_ATTR_DELTA_L_FREQUENCIES_USER
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequencyList,
                    RSVNA_VAL_DELTA_L_FREQUENCIES_CURRENT, RSVNA_VAL_DELTA_L_FREQUENCIES_USER),
                2, "Frequency List")

    viCheckParm(RsCore_GetAttributeViRawStringUnknownLen(instrSession, NULL,
                    attrArr[frequencyList], RS_VAL_DIRECT_USER_CALL, &response),
                4, "Frequencies");
    viCheckParm(RsCore_Convert_String_To_ViReal64ArrayUserBuffer(instrSession, response,
                    arraySize, frequencies, returnedValues),
                4, "Frequencies");

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a frequency to the list of custom user frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Adds a frequency to the list of custom user frequencies.
ViStatus _VI_FUNC rsvna_DeltaLAddUserFrequency(ViSession instrSession,
                                               ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_ADD_USER_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a frequency from the list of custom user frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Deletes a frequency from the list of custom user frequencies.
ViStatus _VI_FUNC rsvna_DeltaLDeleteUserFrequency(ViSession instrSession,
                                                  ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_DELETE_USER_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the list of custom user frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeltaLDeleteAllUserFrequencies(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_DELETE_ALL_USER_FREQUENCIES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tells the firmware to use either default values for frequency step
/// HIFN  size and IF bandwidth, or the settings of the active channel for the
/// HIFN  Delta-L measurement.
/// HIFN     The resulting frequency step size and IF bandwidth can be queried
/// HIFN  using rsvna_QueryDeltaLSweepControlConfiguration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepControl/Tells the firmware to use either default values for frequency step
/// HIPAR sweepControl/size and IF bandwidth, or the settings of the active channel for the
/// HIPAR sweepControl/Delta-L measurement.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLSweepControl(ViSession instrSession,
                                                    ViInt32   sweepControl)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_CONTROL, sweepControl),
            2, "Sweep Control");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the Delta-L measurement, the firmware can either use a default
/// HIFN  sweep configuration, or the active channel setting (see
/// HIFN  rsvna_ConfigureDeltaLSweepControl).
/// HIFN     This query returns the resulting frequency and bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stepSize/Returns the resulting frequency step size.
/// HIPAR ifBandwidth/Returns the resulting IF bandwidth.
ViStatus _VI_FUNC rsvna_QueryDeltaLSweepControlConfiguration(ViSession instrSession,
                                                             ViReal64* stepSize,
                                                             ViReal64* ifBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_CONTROL_STEP, stepSize),
            2, "Step Size");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_CONTROL_IF_BANDWIDTH, ifBandwidth),
            3, "IF Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether dynamic bandwidth at low frequencies is used for
/// HIFN  Delta-L.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR controlDynamicBandwidth/Defines whether dynamic bandwidth at low frequencies is used for
/// HIPAR controlDynamicBandwidth/Delta-L.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLSweepControlDynamicBandwidth(
                            ViSession instrSession,
                            ViBoolean controlDynamicBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_CONTROL_DYNAMIC_BANDWIDTH, controlDynamicBandwidth),
            2, "Control Dynamic Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the sweep range of the Delta-L measurement channel.
/// HIFN  Furthermore, the VNA collects data for the LPI computation at this
/// HIFN  frequency (in addition to RSVNA_ATTR_DELTA_L_FREQUENCIES_CURRENT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimum/Defines the minimum of the sweep range of the Delta-L measurement
/// HIPAR minimum/channel. The entered value is set as
/// HIPAR minimum/RSVNA_ATTR_STIMULUS_FREQUENCY_START.
/// HIPAR maximum/Defines the minimum of the sweep range of the Delta-L measurement
/// HIPAR maximum/channel. The entered value is set as
/// HIPAR maximum/RSVNA_ATTR_STIMULUS_FREQUENCY_STOP.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLSweepRange(ViSession instrSession,
                                                  ViReal64  minimum,
                                                  ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_RANGE_MINIMUM, minimum),
            2, "Minimum");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_DELTA_L_SWEEP_RANGE_MAXIMUM, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, then each of the new traces is displayed in a separate
/// HIFN  diagram. Otherwise, the VNA places all traces in a single diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If enabled, then each of the new traces is displayed in a separate
/// HIPAR enabled/diagram. Otherwise, the VNA places all traces in a single diagram.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLUseRecommendedTraceConfigurationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DELTA_L_USE_RECOMMENDED_TRACE_CONFIGURATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the Time Domain Analysis option R&SZNB/ZNBT-K2 is installed, this
/// HIFN  command allows you to add the TDR-based converted impedance traces to
/// HIFN  the recommended trace configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/If the Time Domain Analysis option R&SZNB/ZNBT-K2 is installed, this
/// HIPAR enabled/command allows you to add the TDR-based converted impedance traces to
/// HIPAR enabled/the recommended trace configuration.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLShowTDRImpededancesEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DELTA_L_SHOW_TDR_IMPEDEDANCES_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads coupon measurement results for the selected analysis line from a
/// HIFN  s2p or s4p Touchstone file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects a line of the Delta-L analysis.
/// HIPAR filename/Path and filename of the coupon measurement results to load.
ViStatus _VI_FUNC rsvna_DeltaLLoadMeasurementFile(ViSession instrSession,
                                                  ViInt32   line,
                                                  ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_LOAD_1L_MEASUREMENT_FILE,
             RSVNA_ATTR_DELTA_L_LOAD_2L_MEASUREMENT_FILE,
             RSVNA_ATTR_DELTA_L_LOAD_3L_MEASUREMENT_FILE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSVNA_VAL_DELTA_L_1L, RSVNA_VAL_DELTA_L_3L),
            2, "Line");

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, attrArr[line], filename),
            3, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a port for the selected Delta-L analysis line. For
/// HIFN  each measurement, either two or four ports must be active. If the
/// HIFN  1-length method is used, then four ports must be active for the first
/// HIFN  line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects a line of the Delta-L analysis.
/// HIPAR port/Selects a port.
/// HIPAR enabled/Enables or disables this port for the selected Delta-L analysis line.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLPortEnabled(ViSession instrSession,
                                                   ViInt32   line,
                                                   ViInt32   port,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  portCount = 0;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_1L_PORT_ENABLED,
             RSVNA_ATTR_DELTA_L_2L_PORT_ENABLED,
             RSVNA_ATTR_DELTA_L_3L_PORT_ENABLED
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSVNA_VAL_DELTA_L_1L, RSVNA_VAL_DELTA_L_3L),
            2, "Line");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Port", &portCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, RSVNA_REPCAP_PORT_PT1, portCount),
            3, "Port");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DELTA_L_1L_PORT_ENABLED,
            "Port", port - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, attrArr[line], enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the length of an analysis line in Delta-L measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects a line of the Delta-L analysis.
/// HIPAR length/The length of the analysis line.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLLineLength(ViSession instrSession,
                                                  ViInt32   line,
                                                  ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_1L_LENGTH,
             RSVNA_ATTR_DELTA_L_2L_LENGTH,
             RSVNA_ATTR_DELTA_L_3L_LENGTH
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSVNA_VAL_DELTA_L_1L, RSVNA_VAL_DELTA_L_3L),
            2, "Line");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, attrArr[line], length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Measures the full set of S-parameters at the active ports for the
/// HIFN  specified line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects a line of the Delta-L analysis.
ViStatus _VI_FUNC rsvna_DeltaLMeasureLine(ViSession instrSession,
                                          ViInt32   line)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[] =
             {
             RSVNA_ATTR_DELTA_L_MEASURE_1L,
             RSVNA_ATTR_DELTA_L_MEASURE_2L,
             RSVNA_ATTR_DELTA_L_MEASURE_3L
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, RSVNA_VAL_DELTA_L_1L, RSVNA_VAL_DELTA_L_3L),
            2, "Line");

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, attrArr[line], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Runs the AITT-DL tool with the collected measurement data and the
/// HIFN  configured Delta-L settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeltaLRun(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_RUN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the 1-length method is used
/// HIFN  (RSVNA_ATTR_DELTA_L_CHARACTERIZATION_METHOD M1L), the VNA firmware
/// HIFN  builds up a cache of coupon measurements (Touchstone files) in the
/// HIFN  DeltaL\1LMeasurementCache subfolder of the user data folder
/// HIFN  C:/Users/Public/Documents/Rohde-Schwarz/Vna.
/// HIFN     This function clears this cache.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeltaLClearAll1LMeasurements(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_CLEAR_ALL_1L_MEASUREMENTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the 1-length method is used
/// HIFN  (RSVNA_ATTR_DELTA_L_CHARACTERIZATION_METHOD M1L), the VNA firmware
/// HIFN  builds up a cache of coupon measurements (Touchstone files) in the
/// HIFN  DeltaL\1LMeasurementCache subfolder of the user data folder
/// HIFN  C:/Users/Public/Documents/Rohde-Schwarz/Vna.
/// HIFN     This function deletes the selected file from this cache.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/If the 1-length method is used
/// HIPAR filename/(RSVNA_ATTR_DELTA_L_CHARACTERIZATION_METHOD M1L), the VNA firmware
/// HIPAR filename/builds up a cache of coupon measurements (Touchstone files) in the
/// HIPAR filename/DeltaL\1LMeasurementCache subfolder of the user data folder
/// HIPAR filename/C:/Users/Public/Documents/Rohde-Schwarz/Vna.
ViStatus _VI_FUNC rsvna_DeltaLClear1LMeasurementFile(ViSession instrSession,
                                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_CLEAR_1L_MEASUREMENT_FILE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the working directory of the Delta-L tool.
/// HIFN
/// HIFN     This function deletes the selected file from this cache.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Defines the working directory of the Delta-L tool.
ViStatus _VI_FUNC rsvna_ConfigureDeltaLDirectory(ViSession instrSession,
                                                 ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the working directory of the Delta-L tool to its default
/// HIFN  C:\Users\Public\Documents\Rohde-Schwarz\Vna\DeltaL.
/// HIFN
/// HIFN     This function deletes the selected file from this cache.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DeltaLDirectoryDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DELTA_L_DIRECTORY_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures offset compensation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offsetCompensation/Selects the type of compensation that the network analyzer calculates
/// HIPAR offsetCompensation/from the acquired compensation data.
ViStatus _VI_FUNC rsvna_ConfigureFixtureCompensationOffsetCorrection(
                            ViSession instrSession,
                            ViInt32   offsetCompensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (offsetCompensation)
    {
    case RSVNA_VAL_OFFSET_COMPENSATION_AUTO_LENGTH:
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_FIXTURE_COMPENSATION_OFFSET_CORRECTION_TYPE, RSVNA_VAL_FIXTURE_COMPENSATION_AUTO));
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_FIXTURE_COMPENSATION_OFFSET_CORRECTION_AUTO_TYPE, RSVNA_VAL_FIXTURE_COMPEMSATION_AUTO));
        break;

    case RSVNA_VAL_OFFSET_COMPENSATION_AUTO_LENGTH_LOSS:
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_FIXTURE_COMPENSATION_OFFSET_CORRECTION_TYPE, RSVNA_VAL_FIXTURE_COMPENSATION_AUTO));
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_FIXTURE_COMPENSATION_OFFSET_CORRECTION_AUTO_TYPE, RSVNA_VAL_FIXTURE_COMPEMSATION_AUTO_LOSS));
        break;

    case RSVNA_VAL_OFFSET_COMPENSATION_DIRECT:
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_FIXTURE_COMPENSATION_OFFSET_CORRECTION_TYPE, RSVNA_VAL_FIXTURE_COMPENSATION_DIRECT));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, offsetCompensation), 2, "Offset Compensation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Prepares the analyzer for fixture compensation comprising a single or
/// HIFN  a series of fixture compensation sweeps
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ResetFixtureCompensation(ViSession instrSession,
                                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_FIXTURE_COMPENSATION_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a fixture compensation sweep in order to acquire  measurement
/// HIFN  data for a test fixture that has its inner conductor terminated with
/// HIFN  the selected standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR arraySize/Size of array containing port numbers.
/// HIPAR terminatedType/Selects standard type, open or short circuit.
/// HIPAR portNumbers/Array of port numbers for perform calibration.
ViStatus _VI_FUNC rsvna_StartFixtureCompensation(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   arraySize,
                                                 ViInt32   terminatedType,
                                                 ViInt32   portNumbers[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:FIXT %s,", channel, terminatedType ? "SHOR" : "OPEN");
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, portNumbers, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Completes a fixture compensation, storing and applying the acquired
/// HIFN  data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_ApplyFixtureCompensation(ViSession instrSession,
                                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);
    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_FIXTURE_COMPENSATION_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads "Direct Compensation" data from the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR fixtureFile/Path to the "Direct Compensation" data file, either absolute or
/// HIPAR terminatedType/Selects standard type, open or short circuit.
/// HIPAR arraySize/Size of array containing port numbers.
/// HIPAR portNumbers/Array of port numbers for perform calibration.
ViStatus _VI_FUNC rsvna_ImportFixtureCompensation(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  fixtureFile,
                                                  ViInt32   terminatedType,
                                                  ViInt32   arraySize,
                                                  ViInt32   portNumbers[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:FIXT:IMP '%s',%s,", channel, fixtureFile, terminatedType ? "SHOR" : "OPEN");
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, portNumbers, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves "Direct Compensation" data from the specified file. The function
/// HIFN  first acquires the required data, just as a function
/// HIFN  rsvna_StartFixtureCompensation would do.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR fixtureFile/Path to the "Direct Compensation" data file, either absolute or
/// HIPAR terminatedType/Selects standard type, open or short circuit.
/// HIPAR arraySize/Size of array containing port numbers.
/// HIPAR portNumbers/Array of port numbers for perform calibration.
ViStatus _VI_FUNC rsvna_ExportFixtureCompensation(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  fixtureFile,
                                                  ViInt32   terminatedType,
                                                  ViInt32   arraySize,
                                                  ViInt32   portNumbers[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS%ld:CORR:COLL:FIXT:EXP '%s',%s,", channel, fixtureFile, terminatedType ? "SHOR" : "OPEN");
    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, portNumbers, arraySize));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the Automatic Harmonic Grid function ON or OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Turns the Automatic Harmonic Grid function ON or OFF.
ViStatus _VI_FUNC rsvna_ConfigureHarmonicAutomaticGridEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HARMONIC_AUTOMATIC_GRID_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_HARMONIC_AUTOMATIC_GRID_ENABLED, enabled),
            3, "enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether the current frequency grid is harmonic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR isHarmonic/Queries whether the current frequency grid is harmonic.
ViStatus _VI_FUNC rsvna_QueryIsCurrentFrequencyHarmonic(ViSession  instrSession,
                                                        ViInt32    channel,
                                                        ViBoolean* isHarmonic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HARMONIC_IS_CURRENT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_HARMONIC_IS_CURRENT, isHarmonic),
            3, "Is Harmonic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows you to activate pipelining of measurement data for the
/// HIFN  alternated driving mode ([SENSe<Ch>:]COUPle ALL | AUTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pipelining/Allows you to activate pipelining of measurement data for the
/// HIPAR pipelining/alternated driving mode ([SENSe<Ch>:]COUPle ALL | AUTO).
ViStatus _VI_FUNC rsvna_ConfigurePipeliningState(ViSession instrSession,
                                                 ViInt32   pipelining)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_PIPELINING_STATE, pipelining),
            2, "Pipelining");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the availability of channel results (to be queried, e.g.,
/// HIFN  using CALCulate<Chn>:DATA) in single-sweep mode.
/// HIFN     The string result '<true|false>,<channel nos>' indicates:
/// HIFN     - false, if the sweep is still running
/// HIFN     - true, if the sweep has been completed
/// HIFN     - The comma-separated list of channels nos for which new measurement
/// HIFN  data are available (since the last call)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR result/Queries the availability of channel results (to be queried, e.g.,
/// HIPAR result/using CALCulate<Chn>:DATA) in single-sweep mode.
/// HIPAR returnedBytes/The length of the response received from the instrument. If this is
/// HIPAR returnedBytes/longer than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryPipeliningChannelResults(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViChar    result[],
                                                      ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_PIPELINING_CHANNEL_RESULTS,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, result, returnedBytes),
                3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables the mixer delay measurement and switches back to normal (non
/// HIFN  frequency-converting) mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_DisableMixerDelay(ViSession instrSession,
                                          ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_DISABLE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_DISABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts a calibration sweep for the mixer delay measurement. As a
/// HIFN  prerequisite, the calibrated channel must have at least one mixer delay
/// HIFN  trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
ViStatus _VI_FUNC rsvna_MixerDelayCalibrationStart(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GENERIC_DEVICES_DELETE_ADDRESS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_CALIBRATION_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frequency difference between the upper and lower tone
/// HIFN  (aperture).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR deltaFrequency/Defines the frequency difference between the upper and lower tone
/// HIPAR deltaFrequency/(aperture).
ViStatus _VI_FUNC rsvna_ConfigureMixerDelay2ToneDeltaFrequency(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  deltaFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_2_TONE_DELTA_FREQUENCY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_2_TONE_DELTA_FREQUENCY, deltaFrequency),
            3, "Delta Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a constant mixer delay value, to be used as a reference for a
/// HIFN  mixer delay measurement calibration. The setting takes effect after
/// HIFN  constant mixer delay is enabled; see example below.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR constantDelay/Defines a constant mixer delay value, to be used as a reference for a
/// HIPAR constantDelay/mixer delay measurement calibration. The setting takes effect after
/// HIPAR constantDelay/constant mixer delay is enabled; see example below.
ViStatus _VI_FUNC rsvna_ConfigureMixerDelayConstantDelay(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViReal64  constantDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_CONSTANT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_CONSTANT_DELAY, constantDelay),
            3, "Constant Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If checked, zou can specify a constant calibration delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/If checked, zou can specify a constant calibration delay.
ViStatus _VI_FUNC rsvna_MixerDelayUseConstantDelay(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_USE_CONSTANT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_USE_CONSTANT_DELAY, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Qualifies whether the analyzer applies the current mixer delay
/// HIFN  measurement calibration data.
/// HIFN
/// HIFN     Tip:
/// HIFN     The analyzer automatically applies the calibration data when a
/// HIFN  calibration file is loaded (MMEMory:LOAD:MDCData) or after a
/// HIFN  calibration sweep has been completed
/// HIFN  ([SENSe<Ch>:]FREQuency:MDELay:ACQuire).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR active/Qualifies whether the analyzer applies the current mixer delay
/// HIPAR active/measurement calibration data.
ViStatus _VI_FUNC rsvna_ConfigureMixerDelayCalibrationActive(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViBoolean active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_CALIBRATION_ACTIVE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_CALIBRATION_ACTIVE, active),
            3, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select the port that generates the lower tone of the two-tone signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR lowerTone/Select the port that generates the lower tone of the two-tone signal.
ViStatus _VI_FUNC rsvna_ConfigureMixerDelayLowerTone(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   lowerTone)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_LOWER_TONE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_LOWER_TONE, lowerTone),
            3, "Lower Tone");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Group Delay Receiver Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR receiverMode/Sets the Group Delay Receiver Mode.
ViStatus _VI_FUNC rsvna_ConfigureMixerDelayGroupDelayReceiverMode(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   receiverMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_MIXER_DELAY_GROUP_DELAY_RECEIVER_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_MIXER_DELAY_GROUP_DELAY_RECEIVER_MODE, receiverMode),
            3, "Receiver Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the source for the upper tone signal that is used for the
/// HIFN  mixer delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR signal/Upper tone source.
/// HIPAR portNumber/Number of the port for the internal source or of the generator (to be
/// HIPAR portNumber/omitted for source NONE).
ViStatus _VI_FUNC rsvna_ConfigureMixerDelayUpperTone(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   signal,
                                                     ViInt32   portNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2sig;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_MIXER_DELAY_UPPER_TONE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngUpperToneSource, signal, &p2sig),
                3, "Signal");

    if (signal == RSVNA_VAL_UPPER_TONE_SOURCE_NONE)
    {
        snprintf(params, RS_PARAM_BUF_SIZE, "%s", p2sig);
    }
    else
    {
        snprintf(params, RS_PARAM_BUF_SIZE, "%s,%ld", p2sig, portNumber);
    }

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_MIXER_DELAY_UPPER_TONE, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  Loads the known delay values of a calibration mixer, to be used as a
/// HIFN  reference for a mixer delay measurement calibration. Mixer delay
/// HIFN  measurements are controlled using the SENSe<Ch>:FREQuency:MDELay...
/// HIFN  commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelID/Channel Number.
/// HIPAR filename/String parameter to specify the name and directory of the loaded file.
/// HIPAR filename/The default extension (manual control) for files containing known
/// HIPAR filename/delays is *.csv, although other extensions are allowed. If no path is
/// HIPAR filename/specified the analyzer searches the current directory, to be queried
/// HIPAR filename/with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_LoadMixerDelayKnownDelayValues(ViSession instrSession,
                                                       ViInt32   channelID,
                                                       ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s'", channelID, filename);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HIDDEN_MIXER_DELAY_LOAD_KNOWN_DELAY_VALUES, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads correction data for a mixer delay measurement calibration from a
/// HIFN  specified file and assigns it to a channel with a specified number.
/// HIFN  Mixer delay measurements are controlled using the
/// HIFN  SENSe<Ch>:FREQuency:MDELay... commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelID/Channel Number.
/// HIPAR filename/String parameter to specify the name and directory of the loaded file.
/// HIPAR filename/The default extension (manual control) for files containing known
/// HIPAR filename/delays is *.csv, although other extensions are allowed. If no path is
/// HIPAR filename/specified the analyzer searches the current directory, to be queried
/// HIPAR filename/with MMEMory:CDIRectory?.
ViStatus _VI_FUNC rsvna_LoadMixerDelayCorrectionData(ViSession instrSession,
                                                     ViInt32   channelID,
                                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s'", channelID, filename);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_MIXER_DELAY_LOAD_CORRECTION_DATA, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stores the correction data for a mixer delay measurement calibration
/// HIFN  to a specified file. Mixer delay measurements are controlled using the
/// HIFN  SENSe<Ch>:FREQuency:MDELay... commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR storeCorrectionData/Stores the correction data for a mixer delay measurement calibration
/// HIPAR storeCorrectionData/to a specified file. Mixer delay measurements are controlled using the
/// HIPAR storeCorrectionData/SENSe<Ch>:FREQuency:MDELay... commands.
ViStatus _VI_FUNC rsvna_StoreMixerDelayCorrectionData(ViSession instrSession,
                                                      ViInt32   channelID,
                                                      ViString  storeCorrectionData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (params, RS_PARAM_BUF_SIZE, "%ld,'%s'", channelID, storeCorrectionData);

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_HIDDEN_MIXER_DELAY_STORE_CORRECTION_DATA, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the duration of the data acquisition for pulse measurements in
/// HIFN  channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR acquisitionTime/Defines the duration of the data acquisition for pulse measurements in
/// HIPAR acquisitionTime/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigurePulseAcquisitionTime(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64  acquisitionTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_ACQUISITION_TIME,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_ACQUISITION_TIME, acquisitionTime),
            3, "Acquisition Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enforces identical pulse generator settings for all ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Enforces identical pulse generator settings for all ports.
ViStatus _VI_FUNC rsvna_ConfigurePulseCoupledEnabled(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_COUPLED_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_COUPLED_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the duty cycle of the generated pulses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR fixedDutyCycleEnabled/Defines whether the analyzer firmware shall enforce a fixed duty cycle
/// HIPAR fixedDutyCycleEnabled/for all pulse generators.
/// HIPAR dutyCycle/Defines the duty cycle of the generated pulses.
ViStatus _VI_FUNC rsvna_ConfigurePulseDutyCycle(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean fixedDutyCycleEnabled,
                                                ViReal64  dutyCycle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_FIXED_DUTY_CYCLE_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_FIXED_DUTY_CYCLE_ENABLED, fixedDutyCycleEnabled),
            3, "Fixed Duty Cycle Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_DUTY_CYCLE, dutyCycle),
            4, "Duty Cycle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Pulse Generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR period/Sets the Pulse Generator Period.
/// HIPAR source/Sets the source for Pulse Generator.
/// HIPAR trainPeriod/Sets the train period.
ViStatus _VI_FUNC rsvna_ConfigurePulseGenerator(ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  period,
                                                ViInt32   source,
                                                ViReal64  trainPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_PERIOD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_PERIOD, period),
            3, "Period");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_SOURCE, source),
            4, "Source");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_PERIOD, trainPeriod),
            5, "Train Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches between internal and external pulse control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Switches between internal and external pulse control.
ViStatus _VI_FUNC rsvna_ConfigurePulseInternalEnabled(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_INTERNAL_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_INTERNAL_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the measurement delay (acquisition delay) relative to the
/// HIFN  pulse start.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR measurementDelay/Defines the measurement delay (acquisition delay) relative to the
/// HIPAR measurementDelay/pulse start.
ViStatus _VI_FUNC rsvna_ConfigurePulseMeasurementDelay(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViReal64  measurementDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_MEASUREMENT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_MEASUREMENT_DELAY, measurementDelay),
            3, "Measurement Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the (common) pulse period of the internal pulse generators and
/// HIFN  modulators.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR pulsePeriod/Defines the (common) pulse period of the internal pulse generators and
/// HIPAR pulsePeriod/modulators.
ViStatus _VI_FUNC rsvna_ConfigurePulsePeriod(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  pulsePeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PERIOD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_PERIOD, pulsePeriod),
            3, "Pulse Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Sync measurement to pulses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR sync/Sets the Sync measurement to pulses.
ViStatus _VI_FUNC rsvna_ConfigurePulseSyncMeasurementtoPulses(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean sync)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_SYNC_MEASUREMENT_TO_PULSES,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_SYNC_MEASUREMENT_TO_PULSES, sync),
            3, "Sync");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the state of the Pulse Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Sets the state of the Pulse Measurement.
ViStatus _VI_FUNC rsvna_ConfigurePulseMeasurementEnabled(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_MEASUREMENT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_MEASUREMENT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the acquisition delay of the ALC measurement relative to the
/// HIFN  master pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR alcMeasurementDelay/Defines the acquisition delay of the ALC measurement relative to the
/// HIPAR alcMeasurementDelay/master pulse.
ViStatus _VI_FUNC rsvna_ConfigurePulseALCMeasurementDelay(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  alcMeasurementDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_ALC_MEASUREMENT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_ALC_MEASUREMENT_DELAY, alcMeasurementDelay),
            3, "ALC Measurement Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Pulse Generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR enabled/Sets the Pulse Generator State.
/// HIPAR delay/Sets the Pulse Generator Delay.
/// HIPAR width/Sets the Pulse Generator Width.
ViStatus _VI_FUNC rsvna_ConfigurePulseMeasurementGenerator(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   generatorNumber,
                                                           ViBoolean enabled,
                                                           ViReal64  delay,
                                                           ViReal64  width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_ENABLED,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_DELAY, delay),
            5, "Delay");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_WIDTH, width),
            6, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the Pulse Generator Train state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR enabled/Sets the Pulse Generator Train state.
ViStatus _VI_FUNC rsvna_ConfigurePulseGeneratorTrainEnabled(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   generatorNumber,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_ENABLED,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Train Data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR arraySize/Number of array elements.
/// HIPAR element/Set pulse train element active.
/// HIPAR startTime/Set Element Start Time.
/// HIPAR stopTime/Set Element Stop Time.
ViStatus _VI_FUNC rsvna_ConfigurePulseGeneratorTrainData(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   generatorNumber,
                                                         ViInt32   arraySize,
                                                         ViBoolean element[],
                                                         ViReal64  startTime[],
                                                         ViReal64  stopTime[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  i = 0;
    ViChar   *p2char;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_PULSE_GENERATOR_TRAIN_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_PULSE_GENERATOR_TRAIN_DATA,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    p2char = params;

    for (i = 0; i < arraySize; i++)
    {
        p2char += snprintf (p2char, RS_MAX_MESSAGE_BUF_SIZE, "%s,%.12lG,%.12lG,", (element[i] == VI_FALSE) ? "OFF":"ON", startTime[i], stopTime[i]);
    }

    *--p2char = '\0';

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_HIDDEN_PULSE_GENERATOR_TRAIN_DATA, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all Train Data.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSVNA_ATTR_PULSE_GENERATOR_TRAIN_DELETE_ALL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe<Ch>:]PULSe:GENerator<Id>:TRAin:DELete:ALL
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
ViStatus _VI_FUNC rsvna_PulseGeneratorTrainDeleteAll(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   generatorNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_DELETE_ALL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_DELETE_ALL,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_TRAIN_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR segment/Segment number.
/// HIPAR segmentCount/Sets the number of segments.
ViStatus _VI_FUNC rsvna_ConfigurePulseGeneratorTrainCount(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   generatorNumber,
                                                          ViInt32   segment,
                                                          ViInt32   segmentCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_COUNT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_COUNT,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_COUNT,
            "TrainSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap3),
            4, "Segment");
    strcat (repCap, ",");
    strcat (repCap, repCap2);
    strcat (repCap, ",");
    strcat (repCap, repCap3);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_COUNT, segmentCount),
            5, "Segment Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Pulse Generator Train.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR generatorNumber/Generator number.
/// HIPAR segment/Segment number.
/// HIPAR enabled/Set pulse train segment active/inactive.
/// HIPAR startTime/Sets the Start time of the segment of the pulse train.
/// HIPAR stopTime/Sets the Stop time of the segment of the pulse train.
ViStatus _VI_FUNC rsvna_ConfigurePulseGeneratorTrain(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   generatorNumber,
                                                     ViInt32   segment,
                                                     ViBoolean enabled,
                                                     ViReal64  startTime,
                                                     ViReal64  stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_ENABLED,
            "Generator", generatorNumber - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Generator Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_ENABLED,
            "TrainSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap3),
            4, "Segment");
    strcat (repCap, ",");
    strcat (repCap, repCap2);
    strcat (repCap, ",");
    strcat (repCap, repCap3);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_ENABLED, enabled),
            5, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_START_TIME, startTime),
                6, "Start Time");

        viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_GENERATOR_SEGMENT_STOP_TIME, stopTime),
                7, "Stop Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Pulse Measurement Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Activates or deactivates the internal pulse modulator at VNA port
/// HIPAR enabled/<Pt>.
/// HIPAR delay/Defines the port-specific delay of the pulses generated by the
/// HIPAR delay/respective internal pulse generator.
/// HIPAR width/Defines the port-specific width of the pulses generated by the
/// HIPAR width/respective internal pulse generator.
ViStatus _VI_FUNC rsvna_ConfigurePulsePort(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   physicalPort,
                                           ViBoolean enabled,
                                           ViReal64  delay,
                                           ViReal64  width)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_DELAY, delay),
            5, "Delay");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_WIDTH, width),
            6, "Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures Pulse Port Inverted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR input/If External Pulse Control is used, this defines the polarity of the
/// HIPAR input/incoming generator pulses at PuMo In/Out <Pt>.
/// HIPAR output/If pulse generator <Pt> is output to PuMo In/Out <Pt> (see
/// HIPAR output/[SENSe<Ch>:]PULSe:PORT<Pt>:EXTernal:OUTPut[:STATe]), this defines the
/// HIPAR output/polarity of the outgoing pulses.
ViStatus _VI_FUNC rsvna_ConfigurePulsePortInverted(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   physicalPort,
                                                   ViBoolean input,
                                                   ViBoolean output)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_INPUT_INVERTED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_INPUT_INVERTED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_INPUT_INVERTED, input),
            4, "Input");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_OUTPUT_INVERTED, output),
            5, "Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If Internal Pulse Control is used, this enables/disables output of
/// HIFN  pulse generator <Pt> at PuMo In/Out <Pt>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/If Internal Pulse Control is used, this enables/disables output of
/// HIPAR enabled/pulse generator <Pt> at PuMo In/Out <Pt>.
ViStatus _VI_FUNC rsvna_ConfigurePulsePortExternalOutputEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   physicalPort,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_EXTERNAL_OUTPUT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_PULSE_PORT_EXTERNAL_OUTPUT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_PULSE_PORT_EXTERNAL_OUTPUT_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures SOurce Coherence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Turns source coherence mode (globally) on or off. Defined coherence
/// HIPAR enabled/for individual analyzer ports is enabled or disabled using
/// HIPAR enabled/SOURce<Ch>:CMODe:PORT<Pt>[:STATe].
/// HIPAR optimizedEnabled/Allows to select between optimized and raw phase control in source
/// HIPAR optimizedEnabled/coherence mode.
ViStatus _VI_FUNC rsvna_ConfigureSourceCoherence(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled,
                                                 ViBoolean optimizedEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_COHERENCE_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SOURCE_COHERENCE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SOURCE_COHERENCE_OPTIMIZED_ENABLED, optimizedEnabled),
            4, "Optimized Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Source Coherence Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Enables or disables defined phase coherence at physical port <Pt> for
/// HIPAR enabled/channel <Ch>.
/// HIPAR phase/Defines the phase for the coherent signal at port no. <PhyPt>.
/// HIPAR phaseSpan/Phase Coherent Port Phase Span.
ViStatus _VI_FUNC rsvna_ConfigureSourceCoherencePort(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   physicalPort,
                                                     ViBoolean enabled,
                                                     ViReal64  phase,
                                                     ViReal64  phaseSpan)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_COHERENCE_PORT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SOURCE_COHERENCE_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Physical Port");
    strcat (repCap, ",");
    strcat (repCap, repCap2);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SOURCE_COHERENCE_PORT_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_COHERENCE_PORT_PHASE, phase),
            5, "Phase");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SOURCE_COHERENCE_PORT_PHASE_SPAN, phaseSpan),
            6, "Phase Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the 'True Differential Mode' on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR enabled/Switches the 'True Differential Mode' on or off. This control is
/// HIPAR enabled/available only if a suitable balanced port configuration is active.
/// HIPAR compensateImbalanceEnabled/Selects the calculation method for S-parameters, ratios and derived
/// HIPAR compensateImbalanceEnabled/quantities during an amplitude imbalance or phase imbalance sweep.
ViStatus _VI_FUNC rsvna_ConfigureTDIFMode(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean enabled,
                                          ViBoolean compensateImbalanceEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDIF_MODE_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_COMPENSATE_IMBALANCE_ENABLED, compensateImbalanceEnabled),
            4, "Compensate Imbalance Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the TDIF Mode Phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR port/Selects balanced logical port <lport no> as the sweep port of a phase
/// HIPAR port/imbalance sweep in true differential mode. The balanced port must be
/// HIPAR port/defined using SOURce<Ch>:LPORt<LogPt> before it can be selected.
/// HIPAR start/Defines the start phase difference of a phase imbalance sweep in 'True
/// HIPAR start/Differential Mode'.
/// HIPAR stop/Defines the stop phase difference of a phase imbalance sweep in 'True
/// HIPAR stop/Differential Mode'.
ViStatus _VI_FUNC rsvna_ConfigureTDIFModeImbalancePhase(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   port,
                                                        ViReal64  start,
                                                        ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDIF_MODE_IMBALANCE_PHASE_PORT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_PHASE_PORT, port),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_PHASE_START, start),
            4, "Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_PHASE_STOP, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the TDIF Mode Amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR port/Selects balanced logical port <lport no> as the sweep port of an
/// HIPAR port/amplitude imbalance sweep in true differential mode. The balanced port
/// HIPAR port/must be defined using SOURce<Ch>:LPORt<LogPt> before it can be
/// HIPAR port/selected.
/// HIPAR start/Defines the start ower difference of an amplitude imbalance sweep in
/// HIPAR start/true differential mode.
/// HIPAR stop/Defines the stop power difference of an amplitude imbalance sweep in
/// HIPAR stop/true differential mode.
ViStatus _VI_FUNC rsvna_ConfigureTDIFModeImbalanceAmplitude(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   port,
                                                            ViReal64  start,
                                                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDIF_MODE_IMBALANCE_AMPLITUDE_PORT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_AMPLITUDE_PORT, port),
            3, "Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_AMPLITUDE_START, start),
            4, "Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDIF_MODE_IMBALANCE_AMPLITUDE_STOP, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the compression value for a gain compression measurement in
/// HIFN  channel <Ch>.
/// HIFN
/// HIFN     The gain compression measurement searches for the <CompVal>dB
/// HIFN  compression points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR gainCompression/Defines the compression value for a gain compression measurement in
/// HIPAR gainCompression/channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionValue(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64  gainCompression)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_VALUE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_VALUE, gainCompression),
            3, "Gain Compression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of points for all power sweeps of a gain
/// HIFN  compression measurement in channel <Ch>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR powerPoints/Defines the number of points for all power sweeps of a gain
/// HIPAR powerPoints/compression measurement in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionPowerPoints(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   powerPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_POWER_POINTS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_POWER_POINTS, powerPoints),
            3, "Power Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures Power Point Position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR start/Defines the start and stop power for all power sweeps of a gain
/// HIPAR start/compression measurement in channel <Ch>.
/// HIPAR stop/Defines the start and stop power for all power sweeps of a gain
/// HIPAR stop/compression measurement in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionPowerPointPosition(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_START_POWER_POINT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_START_POWER_POINT, start),
            3, "Start");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_STOP_POWER_POINT, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Gain Compression Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR receivingPort/Defines the driving and receiving port for a gain compression
/// HIPAR receivingPort/measurement in channel <Ch>.
/// HIPAR drivingPort/Defines the driving and receiving port for a gain compression
/// HIPAR drivingPort/measurement in channel <Ch>.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionPort(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   receivingPort,
                                                     ViInt32   drivingPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_RECEIVING_PORT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_RECEIVING_PORT, receivingPort),
            3, "Receiving Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_DRIVING_PORT, drivingPort),
            4, "Driving Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For two-dimensional gain compression measurements, this command
/// HIFN  activates or deactivates skipping the linear part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR skip/For two-dimensional gain compression measurements, this command
/// HIPAR skip/activates or deactivates skipping the linear part.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionSkip(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean skip)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_SKIP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_SKIP, skip),
            3, "Skip");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For two-dimensional gain compression measurements, this command allows
/// HIFN  you to define the power sweep range to be skipped, if skipping the
/// HIFN  linear part is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR skipOffset/For two-dimensional gain compression measurements, this command allows
/// HIPAR skipOffset/you to define the power sweep range to be skipped, if skipping the
/// HIPAR skipOffset/linear part is active.
ViStatus _VI_FUNC rsvna_ConfigureGainCompressionSkipOffset(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViReal64  skipOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_GAIN_COMPRESSION_SKIP_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_GAIN_COMPRESSION_SKIP_OFFSET, skipOffset),
            3, "Skip Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the detector for the spectrum measurement, i.e. how the
/// HIFN  measurement result for a particular sweep point is derived from the
/// HIFN  related sample values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR spectrumDetector/Defines the detector for the spectrum measurement, i.e. how the
/// HIPAR spectrumDetector/measurement result for a particular sweep point is derived from the
/// HIPAR spectrumDetector/related sample values.
ViStatus _VI_FUNC rsvna_ConfigureSpectrumDetector(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   spectrumDetector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SPECTRUM_DETECTOR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SPECTRUM_DETECTOR, spectrumDetector),
            3, "Spectrum Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the image rejection strategy: the higher the number of data
/// HIFN  acquisitions, the better the image rejection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR imageRejection/Selects the image rejection strategy: the higher the number of data
/// HIPAR imageRejection/acquisitions, the better the image rejection.
ViStatus _VI_FUNC rsvna_ConfigureSpectrumImageRejection(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   imageRejection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE]= "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SPECTRUM_IMAGE_REJECTION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SPECTRUM_IMAGE_REJECTION, imageRejection),
            3, "Image Rejection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables/enables zero padding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR detectorZeroPadding/Disables/enables zero padding.
ViStatus _VI_FUNC rsvna_ConfigureSpectrumDetectorZeroPadding(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViBoolean detectorZeroPadding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SPECTRUM_DETECTOR_ZERO_PADDING,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SPECTRUM_DETECTOR_ZERO_PADDING, detectorZeroPadding),
            3, "Detector Zero Padding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new diagram area, identified by its area
/// HIFN  number <Wnd>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
ViStatus _VI_FUNC rsvna_DiagramAreaAdd(ViSession instrSession,
                                       ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    snprintf(repCap_cmd, RS_REPCAP_BUF_SIZE, "%ld", window);

    checkErr(RsCore_AddRepCap(instrSession, "Window", repCap, repCap_cmd));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_DIAGRAM_STATE, VI_TRUE),
            2, "Window");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a diagram area including all traces displayed in
/// HIFN  the diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
ViStatus _VI_FUNC rsvna_DiagramAreaDelete(ViSession instrSession,
                                          ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SynchronizeDynamicRepeatedCapability(instrSession, RSVNA_VAL_WINDOW));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    checkErr(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TRACE_DIAGRAM_STATE, VI_FALSE));

    checkErr(RsCore_RemoveRepCap(instrSession, "Window", repCap));

    checkErr(rsvna_SynchronizeDynamicRepeatedCapability(instrSession, RSVNA_VAL_WINDOW));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function maximizes all diagram areas in the active setup or
/// HIFN  restores the previous display configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR diagramArea/Maximizes all diagram areas in the active setup or restores the
/// HIPAR diagramArea/previous display configuration.
ViStatus _VI_FUNC rsvna_DiagramAreaMaximize(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   diagramArea)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_MAXIMIZE_ALL,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap,
                    RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_MAXIMIZE_ALL,
                    diagramArea?VI_TRUE:VI_FALSE
                ), 3, "Diagram Area");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a title for diagram area <Wnd>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR title/Displays or hides the title for area number <Wnd>.
/// HIPAR titleString/String variable for the title. The length of the title is practically
/// HIPAR titleString/unlimited but should be kept short enough to be displayed in the
/// HIPAR titleString/diagrams.
ViStatus _VI_FUNC rsvna_DiagramAreaTitle(ViSession instrSession,
                                         ViInt32   window,
                                         ViBoolean title,
                                         ViString  titleString)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_SHOW_TITLE,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_SHOW_TITLE, title),
            3, "Title");

    if (title)
    {
        viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_TITLE, titleString),
                4, "Title String");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a name for diagram area <Wnd>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR areaName/String variable for the name.
ViStatus _VI_FUNC rsvna_DiagramAreaName(ViSession instrSession,
                                        ViInt32   window,
                                        ViString  areaName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_NAME,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_NAME, areaName),
            3, "Area Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the numbers and names of all diagram areas in
/// HIFN  the current recall set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reserved/Reserved for future use.
/// HIPAR catalog/Returns string with comma-separated list of trace numbers and names.
/// HIPAR catalog/If all traces have been deleted the response is an empty string ("").
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_DiagramAreaCatalog(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViChar    catalog[],
                                           ViInt32   bufferSize)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_DIAGRAM_AREA_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the numbers and names of all traces in diagram
/// HIFN  area no. <Wnd>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Number of the diagram area.
/// HIPAR catalog/Returns string with comma-separated list of trace numbers and names.
/// HIPAR catalog/If all traces have been deleted the response is an empty string ("").
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/'Catalog' parameter.
ViStatus _VI_FUNC rsvna_TraceDiagramAreaCatalog(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   bufferSize,
                                                ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISPLAY_TRACE_DIAGRAM_AREA_CATALOG,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISPLAY_TRACE_DIAGRAM_AREA_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function magnifies a rectangular portion of the diagram (zoom
/// HIFN  window) to fill the entire diagram area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Number of the diagram area.
/// HIPAR window_Trace/Trace number used to distinguish the traces of the same diagram area
/// HIPAR window_Trace/<Wnd>.
/// HIPAR zoomState/This control displays or hides trace(s).
/// HIPAR topEdge/Specifies the upper edge of the zoom window. In contrast to manual
/// HIPAR topEdge/control, all or part of the zoom window may be outside the original
/// HIPAR topEdge/diagram.
/// HIPAR bottomEdge/Specifies the lower edge of the zoom window. In contrast to manual
/// HIPAR bottomEdge/control, all or part of the zoom window may be outside the original
/// HIPAR bottomEdge/diagram.
/// HIPAR leftEdge/Specifies the start value of the zoom window (left border). In
/// HIPAR leftEdge/contrast to manual control, all or part of the zoom window may be
/// HIPAR leftEdge/outside the original diagram.
/// HIPAR rightEdge/Specifies the stop value of the zoom window (right border). In
/// HIPAR rightEdge/contrast to manual control, all or part of the zoom window may be
/// HIPAR rightEdge/outside the original diagram.
ViStatus _VI_FUNC rsvna_ConfigureTraceZoom(ViSession instrSession,
                                           ViInt32   window,
                                           ViInt32   window_Trace,
                                           ViBoolean zoomState,
                                           ViReal64  topEdge,
                                           ViReal64  bottomEdge,
                                           ViReal64  leftEdge,
                                           ViReal64  rightEdge)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, topEdge, -89.99, 232.0), 4, "Top Edge");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, bottomEdge, -390.0, 9.99), 5, "Bottom Edge");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, topEdge, bottomEdge, INT_MAX),
            5, "Bottom Edge");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, rightEdge, leftEdge, INT_MAX),
            6, "Left Edge");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:ZOOM:TOP %.12lG", window, window_Trace, topEdge);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:ZOOM:BOTT %.12lG", window, window_Trace, bottomEdge);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:ZOOM:STAR %.12lG", window, window_Trace, leftEdge);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:ZOOM:STOP %.12lG", window, window_Trace, rightEdge);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC%ld:ZOOM:STAT %s", window, window_Trace, offOnArr[zoomState]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the zoom overview. The upper part of the diagram
/// HIFN  shows the original diagram for comparison. You can move the zoomed part
/// HIFN  of the trace by moving the marked zoom area in the overview.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Number of the diagram area.
/// HIPAR zoomOverviewState/This control activates the zoom overview for the selected window.
ViStatus _VI_FUNC rsvna_ConfigureZoomOverview(ViSession instrSession,
                                              ViInt32   window,
                                              ViBoolean zoomOverviewState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_ZOOM_OVERVIEW,
            "Window", window -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_ZOOM_OVERVIEW, zoomOverviewState),
            3, "Zoom Overview State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function arranges the display windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR layoutMode/Arranges the display windows.
ViStatus _VI_FUNC rsvna_ConfigureSplitType(ViSession instrSession,
                                           ViInt32   layoutMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_DISPLAY_SPLIT_TYPE, layoutMode),
            2, "Layout Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of rows and columns if split type
/// HIFN  is set to "Rows + Columns".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rows/Defines the number of rows.
/// HIPAR columns/Defines the number of columns.
ViStatus _VI_FUNC rsvna_ConfigureSplitGrid(ViSession instrSession,
                                           ViInt32   rows,
                                           ViInt32   columns)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rows, 1, 16),
            2, "Rows");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, columns, 1, 16),
            3, "Columns");

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld,%ld", rows, columns);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL,
        RSVNA_ATTR_HIDDEN_DISPLAY_GRID_ROWS_AND_COLUMNS,
        RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a horizontal or vertical display layout and
/// HIFN  provides it with an identifier (<LayoutId>) and selects it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR layoutID/Current number, used by other DISPlay:LAYout... commands to reference
/// HIPAR layoutID/the created layout.
/// HIPAR layoutFormatMode/Selects the horizontal or vertical layout. Layouts are defined row by
/// HIPAR layoutFormatMode/row (horizontal layouts) or column by column (vertical layouts).
/// HIPAR layoutData/Layouts are defined row by row (horizontal layouts) or column by
/// HIPAR layoutData/column (vertical layouts).
ViStatus _VI_FUNC rsvna_ConfigureLayoutSimplified(ViSession instrSession,
                                                  ViInt32   layoutID,
                                                  ViInt32   layoutFormatMode,
                                                  ViString  layoutData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString layoutFormatModeArr[] = {"Horizontal", "Vertical"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, layoutFormatMode, RSVNA_VAL_LAYOUT_FORMAT_HORIZONTAL, RSVNA_VAL_LAYOUT_FORMAT_VERTICAL),
            2, "Layout Format Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:LAY:DEF %ld,%s,'%s'", layoutID, layoutFormatModeArr[layoutFormatMode], layoutData);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SELECT_DISPLAY_LAYOUT, layoutID),
            2, "Layout ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates and displays a horizontal or vertical display
/// HIFN  layout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR layoutData/Defines the number of diagrams and their position (executable format).
ViStatus _VI_FUNC rsvna_ConfigureLayout(ViSession instrSession,
                                        ViString  layoutData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:LAY:EXEC '%s'", layoutData);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a nested layout, inserting a sub-layout into one
/// HIFN  of the diagrams of a main layout and selects it. Main layout and
/// HIFN  sub-layout must be defined previously - for example using function
/// HIFN  rsvna_ConfigureLayoutSimplified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mainLayoutID/Current number of main layout.
/// HIPAR diagramNumber/Diagram number in the main layout.
/// HIPAR subLayoutID/Current number of sub-layout.
ViStatus _VI_FUNC rsvna_JoinLayouts(ViSession instrSession,
                                    ViInt32   mainLayoutID,
                                    ViInt32   diagramNumber,
                                    ViInt32   subLayoutID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:LAY:JOIN %ld,%ld,%ld", mainLayoutID, diagramNumber, subLayoutID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SELECT_DISPLAY_LAYOUT, mainLayoutID),
            2, "Main Layout ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a separate diagram for each trace in the active recall set and
/// HIFN  automatically arranges those diagrams in the diagram area. Existing
/// HIFN  diagrams are deleted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DisplaySplitAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_SPLIT_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the color scheme for all diagram areas in the
/// HIFN  active setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorScheme/Selects the color scheme for all diagram areas in the active setup.
ViStatus _VI_FUNC rsvna_ConfigureColorScheme(ViSession instrSession,
                                             ViInt32   colorScheme)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_SELECT_COLOR_SCHEME, colorScheme),
            2, "Color Scheme");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the color of all display elements based on the
/// HIFN  Red/Green/Blue color model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR element/Selects the display elements.
/// HIPAR red/Sets the red color.
/// HIPAR green/Sets the green color.
/// HIPAR blue/Sets the blue color.
/// HIPAR traceStyle/Selects the trace style.
/// HIPAR traceWidth/Selects the trace width.
ViStatus _VI_FUNC rsvna_ConfigureRGBColor(ViSession instrSession,
                                          ViInt32   element,
                                          ViReal64  red,
                                          ViReal64  green,
                                          ViReal64  blue,
                                          ViInt32   traceStyle,
                                          ViInt32   traceWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, element, RSVNA_VAL_ELEMENT_BACKGROUND, RSVNA_VAL_ELEMENT_TRACE16),
            2, "Element");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, red, 0.0, 1.0), 3, "Red");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, green, 0.0, 1.0), 4, "Green");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, blue, 0.0, 1.0), 5, "Blue");

    if (element >= RSVNA_VAL_ELEMENT_TRACE1)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceStyle, RSVNA_VAL_TRACE_STYLE_SOLID, RSVNA_VAL_TRACE_STYLE_DDDOTTED),
                6, "Trace Style");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceWidth, 1, 20),
                7, "Trace Width");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP%ld:RGB %.12lG,%.12lG,%.12lG,%s,%ld",
                element, red, green, blue, traceStyleArr[traceStyle], traceWidth);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP%ld:RGB %.12lG,%.12lG,%.12lG", element, red, green, blue);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the trace color schemes in different diagram
/// HIFN  areas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceName/Trace name.
/// HIPAR red/Sets the red color.
/// HIPAR green/Sets the green color.
/// HIPAR blue/Sets the blue color.
/// HIPAR traceStyle/Selects the trace style.
/// HIPAR traceWidth/Selects the trace width.
ViStatus _VI_FUNC rsvna_ConfigureTraceRGBColor(ViSession instrSession,
                                               ViString  traceName,
                                               ViReal64  red,
                                               ViReal64  green,
                                               ViReal64  blue,
                                               ViInt32   traceStyle,
                                               ViInt32   traceWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, red, 0.0, 1.0), 3, "Red");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, green, 0.0, 1.0), 4, "Green");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, blue, 0.0, 1.0), 5, "Blue");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceStyle, RSVNA_VAL_TRACE_STYLE_SOLID, RSVNA_VAL_TRACE_STYLE_DDDOTTED),
            6, "Trace Style");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceWidth, 1, 20),
            7, "Trace Width");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP:TRAC:RGB '%s',%.12lG,%.12lG,%.12lG,%s,%ld",
            traceName, red, green, blue, traceStyleArr[traceStyle], traceWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores a color scheme to a specified VNA color scheme
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be created. If no path is specified the analyzer uses the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?. The default
/// HIPAR fileName/extension (manual control) for color scheme files is *.ColorScheme,
/// HIPAR fileName/although other extensions are allowed.
ViStatus _VI_FUNC rsvna_SaveColorScheme(ViSession instrSession,
                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_COLOR_SCHEME_SAVE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores a color scheme to a specified VNA color scheme
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be created. If no path is specified the analyzer uses the current
/// HIPAR fileName/directory, to be queried with MMEMory:CDIRectory?. The default
/// HIPAR fileName/extension (manual control) for color scheme files is *.ColorScheme,
/// HIPAR fileName/although other extensions are allowed.
ViStatus _VI_FUNC rsvna_SaveColorSchemeHCOPY(ViSession instrSession,
                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_COLOR_SCHEME_HCOPY_SAVE, fileName),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function loads a color scheme from a specified VNA color scheme
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be loaded. The default extension (manual control) for color scheme
/// HIPAR fileName/files is *.ColorScheme, although other extensions are allowed.
ViStatus _VI_FUNC rsvna_RecallColorSchemeHCOPY(ViSession instrSession,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_COLOR_SCHEME_HCOPY_LOAD, fileName),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays all traces of the active recall set in a single diagram,
/// HIFN  which is maximized to occupy the whole screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsvna_DisplayOverlayAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_OVERLAY_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the display of X-axis grid labels in cartesian
/// HIFN  diagrams with linear scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xAxisGridLabels/Enables or disables the display of X-axis grid labels in cartesian
/// HIPAR xAxisGridLabels/diagrams with linear scale.
ViStatus _VI_FUNC rsvna_ConfigureDisplayXAxisGridLabels(ViSession instrSession,
                                                        ViBoolean xAxisGridLabels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_DISPLAY_X_AXIS_GRID_LABELS, xAxisGridLabels),
            2, "X Axis Grid Labels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads a color scheme from a specified VNA color scheme
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/This control specifies the name and directory of the cal kit file to
/// HIPAR fileName/be loaded. The default extension (manual control) for color scheme
/// HIPAR fileName/files is *.ColorScheme, although other extensions are allowed.
ViStatus _VI_FUNC rsvna_RecallColorScheme(ViSession instrSession,
                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_COLOR_SCHEME_LOAD, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the "length" of the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR maxDUTDelay/Sets the expected maximum time delay through the DUT.
/// HIPAR maxElectricalLength/Sets the expected maximum electrical length of the DUT.
/// HIPAR maxMechanicalLength/Sets the expected maximum mechanical length of the DUT.
ViStatus _VI_FUNC rsvna_ConfigureTDRStimulusLength(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  maxDUTDelay,
                                                   ViReal64  maxElectricalLength,
                                                   ViReal64  maxMechanicalLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_STIMULUS_MAX_DUT_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_MAX_DUT_DELAY, maxDUTDelay),
            3, "Max DUT Delay");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_MAX_ELECTRICAL_LENGTH, maxElectricalLength),
            4, "Max Electrical Length");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_MAX_MECHANICAL_LENGTH, maxMechanicalLength),
            5, "Max Mechanical Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the (relative) permittivity of the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR permitivity/Sets the (relative) permittivity of the DUT.
ViStatus _VI_FUNC rsvna_ConfigureTDRStimulusPermitivity(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViReal64  permitivity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_STIMULUS_PERMITIVITY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_PERMITIVITY, permitivity),
            3, "permitivity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the velocity factor of the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR velocityFactor/Sets the velocity factor of the DUT.
ViStatus _VI_FUNC rsvna_ConfigureTDRStimulusVelocityFactor(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViReal64  velocityFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_STIMULUS_VELOCITY_FACTOR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_VELOCITY_FACTOR, velocityFactor),
            3, "Velocity Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the minimum rise time the user wishes to measure on the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR riseTime/Sets the minimum rise time the user wishes to measure on the DUT.
/// HIPAR definition/Selects between rise time definitions 10%-90% (default) and 20%-80%.
ViStatus _VI_FUNC rsvna_ConfigureTDRStimulusRiseTime(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViReal64  riseTime,
                                                     ViInt32   definition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_STIMULUS_RISE_TIME,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_RISE_TIME, riseTime),
            3, "Rise Time");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_STIMULUS_RISE_TIME_DEFINITION, definition),
            4, "Definition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the active trace in the selected channel shall be
/// HIFN  represented as an eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Defines whether the active trace in the selected channel shall be
/// HIPAR enabled/represented as an eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeEnabled(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the simulated bit stream.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR bitStream/Defines the type of bit stream to be simulated for the related eye
/// HIPAR bitStream/diagram.
/// HIPAR symbolRate/Defines the data rate of the bit stream generator for the related eye
/// HIPAR symbolRate/diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeBitStream(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   bitStream,
                                                 ViInt64   symbolRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_BIT_STREAM,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_BIT_STREAM, bitStream),
            3, "Bit Stream");

    viCheckParm(rsvna_SetAttributeViInt64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_SYMBOL_RATE, symbolRate),
                4, "Symbol Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the length of the pseudo-random binary sequence to be
/// HIFN  simulated for the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR length/Defines the length of the pseudo-random binary sequence to be
/// HIPAR length/simulated for the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeBitStreamPRBSLength(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViInt32   length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_BIT_STREAM_PRBS_LENGTH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_BIT_STREAM_PRBS_LENGTH, length),
            3, "length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the length of a user-defined bit stream to be simulated for
/// HIFN  the related eye diagram. The user-defined bit pattern (the default 10
/// HIFN  or loaded from file) is repeated until the specified length is reached.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR length/Defines the length of a user-defined bit stream to be simulated for
/// HIPAR length/the related eye diagram. The user-defined bit pattern (the default 10
/// HIPAR length/or loaded from file) is repeated until the specified length is reached.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeBitStreamBitLength(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt64   length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_BIT_STREAM_BIT_LENGTH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_BIT_STREAM_BIT_LENGTH, length),
                3, "length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a user-defined bit pattern for the related eye diagram from a
/// HIFN  7-bit ASCII file. Such a bit stream file must be a 7-bit ASCII
/// HIFN  compatible text file; the Byte Order Mark (BOM) that is common with UTF
/// HIFN  encodings is not allowed. Only the ASCII codes 0x30
/// HIFN     (0) and 0x31 (1) will be interpreted, all other codes including
/// HIFN  whitespace and line endings are ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/String parameter containing the path and file name of the bit pattern
/// HIPAR fileName/file. If the path is omitted, the current directory is used
ViStatus _VI_FUNC rsvna_TDREyeLoadBitStream(ViSession instrSession,
                                            ViString  traceName,
                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", traceName, fileName);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_BIT_STREAM_LOAD, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures simulated levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR modulation/Defines the modulation type of the signal to be simulated for the
/// HIPAR modulation/related eye diagram.
/// HIPAR highLevel/Defines the highest (nominal) voltage level of the simulated digital
/// HIPAR highLevel/signal generating the related eye diagram.
/// HIPAR lowLevel/Defines the lowest (nominal) voltage level of the simulated digital
/// HIPAR lowLevel/signal generating the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeSimulatedLevels(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   modulation,
                                                       ViReal64  highLevel,
                                                       ViReal64  lowLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MODULATION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MODULATION, modulation),
            3, "Modulation");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_HIGH_LEVEL, highLevel),
            4, "High Level");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_LOW_LEVEL, lowLevel),
            5, "Low Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the simulated rise time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lowPass/Enables/disables a single pole low pass filter in the binary signal
/// HIPAR lowPass/generator simulation of the related eye diagram measurement.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR riseTime/Sets the rise time of the low pass in the binary signal generator
/// HIPAR riseTime/simulation of the related eye diagram.
/// HIPAR definition/Selects the appropriate rise time definition for the low pass in the
/// HIPAR definition/binary signal generator simulation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeSimulatedRiseTime(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean lowPass,
                                                         ViReal64  riseTime,
                                                         ViInt32   definition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_LOW_PASS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_LOW_PASS, lowPass),
            3, "Low Pass");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_RISE_TIME, riseTime),
            4, "Rise Time");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_RISE_TIME_DEFINITION, definition),
            5, "Definition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows to shorten the calculation chain of the related eye diagram
/// HIFN  without deactivating the building blocks at the tail end.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR view/Allows to shorten the calculation chain of the related eye diagram
/// HIPAR view/without deactivating the building blocks at the tail end.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeView(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   view)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_VIEW,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_VIEW, view),
            3, "View");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR encoder/Enables or disables 8b/10b encoding in the bit stream simulation of
/// HIPAR encoder/the related eye diagram.
/// HIPAR scrambler/Enables/disables scrambling of the (encoded) bit stream.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeGenerator(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean encoder,
                                                 ViBoolean scrambler)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_GENERATOR_ENCODER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_GENERATOR_ENCODER, encoder),
            3, "Encoder");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_GENERATOR_SCRAMBLER, scrambler),
            4, "Scrambler");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates pre-emphasis in the calculation chain of the
/// HIFN  related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates/deactivates pre-emphasis in the calculation chain of the
/// HIPAR enabled/related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeEmphasisEnabled(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_EMPHASIS_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_EMPHASIS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the emphasis cursor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR preCursor/Sets the weight of the pre-cursor tap for the pre-emphasis FIR filter
/// HIPAR preCursor/in the calculation chain of the related eye diagram.
/// HIPAR postCursor1/Sets the weights of the post-cursor 1 for the pre-emphasis FIR filter
/// HIPAR postCursor1/in the calculation chain of the related eye diagram.
/// HIPAR postCursor2/Sets the weights of the post-cursor 2 for the pre-emphasis FIR filter
/// HIPAR postCursor2/in the calculation chain of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeEmphasisCursor(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64  preCursor,
                                                      ViReal64  postCursor1,
                                                      ViReal64  postCursor2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapCursor[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_EMPHASIS_PRE_CURSOR,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_EMPHASIS_PRE_CURSOR, preCursor),
            3, "Pre Cursor");

    snprintf(repCapCursor, RS_REPCAP_BUF_SIZE, "%s,Cursor1", repCap);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCapCursor, RSVNA_ATTR_TDR_EYE_EMPHASIS_POST_CURSOR, postCursor1),
            4, "Post Cursor1");

    snprintf(repCapCursor, RS_REPCAP_BUF_SIZE, "%s,Cursor2", repCap);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCapCursor, RSVNA_ATTR_TDR_EYE_EMPHASIS_POST_CURSOR, postCursor2),
            5, "Post Cursor2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Gaussian noise in the generator simulation of the related
/// HIFN  eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables Gaussian noise in the generator simulation of the
/// HIPAR enabled/related eye diagram.
/// HIPAR rms/Defines the root mean square (RMS) noise level in the generator
/// HIPAR rms/simulation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeNoise(ViSession instrSession,
                                             ViInt32   channel,
                                             ViBoolean enabled,
                                             ViReal64  rms)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_NOISE_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_NOISE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_NOISE_RMS, rms),
            4, "RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows to (temporarily) switch between the measured DUT and an ideal
/// HIFN  one (with flat frequency response) in the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dutMode/Allows to (temporarily) switch between the measured DUT and an ideal
/// HIPAR dutMode/one (with flat frequency response) in the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeDUTMode(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   dutMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_DUT_MODE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_DUT_MODE, dutMode),
            3, "DUT Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the CTLE at the receiver simulation of the related
/// HIFN  eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables the CTLE at the receiver simulation of the related
/// HIPAR enabled/eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeEqualizationEnabled(ViSession instrSession,
                                                           ViInt32   channel,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_EQUALIZATION_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_EQUALIZATION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the CTLE equalizer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR dc/Specifies the DC gain of the CTLE (a two-pole filter with single zero)
/// HIPAR dc/at the receiver simulation of the related eye diagram.
/// HIPAR zero/Specifies the zero of the equalizer (a two-pole filter with single
/// HIPAR zero/zero) used at the receiver simulation of the related eye diagram.
/// HIPAR pole1/Specifies the pole 1 of the CTLE used at the receiver simulation of
/// HIPAR pole1/the related eye diagram.
/// HIPAR pole2/Specifies the pole 2 of the CTLE used at the receiver simulation of
/// HIPAR pole2/the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeEqualizationCTLEEqualizer(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  dc,
                            ViReal64  zero,
                            ViReal64  pole1,
                            ViReal64  pole2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPole[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_EQUALIZATION_CTLE_DC,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_EQUALIZATION_CTLE_DC, dc),
            3, "DC");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_EQUALIZATION_CTLE_ZERO, zero),
            4, "Zero");

    snprintf(repCapPole, RS_REPCAP_BUF_SIZE, "%s,Pole1", repCap);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCapPole, RSVNA_ATTR_TDR_EYE_EQUALIZATION_CTLE_POLE, pole1),
            4, "Pole1");

    snprintf(repCapPole, RS_REPCAP_BUF_SIZE, "%s,Pole2", repCap);

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCapPole, RSVNA_ATTR_TDR_EYE_EQUALIZATION_CTLE_POLE, pole2),
            5, "Pole2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the transfer rate of the bit stream generator for the related
/// HIFN  eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR units/Defines the transfer rate of the bit stream generator for the related
/// HIPAR units/eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeTransferRateUnits(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   units)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_TRANSFER_RATE_UNITS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_TRANSFER_RATE_UNITS, units),
            3, "Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows/hides the Info Window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Shows/hides the Info Window.
ViStatus _VI_FUNC rsvna_ConfigureDisplayInfoWindowEnabled(ViSession instrSession,
                                                          ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_DISPLAY_INFO_WINDOW_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the bandfilter search results to be displayed in the Info
/// HIFN  Window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related trace.
/// HIPAR bandfilterItem/Bandfilter search result.
/// HIPAR displayed/Result <BandfilterItem> of trace <TraceName> is displayed.
ViStatus _VI_FUNC rsvna_ConfigureDisplayInfoWindowBandfilterSearchResults(
                            ViSession instrSession,
                            ViString  traceName,
                            ViInt32   bandfilterItem,
                            ViBoolean displayed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  bandfilterItemString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDisplayInfoWindowBandfilter,
                    bandfilterItem, &bandfilterItemString), 3, "Bandfilter Item");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s',%s,%s", traceName, bandfilterItemString, offOnArr[displayed]);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_INFO_WINDOW_BANDFILTER_SEARCH_RESULTS, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the jitter functionality in the generator simulation of the
/// HIFN  related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates the jitter functionality in the generator simulation of the
/// HIPAR enabled/related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeJitterEnabled(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_JITTER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures random eye jitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables random jitter insertion in the generator simulation
/// HIPAR enabled/of the related eye diagram.
/// HIPAR standardDeviation/Defines the standard deviation of the random jitter in the generator
/// HIPAR standardDeviation/simulation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeJitterRandom(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean enabled,
                                                    ViReal64  standardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_JITTER_RANDOM_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_RANDOM_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_RANDOM_STANDARD_DEVIATION, standardDeviation),
            4, "Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures periodic jitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables periodic jitter insertion in the generator simulation
/// HIPAR enabled/of the related eye diagram.
/// HIPAR magnitude/Defines the magnitude of the periodic jitter in the generator
/// HIPAR magnitude/simulation of the related eye diagram.
/// HIPAR frequency/Defines the frequency of the periodic jitter in the generator
/// HIPAR frequency/simulation of the related eye diagram.
/// HIPAR phase/Defines the phase of the periodic jitter in the generator simulation
/// HIPAR phase/of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeJitterPeriodic(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled,
                                                      ViReal64  magnitude,
                                                      ViReal64  frequency,
                                                      ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_JITTER_PERIODIC_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_PERIODIC_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_PERIODIC_MAGNITUDE, magnitude),
            4, "Magnitude");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_PERIODIC_FREQUENCY, frequency),
            5, "Frequency");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_PERIODIC_PHASE, phase),
            6, "Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures dirac jitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables Dirac jitter insertion in the generator simulation of
/// HIPAR enabled/the related eye diagram.
/// HIPAR delta/Defines the magnitude of the Dirac jitter in the generator simulation
/// HIPAR delta/of the related eye diagram.
/// HIPAR probability/Defines the probability of the Dirac jitter in the generator
/// HIPAR probability/simulation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeJitterDirac(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled,
                                                   ViReal64  delta,
                                                   ViReal64  probability)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_JITTER_DIRAC_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_DIRAC_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_DIRAC_DELTA, delta),
            4, "Delta");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_DIRAC_PROBABILITY, probability),
            5, "Probability");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables user-defined jitter insertion in the generator
/// HIFN  simulation of the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables user-defined jitter insertion in the generator
/// HIPAR enabled/simulation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeJitterUserEnabled(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_JITTER_USER_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_JITTER_USER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads user-defined jitter from a 7-bit ASCII file into the generator
/// HIFN  simulation of an eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/String parameter containing the path and file name of the jitter file.
/// HIPAR fileName/If the path is omitted, the current directory is used.
ViStatus _VI_FUNC rsvna_TDREyeLoadUserJitter(ViSession instrSession,
                                             ViString  traceName,
                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", traceName, fileName);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_JITTER_LOAD_USER, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the visibility of the configured eye mask in the related eye
/// HIFN  diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR show/Defines the visibility of the configured eye mask in the related eye
/// HIPAR show/diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskShow(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean show)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_SHOW,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_SHOW, show),
            3, "Show");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the eye mask test shall be run after every
/// HIFN  recalculation of the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Defines whether the eye mask test shall be run after every
/// HIPAR enabled/recalculation of the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskEnabled(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically creates an eye mask, based on the current eye
/// HIFN  measurement settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_TDREyeMaskAuto(ViSession instrSession,
                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_AUTO,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the R&S ZNB/ZNBT should make an audible beep on mask
/// HIFN  failures in the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR beep/Defines whether the R&S ZNB/ZNBT should make an audible beep on mask
/// HIPAR beep/failures in the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskFailBeep(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean beep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_FAIL_BEEP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_FAIL_BEEP, beep),
            3, "Beep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns 'Pass' or 'Fail' to indicate the result of the limit check in
/// HIFN  the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR status/Returns 'Pass' or 'Fail' to indicate the result of the limit check in
/// HIPAR status/the related eye diagram.
ViStatus _VI_FUNC rsvna_QueryTDREyeMaskFailStatus(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_FAIL_STATUS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_FAIL_STATUS, status),
            3, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the mask polygon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates/deactivates the polygon area in the eye mask of the related
/// HIPAR enabled/eye diagram.
/// HIPAR shape/Defines the shape of the center polygon (octagon, hexagon or
/// HIPAR shape/rectangle) in the eye mask of the related eye diagram.
/// HIPAR widthMain/Defines the main width of the center polygon in the mask of the
/// HIPAR widthMain/related eye diagram.
/// HIPAR heightMain/Defines the main height of the center polygon in the mask of the
/// HIPAR heightMain/related eye diagram.
/// HIPAR widthMinor/Defines the minor width of the center polygon in the mask of the
/// HIPAR widthMinor/related eye diagram. Not applicable for Rectangle shape masks.
/// HIPAR heightMinor/Defines the minor height of the center polygon in the mask of the
/// HIPAR heightMinor/related eye diagram. Not applicable for Rectangle  or Hexagon
/// HIPAR heightMinor/shape masks.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskPolygon(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean enabled,
                                                   ViInt32   shape,
                                                   ViReal64  widthMain,
                                                   ViReal64  heightMain,
                                                   ViReal64  widthMinor,
                                                   ViReal64  heightMinor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_SHAPE, shape),
            4, "Shape");

    if (enabled)
    {
        if (shape == RSVNA_VAL_POLYGON_SHAPE_RECTANGLE)
        {
            viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_WIDTH_MAIN, widthMain),
                    5, "Width Main");
        }
        else
        {
            snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG", widthMain, widthMinor);

            checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_WIDTH, parameter));
        }

        if ((shape == RSVNA_VAL_POLYGON_SHAPE_RECTANGLE) || (shape == RSVNA_VAL_POLYGON_SHAPE_HEXAGON))
        {
            viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_HEIGHT_MAIN, heightMain),
                    6, "Height Main");
        }
        else
        {
            snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG", heightMain, heightMinor);

            checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_POLYGON_HEIGHT, parameter));
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures top mask area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates/deactivates the respective area in the eye mask of the
/// HIPAR enabled/related eye diagram.
/// HIPAR width/Defines the width of the top rectangle in the eye mask of the related
/// HIPAR width/eye diagram.
/// HIPAR height/Defines the offset of the top rectangle in the mask of the related eye
/// HIPAR height/diagram. The offset is specified relative to the vertical center of the
/// HIPAR height/eye mask.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskTop(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean enabled,
                                               ViReal64  width,
                                               ViReal64  height)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_TOP_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_TOP_ENABLED, enabled),
            3, "enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_TOP_WIDTH, width),
            4, "width");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_TOP_HEIGHT, height),
            5, "height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the bottom area in the eye mask of the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Activates/deactivates the bottom area in the eye mask of the related
/// HIPAR enabled/eye diagram.
/// HIPAR width/Defines the width of the bottom rectangle in the eye mask of the
/// HIPAR width/related eye diagram.
/// HIPAR height/Defines the offset of the bottom rectangle in the mask of the related
/// HIPAR height/eye diagram. The offset is specified relative to the vertical center of
/// HIPAR height/the eye mask.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskBottom(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean enabled,
                                                  ViReal64  width,
                                                  ViReal64  height)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_BOTTOM_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_BOTTOM_ENABLED, enabled),
            3, "enabled");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_BOTTOM_WIDTH, width),
            4, "width");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_BOTTOM_HEIGHT, height),
            5, "height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the horizontal center of the eye mask in the related eye
/// HIFN  diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR width/Defines the horizontal center of the eye mask in the related eye
/// HIPAR width/diagram.
/// HIPAR height/Defines the vertical center of the eye mask in the related eye
/// HIPAR height/diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskCenter(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViReal64  width,
                                                  ViReal64  height)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_CENTER_WIDTH,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_CENTER_WIDTH, width),
            3, "width");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_CENTER_HEIGHT, height),
            4, "height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the fail condition for the eye mask test of the
/// HIFN  related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR failCondition/Defines whether the fail condition for the eye mask test of the
/// HIPAR failCondition/related eye diagram is specified in absolute or relative terms.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR toleranceRate/Defines the violation tolerance (i.e. the number of bad samples) or
/// HIPAR toleranceRate/the violation rate (i.e. the share of bad samples) for the mask test in
/// HIPAR toleranceRate/the related eye diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskFailCondition(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   failCondition,
                                                         ViInt32   toleranceRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_FAIL_CONDITION,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_FAIL_CONDITION, failCondition),
            3, "Fail Condition");

    if (failCondition == RSVNA_VAL_FAIL_CONDITION_SAMPLES)
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_VIOLATION_TOLERANCE, toleranceRate),
                4, "Tolerance Rate");
    }
    else
    {
        viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_VIOLATION_RATE, toleranceRate),
                4, "Tolerance Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the detailed results of the mask test in the related eye
/// HIFN  diagram (i.e. the contents of the corresponding result info field).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR bufferSize/Size of output array.
/// HIPAR data/Returns the detailed results of the mask test in the related eye
/// HIPAR data/diagram (i.e. the contents of the corresponding result info
/// HIPAR data/field).Attribute:
ViStatus _VI_FUNC rsvna_QueryTDREyeMaskData(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   bufferSize,
                                            ViChar    data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MASK_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MASK_DATA,
            bufferSize, data),
            3, "data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a user-defined eye mask from a 7bit ASCII file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/String parameter containing the path and file name of the eye mask
/// HIPAR fileName/file. If the path is omitted, the current directory is used
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskLoad(ViSession instrSession,
                                                ViString  traceName,
                                                ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", traceName, fileName);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_MASK_LOAD, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a user-defined eye mask to a 7bit ASCII file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/String parameter containing the path and file name of the eye mask
/// HIPAR fileName/file. If the path is omitted, the current directory is used
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskStore(ViSession instrSession,
                                                 ViString  traceName,
                                                 ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", traceName, fileName);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_MASK_STORE, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the detailed results of the mask test in the related eye diagram
/// HIFN  to an ASCII file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/Mandatory string parameter containing the path and file name of the
/// HIPAR fileName/file. If the path is omitted, the current directory is used
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMaskStoreResults(ViSession instrSession,
                                                        ViString  traceName,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s'", traceName, fileName);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_MASK_STORE_RESULTS, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the visibility of the result info field in the related eye
/// HIFN  diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Defines the visibility of the result info field in the related eye
/// HIPAR enabled/diagram.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMeasurementEnabled(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MEASUREMENT_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MEASUREMENT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measurement results of the related eye diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR data/Returns the measurement results of the related eye diagram.Attribute:
/// HIPAR channel/Channel number used to identify the active trace.
ViStatus _VI_FUNC rsvna_QueryTDREyeMeasurementData(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   bufferSize,
                                                   ViChar    data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MEASUREMENT_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MEASUREMENT_DATA,
            bufferSize, data),
            3, "data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper threshold that are used to calculate the
/// HIFN  transition times (rise/fall time) in an eye measurement. Use
/// HIFN  rsvna_ConfigureTDREyeMeasurementThresholdCustom to define custom lower
/// HIFN  and upper thresholds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR threshold/Defines the lower and upper threshold that are used to calculate the
/// HIPAR threshold/transition times (rise/fall time) in an eye measurement.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMeasurementThreshold(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MEASUREMENT_THRESHOLD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MEASUREMENT_THRESHOLD, threshold),
            3, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper threshold that are used to calculate the
/// HIFN  transition times (rise/fall time) in an eye measurement. Specify the
/// HIFN  thresholds manually as integers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR lower/Lower rise time threshold as integer percentage.
/// HIPAR upper/Upper rise time threshold as integer percentage.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMeasurementThresholdCustom(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   lower,
                            ViInt32   upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lower, 0, 100),
            3, "Lower");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upper, 0, 100),
            4, "Upper");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_EYE_MEASUREMENT_THRESHOLD_CUSTOM,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld", lower, upper);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_EYE_MEASUREMENT_THRESHOLD_CUSTOM, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows to export eye diagram results to an ASCII file (csv), with the
/// HIFN  possibility to specify a decimal separator (comma or point) and a field
/// HIFN  separator (semicolon, comma, tab or space).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceName/Name of the related eye diagram.
/// HIPAR fileName/Mandatory string parameter containing the path and file name of the
/// HIPAR fileName/file. If the path is omitted, the current directory is used
/// HIPAR decimalSeparator/Decimal separator.
/// HIPAR fieldSeparator/5Field separator.
ViStatus _VI_FUNC rsvna_ConfigureTDREyeMeasurementExport(ViSession instrSession,
                                                         ViString  traceName,
                                                         ViString  fileName,
                                                         ViInt32   decimalSeparator,
                                                         ViInt32   fieldSeparator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  decSepString;
    ViChar*  fieldSepString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngDecimalSeparator,
                    decimalSeparator, &decSepString), 4, "Decimal Separator");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsvna_rngFieldSeparator,
                    fieldSeparator, &fieldSepString), 5, "Field Separator");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s','%s',%s,%s",
             traceName, fileName, decSepString, fieldSepString);

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_TDR_EYE_MEASUREMENT_EXPORT, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the Rise Time Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables the Rise Time Measurement.
ViStatus _VI_FUNC rsvna_ConfigureRiseTimeEnabled(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_RISE_TIME_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_TDR_RISE_TIME_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the standard lower/upper threshold for the rise time measurement.
/// HIFN  Use rsvna_ConfigureRiseTimeThresholdCustom to set custom lower and
/// HIFN  upper threshold values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR threshold/Defines the lower/upper threshold for the rise time measurement.
ViStatus _VI_FUNC rsvna_ConfigureRiseTimeThreshold(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_RISE_TIME_THRESHOLD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_TDR_RISE_TIME_THRESHOLD, threshold),
            3, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper threshold for the rise time measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR lower/Lower rise time threshold as integer percentage.
/// HIPAR upper/Upper rise time threshold as integer percentage.
ViStatus _VI_FUNC rsvna_ConfigureRiseTimeThresholdCustom(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   lower,
                                                         ViInt32   upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lower, 0, 100),
            3, "Lower");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upper, 0, 100),
            4, "Upper");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_RISE_TIME_THRESHOLD_CUSTOM,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld", lower, upper);

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_TDR_RISE_TIME_THRESHOLD_CUSTOM, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the rise time measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR arraySize/Size of output array.
/// HIPAR data/Queries the results of the rise time measurement.
/// HIPAR returnedValues/Number of returned values.
ViStatus _VI_FUNC rsvna_QueryRiseTimeData(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   arraySize,
                                          ViReal64  data[],
                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 3, "Array size");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_TDR_RISE_TIME_DATA,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSVNA_ATTR_TDR_RISE_TIME_DATA, repCap, &cmd));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, returnedValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Skew Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR enabled/Enables/disables the Skew Measurement.
/// HIPAR referenceTrace/Selects the reference trace for the skew measurement.
/// HIPAR position/Defines the position of the skew measurement.
/// HIPAR failBeep/Defines whether the R&S ZNB/ZNBT should make an audible beep on skew
/// HIPAR failBeep/limit violations.
ViStatus _VI_FUNC rsvna_ConfigureSkew(ViSession instrSession,
                                      ViInt32   channel,
                                      ViBoolean enabled,
                                      ViString  referenceTrace,
                                      ViInt32   position,
                                      ViBoolean failBeep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SKEW_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SKEW_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_SKEW_REFERENCE_TRACE, referenceTrace),
            4, "Reference Trace");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_SKEW_POSITION, position),
            5, "Position");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SKEW_FAIL_BEEP, failBeep),
            6, "Fail Beep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the skew limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR checkOn/Activates/deactivates the skew limit check.
/// HIPAR skewLimit/Defines the limit value for the skew check.
ViStatus _VI_FUNC rsvna_ConfigureSkewLimitCheck(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean checkOn,
                                                ViReal64  skewLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SKEW_CHECK_ON,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SKEW_CHECK_ON, checkOn),
            3, "Check On");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SKEW_LIMIT, skewLimit),
            4, "Skew Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates whether the skew has passed or failed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR skewFail/Indicates whether the skew has passed or failed.
ViStatus _VI_FUNC rsvna_QuerySkewFail(ViSession  instrSession,
                                      ViInt32    channel,
                                      ViBoolean* skewFail)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SKEW_FAIL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_SKEW_FAIL, skewFail),
            3, "Skew Fail");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the skew measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR skewResult/Queries the results of the skew measurement.
ViStatus _VI_FUNC rsvna_QuerySkewResult(ViSession instrSession,
                                        ViInt32   channel,
                                        ViReal64* skewResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SKEW_RESULT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsvna_GetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_SKEW_RESULT, skewResult),
            3, "skewResult");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the skew measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number used to identify the active trace.
/// HIPAR arraySize/Size of output array.
/// HIPAR results/Queries the results of the skew measurement.
/// HIPAR returnedValues/Number of returned values.
ViStatus _VI_FUNC rsvna_QuerySkewAllResults(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   arraySize,
                                            ViReal64  results[],
                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1), 3, "Array size");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_SKEW_ALL_RESULTS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSVNA_ATTR_SKEW_ALL_RESULTS, repCap, &cmd));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the converter 'LO Out' port operates as independent CW
/// HIFN  source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Defines whether the converter 'LO Out' port operates as independent CW
/// HIPAR enabled/source.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorEnabled(ViSession instrSession,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_INDEPENDENT_GENERATOR_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If the converter 'LO Out' port is configured as an independent CW
/// HIFN  source, this command sets the frequency of the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/If the converter 'LO Out' port is configured as an independent CW
/// HIPAR frequency/source, this command sets the frequency of the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorFrequency(
                            ViSession instrSession,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_INDEPENDENT_GENERATOR_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If the converter 'LO Out' port is configured as an independent CW
/// HIFN  source, this command sets the power of the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR power/If the converter 'LO Out' port is configured as an independent CW
/// HIPAR power/source, this command sets the power of the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPower(ViSession instrSession,
                                                           ViReal64  power)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, NULL, RSVNA_ATTR_INDEPENDENT_GENERATOR_POWER, power),
            2, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If the converter 'LO Out' port is configured as an independent CW
/// HIFN  source, this attribute toggles the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfOff/If the converter 'LO Out' port is configured as an independent CW
/// HIPAR rfOff/source, this control toggles the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorRFOff(ViSession instrSession,
                                                           ViBoolean rfOff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, NULL, RSVNA_ATTR_INDEPENDENT_GENERATOR_RF_OFF, rfOff),
            2, "RF Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines whether port <Pt> operates as independent CW source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR enabled/Defines whether port <Pt> operates as independent CW source.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPortEnabled(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_ENABLED,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If RF port <Pt> is configured as an independent CW source, this
/// HIFN  command sets the frequency of the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR frequency/If RF port <Pt> is configured as an independent CW source, this
/// HIPAR frequency/command sets the frequency of the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPortFrequency(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_FREQUENCY,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If RF port <Pt> is configured as an independent CW source, this
/// HIFN  command sets the power of the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR power/If RF port <Pt> is configured as an independent CW source, this
/// HIPAR power/command sets the power of the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPortPower(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_POWER,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_POWER, power),
            3, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If RF port <Pt> is configured as an independent CW source, this
/// HIFN  command toggles the CW signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR rfOff/If RF port <Pt> is configured as an independent CW source, this
/// HIPAR rfOff/command toggles the CW signal.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPortRFOff(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViBoolean rfOff)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_RF_OFF,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_RF_OFF, rfOff),
            3, "RF Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If RF port <Pt> is configured as an independent CW source and equipped
/// HIFN  with a source step attenuator, this command sets the source attenuation
/// HIFN  to be applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalPort/Physical port number.
/// HIPAR stepAttenuator/If RF port <Pt> is configured as an independent CW source and equipped
/// HIPAR stepAttenuator/with a source step attenuator, this command sets the source attenuation
/// HIPAR stepAttenuator/to be applied.
ViStatus _VI_FUNC rsvna_ConfigureIndependentGeneratorPortStepAttenuator(
                            ViSession instrSession,
                            ViInt32   physicalPort,
                            ViReal64  stepAttenuator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_STEP_ATTENUATOR,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Physical Port");

    viCheckParm(rsvna_SetAttributeViReal64(instrSession, repCap, RSVNA_ATTR_INDEPENDENT_GENERATOR_PORT_STEP_ATTENUATOR, stepAttenuator),
            3, "Step Attenuator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the markers to be displayed in the Info Window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Marker number
/// HIPAR tracename/Name of the related trace.
/// HIPAR displayed/Marker <Mk> of trace <TraceName> is displayed.
ViStatus _VI_FUNC rsvna_ConfigureDisplayInfoWindowMarkerSearchResults(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViString  tracename,
                            ViBoolean displayed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, marker, 1), 2, "Marker");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s',%s", tracename, offOnArr[displayed]);

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_DISPLAY_INFO_WINDOW_MARKER_SEARCH_RESULTS,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    checkErr(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_DISPLAY_INFO_WINDOW_MARKER_SEARCH_RESULTS, parameter));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the identifiers of the available dialog opener actions as a
/// HIFN  comma-separated list of strings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR catalog/Displays the identifiers of the available dialog opener actions as a
/// HIPAR catalog/comma-separated list of strings.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryMenuKeyDialogCatalog(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    catalog[],
                                                  ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViRawStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_DISPLAY_MENU_KEY_DIALOG_CATALOG, RS_VAL_DIRECT_USER_CALL,
                    bufferSize, catalog, responseLength),
                3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Opens the dialog identified by <MenuKey>. Use
/// HIFN  rsvna_QueryMenuKeyDialogCatalog to retrieve a list of the available
/// HIFN  opener IDs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR menuKey/The opener ID of the selected dialog.
ViStatus _VI_FUNC rsvna_DisplayMenuKeyDialog(ViSession instrSession,
                                             ViString  menuKey)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_MENU_KEY_DIALOG, menuKey),
            2, "Menu Key");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the identifiers of the available softtool tabs as a
/// HIFN  comma-separated list of strings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR catalog/Displays the identifiers of the available softtool tabs as a
/// HIPAR catalog/comma-separated list of strings.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryMenuKeyTabCatalog(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    catalog[],
                                               ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViRawStringWithRealLen(instrSession, NULL,
                    RSVNA_ATTR_DISPLAY_MENU_KEY_TAB_CATALOG, RS_VAL_DIRECT_USER_CALL,
                    bufferSize, catalog, responseLength),
                3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the softtool tab with identifier <MenuKey>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR menuKey/Activates the softtool tab with identifier <MenuKey>.
ViStatus _VI_FUNC rsvna_DisplayMenuKeyTab(ViSession instrSession,
                                          ViString  menuKey)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, NULL, RSVNA_ATTR_DISPLAY_MENU_KEY_TAB, menuKey),
            2, "Menu Key");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies whether a preset (SYSTem:PRESet; *RST) affects
/// HIFN  the active recall set only or all open recall sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR presetScope/This control specifies whether a preset (SYSTem:PRESet; *RST) affects
/// HIPAR presetScope/the active setup only or all open setups.
ViStatus _VI_FUNC rsvna_ConfigurePresetScope(ViSession instrSession,
                                             ViInt32   presetScope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_PRESET_SCOPE, presetScope),
            2, "Preset Scope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the behavior of the *RST and
/// HIFN  SYSTem:PRESet[:DUMMy] commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR align_RSTtoUserDefPreset/Defines the behavior of the *RST and SYSTem:PRESet commands.
ViStatus _VI_FUNC rsvna_ConfigureRemotePreset(ViSession instrSession,
                                              ViBoolean alignRSTToUserDefPreset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_REMOTE_PRESET, alignRSTToUserDefPreset),
            2, "Align RST to User Def Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the analyzer firmware always starts with the
/// HIFN  configured system preset (factory or user defined) or with the previous
/// HIFN  state. This is a global setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startInPreset/Defines whether the analyzer firmware always starts with the
/// HIPAR startInPreset/configured system preset (factory or user defined) or with the previous
/// HIPAR startInPreset/state. This is a global setting.
ViStatus _VI_FUNC rsvna_ConfigureStartInPreset(ViSession instrSession,
                                               ViBoolean startInPreset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_START_IN_PRESET, startInPreset),
            2, "Start In Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs a factory preset of all instrument settings
/// HIFN  (i.e. all open recall sets) or of the active recall set, depending on
/// HIFN  the function rsvna_ConfigurePresetScope settings, and deletes channel 1
/// HIFN  in the active recall set. As a result, the active recall set contains
/// HIFN  no channels, traces, and diagram areas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_FactoryPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_FACTORY_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the behavior of the Restart Sweep function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setAllTracesto0/Defines the behavior of the INITiate<Ch>[:IMMediate][:DUMMy] and
/// HIPAR setAllTracesto0/INITiate[:IMMediate]:ALL.
ViStatus _VI_FUNC rsvna_ConfigureSetAllTracesto0(ViSession instrSession,
                                                 ViBoolean setAllTracesto0)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_SET_ALL_TRACES_TO_0, setAllTracesto0),
            3, "Set All Traces to 0");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a factory preset or a user-defined preset and
/// HIFN  sets the name of a recall set file (.znx) to be used for a user-defined
/// HIFN  preset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR userDefinedPreset/This control selects a factory preset or a user-defined preset.
/// HIPAR userDefinedPresetFile/This control specifies the name and directory of the recall set file
/// HIPAR userDefinedPresetFile/to be loaded. The default extension (manual control) for setup files is
/// HIPAR userDefinedPresetFile/*.znx, although other extensions are allowed. If no path is specified
/// HIPAR userDefinedPresetFile/the analyzer searches the default directory.
ViStatus _VI_FUNC rsvna_ConfigureUserDefinedPreset(ViSession instrSession,
                                                   ViBoolean userDefinedPreset,
                                                   ViString  userDefinedPresetFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_USER_DEFINED_PRESET_STATE, userDefinedPreset),
            2, "User Defined Preset");

    if (userDefinedPreset == VI_TRUE)
    {
        viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_USER_DEFINED_PRESET_NAME, userDefinedPresetFile),
                3, "User Defined Preset File");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a calibration from the calibration pool that shall be
/// HIFN  restored during a user-defined preset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR userDefinedPresetCalibration/This control selects a factory preset or a user-defined preset.
ViStatus _VI_FUNC rsvna_ConfigureUserDefinedPresetCalibration(
                            ViSession instrSession,
                            ViString  userDefinedPresetCalibration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_USER_DEFINED_PRESET_CAL, userDefinedPresetCalibration),
            2, "User Defined Preset Calibration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the averaging during calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR averaging/This control selects the averaging during calibration.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationAutoAveraging(ViSession instrSession,
                                                          ViInt32   averaging)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_CALIBRATION_AUTO_AVERAGING, averaging),
            2, "Avaraging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function locks or unlocks the local controls of the analyzer.
/// HIFN  This includes the front panel keys, the keyboard, or other local
/// HIFN  interfaces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR keyLockout/Locks or unlocks the local controls of the analyzer.
ViStatus _VI_FUNC rsvna_ConfigureKeyLock(ViSession instrSession,
                                         ViBoolean keyLockout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_KEY_LOCK, keyLockout),
            2, "Key Lockout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function locks/unlocks the touchscreen functionality.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Locks/unlocks the touchscreen functionality.
ViStatus _VI_FUNC rsvna_ConfigureTouchscreenLock(ViSession instrSession,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_TOUCHSCREEN_LOCK, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function labels the functionality of a function key to
/// HIFN a user-defined key in the remote display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR key/Number of the user key.
/// HIPAR label/Label for user key no. 1 to 8
/// HIPAR function/Name of a function key, e.g. 'S11', 'Start' etc.
ViStatus _VI_FUNC rsvna_ConfigureUserKeyFunction(ViSession instrSession,
                                                 ViInt32   key,
                                                 ViString  label,
                                                 ViString  function)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, key, 0, 8),
            2, "Key");

    if (key == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:USER:KEY %ld", key);
        checkErr(RsCore_Write(instrSession, cmd));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:USER:KEY %ld,'%s'", key, label);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsvna_CheckStatus(instrSession));

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:USER:KEY:FUNC %ld,'%s'", key, function);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the Remote Language for the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR language/This parameter selects the command set.
ViStatus _VI_FUNC rsvna_ConfigureRemoteLanguage(ViSession instrSession,
                                                ViInt32   language)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_REMOTE_LANGUAGE, language),
            2, "Language");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets a channel-dependent eight-bit binary value
/// HIFN  (00000000 ... 11111111) to control eight independent
/// HIFN     output signals at the USER PORT connector (lines 8, 9, 10, 11 and
/// HIFN  lines 16, 17, 18, 19).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelBits/This control sets a channel-dependent eight-bit binary value to
/// HIPAR channelBits/control eight independent output signals at the USER PORT connector
/// HIPAR channelBits/(lines 8, 9, 10, 11 and lines 16, 17, 18, 19). The output signals are
/// HIPAR channelBits/3.3 V TTL signals which can be used to differentiate between up to 255
/// HIPAR channelBits/independent analyzer states.
/// HIPAR pin1617/Defines the usage of pins pins 16 to 19 of the USER PORT connector.
ViStatus _VI_FUNC rsvna_ConfigureChannelBits(ViSession instrSession,
                                             ViInt32   channel,
                                             ViString  channelBits,
                                             ViInt32   pin1617)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT, channel);

    viCheckParm(rsvna_SetAttributeViString(instrSession, repCap, RSVNA_ATTR_USER_PORT_CHANNEL_BITS, channelBits),  // NOLINT(readability-suspicious-call-argument)
            2, "Channel Bits");

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "",
                    RSVNA_ATTR_USER_PORT_PINS_USAGE,
                    (ViBoolean) pin1617
                ), 3, "Pin 16 17");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables segment bits for the sweep segments in channel no.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR segment/Number of the segment.
/// HIPAR enabled/Enables or disables segment bits for the sweep segments in channel no.
/// HIPAR enabled/<Ch>; see OUTPut<Ch>:UPORt:SEGMent<Seg>[:VALue]. The command is valid
/// HIPAR enabled/for segmented frequency sweep.
/// HIPAR value/Sets or queries a sweep segment-dependent four-bit binary value to
/// HIPAR value/control four independent output signals at the USER PORT connector
/// HIPAR value/(lines 16, 17, 18, 19). The output signals are 3.3 V TTL signals which
/// HIPAR value/can be used to differentiate between up to 16 independent analyzer
/// HIPAR value/states for each channel.
ViStatus _VI_FUNC rsvna_ConfigureUserPortOptionalColumns(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViInt32   segment,
                                                         ViBoolean enabled,
                                                         ViInt32   value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, channel, 1), 2, "Channel");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, segment, 1), 3, "Segment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, CHAN_RC_FORMAT ",Seg%ld", channel, segment);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_USER_PORT_OPTIONAL_COLUMNS_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_USER_PORT_OPTIONAL_COLUMNS, value),
            5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function generates a system report and writes it to the specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reportFileName/This control specifies the file name. If no path is specified, the
/// HIPAR reportFileName/file is stored to the directory C:/Users/Public/Documents/Vna/Report;
/// HIPAR reportFileName/the extension *.zip is appended automatically.
/// HIPAR timeout/Sets the timeout for the operation to complete.
ViStatus _VI_FUNC rsvna_SaveReport(ViSession instrSession,
                                   ViInt32   timeout,
                                   ViString  reportFileName)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_GetOPCTimeout(instrSession, &old_timeout));

    checkErr(rsvna_SetOPCTimeout(instrSession, timeout));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_SAVE_REPORT, reportFileName),
            2, "Report File Name");

    checkErr(rsvna_SetOPCTimeout(instrSession, old_timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables logging of all remote control commands
/// HIFN  transferred to the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR remoteLogging/Enables logging of all remote control commands transferred to the
/// HIPAR remoteLogging/analyzer.
ViStatus _VI_FUNC rsvna_ConfigureRemoteLogging(ViSession instrSession,
                                               ViBoolean remoteLogging)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_REMOTE_LOGGING, remoteLogging),
            2, "Remote Logging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the character encoding used at the remote
/// HIFN  interface. The selected encoding applies to directory and file names,
/// HIFN  CalKit names, CalUnit characterizations and display titles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR encoding/Selects the character encoding used at the remote interface.
ViStatus _VI_FUNC rsvna_ConfigureRemoteEncoding(ViSession instrSession,
                                                ViInt32   encoding)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_REMOTE_ENCODING, encoding),
            2, "Encoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and disables the calibration manager.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Enables and disables the calibration manager.
ViStatus _VI_FUNC rsvna_ConfigureCalibrationManagerState(ViSession instrSession,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_CALIBRATION_MANAGER_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a title for the remote display, i.e. for the screen that is
/// HIFN  shown at the instrument, if a remote session is established and the
/// HIFN  standard display is switched off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayTitle/Defines a title for the remote display, i.e. for the screen that is
/// HIPAR displayTitle/shown at the instrument, if a remote session is established and the
/// HIPAR displayTitle/standard display is switched off.
ViStatus _VI_FUNC rsvna_ConfigureDisplayTitle(ViSession instrSession,
                                              ViString  displayTitle)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_TITLE, displayTitle),
            2, "Display Title");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum number of trace colors (trace properties for the
/// HIFN  user defined color scheme defined using DISPlay:CMAP<DispEl>:RGB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofTraceColours/Defines the maximum number of trace colors (trace properties for the
/// HIPAR numberofTraceColours/user defined color scheme defined using DISPlay:CMAP<DispEl>:RGB.
ViStatus _VI_FUNC rsvna_ConfigureDisplayNumberofTraceColours(ViSession instrSession,
                                                             ViInt32   numberofTraceColours)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_DISPLAY_NUMBER_OF_TRACE_COLOURS, numberofTraceColours),
            2, "Number of Trace Colours");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches on or off the update of all display
/// HIFN elements during remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayInRemote/Switches the display on or off while the analyzer is in the remote state.
ViStatus _VI_FUNC rsvna_ConfigureDisplayUpdate(ViSession instrSession,
                                               ViBoolean displayInRemote)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, displayInRemote),
            2, "Display In Remote");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches the display on and show the current
/// HIFN trace. This can be used for occasional checks of the measurement
/// HIFN results or settings. The measurement is continued, however, the
/// HIFN measurement results are not updated. Compared to the ON setting,
/// HIFN ONCE does not slow down the measurement speed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsvna_DisplayUpdateOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of all SCPI commands supported by the related parser
/// HIFN  (or the default parser, if omitted).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parser/Optionally specifies a parser. Leave blank (or NULL) to use the
/// HIPAR parser/default parser.
/// HIPAR bufferSize/Size of output array.
/// HIPAR scpiCommands/Returns a list of all SCPI commands supported by the related parser.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QuerySystemSCPICommands(ViSession instrSession,
                                                ViString  parser,
                                                ViInt32   bufferSize,
                                                ViChar    scpiCommands[],
                                                ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   qParam[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if ((parser != NULL) && (strlen(parser) > 0))
    {
        snprintf(qParam, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "'%s'", parser);
    }

    viCheckParm(RsCore_GetAttributeViStringWithQparam(instrSession, NULL,
                    RSVNA_ATTR_SYSTEM_SCPI_COMMANDS, qParam,
                    bufferSize, scpiCommands, responseLength), 4, "SCPI Commands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the implemented SCPI commands and their parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR scpiCommands/Queries the implemented SCPI commands and their parameters.
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_QueryImplementedSCPICommands(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    scpiCommands[],
                                                     ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithQparam(instrSession, NULL,
                    RSVNA_ATTR_QUERY_IMPLEMENTED_SCPI_COMMANDS, NULL,
                    bufferSize, scpiCommands, responseLength), 2, "SCPI Commands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the full syntax of the specified command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/String containing the command you want to query.
/// HIPAR bufferSize/Size of output array.
/// HIPAR syntax/Queries the implemented SCPI commands and their parameters.Attribute:
/// HIPAR responseLength/The length of the response from the instrument, in bytes. If this is
/// HIPAR responseLength/larger than the allocated Buffer Size, some of the response was lost.
ViStatus _VI_FUNC rsvna_SystemHelpSyntax(ViSession instrSession,
                                         ViString  command,
                                         ViInt32   bufferSize,
                                         ViChar    syntax[],
                                         ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
                2, "Command");

    snprintf(parameter, RS_MAX_MESSAGE_BUF_SIZE, "'%s'", command);

    checkErr(RsCore_GetAttributeViStringWithQparam(instrSession, NULL,
        RSVNA_ATTR_HIDDEN_SYSTEM_HELP_SYNTAX,
        parameter, bufferSize, syntax, responseLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Suppresses/unsuppresses Device Clear GPIB interface messages (DCL, SDC).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gpibSupresses/Suppresses/unsuppresses of Device Clear GPIB interface messages (DCL,
/// HIPAR gpibSupresses/SDC).
ViStatus _VI_FUNC rsvna_ConfigureGPIBSupresses(ViSession instrSession,
                                               ViBoolean gpibSupresses)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, "", RSVNA_ATTR_GPIB_SUPRESSES, gpibSupresses),
            2, "GPIB Supresses");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or gets the host name of the instrument.
/// HIFN     The default host name is ZNB-<serial no.> for R&S, ZNBZNBT-<serial
/// HIFN  no.> for R&S ZNBT
/// HIFN     The default host name is ZND-<serial no.>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hostname/Sets or gets the host name of the instrument.
ViStatus _VI_FUNC rsvna_ConfigureHostname(ViSession instrSession,
                                          ViString  hostname)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_HOSTNAME, hostname),
            2, "Hostname");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Keysight-compatible command to open the Start Cal Tab or the
/// HIFN  Calibration Kits Dialog.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dialog/Keysight-compatible command to open the Start Cal Tab or the
/// HIPAR dialog/Calibration Kits Dialog.
ViStatus _VI_FUNC rsvna_DisplayCorrectionWizardDialog(ViSession instrSession,
                                                      ViInt32   dialog)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_CORRECTION_WIZARD_DIALOG, dialog),
            2, "Dialog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the frequency of the incoming reference clock signal at the
/// HIFN  Reference In/Out SMA connector (see [SENSe<Ch>:]ROSCillator[:SOURce]).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputFrequency/Specifies the frequency of the incoming reference clock signal at the
/// HIPAR inputFrequency/Reference In/Out SMA connector (see [SENSe<Ch>:]ROSCillator[:SOURce]).
ViStatus _VI_FUNC rsvna_ConfigureReferenceOscillatorSMAInputFrequency(
                            ViSession instrSession,
                            ViInt32   inputFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_REFERENCE_OSCILLATOR_SMA_INPUT_FREQUENCY, inputFrequency),
            2, "Input Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines whether a reference clock signal shall be output to the
/// HIFN  Reference In/Out SMA connector.
/// HIFN
/// HIFN     Ineffective if this connector is used as reference clock source (see
/// HIFN  [SENSe<Ch>:]ROSCillator[:SOURce]).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputFrequency/Defines whether a reference clock signal shall be output to the
/// HIPAR outputFrequency/Reference In/Out SMA connector.
ViStatus _VI_FUNC rsvna_ConfigureReferenceOscillatorSMAOutputFrequency(
                            ViSession instrSession,
                            ViInt32   outputFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_REFERENCE_OSCILLATOR_SMA_OUTPUT_FREQUENCY, outputFrequency),
            2, "Output Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the User Defined LED at the front panel of the R&SZNA. This
/// HIFN  is a persistent setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ledColor/Configures the User Defined LED at the front panel of the R&SZNA. This
/// HIPAR ledColor/is a persistent setting.
ViStatus _VI_FUNC rsvna_ConfigureUserDefinedLedColor(ViSession instrSession,
                                                     ViInt32   ledColor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_USER_DEFINED_LED_COLOR, ledColor),
            2, "LED Color");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to define how the R&SZNA turns the RF power off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfOffBehavior/Allows you to define how the R&SZNA turns the RF power off.
ViStatus _VI_FUNC rsvna_ConfigureRFOffBehavior(ViSession instrSession,
                                               ViInt32   rfOffBehavior)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_RF_OFF_BEHAVIOR, rfOffBehavior),
            2, "RF Off Behavior");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Allows you to select between different preset modes.
/// HIFN
/// HIFN     GLOBal and RFOFf presets can only be performed from the analyzer GUI.
/// HIFN  FILE presets can also be performed via remote commands, if
/// HIFN  SYSTem:PRESet:REMote[:STATe] is set to ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR systemPresetMode/Allows you to select between different preset modes.
ViStatus _VI_FUNC rsvna_ConfigureSystemPresetMode(ViSession instrSession,
                                                  ViInt32   systemPresetMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_SYSTEM_PRESET_MODE, systemPresetMode),
            2, "System Preset Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Like the *TST?, this operator check performs a self test of the R&S
/// HIFN  ZNB/ZNBT and returns its result. However, the role of boolean results 0
/// HIFN  and 1 is exactly opposite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selftest/Like the *TST?, this operator check performs a self test of the R&S
/// HIPAR selftest/ZNB/ZNBT and returns its result. However, the role of boolean results 0
/// HIPAR selftest/and 1 is exactly opposite.
ViStatus _VI_FUNC rsvna_QueryOPCCheckSelftest(ViSession instrSession,
                                              ViInt32*  selftest)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_OPC_CHECK_SELFTEST, selftest),
            2, "Self Test");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines and executes a power level operator check on port <Port> and
/// HIFN  returns its boolean result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR power/Source power of port <Port>.
/// HIPAR limitLines/String parameter specifying the name and directory of the limit line
/// HIPAR limitLines/file to be loaded.
/// HIPAR checkPort/Boolean result of the Check Port.
ViStatus _VI_FUNC rsvna_QueryLevelOPCCheckPort(ViSession  instrSession,
                                               ViInt32    physicalPort,
                                               ViReal64   power,
                                               ViString   limitLines,
                                               ViBoolean* checkPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   params[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_HIDDEN_POWER_LEVEL_OPC_CHECK_PORT,
            "Port", physicalPort - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    snprintf (params, RS_PARAM_BUF_SIZE, "%.12lG,'%s'", power, limitLines);

    checkErr(RsCore_GetAttributeViBooleanWithQparam(instrSession, repCap, RSVNA_ATTR_HIDDEN_POWER_LEVEL_OPC_CHECK_PORT, params, checkPort));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the format of the factory ID string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR formatIdentify/Defines the format of the factory ID string.
ViStatus _VI_FUNC rsvna_ConfigureFormatIdentify(ViSession instrSession,
                                                ViInt32   formatIdentify)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, NULL, RSVNA_ATTR_FORMAT_IDENTIFY, formatIdentify),
            2, "Format Identify");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function a write data to ports E, F, G, H. To write data to a
/// HIFN  port, the port must be configured as an output port (see example). The
/// HIFN  port lines have negative logic: A "0" at a pin corresponds to a high
/// HIFN  signal, a "1" to a low signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/This Control defines the output port of the Universal Interface.
/// HIPAR data/Port data.
ViStatus _VI_FUNC rsvna_ConfigureUIData(ViSession instrSession,
                                        ViInt32   port,
                                        ViInt32   data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_UI_PORT_DATA,
            "UIPort", port - 4, RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_UI_PORT_DATA, data),
            3, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the direction of the data flow at ports A, B, C, D.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This Control defines direction of the data flow at ports A, B,
/// HIFN C, D.
/// HIPAR data/Write data to ports A, B, C, D.
ViStatus _VI_FUNC rsvna_ConfigureUIDataFlowDirection(ViSession instrSession,
                                                     ViInt32   port,
                                                     ViInt32   mode,
                                                     ViInt32   data)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_UI_PORT_DATA_MODE,
            "UIPortInit", port , RS_REPCAP_BUF_SIZE, repCap),
            2, "Port");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_UI_PORT_DATA_MODE, mode),
            3, "Mode");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_UI_PORT_WRITING_DATA, data),
            4, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the state of the output ports (pin 3 or 4) of the
/// HIFN  Universal Interface connector after the next negative pulse on
/// HIFN  the /INPUT1 line (pin 2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputPortNumber/Sets the output port number.
/// HIPAR data/Writes a 0 or 1 to the output ports (pin 3 or 4) of the
/// HIRET Universal Interface connector. The port lines have negative logic: A "0" corresponds
/// HIRET to a high signal, a "1" to a low signal.
/// HIPAR state/Defines the state of the output ports, (pin 3 or 4) of the
/// HIRET Universal Interface connector after the next negative pulse on the /INPUT1 line (pin 2).
ViStatus _VI_FUNC rsvna_ConfigureUIOutputPortsState(ViSession instrSession,
                                                    ViInt32   outputPortNumber,
                                                    ViInt32   data,
                                                    ViInt32   state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "OutPort%ld", outputPortNumber);

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_UI_OUTPUT_PORT_DATA, data),
            3, "Data");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_UI_OUTPUT_PORT_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the logic of the Data ports A-H on the
/// HIFN  Handler connector. Some of these lines are connected internally to the
/// HIFN  AuxIO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR logic/Sets the ports logic.
ViStatus _VI_FUNC rsvna_ConfigureUIDataPortsLogic(ViSession instrSession,
                                                  ViInt32   logic)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_DATA_PORTS_LOGIC, logic),
            2, "Logic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures Pass Fail.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR logic/Sets the logic of the PassFail line.
/// HIPAR defaultState/Sets the default logical pass/fail state This is the state the
/// HIPAR defaultState/pass/fail line stays in until a failure occurs (if the mode is NO
/// HIPAR defaultState/Wait), or until an end-of-sweep condition occurs (if the mode is PASS
/// HIPAR defaultState/or FAIL).
/// HIPAR scope/Sets the scope of pass/fail testing. The pass/fail line on the
/// HIPAR scope/material handler port can report the status of all measurements on each
/// HIPAR scope/channel, or all measurements on all channels.
/// HIPAR policy/Sets the policy used to determine how global pass/fail is computed.
ViStatus _VI_FUNC rsvna_ConfigureUIPassFail(ViSession instrSession,
                                            ViInt32   logic,
                                            ViInt32   defaultState,
                                            ViInt32   scope,
                                            ViInt32   policy)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_PASS_FAIL_LOGIC, logic),
            2, "Logic");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_PASS_FAIL_DEFAULT_STATE, defaultState),
            3, "Default State");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_PASS_FAIL_SCOPE, scope),
            4, "Scope");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_PASS_FAIL_POLICY, policy),
            5, "Policy");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the event that will cause the Handler Sweep
/// HIFN  End line to strobe. The strobe is at least 10ms in duration, and is
/// HIFN  activated when all calculations for the associated measurement are
/// HIFN  complete. This line is connected internally to the Sweep End line of
/// HIFN  the AUX IO connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR event/Specifies the event that will cause the Handler Sweep End line to
/// HIPAR event/strobe.
ViStatus _VI_FUNC rsvna_ConfigureUISweepEndEvent(ViSession instrSession,
                                                 ViInt32   event)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_SWEEP_END_EVENT, event),
            2, "Event");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the digital signal that is routed to pin 20 and pin 21 of the Universal Interface connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Selects the operation.
/// HIPAR digitalSignal/This control selects the digital signal that is routed to pin 20
/// HIRET and pin 21 of the Universal Interface connector.
ViStatus _VI_FUNC rsvna_ConfigureUIDigitalSignal(ViSession instrSession,
                                                 ViInt32   signal,
                                                 ViBoolean digitalSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", SignalRoutingArr[signal]);

    viCheckParm(rsvna_SetAttributeViBoolean(instrSession, repCap, RSVNA_ATTR_UI_DIGITAL_SIGNAL, digitalSignal),
            3, "Digital Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent pass/fail status value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR status/Pass/fail status value.
ViStatus _VI_FUNC rsvna_QueryUIPassFailStatus(ViSession instrSession,
                                              ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_PASS_FAIL_STATUS, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries a hardware latch that captures high to low
/// HIFN  transitions on Input1 of the Material Handler IO.  Reading the latch
/// HIFN  causes it to reset and is ready for the next transition. The hardware
/// HIFN  latch is only capable of capturing one transition per query. Additional
/// HIFN  transitions are ignored until after the  next query.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR status/Transition status
ViStatus _VI_FUNC rsvna_QueryUITransitionStatus(ViSession instrSession,
                                                ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, "", RSVNA_ATTR_UI_TRANSITION_STATUS, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the source of the reference oscillator signal and
/// HIFN  frequency of the external reference oscillator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reference/This control selects the source of the reference oscillator signal.
ViStatus _VI_FUNC rsvna_ConfigureReference(ViSession instrSession,
                                           ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_REFERENCE_OSCILATOR_SOURCE, reference),
            2, "Reference");

    if (reference == RSVNA_VAL_EXT)
    {
        checkErr(rsvna_SetAttributeViInt32(instrSession, "", RSVNA_ATTR_EXTERNAL_REFERENCE_OSCILATOR_FREQUENCY, 10000000));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the Enable, NTransition, and PTransition bits
/// HIFN  of the questionable registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR ptransition/register.
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR ntransition/register.
ViStatus _VI_FUNC rsvna_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViInt32   enable,
                                          ViInt32   ptransition,
                                          ViInt32   ntransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_reg[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (registerOperation == RSVNA_VAL_STATUS_REGISTER_PRESET)
    {
        checkErr(rsvna_SetAttributeViString(instrSession, "", RSVNA_ATTR_PRESET_STATUS_REGISTERS, NULL));
        goto Error;
    }

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_QUESTIONABLE_REGISTER,
            "QRegister", questionableRegister , RS_REPCAP_BUF_SIZE, repCap_reg),
            2, "Questionable Register");

    strcpy(repCap, repCap_reg);
    strcat(repCap, ",");
    strcat(repCap, "RegEnable");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUESTIONABLE_REGISTER, enable),
            3, "Enable");

    strcpy(repCap, repCap_reg);
    strcat(repCap, ",");
    strcat(repCap, "RegNTR");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUESTIONABLE_REGISTER, ntransition),
            5, "NTR Transition");

    strcpy(repCap, repCap_reg);
    strcat(repCap, ",");
    strcat(repCap, "RegPTR");

    viCheckParm(rsvna_SetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUESTIONABLE_REGISTER, ptransition),
            4, "PTR Transition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR questionableRegister/This control selects the registers that will be queried.
/// HIPAR registerPart/Selects register part to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsvna_GetStatusRegister(ViSession instrSession,
                                          ViInt32   questionableRegister,
                                          ViInt32   registerPart,
                                          ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_reg[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_part[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_QUESTIONABLE_REGISTER,
            "QRegister", questionableRegister , RS_REPCAP_BUF_SIZE, repCap_reg),
            2, "Questionable Register");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSVNA_ATTR_QUESTIONABLE_REGISTER,
            "RegisterPart", registerPart , RS_REPCAP_BUF_SIZE, repCap_part),
            3, "Register Part");

    strcpy(repCap, repCap_reg);
    strcat(repCap, ",");
    strcat(repCap, repCap_part);

    viCheckParm(rsvna_GetAttributeViInt32(instrSession, repCap, RSVNA_ATTR_QUESTIONABLE_REGISTER, registerValue),
            4, "Register Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsvna_GetError(ViSession instrSession,
                                 ViStatus* errorCode,
                                 ViInt32   bufferSize,
                                 ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsvna_utility_error_message, errorCode, bufferSize, description));

Error:
    // Unlock Session
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
ViStatus _VI_FUNC rsvna_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsvna_WriteInstrData(ViSession instrSession,
                                       ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, writeBuffer));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsvna_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 10000000), 2, "Command(null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsvna_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 10000000), 2, "Command(null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsvna_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 10000000), 2, "Command(null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsvna_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViInt32   bufferSize,
                                      ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViStatus warning = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 10000000), 2, "Command(null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, command, bufferSize, value, NULL));
    checkErrKeepWarn(rsvna_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rsvna_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsvna_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsvna_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function.
ViStatus _VI_FUNC rsvna_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Set OPC Timeout
 * Purpose:  Sets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rsvna_SetOPCTimeout(ViSession instrSession,
                                      ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get OPC Timeout
 * Purpose:  Gets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rsvna_GetOPCTimeout(ViSession instrSession,
                                      ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeId
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsvna_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
